Package: GSEAlens
Type: Package
Title: Gene Set Enrichment Analysis Interactive Explorer
Version: 0.99.35
Authors@R: c(person(given = "Shenhui", family = "Xu", role = c("aut", "cre"), email = "sealgod@qq.com", comment = c(ORCID = "0000-0002-2616-5132")), person(given = "Yuanhang", family = "Zhao", role = "ctb", email = "zy0928hang@163.com"), person(given = "Mireia", family = "Ramos-Rodríguez", role = "rev", comment = c(URL = "https://github.com/mireia-bioinfo")))
Description: GSEAlens provides an interactive exploration layer on top of
    standard Bioconductor RNA-seq workflows. It consumes fitted model objects
    from limma (MArrayLM) or DESeq2 (DESeqDataSet) as input and accepts
    expression matrices and sample metadata as SummarizedExperiment objects,
    ensuring interoperability with core Bioconductor data containers. For core
    computation, GSEAlens wraps clusterProfiler::GSEA() as its statistical
    engine (thereby inheriting the fgsea fast GSEA methodology) and draws on
    MSigDB gene set collections via the msigdbr package from CRAN; multi-contrast
    parallel computation is handled by future (future::multisession).
    Visualization output relies on Bioconductor graphics packages including
    enrichplot, ComplexHeatmap, and circlize, producing figures suitable for
    publication pipelines. The package also includes a built-in Shiny application
    for interactive exploration of enrichment results after DEG analysis, with
    the ability to export self-contained reproducible R scripts.
License: MIT + file LICENSE
URL: https://github.com/DDL095/GSEAlens
BugReports: https://github.com/DDL095/GSEAlens/issues
Encoding: UTF-8
RoxygenNote: 8.0.0
biocViews: GeneSetEnrichment, Visualization, ShinyApps, Software
Imports: stats, utils, enrichplot, enrichit, grDevices, graphics,
        methods, shiny, shinycssloaders, DT, plotly, ggplot2, rlang,
        igraph, dplyr, tidyr, tibble, stringr, patchwork,
        ComplexHeatmap, circlize, grid, clusterProfiler, limma, edgeR,
        DESeq2, SummarizedExperiment, S4Vectors, msigdbr, future,
        future.apply, htmltools, htmlwidgets, jsonlite, clipr,
        progressr, visNetwork, shinyjs, withr
Suggests: testthat (>= 3.0.0), BiocStyle, knitr, rmarkdown, airway,
        ggrepel
Depends: R (>= 4.6.0)
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/GSEAlens
git_branch: devel
git_last_commit: d1807ab
git_last_commit_date: 2026-08-19
Repository: Bioconductor 3.24
Date/Publication: 2026-08-19
NeedsCompilation: no
Packaged: 2026-08-19 22:25:37 UTC; biocbuild
Author: Shenhui Xu [aut, cre] (ORCID: <https://orcid.org/0000-0002-2616-5132>),
  Yuanhang Zhao [ctb],
  Mireia Ramos-Rodríguez [rev] (URL: https://github.com/mireia-bioinfo)
Maintainer: Shenhui Xu <sealgod@qq.com>

# Generated by roxygen2: do not edit by hand

S3method(get_contrast_registry,GseaEnv)
S3method(get_contrast_registry,GseaRes)
S3method(get_de_table,GseaRes)
S3method(get_expr_matrix,GseaEnv)
S3method(get_expr_matrix,GseaRes)
S3method(get_geneset_info,GseaEnv)
S3method(get_geneset_info,GseaRes)
S3method(get_sample_meta,GseaEnv)
S3method(get_sample_meta,GseaRes)
S3method(print,GseaBenchmarkAligned)
S3method(print,GseaEnv)
S3method(print,GseaRes)
export(batch_calc_gsea)
export(build_edge_list_safely)
export(build_gsea_pathways)
export(build_hubgene_network)
export(calculate_overlap_ratio)
export(color_by_direction)
export(creat_addition_data_rdsfile)
export(create_addition_template)
export(create_gsea_env)
export(create_gsea_res)
export(create_gsea_task)
export(extract_gsea_task)
export(extract_hub_genes)
export(generate_boxplot_data_code)
export(generate_boxplot_image_code)
export(generate_combined_plot_code)
export(generate_de_volcano_code)
export(generate_dotplot_code)
export(generate_gsea_html_report)
export(generate_hubgene_code)
export(generate_hubgene_hover_text)
export(generate_joint_canvas_code)
export(generate_network_code)
export(generate_pathway_plot_code)
export(generate_volcano_code)
export(get_contrast_registry)
export(get_core_genes_for_pathway)
export(get_core_genes_list)
export(get_de_table)
export(get_expr_matrix)
export(get_geneset_info)
export(get_hubgene_legend)
export(get_sample_meta)
export(get_term_genes)
export(import_gsea_capsule)
export(inspect_gsea_env)
export(inspect_gsea_res)
export(launch_gsea_app)
export(plot_directional_gsea)
export(plot_gsea_memory)
export(prepare_hubgene_nodes)
export(read_addition_data)
export(setup_gsea_env)
export(validate_param)
importFrom(ComplexHeatmap,Heatmap)
importFrom(ComplexHeatmap,HeatmapAnnotation)
importFrom(ComplexHeatmap,draw)
importFrom(ComplexHeatmap,rowAnnotation)
importFrom(DESeq2,DESeq)
importFrom(DESeq2,counts)
importFrom(DESeq2,estimateSizeFactors)
importFrom(DESeq2,results)
importFrom(DESeq2,resultsNames)
importFrom(DESeq2,vst)
importFrom(DT,dataTableOutput)
importFrom(DT,datatable)
importFrom(DT,formatStyle)
importFrom(DT,renderDataTable)
importFrom(DT,styleEqual)
importFrom(DT,styleInterval)
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,metadata)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowData)
importFrom(circlize,colorRamp2)
importFrom(clipr,write_clip)
importFrom(clusterProfiler,GSEA)
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,coalesce)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,if_else)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,right_join)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,slice_head)
importFrom(dplyr,slice_tail)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(edgeR,DGEList)
importFrom(edgeR,calcNormFactors)
importFrom(edgeR,cpm)
importFrom(edgeR,rpkm)
importFrom(enrichit,gseaScores)
importFrom(enrichplot,cnetplot)
importFrom(enrichplot,dotplot)
importFrom(enrichplot,gseaplot)
importFrom(enrichplot,heatplot)
importFrom(enrichplot,ridgeplot)
importFrom(enrichplot,upsetplot)
importFrom(future,multisession)
importFrom(future,plan)
importFrom(future,sequential)
importFrom(future.apply,future_lapply)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotate)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_jitter)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_rect)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,labs)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_alpha_manual)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_gradient2)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_size_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_minimal)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.off)
importFrom(grDevices,png)
importFrom(graphics,abline)
importFrom(graphics,barplot)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,text)
importFrom(grid,gpar)
importFrom(grid,grid.text)
importFrom(grid,unit)
importFrom(htmltools,HTML)
importFrom(htmltools,tags)
importFrom(htmlwidgets,JS)
importFrom(htmlwidgets,saveWidget)
importFrom(igraph,V)
importFrom(igraph,ecount)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,layout_in_circle)
importFrom(igraph,layout_with_fr)
importFrom(igraph,layout_with_kk)
importFrom(igraph,vcount)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,toJSON)
importFrom(limma,contrasts.fit)
importFrom(limma,eBayes)
importFrom(limma,makeContrasts)
importFrom(limma,topTable)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,slot)
importFrom(methods,slotNames)
importFrom(msigdbr,msigdbr)
importFrom(msigdbr,msigdbr_collections)
importFrom(patchwork,plot_annotation)
importFrom(patchwork,wrap_plots)
importFrom(plotly,add_annotations)
importFrom(plotly,event_data)
importFrom(plotly,ggplotly)
importFrom(plotly,layout)
importFrom(plotly,plot_ly)
importFrom(plotly,plotlyOutput)
importFrom(plotly,renderPlotly)
importFrom(plotly,style)
importFrom(progressr,handlers)
importFrom(progressr,progressor)
importFrom(rlang,.data)
importFrom(rlang,`%||%`)
importFrom(shiny,HTML)
importFrom(shiny,NS)
importFrom(shiny,actionButton)
importFrom(shiny,br)
importFrom(shiny,div)
importFrom(shiny,fluidPage)
importFrom(shiny,h4)
importFrom(shiny,hr)
importFrom(shiny,icon)
importFrom(shiny,invalidateLater)
importFrom(shiny,isolate)
importFrom(shiny,mainPanel)
importFrom(shiny,modalDialog)
importFrom(shiny,moduleServer)
importFrom(shiny,need)
importFrom(shiny,observeEvent)
importFrom(shiny,reactive)
importFrom(shiny,reactiveVal)
importFrom(shiny,renderPlot)
importFrom(shiny,renderText)
importFrom(shiny,req)
importFrom(shiny,selectInput)
importFrom(shiny,shinyApp)
importFrom(shiny,showModal)
importFrom(shiny,showNotification)
importFrom(shiny,sidebarLayout)
importFrom(shiny,sidebarPanel)
importFrom(shiny,span)
importFrom(shiny,tabPanel)
importFrom(shiny,tabsetPanel)
importFrom(shiny,tagList)
importFrom(shiny,tags)
importFrom(shiny,textInput)
importFrom(shiny,titlePanel)
importFrom(shiny,updateSelectInput)
importFrom(shiny,updateSelectizeInput)
importFrom(shiny,validate)
importFrom(shinycssloaders,withSpinner)
importFrom(shinyjs,disable)
importFrom(shinyjs,enable)
importFrom(stats,aggregate)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,quantile)
importFrom(stats,reorder)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,terms)
importFrom(stats,var)
importFrom(stringr,str_c)
importFrom(stringr,str_detect)
importFrom(stringr,str_replace)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_sub)
importFrom(stringr,str_to_title)
importFrom(stringr,str_trim)
importFrom(stringr,str_wrap)
importFrom(tibble,as_tibble)
importFrom(tibble,deframe)
importFrom(tibble,enframe)
importFrom(tibble,rownames_to_column)
importFrom(tibble,tibble)
importFrom(tidyr,drop_na)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,separate)
importFrom(tidyr,unite)
importFrom(tools,file_ext)
importFrom(utils,combn)
importFrom(utils,data)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,read.csv)
importFrom(utils,tail)
importFrom(visNetwork,renderVisNetwork)
importFrom(visNetwork,visEvents)
importFrom(visNetwork,visInteraction)
importFrom(visNetwork,visLayout)
importFrom(visNetwork,visNetwork)
importFrom(visNetwork,visNetworkOutput)
importFrom(visNetwork,visOptions)
importFrom(visNetwork,visPhysics)
importFrom(withr,local_seed)
importFrom(withr,with_seed)

The name "lens" in GSEAlens symbolizes that this package acts like a magnifying glass, exploring relevant GSEA pathways.

GSEAlens provides a web-based interface to display pathway introductions and descriptions, incorporating AI-assisted export functionality for pathway enrichment results. This makes GSEA enrichment analysis more convenient, while also allowing users to import translated pathway annotations and summaries as needed.

By packaging the workflow and standardizing input formats, this R package simplifies the viewing and exploration process of GSEA enrichment analysis.

Since multi-group comparisons are more convenient using the no-intercept group specification in the limma-voom pipeline, the code in this project is based on the no-intercept group setting for result analysis and processing.

Based on the reference `Fast gene set enrichment analysis`,`https://www.biorxiv.org/content/10.1101/060012v3`, GSEAlens accepts Wald values from DESeq2::result (`result$stat`) and t-values from limma::fit (`fit$t`) as input for fgsea(Based o `clusterProfiler::GSEA`) and performs ranking(draw based on `GseaVis`). The corresponding comparisons are automatically generated based on the group information contained in the input data. Therefore, users need to specify the target categorical factor (e.g., condition or group). The DESeq2 pipeline allows additional additive covariates (e.g., batch, subject, sex) in the model for correction. In the limma-voom pipeline, the group needs to be specified in a no-intercept manner `~0+group`) DESeq2 supports `~ group`, `~ batch + group`, and `~ subject + group`, but ultimately only supports group-based analysis. GSEAlens does not currently support the generation and exploration of batch GSEA comparison results with interaction terms, continuous variable effects, or complex custom contrasts.

## Installation

GSEAlens can be installed from Bioconductor (recommended for stable releases):

```r
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("GSEAlens")
```

For the development version hosted on GitHub:

```r
if (!requireNamespace("pak", quietly = TRUE))
    install.packages("pak")
pak::pkg_install("DDL095/GSEAlens")
```

# Data Generation

This directory contains scripts and documentation describing how the example data shipped with GSEAlens is generated.

## Distribution layout

GSEAlens ships two kinds of example data:

| Directory | Contents | Loaded via |
| --- | --- | --- |
| `inst/extdata/` | `pathway_annotations.csv` (raw CSV) | `system.file("extdata", "pathway_annotations.csv", package = "GSEAlens")` |
| `data/` | Six pre-processed `.rda` files (see below) | `data(<name>, package = "GSEAlens")` |

The split follows the Bioconductor guideline: `inst/extdata/` for raw
text-like files (CSV), and `data/` for processed R objects that are
loadable via `data()`.

## pathway_annotations.csv

**Source**: Manually curated pathway annotation file for demonstration and custom pathway visualization purposes.

**Columns**:

- `ID`: Pathway identifier (e.g., HALLMARK_HYPOXIA, REACTOME_EXAMPLE)
- `Brief_Description_CN`: Brief description in Simplified Chinese
- `Brief_Description_EN`: Brief description in English
- `Abstract`: Detailed pathway abstract text
- `Custom_Note`: User-defined custom notes for the pathway

**Licensing**: This file is provided as example data for GSEAlens package functionality demonstration.

**Generation Method**:

1. Pathway IDs were selected from commonly used gene set collections (MSigDB Hallmark, Reactome, KEGG)
2. Descriptions were curated based on official pathway documentation
3. The file can be extended by users to add custom pathway annotations for their own gene sets

**Usage in GSEAlens**:
This annotation file can be loaded via the `create_addition_data()` function to provide custom pathway descriptions in the Shiny application.

## Pre-computed Vignette Objects

The main vignette (`vignettes/GSEAlens.Rmd` and `vignettes/GSEAlens-vignette-zh.Rmd`)
consumes six pre-computed `.rda` files from `data/`. They exist so the
vignette can set `eval=TRUE` on its data-loading and object-assembly chunks
without re-running the slow upstream steps (DESeq2 DESeq, limma-voom, msigdbr
bulk loading) on every Bioconductor build.

| File                                  | Generator script                | Contents                                                                                                                                                                                                                                                                                                                                |
| ------------------------------------- | ------------------------------- | --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| `gsea_pathwaysets_toy_hallmark.rda` | `make_gsea_pathwaysets_toy.R` | `build_gsea_pathways("HS", "H")` result; 50 Hallmark pathways x 4384 genes (~22 KB)                                                                                                                                                                                                                                                   |
| `gsea_pathwaysets_toy.rda`          | `make_gsea_pathwaysets_toy.R` | `build_gsea_pathways("HS", c("H","C2:CP:KEGG_LEGACY"))` result; 236 pathways x 7337 genes (~46 KB)                                                                                                                                                                                                                                    |
| `preprocessed_limma.rda`            | `make_preprocessed_inputs.R`  | `list(fit = ..., gsea_limma_voom_data = ...)` from airway, limma-voom (~1 MB)                                                                                                                                                                                                                                                         |
| `preprocessed_dds_se.rda`           | `make_preprocessed_inputs.R`  | DESeq-transformed RangedSummarizedExperiment (13246 x 8) (~3 MB, **slimmed**; see below)                                                                                                                                                                                                                         |
| `preprocessed_dds.rda`              | `make_preprocessed_inputs.R`  | DESeqDataSet from count matrix (13246 x 8) (~4 MB)                                                                                                                                                                                                                                                                                      |
| `precomputed_gseares.rda`           | `make_precomputed_gseares.R`  | `batch_calc_gsea()` result ("GseaRes" list) from `preprocessed_limma.rda` + `gsea_pathwaysets_toy.rda`; 2 contrasts, ~20 enriched pathways each (~1.3 MB). Used by accessor examples (`get_expr_matrix`, `get_de_table`, `extract_gsea_task`, `inspect_gsea_res`, ...) so they can run in < 1 s without recomputing GSEA. |

**Regeneration**:

```sh
# From the package root:
Rscript inst/scripts/make_gsea_pathwaysets_toy.R
Rscript inst/scripts/make_preprocessed_inputs.R
Rscript inst/scripts/make_precomputed_gseares.R
# After regeneration, convert .rds to .rda so data() can load them:
Rscript inst/scripts/convert_data_to_rda.R
```

Run these whenever `msigdbr`, `airway`, `DESeq2`, `limma`, `clusterProfiler`,
or `fgsea` see a version bump, or when `build_gsea_pathways()` /
`batch_calc_gsea()` / the preprocessing vignette change their filtering logic.

**Note on dependency order**: `make_precomputed_gseares.R` consumes the
outputs of the other two scripts, so re-run it **last** if you regenerate
all three. `convert_data_to_rda.R` must always be run last because it
deletes the `.rds` intermediates.

**Versions used for the current shipped copies**:

- msigdbr 26.1.0, airway 1.32.0, DESeq2 1.52.0, limma 3.68.2, edgeR 4.10.0
- clusterProfiler 4.16.0, fgsea 1.36.0
- Generated on 2026-06-28

**Why `preprocessed_dds_se.rds` is "slimmed"**:

The raw `DESeqDataSet` produced by `DESeq()` on the airway data is ~8 MB on
disk, which exceeds the Bioconductor 5 MB extdata limit. The `slim_dds_se()`
helper inside `make_preprocessed_inputs.R` trims three categories of data
that GSEAlens never reads, reducing the file to ~3 MB without changing any
GSEAlens-visible behaviour:

| Removed                                | Reason                                                                                                                                  |
| -------------------------------------- | --------------------------------------------------------------------------------------------------------------------------------------- |
| `mu` / `H` / `cooks` assays            | DESeq() fitting intermediates. GSEAlens only needs `counts` + the Wald statistics in `mcols` (via `DESeq2::results()`).                 |
| `GRangesList` -> `GRanges` (1 per gene) | Each gene's ~20 transcript sub-ranges carry coordinate info GSEAlens never inspects.                                                    |
| 5 Ensembl coordinate columns           | `gene_seq_start`, `gene_seq_end`, `seq_name`, `seq_strand`, `seq_coord_system`. `gene_id` / `gene_name` / `symbol` / `gene_biotype` are **kept**. |

Verified post-slimming: `validObject()` passes, `DESeq2::results(dds_se)`
returns bit-identical `log2FoldChange` / `pvalue` / `padj`, and all GSEAlens
vignette chunks produce the same output. Users who want the full, untrimmed
`DESeqDataSet` for teaching DESeq2 itself should re-run this script with
`slim_dds_se()` removed, or follow the preprocessing vignette.

# Section: Joint Canvas Module ----



#' @title Joint GSEA Fill Canvas Module UI (Display-Only Version)

#' @description

#'   All controls have been moved to the sidebar; only canvas results are displayed here.

#'   This module provides a display interface for joint GSEA visualization results

#'   with export functionality for reproducible R code generation.

#' @keywords internal



mod_joint_canvas_ui <- function(id) {

    ns <- shiny::NS(id)

    shiny::tagList(

    shiny::div(

            class = "white-box", style = "min-height: 900px;",

            shiny::h4("GSEA Joint Canvas"),

            shiny::uiOutput(ns("canvas_info")),

            shiny::plotOutput(ns("canvas_plot"), height = "auto", width = "100%")

    ),

    shiny::hr(),

    shiny::fluidRow(

            shiny::column(12, shiny::div(

        class = "white-box",

        shiny::h4("Export Image / Code"),

        shiny::fluidRow(

                    shiny::column(12,

            shiny::actionButton(

                            ns("open_export_modal"),

                            label = "Export Publication Plot",

                            class = "btn-success",

                            icon  = shiny::icon("file-image"),

                            style = "width: 100%;"

            )

                    )

        ),

        shiny::tags$small(style = "color:#666; display:block; margin-top:6px;",

                    "Image export uses ggplot2::ggsave (PDF/PNG/SVG/TIFF). Reproducible R code is available inside the modal via 'Copy R Code'.")

            ))

    )

    )

}



# Section: Joint Canvas Server Module ----



#' @title Joint GSEA Fill Canvas Module Server (Sidebar Control Version)

#' @description

#'   Server-side logic for the joint GSEA canvas display module.

#'   This module:

#'   \enumerate{

#'     \item Receives sidebar control parameters (arrangement mode)

#'     \item Applies no maximum row limit

#'     \item Maintains aesthetic consistency with multi_plot

#'     \item Generates combined visualization with multiple contrasts and pathways

#'     \item Provides R code export functionality for reproducibility

#'   }

#' @param id Character string identifying the module namespace.

#' @param gsea_res Reactive expression containing GSEA results.

#' @param data_prep_list Reactive list containing data preparation parameters

#'   including contrasts, ncol, and custom colors.

#' @param table_result Reactive list containing pathway selection results.

#' @keywords internal



mod_joint_canvas_server <- function(id, gsea_res, data_prep_list, table_result) {

    shiny::moduleServer(id, function(input, output, session) {

    ns <- session$ns



    # 存储当前画布结果

    canvas_result <- shiny::reactiveVal(NULL)



    shiny::observeEvent(data_prep_list$joint_generate(), {

            contrasts <- data_prep_list$joint_contrasts()

            ncol_val <- data_prep_list$joint_ncol()



            shiny::req(contrasts, ncol_val)

            if (length(contrasts) == 0) {

        shiny::showNotification("Please select contrast groups in the left control panel first", type = "error")

        return()

            }



            data_list <- data_prep_list$data()

            shiny::req(data_list)



            plot_subtype <- data_list$plot_subtype

            custom_colors <- data_list$custom_colors



            pathways <- table_result$selected_pathways()

            shiny::req(pathways)

            if (length(pathways) == 0) {

        shiny::showNotification("Please check the pathways for 'Joint Display' in the main workspace first", type = "error")

        return()

            }



            # 验证ncol

            if (is.null(ncol_val) || ncol_val < 1) ncol_val <- 2



            # 解析颜色

            colors <- trimws(strsplit(custom_colors, ",")[[1]])

            if (length(colors) < length(pathways)) {

        colors <- rep(colors, length.out = length(pathways))

            }



            shiny::incProgress(0.1, detail = "Extracting GSEA tasks...")



            plot_list <- list()



            for (i in seq_along(contrasts)) {

        contrast_id <- contrasts[i]



        task_obj <- tryCatch(

                    {

            extract_gsea_task(gsea_res, contrast_id, "ALL")

                    },

                    error = function(e) {

            message(sprintf("Extraction failed for %s: %s", contrast_id, e$message))

            NULL

                    }

        )



        if (is.null(task_obj)) {

                    p <- ggplot2::ggplot() +

            ggplot2::annotate("text",

                            x = 0.5, y = 0.5,

                            label = sprintf("Not found:\n%s", contrast_id),

                            size = 3, color = "red"

            ) +

            ggplot2::theme_void()

                    plot_list[[i]] <- p

                    next

        }



        main_title <- if (length(pathways) == 1) {

                    # Single pathway: show pathway name instead of generic count
                    pw_label <- pathways[1]
                    tit <- unlist(strsplit(pw_label, split = "_"))
                    if (length(tit) > 1) {
            pw_label <- paste(tools::toTitleCase(tolower(tit[-1])), collapse = " ")
                    } else {
            pw_label <- tools::toTitleCase(tolower(pw_label))
                    }
                    sprintf("%s: %s", gsub("_vs_", " vs ", contrast_id), pw_label)

        } else {

                    sprintf(

            "%s [%d pathways]",

            gsub("_vs_", " vs ", contrast_id),

            length(pathways)

                    )

        }


        p <- tryCatch(

                    {

            plot_directional_gsea(

                            directional_gsea_obj = task_obj,

                            target_pathways = pathways,

                            subPlot = as.numeric(plot_subtype), # 使用主控制栏的subPlot

                            curveCol = colors,

                            main_title = main_title,

                            add_pval = FALSE,

                            show_contrast_in_axis = TRUE

            )

                    },

                    error = function(e) {

            ggplot2::ggplot() +

                            ggplot2::annotate("text",

                x = 0.5, y = 0.5,

                label = sprintf("Error:\n%s", substr(e$message, 1, 80)),

                size = 3, color = "red"

                            ) +

                            ggplot2::theme_void()

                    }

        )



        plot_list[[i]] <- p



        if (i %% 2 == 0 || i == length(contrasts)) {

                    shiny::incProgress(0.6 * i / length(contrasts),

            detail = sprintf("Plotting %d/%d ...", i, length(contrasts))

                    )

        }

            }



            if (length(plot_list) == 0) {

        shiny::showNotification("No valid plots can be generated", type = "error")

        return()

            }



            n_plots <- length(plot_list)

            actual_nrow <- ceiling(n_plots / ncol_val)



            combined_plot <- patchwork::wrap_plots(

        plot_list,

        ncol = ncol_val,

        nrow = actual_nrow,

        byrow = TRUE,

        guides = "collect"

            ) + patchwork::plot_annotation(

        title = if (length(pathways) == 1) {

                    pw_label <- pathways[1]
                    tit <- unlist(strsplit(pw_label, split = "_"))
                    if (length(tit) > 1) {
            pw_label <- paste(tools::toTitleCase(tolower(tit[-1])), collapse = " ")
                    } else {
            pw_label <- tools::toTitleCase(tolower(pw_label))
                    }
                    sprintf("Joint GSEA Canvas: %d contrast groups x 1 pathway (%s)", n_plots, pw_label)

        } else {

                    sprintf("Joint GSEA Canvas: %d contrast groups x %d pathways", n_plots, length(pathways))

        },

        subtitle = sprintf("Arrangement: %s", paste(contrasts, collapse = " -> ")),

        theme = ggplot2::theme(

                    plot.title = ggplot2::element_text(size = 12, face = "bold", hjust = 0.5),

                    plot.subtitle = ggplot2::element_text(size = 8, color = "gray50", hjust = 0.5),

                    plot.margin = ggplot2::margin(10, 10, 10, 10)

        )

            )



            canvas_result(list(

        plot = combined_plot,

        n_plots = n_plots,

        ncol = ncol_val,

        nrow = actual_nrow,

        contrasts = contrasts,

        pathways = pathways

            ))



            shiny::incProgress(1.0, detail = "Done!")

    })





    # Subsection: Export Image Modal ----

    # ------------------------------------------------------------

    # Joint Canvas is a patchwork object (canvas_result()$plot), so we can

    # ggsave it directly without any code generation. This mirrors the

    # export centers in modules 10/13/16 but is simpler because the plot

    # object already exists (no eval() needed).

    # ------------------------------------------------------------

    .build_canvas_export_modal <- function() {

            shiny::modalDialog(

        title = "Export Publication Plot - Joint Canvas",

        size = "l", footer = NULL, easyClose = TRUE,

        shiny::fluidRow(

                    shiny::column(5,

            shiny::h5("Dimensions"),

            shiny::fluidRow(

                            shiny::column(6,

                shiny::numericInput(ns("jc_width"),  "Width (inch)",  value = 16, min = 4, max = 40)

                            ),

                            shiny::column(6,

                shiny::numericInput(ns("jc_height"), "Height (inch)", value = 12, min = 4, max = 40)

                            )

            ),

            shiny::numericInput(ns("jc_dpi"), "DPI", value = 300, min = 72, max = 600),



            shiny::h6("Canvas Margin (pt)"),

            shiny::fluidRow(

                            shiny::column(3, shiny::numericInput(ns("jc_margin_top"),    "Top",    value = 10, min = 0, max = 80)),

                            shiny::column(3, shiny::numericInput(ns("jc_margin_bottom"), "Bottom", value = 10, min = 0, max = 80)),

                            shiny::column(3, shiny::numericInput(ns("jc_margin_left"),   "Left",   value = 10, min = 0, max = 80)),

                            shiny::column(3, shiny::numericInput(ns("jc_margin_right"),  "Right",  value = 10, min = 0, max = 80))

            ),

            shiny::hr(),

            shiny::h5("Download"),

            shiny::fluidRow(

                            shiny::column(6,

                shiny::downloadButton(ns("jc_download_pdf"),

                                    "Download PDF",

                                    class = "btn-danger btn-block",

                                    icon  = shiny::icon("file-pdf"))

                            ),

                            shiny::column(6,

                shiny::downloadButton(ns("jc_download_png"),

                                    "Download PNG",

                                    class = "btn-primary btn-block",

                                    icon  = shiny::icon("file-image"))

                            )

            ),

            shiny::hr(),

            shiny::fluidRow(

                            shiny::column(6,

                shiny::selectInput(ns("jc_format"), "Other format",

                                    choices = c("SVG" = "svg", "TIFF" = "tiff"),

                                    selected = "svg")

                            ),

                            shiny::column(6,

                shiny::div(style = "margin-top: 22px;",

                                    shiny::downloadButton(ns("jc_download_other"),

                    "Download", class = "btn-default btn-block")

                )

                            )

            ),

            shiny::hr(),

            shiny::fluidRow(
                            shiny::column(6,
                shiny::actionButton(ns("jc_copy_code"),
                                    "Copy R Code",
                                    class = "btn-info btn-block",
                                    icon = shiny::icon("clipboard"))
                            ),
                            shiny::column(6,
                shiny::actionButton(ns("jc_dismiss"), "Close",
                                    class = "btn-default btn-block")
                            )
            )
                    ),

                    shiny::column(7,

            shiny::h5("Live Preview"),

            shiny::div(style = "background:#f5f5f5; border:1px solid #ddd; border-radius:4px; padding:8px; width:100%; height:520px; display:flex; align-items:center; justify-content:center; overflow:auto;",

                            shiny::plotOutput(ns("jc_preview")) |>

                shinycssloaders::withSpinner(type = 6, color = "#28a745")

            ),

            shiny::tags$small(style = "color: #666; display:block; margin-top:6px;",

                            "The Joint Canvas is a patchwork of GSEA sub-plots. Larger widths are recommended for multi-contrast layouts. Preview auto-scaled to fit while keeping the export aspect ratio.")

                    )

        )

            )

    }



    shiny::observeEvent(input$open_export_modal, {

            if (is.null(canvas_result()) || is.null(canvas_result()$plot)) {

        shiny::showNotification("Generate a canvas first.", type = "warning"); return()

            }

            shiny::showModal(.build_canvas_export_modal())

    })

    shiny::observeEvent(input$jc_dismiss, shiny::removeModal())



    # Preview device matching export physical size (2026-07-06 v3):
    # res fixed at 72 (Shiny renderPlot doesn't accept res as function),
    # pixel dims = inch * 72, physical size matches export.
    .jc_preview_dims <- function(w_in, h_in, res = 72, max_px = 1600) {

            w_in <- suppressWarnings(as.numeric(w_in[1]))

            h_in <- suppressWarnings(as.numeric(h_in[1]))

            if (length(w_in) == 0 || is.na(w_in) || w_in <= 0) w_in <- 16

            if (length(h_in) == 0 || is.na(h_in) || h_in <= 0) h_in <- 12

            scale <- min(1, max_px / (max(w_in, h_in) * res))

            list(width  = round(w_in * res * scale),

                        height = round(h_in * res * scale))

    }



    # Post-process: apply user-adjustable canvas margin at the OUTER level
    # (whole canvas) only. Previously this used the `&` operator which
    # applies theme to EVERY sub-plot, overwriting the per-panel
    # plot.margin = 0 set in vis-gsea-core.R and inflating the gaps
    # between Panel 1 (RES), Panel 2 (hit strips) and Panel 3 (ranked
    # list) inside each contrast's GSEA plot. That caused the visual
    # mismatch between the Shiny preview (loose) and the Copy-R-Code
    # standalone rendering (tight, since the copied code does not carry
    # the `&` post-processing).
    #
    # Fix: wrap the canvas in patchwork::wrap_elements(full = ...) so the
    # outer margin only pads the canvas as a whole. Title/subtitle from
    # plot_annotation are preserved (they live on cr$plot); the wrap's
    # outer theme only controls the canvas breathing room.
    .jc_preview_plot <- shiny::reactive({
            cr <- canvas_result()
            shiny::req(cr, cr$plot)
            mt <- if (is.null(input$jc_margin_top)    || is.na(input$jc_margin_top))    10 else input$jc_margin_top
            mb <- if (is.null(input$jc_margin_bottom) || is.na(input$jc_margin_bottom)) 10 else input$jc_margin_bottom
            ml <- if (is.null(input$jc_margin_left)   || is.na(input$jc_margin_left))   10 else input$jc_margin_left
            mr <- if (is.null(input$jc_margin_right)  || is.na(input$jc_margin_right))  10 else input$jc_margin_right
            patchwork::wrap_elements(full = cr$plot) +
                ggplot2::theme(plot.margin = ggplot2::margin(mt, mr, mb, ml))
    })



    output$jc_preview <- shiny::renderPlot(

            {

        .jc_preview_plot()

            },

            res  = 72,

            width  = function() {

        d <- .jc_preview_dims(input$jc_width, input$jc_height); d$width

            },

            height = function() {

        d <- .jc_preview_dims(input$jc_width, input$jc_height); d$height

            }

    )



    .jc_render_to_file <- function(file, fmt) {

            p <- .jc_preview_plot()

            if (is.null(p)) {

        shiny::showNotification("No canvas to export.", type = "warning"); return()

            }

            ggplot2::ggsave(

        file, p,

        width  = if (is.null(input$jc_width))  16 else input$jc_width,

        height = if (is.null(input$jc_height)) 12 else input$jc_height,

        dpi    = if (is.null(input$jc_dpi))   300 else input$jc_dpi,

        device = fmt,

        limitsize = FALSE

            )

    }



    output$jc_download_pdf <- shiny::downloadHandler(

            filename = function() sprintf("GSEAlens_joint_canvas_%s.pdf", Sys.Date()),

            content  = function(file) .jc_render_to_file(file, "pdf")

    )

    output$jc_download_png <- shiny::downloadHandler(

            filename = function() sprintf("GSEAlens_joint_canvas_%s.png", Sys.Date()),

            content  = function(file) .jc_render_to_file(file, "png")

    )

    output$jc_download_other <- shiny::downloadHandler(

            filename = function() sprintf("GSEAlens_joint_canvas_%s.%s", Sys.Date(),

                                    if (is.null(input$jc_format)) "svg" else input$jc_format),

            content  = function(file) .jc_render_to_file(file, input$jc_format)

    )



    # Subsection: Code export (Copy R Code, mirrors module 12/13 pattern) ----

    .jc_export_code <- function() {
            contrasts <- data_prep_list$joint_contrasts()
            pathways  <- table_result$selected_pathways()
            if (length(pathways) == 0 || is.null(contrasts)) return("")
            generate_joint_canvas_code(
        GSEAlens_res    = gsea_res,
        contrast_ids    = contrasts,
        target_pathways = pathways,
        ncol            = data_prep_list$joint_ncol()
            )
    }

    shiny::observeEvent(input$jc_copy_code, {
            code <- .jc_export_code()
            if (!nzchar(code)) {
        shiny::showNotification("Nothing to copy.", type = "warning"); return()
            }
            ok <- tryCatch({ clipr::write_clip(code); TRUE }, error = function(e) FALSE)
            if (isTRUE(ok)) {
        shiny::showNotification("R code copied to clipboard.", type = "message")
            } else {
        shiny::showModal(shiny::modalDialog(
                    title = "Reproducible R Code (copy manually)",
                    size = "l", easyClose = TRUE,
                    shiny::div(
            style = "background:#f5f5f5; padding:15px; border-radius:5px; max-height:500px; overflow:auto;",
            shiny::tags$pre(shiny::code(code),
                            style = "font-size:11px; white-space:pre-wrap; word-break:break-all;")
                    ),
                    footer = shiny::modalButton("Close")
        ))
            }
    })


    # Subsection: Render Canvas ----



    # 渲染画布

    output$canvas_plot <- shiny::renderPlot(

            {

        shiny::req(canvas_result())

        canvas_result()$plot

            },

            height = function() {

        if (is.null(canvas_result())) {

                    return(900)

        }

        nrow <- canvas_result()$nrow

        return(max(900, nrow * 400))

            },

            width = 1600,

            res = 72

    )



    # 画布信息

    output$canvas_info <- shiny::renderUI({

            shiny::req(canvas_result())

            info <- canvas_result()



            shiny::tags$div(

        style = "margin-bottom: 15px; padding: 10px; background: linear-gradient(135deg, #667eea 0%, #764ba2 100%); color: white; border-radius: 8px;",

        shiny::HTML(sprintf(

                    "<strong>Canvas Info:</strong> %d contrast groups | Layout: %d cols x %d rows | %d pathways<br>

                        <small>Arrangement order: %s</small>",

                    info$n_plots, info$ncol, info$nrow, length(info$pathways),

                    paste(info$contrasts, collapse = " -> ")

        ))

            )

    })

    })

}

#' @title HubGene Network Module UI

#' @description

#' Shiny module UI for visualizing HubGene networks using visNetwork.

#' Provides interactive controls for pathway selection modes, physics

#' simulation parameters, node sizing, and network statistics display.

#'

#' @param id Character string used to namespace the module.

#'

#' @return A Shiny tagList containing the module's UI elements.

#'

#' @keywords internal



mod_hubgene_vis_ui <- function(id) {

    ns <- shiny::NS(id)



    shiny::tagList(

    shiny::fluidRow(

            # ─── Left Control Panel ───

            shiny::column(

        width = 3,

        shiny::div(

                    class = "well",

                    style = "padding: 15px; max-height: 90vh; overflow-y: auto;",

                    shiny::h4("HubGene Network (visNetwork)"),

                    shiny::hr(),



                    # ── Mode Selection ──

                    shiny::h5("Pathway Source Mode"),

                    shiny::radioButtons(

            ns("vis_mode"),

            label = NULL,

            choices = c(

                            "Top N" = "mode_topN",

                            "Selected from Table" = "mode_select"

            ),

            selected = "mode_topN",

            width = "100%"

                    ),

                    shiny::hr(),



                    # ── Top N Configuration ──

                    shiny::conditionalPanel(

            condition = sprintf("input['%s'] == 'mode_topN'", ns("vis_mode")),

            shiny::numericInput(

                            ns("vis_topN"),

                            label = "Top N Count:",

                            value = 5,

                            min = 3,

                            max = 50,

                            step = 1

            )

                    ),

                    shiny::numericInput(

            ns("vis_fdr"),

            label = "FDR Threshold:",

            value = 0.25,

            min = 0,

            max = 1,

            step = 0.01

                    ),

                    shiny::numericInput(

            ns("vis_min_hub"),

            label = "Min Hub Degree:",

            value = 2,

            min = 1,

            max = 20,

            step = 1

                    ),

                    shiny::hr(),



                    # ── Physics Engine Controls ──

                    shiny::h5("Physics & Interaction"),

                    shiny::checkboxInput(

            ns("vis_physics"),

            label = "Enable Physics",

            value = TRUE

                    ),

                    shiny::conditionalPanel(

            condition = sprintf("input['%s'] == true", ns("vis_physics")),

            shiny::div(

                            style = "background: #f8f9fa; padding: 10px; border-radius: 5px; margin-top: 10px;",

                            shiny::sliderInput(

                ns("vis_gravitational"),

                label = "Gravitational Constant:",

                min = -2000,

                max = -50,

                value = -400,

                step = 10

                            ),

                            shiny::sliderInput(

                ns("vis_spring_length"),

                label = "Spring Length:",

                min = 25,

                max = 500,

                value = 150,

                step = 5

                            ),

                            shiny::sliderInput(

                ns("vis_spring_constant"),

                label = "Spring Constant:",

                min = 0,

                max = 1.0,

                value = 0.001,

                step = 0.001

                            ),

                            shiny::sliderInput(

                ns("vis_central_gravity"),

                label = "Central Gravity:",

                min = 0,

                max = 1,

                value = 0.3,

                step = 0.05

                            ),

                            shiny::sliderInput(

                ns("vis_damping"),

                label = "Damping:",

                min = 0.01,

                max = 0.9,

                value = 0.09,

                step = 0.01

                            )

            )

                    ),

                    shiny::checkboxInput(

            ns("vis_nodes_draggable"),

            label = "Allow Node Dragging",

            value = TRUE

                    ),

                    shiny::checkboxInput(

            ns("vis_smooth_edges"),

            label = "Smooth Curved Edges",

            value = TRUE

                    ),

                    shiny::hr(),



                    # ── Node Sizing ──

                    shiny::h5("Node Sizing"),

                    shiny::selectInput(

            ns("vis_pw_size_mode"),

            label = "Pathway Node Size Encoding:",

            choices = c(

                            "Fixed size (slider below)"        = "fixed",

                            "By significance (-log10 FDR)"     = "fdr",

                            "By gene-set size (setSize)"       = "setsize"

            ),

            selected = "setsize"

                    ),

                    shiny::helpText(

            style = "color: #666; font-size: 11px;",

            "setSize mode matches enrichplot::cnetplot convention.",

            "Size range scales around the slider value [0.6x, 1.4x]."

                    ),

                    shiny::sliderInput(

            ns("vis_pw_size"),

            label = "Pathway Node Base Size:",

            min = 15,

            max = 60,

            value = 30,

            step = 1

                    ),

                    shiny::sliderInput(

            ns("vis_gene_size"),

            label = "Gene Node Base Size:",

            min = 5,

            max = 30,

            value = 12,

            step = 1

                    ),

                    shiny::hr(),



                    # ── Random Seed ──

                    shiny::numericInput(

            ns("vis_seed"),

            label = "Random Seed:",

            value = 42,

            min = 1,

            max = 9999,

            step = 1

                    ),

                    shiny::hr(),



                    # ── Statistics ──

                    shiny::h5("Network Statistics"),

                    shiny::verbatimTextOutput(ns("vis_stats")) |>

            shiny::tagAppendAttributes(style = "font-size: 10px; max-height: 100px; overflow-y: auto;"),

                    shiny::hr(),



                    # ── Pathway Preview ──

                    shiny::h5("Pathways Preview"),

                    shiny::uiOutput(ns("vis_pathway_list")) |>

            shiny::tagAppendAttributes(style = "max-height: 150px; overflow-y: auto;")

        )

            ),



            # ─── Main Plotting Area ───

            shiny::column(

        width = 9,

        shiny::div(

                    class = "white-box",

                    style = "min-height: 850px;",



                    # Status Bar

                    shiny::div(

            style = "background: linear-gradient(135deg, #667eea 0%, #764ba2 100%); padding: 12px; border-radius: 5px; margin-bottom: 15px; color: white;",

            shiny::uiOutput(ns("vis_status_text"))

                    ),



                    # visNetwork Output

                    visNetwork::visNetworkOutput(

            ns("vis_network"),

            height = "1200px",

            width = "100%"

                    ) |>

            shinycssloaders::withSpinner(type = 6, color = "#28a745"),

                    shiny::hr(),

                    shiny::div(

            style = "text-align: center;",

            shiny::actionButton(

                            ns("open_hubgene_export"),

                            label = "Export Publication Plot",

                            icon = shiny::icon("file-image"),

                            class = "btn-success",

                            style = "width: 60%;"

            )

                    ),

                    shiny::hr()

        )

            )

    )

    )

}





#' @title HubGene Network (visNetwork) Module Server

#' @description

#' Shiny module server for HubGene network visualization using visNetwork.

#' Handles pathway selection, network building, rendering with physics

#' simulation, and interactive node/edge display.

#'

#' @param id Character string used to namespace the module.

#' @param data_prep_list A reactive expression returning a list containing

#'   GSEA results, data frame, contrast groups, and metadata.

#' @param table_controller A list containing reactive expressions for

#'   table interactions, including selected pathways.

#' @param gsea_res A GSEA result object used for symbol mapping.

#'

#' @return A list containing reactive expressions:

#'   \item{final_pathways}{Reactive character vector of selected pathway IDs}

#'

#' @keywords internal



mod_hubgene_vis_server <- function(id, data_prep_list, table_controller, gsea_res) {

    shiny::moduleServer(id, function(input, output, session) {

    ns <- session$ns



    # ──────────────────────────────────────────────────────────────

    # Debug Log

    # ──────────────────────────────────────────────────────────────

    session$userData$debug_log <- character(0)



    add_debug <- function(msg, level = "INFO") {

            timestamp <- format(Sys.time(), "%H:%M:%S")

            new_entry <- sprintf("[%s] [%s] %s", timestamp, level, msg)

            session$userData$debug_log <- c(new_entry, session$userData$debug_log)[seq_len(min(50, length(session$userData$debug_log) + 1L))]

            message(sprintf("[HubGene-vis] %s", new_entry))

    }



    output$vis_debug <- shiny::renderText({

            paste(session$userData$debug_log, collapse = "\n")

    })



    add_debug("Module started")



    # ──────────────────────────────────────────────────────────────

    # 1. Mode State

    # ──────────────────────────────────────────────────────────────



    vis_mode <- shiny::reactiveVal("mode_topN")



    shiny::observeEvent(input$vis_mode, {

            vis_mode(input$vis_mode)

            add_debug(sprintf("Mode: %s", input$vis_mode))

    })



    # ──────────────────────────────────────────────────────────────

    # 2. Data Source

    # ──────────────────────────────────────────────────────────────



    topN_candidates <- shiny::reactive({

            if (vis_mode() != "mode_topN") {

        return(character(0))

            }

            data_list <- data_prep_list$data()

            shiny::req(data_list)

            df <- data_list$df

            top_n <- input$vis_topN

            if (is.null(top_n)) top_n <- 5

            top_n <- max(3, min(top_n, nrow(df)))

            df[seq_len(top_n), "ID"]

    })



    select_candidates <- shiny::reactive({

            if (vis_mode() != "mode_select") {

        return(character(0))

            }

            sel <- table_controller$selected_pathways()

            if (is.null(sel) || length(sel) == 0) {

        return(character(0))

            }

            return(sel)

    })



    candidate_raw <- shiny::reactive({

            switch(vis_mode(),

        "mode_topN" = topN_candidates(),

        "mode_select" = select_candidates(),

        character(0)

            )

    })



    candidate_filtered <- shiny::reactive({

            pathways <- candidate_raw()

            if (length(pathways) == 0) {

        return(character(0))

            }

            data_list <- data_prep_list$data()

            shiny::req(data_list)

            df <- data_list$df

            # IRON FIX (2026-07-14): mode_select = user picked from Table.

            # User intent takes priority - do NOT apply FDR filter again.

            if (vis_mode() == "mode_select") {

        pathways[pathways %in% df$ID]

            } else {

        fdr_thresh <- input$vis_fdr

        if (is.null(fdr_thresh)) fdr_thresh <- 0.25

        fdr_vec <- df$p.adjust[match(pathways, df$ID)]

        names(fdr_vec) <- pathways

        pathways[!is.na(fdr_vec) & fdr_vec < fdr_thresh]

            }

    })



    final_pathways <- shiny::reactiveVal(character(0))



    shiny::observeEvent(candidate_filtered(), {

            final_pathways(candidate_filtered())

            add_debug(sprintf("Pathways: %d", length(candidate_filtered())))

    })



    # ──────────────────────────────────────────────────────────────

    # 3. Network Data

    # ──────────────────────────────────────────────────────────────



    net_data <- shiny::reactive({

            data_list <- data_prep_list$data()

            shiny::req(!is.null(data_list))

            pathway_ids <- final_pathways()

            shiny::req(length(pathway_ids) > 0)



            add_debug(sprintf("Building: %d pathways", length(pathway_ids)))



            net <- build_hubgene_network(

        gsea_task = list(

                    gsea_res = data_list$gsea_res,

                    meta = list(

            left_group = data_list$left_group,

            right_group = data_list$right_group

                    )

        ),

        pathway_ids = pathway_ids,

        min_hub_degree = input$vis_min_hub,

        de_df = NULL,

        res_df = data_list$df,

        seed = input$vis_seed

            )



            if (is.null(net)) {

        add_debug("Build failed", "ERROR")

        return(NULL)

            }



            add_debug(sprintf(

        "Built: %d pw + %d gene + %d edges",

        ifelse(is.null(net$nodes$pathway), 0, nrow(net$nodes$pathway)),

        ifelse(is.null(net$nodes$gene), 0, nrow(net$nodes$gene)),

        ifelse(is.null(net$edges), 0, nrow(net$edges))

            ))

            return(net)

    })



    # ──────────────────────────────────────────────────────────────

    # 4. Render Network

    # ──────────────────────────────────────────────────────────────



    output$vis_network <- visNetwork::renderVisNetwork({

            net <- net_data()

            shiny::req(!is.null(net))



            add_debug("Preparing visNetwork...")



            # Get group information

            data_list <- data_prep_list$data()

            left_group <- data_list$left_group

            right_group <- data_list$right_group



            # ──────────────────────────────────────────────────────────────

            # Fix: Rebuild gene name case mapping

            # ──────────────────────────────────────────────────────────────

            symbol_map <- NULL

            if (!is.null(gsea_res) && !is.null(data_list$contrast_id)) {

        symbol_map <- tryCatch(

                    {

            .rebuild_symbol_map(gsea_res, data_list$contrast_id)

                    },

                    error = function(e) {

            message("[HubGene-vis] symbol_map build failed: ", e$message)

            NULL

                    }

        )

        add_debug(sprintf("symbol_map: %d genes mapped", length(symbol_map)))

            } else {

        add_debug("symbol_map skipped: gsea_res or contrast_id is NULL", "WARN")

            }



            # Node size parameters

            # ------------------------------------------------------------

            # Pathway node size is now driven by a user-selectable encoding

            # (input$vis_pw_size_mode). The slider value (vis_pw_size) acts as

            # the BASE size; the chosen encoding scales around it within

            # [0.6x, 1.4x] to keep visNetwork's force-directed layout stable.

            #   "fixed"   -> constant (legacy behavior)

            #   "fdr"     -> -log10(FDR) normalized to [0.6x, 1.4x] of base

            #   "setsize" -> sqrt(setSize) normalized to [0.6x, 1.4x] of base

            #                (cnetplot convention; sqrt avoids over-weighting

            #                 large gene sets)

            # ------------------------------------------------------------

            pw_base  <- ifelse(is.null(input$vis_pw_size),  30, input$vis_pw_size)

            gene_size <- ifelse(is.null(input$vis_gene_size), 12, input$vis_gene_size)

            pw_size_mode <- if (is.null(input$vis_pw_size_mode)) "setsize"

                                            else input$vis_pw_size_mode



            # Per-pathway scaled size vector (matches row order of net$nodes$pathway)

            #

            # IRON FIX (2026-06-30): build_hubgene_network stores the gene-set

            # size under column "n_core" (NOT "setSize"). The previous code

            # unconditionally accessed net$nodes$pathway$setSize, which returned

            # NULL, breaking sqrt()/range() with

            # "non-numeric argument to mathematical function".

            # Now: detect column name setSize -> n_core -> n_total -> fallback,

            # and coerce all numeric inputs via as.numeric() to defend against

            # character-typed columns from upstream GSEA objects.

            pw_pathway <- net$nodes$pathway

            pw_nrow <- if (is.null(pw_pathway)) 0 else nrow(pw_pathway)



            pw_size <- switch(pw_size_mode,

        "fdr" = {

                    if (pw_nrow > 0) {

            fdr_vals <- suppressWarnings(as.numeric(pw_pathway$FDR))

            fdr_vals[is.na(fdr_vals)] <- 1

            fdr_nl <- -log10(pmax(fdr_vals, 1e-300))

            r <- range(fdr_nl, na.rm = TRUE); if (diff(r) < 1e-6) r <- c(0, 10)

            pw_base * (0.6 + 0.8 * (fdr_nl - r[1]) / diff(r))

                    } else rep(pw_base, 0)

        },

        "setsize" = {

                    if (pw_nrow > 0) {

            # Column-name fallback chain

            sv <- if ("setSize" %in% colnames(pw_pathway)) {

                            suppressWarnings(as.numeric(pw_pathway$setSize))

            } else if ("n_core" %in% colnames(pw_pathway)) {

                            suppressWarnings(as.numeric(pw_pathway$n_core))

            } else if ("n_total" %in% colnames(pw_pathway)) {

                            suppressWarnings(as.numeric(pw_pathway$n_total))

            } else {

                            rep(100, pw_nrow)

            }

            sv[is.na(sv)] <- 100

            r <- range(sv, na.rm = TRUE); if (diff(r) < 1) r <- c(0, 200)

            pw_base * (0.6 + 0.8 * (sqrt(sv) - sqrt(r[1])) /

                                                                    (sqrt(r[2]) - sqrt(r[1])))

                    } else rep(pw_base, 0)

        },

        rep(pw_base, pw_nrow)

            )



            # ──────────────────────────────────────────────────────────────

            # Node Construction (add opacity column)

            # ──────────────────────────────────────────────────────────────



            nodes <- data.frame(

        id = character(),

        label = character(),

        group = character(),

        shape = character(),

        color = character(),

        size = numeric(),

        opacity = numeric(),

        title = character(),

        stringsAsFactors = FALSE

            )



            # Pathway nodes

            if (!is.null(net$nodes$pathway) && nrow(net$nodes$pathway) > 0) {

        # IRON FIX (2026-06-30): build_hubgene_network stores the gene-set

        # size under "n_core" (NOT "setSize"). Accessing row$setSize inside

        # the per-row sprintf returned NULL, which made sprintf produce a

        # zero-length character vector, which in turn made data.frame()

        # fail with "arguments imply differing number of rows: 1, 0".

        # Resolve the size column name once, outside the loop, then index

        # safely with a numeric fallback.

        pw_size_col <- if ("setSize" %in% colnames(net$nodes$pathway)) "setSize"

                                                else if ("n_core" %in% colnames(net$nodes$pathway)) "n_core"

                                                else if ("n_total" %in% colnames(net$nodes$pathway)) "n_total"

                                                else NA_character_

        pw_NES_col  <- if ("NES" %in% colnames(net$nodes$pathway)) "NES" else NA_character_

        pw_FDR_col  <- if ("FDR" %in% colnames(net$nodes$pathway)) "FDR" else NA_character_



        for (i in seq_len(nrow(net$nodes$pathway))) {

                    row <- net$nodes$pathway[i, ]

                    # Safe scalar extraction with NA fallback

                    row_nes <- if (is.na(pw_NES_col)) NA_real_

                                            else suppressWarnings(as.numeric(row[[pw_NES_col]][1]))

                    row_fdr <- if (is.na(pw_FDR_col)) NA_real_

                                            else suppressWarnings(as.numeric(row[[pw_FDR_col]][1]))

                    row_size <- if (is.na(pw_size_col)) NA_integer_

                                            else suppressWarnings(as.integer(row[[pw_size_col]][1]))

                    row_nes[is.na(row_nes)]  <- 0

                    row_fdr[is.na(row_fdr)]  <- 1

                    row_size[is.na(row_size)] <- 0



                    direction <- ifelse(row_nes > 0, left_group, right_group)



                    # Calculate LE connection count

                    le_count <- 0

                    if (!is.null(net$edges) && nrow(net$edges) > 0) {

            le_count <- sum(net$edges$target == row$id & net$edges$is_leading_edge, na.rm = TRUE)

                    }



                    nodes <- rbind(nodes, data.frame(

            id = paste0("pw_", row$id),

            label = gsub("^[^_]*_", "", row$id),

            group = "pathway",

            shape = "diamond",

            color = ifelse(row_nes > 0, "#E41A1C", "#377EB8"),

            size = pw_size[i],

            opacity = 1.0,

            title = sprintf(

                            "<b>%s</b><br>Direction: %s<br>NES: %.3f<br>FDR: %.2e<br>LE Genes: %d<br>Set Size: %d",

                            row$id, direction, row_nes, row_fdr, le_count, row_size

            ),

            stringsAsFactors = FALSE

                    ))

        }

            }



            # Gene nodes (add opacity for distinction)

            if (!is.null(net$nodes$gene) && nrow(net$nodes$gene) > 0) {

        # Resolve gene column names defensively (mirrors the pathway fix

        # above: some upstream paths may rename or drop columns).

        gn_stat_col   <- if ("stat"   %in% colnames(net$nodes$gene)) "stat"   else NA_character_

        gn_degree_col <- if ("degree" %in% colnames(net$nodes$gene)) "degree" else NA_character_



        for (i in seq_len(nrow(net$nodes$gene))) {

                    row <- net$nodes$gene[i, ]

                    # Safe scalar extraction

                    row_stat <- if (is.na(gn_stat_col)) NA_real_

                                            else suppressWarnings(as.numeric(row[[gn_stat_col]][1]))

                    row_degree <- if (is.na(gn_degree_col)) NA_integer_

                        else suppressWarnings(as.integer(row[[gn_degree_col]][1]))

                    row_stat[is.na(row_stat)]    <- 0

                    row_degree[is.na(row_degree)] <- 1



                    # Restore original case

                    gene_label <- .get_display_symbol(row$id, symbol_map)



                    # Check if Leading Edge

                    is_le <- FALSE

                    pw_str <- "None"

                    if (!is.null(net$edges) && nrow(net$edges) > 0) {

            is_le <- any(net$edges$source == row$id & net$edges$is_leading_edge, na.rm = TRUE)

            connected_pws <- unique(net$edges$target[net$edges$source == row$id])

            if (length(connected_pws) > 0) {

                            pw_labels <- gsub("^[^_]*_", "", connected_pws)

                            if (length(pw_labels) <= 5) {

                pw_str <- paste(pw_labels, collapse = ", ")

                            } else {

                pw_str <- paste(pw_labels[seq_len(5)], collapse = ", ")

                pw_str <- paste0(pw_str, sprintf(" (+%d)", length(pw_labels) - 5))

                            }

            }

                    }



                    # Direction

                    if (row_stat > 0) {

            direction <- sprintf("Up in %s", left_group)

                    } else if (row_stat < 0) {

            direction <- sprintf("Up in %s", right_group)

                    } else {

            direction <- "Neutral"

                    }



                    # Opacity: LE=1.0, non-LE=0.5

                    gene_opacity <- ifelse(is_le, 1.0, 0.5)



                    nodes <- rbind(nodes, data.frame(

            id = paste0("gene_", gene_label),

            label = gene_label,

            group = "gene",

            shape = "dot",

            color = ifelse(row_stat > 0, "#E41A1C", ifelse(row_stat < 0, "#377EB8", "#999999")),

            size = gene_size + row_degree * 3,

            opacity = gene_opacity,

            title = sprintf(

                            "<b>%s</b><br>Direction: %s<br>Stat: %.3f<br>Hub Degree: %d<br>LE: %s<br>Connected: %s",

                            gene_label, direction, row_stat, row_degree,

                            ifelse(is_le, "YES", "NO"), pw_str

            ),

            stringsAsFactors = FALSE

                    ))

        }

            }



            # ──────────────────────────────────────────────────────────────

            # Edge Construction (color follows gene stat)

            # ──────────────────────────────────────────────────────────────



            edges <- data.frame(

        from = character(),

        to = character(),

        color = character(),

        width = numeric(),

        dashes = logical(),

        title = character(),

        stringsAsFactors = FALSE

            )



            if (!is.null(net$edges) && nrow(net$edges) > 0) {

        # Resolve edge column names defensively.

        ed_source_col <- if ("source" %in% colnames(net$edges)) "source" else NA_character_

        ed_target_col <- if ("target" %in% colnames(net$edges)) "target" else NA_character_

        ed_le_col     <- if ("is_leading_edge" %in% colnames(net$edges)) "is_leading_edge"

                                                    else NA_character_



        for (i in seq_len(nrow(net$edges))) {

                    row <- net$edges[i, ]



                    ed_source <- if (is.na(ed_source_col)) ""

                                                else as.character(row[[ed_source_col]][1])

                    ed_target <- if (is.na(ed_target_col)) ""

                                                else as.character(row[[ed_target_col]][1])

                    ed_is_le  <- if (is.na(ed_le_col)) FALSE

                                                else isTRUE(as.logical(row[[ed_le_col]][1]))



                    # Get source gene stat value to determine color

                    gene_stat <- 0

                    if (!is.null(net$nodes$gene) && nrow(net$nodes$gene) > 0) {

            if (!is.na(gn_stat_col)) {

                            gene_row <- net$nodes$gene[net$nodes$gene$id == ed_source, ]

                            if (nrow(gene_row) > 0) {

                gene_stat <- suppressWarnings(as.numeric(gene_row[[gn_stat_col]][1]))

                gene_stat <- if (is.na(gene_stat)) 0 else gene_stat

                            }

            }

                    }



                    # Color follows gene stat: light color (with opacity)

                    if (gene_stat > 0) {

            edge_color <- "#E41A1C80"

                    } else if (gene_stat < 0) {

            edge_color <- "#377EB880"

                    } else {

            edge_color <- "#99999980"

                    }



                    # Restore original case

                    gene_display <- .get_display_symbol(ed_source, symbol_map)



                    edges <- rbind(edges, data.frame(

            from = paste0("gene_", gene_display),

            to = paste0("pw_", ed_target),

            color = edge_color,

            width = ifelse(ed_is_le, 3, 1.5),

            dashes = !ed_is_le,

            title = sprintf(

                            "<b>Gene:</b> %s<br><b>Pathway:</b> %s<br><b>Leading Edge:</b> %s",

                            gene_display, ed_target,

                            ifelse(ed_is_le, "YES", "NO")

            ),

            stringsAsFactors = FALSE

                    ))

        }

            }



            add_debug(sprintf("Nodes: %d, Edges: %d", nrow(nodes), nrow(edges)))



            # ──────────────────────────────────────────────────────────────

            # Build visNetwork Object

            # ──────────────────────────────────────────────────────────────



            vis <- visNetwork::visNetwork(nodes, edges, width = "100%", height = "650px")



            # Physics Engine

            physics_enabled <- isTRUE(input$vis_physics)

            grav <- ifelse(is.null(input$vis_gravitational), -400, input$vis_gravitational)



            spring_length <- ifelse(is.null(input$vis_spring_length), 150, input$vis_spring_length)

            spring_constant <- ifelse(is.null(input$vis_spring_constant), 0.01, input$vis_spring_constant)

            damping <- ifelse(is.null(input$vis_damping), 0.09, input$vis_damping)

            central_gravity <- ifelse(is.null(input$vis_central_gravity), 0.3, input$vis_central_gravity)



            if (physics_enabled) {

        vis <- vis |> visNetwork::visPhysics(

                    enabled = TRUE,

                    solver = "barnesHut",

                    barnesHut = list(

            gravitationalConstant = grav,

            centralGravity = central_gravity,

            springLength = spring_length,

            springConstant = spring_constant,

            damping = damping

                    )

        )

        add_debug(sprintf("Physics: ON, solver=barnesHut, grav=%.0f", grav))

            } else {

        vis <- vis |> visNetwork::visPhysics(enabled = FALSE)

        add_debug("Physics: OFF")

            }



            # Layout

            seed <- ifelse(is.null(input$vis_seed), 42, input$vis_seed)

            vis <- vis |> visNetwork::visLayout(randomSeed = seed)



            # Interaction

            vis <- vis |> visNetwork::visInteraction(

        dragNodes = TRUE,

        dragView = TRUE,

        zoomView = TRUE,

        hover = TRUE,

        tooltipDelay = 300,

        navigationButtons = TRUE

            )



            # Click to highlight related nodes and edges

            vis <- vis |> visNetwork::visOptions(

        highlightNearest = TRUE,

        nodesIdSelection = TRUE

            )



            # Click Event

            vis <- vis |> visNetwork::visEvents(

        click = sprintf("function(props) {

            var n = props.nodes[0];

            if(n) Shiny.setInputValue('%s', {id: n, ts: Date.now()}, {priority:'event'});

    }", ns("vis_click"))

            )



            add_debug("Render complete")

            vis

    })



    # ──────────────────────────────────────────────────────────────

    # 5. Events

    # ──────────────────────────────────────────────────────────────



    shiny::observeEvent(input$vis_click, {

            add_debug(sprintf("Click: %s", input$vis_click$id), "CLICK")

    })



    # ──────────────────────────────────────────────────────────────

    # 6. Statistics

    # ──────────────────────────────────────────────────────────────



    output$vis_stats <- shiny::renderPrint({

            net <- tryCatch(net_data(), error = function(e) NULL)

            if (is.null(net)) {

        cat("No data\n")

        return()

            }

            pw <- ifelse(is.null(net$nodes$pathway), 0, nrow(net$nodes$pathway))

            gene <- ifelse(is.null(net$nodes$gene), 0, nrow(net$nodes$gene))

            edge <- ifelse(is.null(net$edges), 0, nrow(net$edges))

            le_edges <- ifelse(is.null(net$edges), 0, sum(net$edges$is_leading_edge, na.rm = TRUE))

            cat(sprintf("Pathways: %d\n", pw))

            cat(sprintf("Hub Genes: %d\n", gene))

            cat(sprintf("Total Edges: %d\n", edge))

            cat(sprintf("LE Edges: %d\n", le_edges))

    })



    # ──────────────────────────────────────────────────────────────

    # 7. Status Bar

    # ──────────────────────────────────────────────────────────────



    output$vis_status_text <- shiny::renderUI({

            net <- tryCatch(net_data(), error = function(e) NULL)

            if (is.null(net)) {

        return(shiny::span("No data", style = "color: #ffcccc;"))

            }

            pw <- ifelse(is.null(net$nodes$pathway), 0, nrow(net$nodes$pathway))

            gene <- ifelse(is.null(net$nodes$gene), 0, nrow(net$nodes$gene))

            edge <- ifelse(is.null(net$edges), 0, nrow(net$edges))

            le_edges <- ifelse(is.null(net$edges), 0, sum(net$edges$is_leading_edge, na.rm = TRUE))

            physics <- if (isTRUE(input$vis_physics)) "ON" else "OFF"



            shiny::tagList(

        shiny::strong(sprintf(

                    "HubGene Network | %d Pathways + %d Hub Genes + %d Edges (LE: %d)",

                    pw, gene, edge, le_edges

        )),

        htmltools::tags$br(),

        htmltools::tags$small(sprintf(

                    "Physics: %s (Barnes-Hut) | %s | tooltipDelay: 300ms ",

                    physics,

                    if (vis_mode() == "mode_select") "no FDR filter" else sprintf("FDR < %.2f", input$vis_fdr %||% 0.25)

        ))

            )

    })



    # ──────────────────────────────────────────────────────────────

    # 8. Pathway Preview

    # ──────────────────────────────────────────────────────────────



    output$vis_pathway_list <- shiny::renderUI({

            pathways <- final_pathways()

            net <- tryCatch(net_data(), error = function(e) NULL)



            if (length(pathways) == 0) {

        return(shiny::div("No pathways", style = "color: #856404;"))

            }

            data_list <- data_prep_list$data()

            df <- data_list$df



            lapply(pathways, function(pid) {

        row_idx <- which(df$ID == pid)

        if (length(row_idx) == 0) {

                    return(NULL)

        }

        row <- df[row_idx[1], ]

        le_count <- 0

        if (!is.null(net) && !is.null(net$edges)) {

                    le_count <- sum(net$edges$target == pid & net$edges$is_leading_edge, na.rm = TRUE)

        }

        shiny::div(

                    style = "background: #f8f9fa; padding: 5px; margin-bottom: 3px; border-left: 3px solid #007bff; font-size: 10px;",

                    shiny::strong(gsub("^[^_]*_", "", pid)),

                    htmltools::tags$br(),

                    sprintf("NES: %.2f | LE: %d", as.numeric(row$NES), le_count)

        )

            })

    })



    # ──────────────────────────────────────────────────────────────

    # 9. Detail Tables

    # ──────────────────────────────────────────────────────────────



    output$vis_node_detail <- DT::renderDataTable({

            net <- tryCatch(net_data(), error = function(e) NULL)

            if (is.null(net) || (is.null(net$nodes$pathway) && is.null(net$nodes$gene))) {

        return(DT::datatable(data.frame(Message = "No nodes"), options = list(dom = "t")))

            }



            # Get symbol_map for displaying original case

            symbol_map <- NULL

            data_list <- tryCatch(data_prep_list$data(), error = function(e) NULL)

            if (!is.null(data_list) && !is.null(data_prep_list$gsea_res) && !is.null(data_list$contrast_id)) {

        symbol_map <- tryCatch(

                    {

            .rebuild_symbol_map(data_prep_list$gsea_res, data_list$contrast_id)

                    },

                    error = function(e) NULL

        )

            }



            rows <- list()



            # Pathway nodes

            if (!is.null(net$nodes$pathway) && nrow(net$nodes$pathway) > 0) {

        for (i in seq_len(nrow(net$nodes$pathway))) {

                    r <- net$nodes$pathway[i, ]

                    le_count <- sum(net$edges$target == r$id & net$edges$is_leading_edge, na.rm = TRUE)

                    total_count <- sum(net$edges$target == r$id, na.rm = TRUE)

                    rows[[length(rows) + 1]] <- data.frame(

            ID = r$id,

            Type = "Pathway",

            NES = round(r$NES, 3),

            Direction = ifelse(r$NES > 0, "Up", "Down"),

            Degree = NA_integer_,

            LE = le_count,

            Total = total_count,

            stringsAsFactors = FALSE

                    )

        }

            }



            # Gene nodes

            if (!is.null(net$nodes$gene) && nrow(net$nodes$gene) > 0) {

        for (i in seq_len(nrow(net$nodes$gene))) {

                    r <- net$nodes$gene[i, ]

                    is_le <- any(net$edges$source == r$id & net$edges$is_leading_edge, na.rm = TRUE)

                    gene_display <- .get_display_symbol(r$id, symbol_map)

                    rows[[length(rows) + 1]] <- data.frame(

            ID = gene_display,

            Type = "Gene",

            NES = round(r$stat, 3),

            Direction = ifelse(r$stat > 0, "Up", ifelse(r$stat < 0, "Down", "Neutral")),

            Degree = r$degree,

            LE = ifelse(is_le, "YES", "NO"),

            Total = NA_integer_,

            stringsAsFactors = FALSE

                    )

        }

            }



            df <- do.call(rbind, rows)

            DT::datatable(df, rownames = FALSE, options = list(pageLength = 10, dom = "t"))

    })



    output$vis_edge_detail <- DT::renderDataTable({

            net <- tryCatch(net_data(), error = function(e) NULL)

            if (is.null(net) || is.null(net$edges) || nrow(net$edges) == 0) {

        return(DT::datatable(data.frame(Message = "No edges"), options = list(dom = "t")))

            }



            # Get symbol_map for displaying original case

            symbol_map <- NULL

            data_list <- tryCatch(data_prep_list$data(), error = function(e) NULL)

            if (!is.null(data_list) && !is.null(data_prep_list$gsea_res) && !is.null(data_list$contrast_id)) {

        symbol_map <- tryCatch(

                    {

            .rebuild_symbol_map(data_prep_list$gsea_res, data_list$contrast_id)

                    },

                    error = function(e) NULL

        )

            }



            rows <- lapply(seq_len(nrow(net$edges)), function(i) {

        r <- net$edges[i, ]

        gene_display <- .get_display_symbol(r$source, symbol_map)

        data.frame(

                    Gene = gene_display,

                    Pathway = r$target,

                    LeadingEdge = ifelse(r$is_leading_edge, "YES", "NO"),

                    Width = ifelse(r$is_leading_edge, 3, 1.5),

                    Style = ifelse(r$is_leading_edge, "Solid", "Dashed"),

                    stringsAsFactors = FALSE

        )

            })



            df <- do.call(rbind, rows)

            DT::datatable(df, rownames = FALSE, options = list(pageLength = 10, dom = "t"))

    })



    add_debug("Ready")



    # ============================================================

    # HubGene Export Center

    # ------------------------------------------------------------

    # Static ggplot2 reproduction of the visNetwork bipartite graph.

    # Reuses generate_hubgene_code() from 15_code_generator.R.

    # ============================================================



    .build_hubgene_export_modal <- function() {

            shiny::modalDialog(

        title = "Export Publication Plot - HubGene Network",

        size = "l", footer = NULL, easyClose = TRUE,

        shiny::fluidRow(

                    shiny::column(5,

            shiny::h5("Dimensions"),

            shiny::fluidRow(

                            shiny::column(6,

                shiny::numericInput(ns("hub_exp_width"),  "Width (inch)",  value = 10, min = 2, max = 24)

                            ),

                            shiny::column(6,

                shiny::numericInput(ns("hub_exp_height"), "Height (inch)", value = 8, min = 2, max = 24)

                            )

            ),

            shiny::numericInput(ns("hub_exp_dpi"), "DPI", value = 300, min = 72, max = 600),



            shiny::h6("Canvas Margin (pt)"),

            shiny::fluidRow(

                            shiny::column(3, shiny::numericInput(ns("hub_exp_margin_top"),    "Top",    value = 18, min = 0, max = 80)),

                            shiny::column(3, shiny::numericInput(ns("hub_exp_margin_bottom"), "Bottom", value = 18, min = 0, max = 80)),

                            shiny::column(3, shiny::numericInput(ns("hub_exp_margin_left"),   "Left",   value = 18, min = 0, max = 80)),

                            shiny::column(3, shiny::numericInput(ns("hub_exp_margin_right"),  "Right",  value = 18, min = 0, max = 80))

            ),

            shiny::hr(),

            shiny::h5("Download"),

            shiny::fluidRow(

                            shiny::column(6,

                shiny::downloadButton(ns("hub_exp_download_pdf"),

                                    "Download PDF",

                                    class = "btn-danger btn-block",

                                    icon  = shiny::icon("file-pdf"))

                            ),

                            shiny::column(6,

                shiny::downloadButton(ns("hub_exp_download_png"),

                                    "Download PNG",

                                    class = "btn-primary btn-block",

                                    icon  = shiny::icon("file-image"))

                            )

            ),

            shiny::hr(),

            shiny::fluidRow(

                            shiny::column(6,

                shiny::selectInput(ns("hub_exp_format"), "Other format",

                                    choices = c("SVG" = "svg", "TIFF" = "tiff"),

                                    selected = "svg")

                            ),

                            shiny::column(6,

                shiny::div(style = "margin-top: 22px;",

                                    shiny::downloadButton(ns("hub_exp_download_other"),

                    "Download", class = "btn-default btn-block")

                )

                            )

            ),

            shiny::hr(),

            shiny::fluidRow(

                            shiny::column(6,

                shiny::actionButton(ns("hub_exp_copy_code"),

                                    "Copy R Code",

                                    class = "btn-info btn-block",

                                    icon = shiny::icon("clipboard"))

                            ),

                            shiny::column(6,

                shiny::actionButton(ns("hub_exp_dismiss"),

                                    "Close",

                                    class = "btn-default btn-block")

                            )

            )

                    ),

                    shiny::column(7,

            shiny::h5("Live Preview"),

            shiny::div(style = "background:#f5f5f5; border:1px solid #ddd; border-radius:4px; padding:8px; width:100%; height:520px; display:flex; align-items:center; justify-content:center; overflow:auto;",

                            shiny::plotOutput(ns("hub_exp_preview")) |>

                shinycssloaders::withSpinner(type = 6, color = "#28a745")

            ),

            shiny::tags$small(style = "color: #666; display:block; margin-top:6px;",

                            "Pathway nodes: diamonds; Gene nodes: circles. Preview auto-scaled to fit while keeping the export aspect ratio.")

                    )

        )

            )

    }



    shiny::observeEvent(input$open_hubgene_export, {

            net <- tryCatch(net_data(), error = function(e) NULL)

            if (is.null(net) || (is.null(net$nodes$pathway) && is.null(net$nodes$gene))) {

        shiny::showNotification("No network to export.", type = "warning"); return()

            }

            shiny::showModal(.build_hubgene_export_modal())

    })

    shiny::observeEvent(input$hub_exp_dismiss, shiny::removeModal())



    # Live preview reactive

    # IRON FIX (2026-06-30): same pattern as pathway/quadrant modules —

    # suppress `print(p)` from opening a separate graphics device, surface

    # errors as notifications, and guard against NA from cleared numericInput.

    #

    # Preview scaling (2026-07-06): the preview pane is a FIXED-SIZE window

    # (~500x480 px). The figure is rendered at the export aspect ratio and

    # scaled to fit inside that window, centered. A tall figure appears as a

    # vertical "pole" thumbnail; a wide figure as a horizontal "pole".

    # Preview device matching export physical size (2026-07-06 v3):
    # Shiny renderPlot does NOT accept res as a function, so res is fixed
    # at 72 and pixel dims are computed as inch * 72 (with a pixel cap).
    # Physical size = width_px / 72 = w_in inches, matching the export.
    .hub_preview_dims <- function(w_in, h_in, res = 72, max_px = 1600) {

            w_in <- suppressWarnings(as.numeric(w_in[1]))

            h_in <- suppressWarnings(as.numeric(h_in[1]))

            if (length(w_in) == 0 || is.na(w_in) || w_in <= 0) w_in <- 10

            if (length(h_in) == 0 || is.na(h_in) || h_in <= 0) h_in <- 8

            scale <- min(1, max_px / (max(w_in, h_in) * res))

            list(width  = round(w_in * res * scale),

                        height = round(h_in * res * scale))

    }

    .hubgene_preview_plot <- shiny::reactive({

            code <- tryCatch(.hubgene_export_code(),

        error = function(e) {

                    shiny::showNotification(

            sprintf("[hub preview] code generation failed: %s", e$message),

            type = "error", duration = 8)

                    ""

        })

            if (!nzchar(code)) return(NULL)

            eval_env <- new.env(parent = globalenv())

            eval_env$p <- NULL

            eval_env$print <- base::invisible
            eval_env$gsea_res <- gsea_res

            tryCatch(eval(parse(text = code), envir = eval_env),

                                error = function(e) {

                                    shiny::showNotification(

                                        sprintf("[hub preview] %s", e$message),

                                        type = "error", duration = 8)

                                    NULL

                                })

            eval_env$p

    })

    output$hub_exp_preview <- shiny::renderPlot(

            {

        p <- .hubgene_preview_plot()

        shiny::req(p)

        p

            },

            res  = 72,

            width  = function() {

        d <- .hub_preview_dims(input$hub_exp_width, input$hub_exp_height); d$width

            },

            height = function() {

        d <- .hub_preview_dims(input$hub_exp_width, input$hub_exp_height); d$height

            }

    )



    .hubgene_export_code <- function() {

            data_list <- data_prep_list$data()

            if (is.null(data_list)) return("")

            seed_val <- if (is.null(input$vis_seed)) 42L else as.integer(input$vis_seed)

            pw_mode  <- if (is.null(input$vis_pw_size_mode)) "setsize" else input$vis_pw_size_mode

            generate_hubgene_code(gsea_res_var = "gsea_res",

                            contrast_id  = data_list$contrast_id,

                            pathways     = final_pathways(),

                            min_hub_degree = if (is.null(input$vis_min_hub)) 2 else input$vis_min_hub,

                            pw_size_mode = pw_mode,

                            seed = seed_val,

                            target_collection = if (is.null(data_list$collections)) "ALL" else data_list$collections,

                            margin_top    = if (is.null(input$hub_exp_margin_top)    || is.na(input$hub_exp_margin_top))    18 else input$hub_exp_margin_top,

                            margin_bottom = if (is.null(input$hub_exp_margin_bottom) || is.na(input$hub_exp_margin_bottom)) 18 else input$hub_exp_margin_bottom,

                            margin_left   = if (is.null(input$hub_exp_margin_left)   || is.na(input$hub_exp_margin_left))   18 else input$hub_exp_margin_left,

                            margin_right  = if (is.null(input$hub_exp_margin_right)  || is.na(input$hub_exp_margin_right))  18 else input$hub_exp_margin_right,

                            fig_width     = if (is.null(input$hub_exp_width)         || is.na(input$hub_exp_width))         10 else input$hub_exp_width,

                            fig_height    = if (is.null(input$hub_exp_height)        || is.na(input$hub_exp_height))         8 else input$hub_exp_height,
                            fig_dpi    = if (is.null(input$hub_exp_dpi)   || is.na(input$hub_exp_dpi))   300 else input$hub_exp_dpi)

    }



    .hubgene_render_to_file <- function(file, fmt) {

            code <- .hubgene_export_code()

            if (!nzchar(code)) return()

            # parent=globalenv() so lexical lookup resolves attached packages.

            # (baseenv() reproduces the v0.99.11 "could not find function" bug +

            # the .htm fallback symptom; see IRON FIX comment in commit log.)

            eval_env <- new.env(parent = globalenv())

            eval_env$p <- NULL

            eval_env$print <- base::invisible   # suppress popup device; ggsave draws itself
            eval_env$gsea_res <- gsea_res

            tryCatch(eval(parse(text = code), envir = eval_env),

                                error = function(e) shiny::showNotification(

                                    sprintf("Render failed: %s", e$message), type = "error", duration = 8))

            if (is.null(eval_env$p)) return()

            ggplot2::ggsave(file, eval_env$p,

                                            width  = if (is.null(input$hub_exp_width))  10 else input$hub_exp_width,

                                            height = if (is.null(input$hub_exp_height))  8 else input$hub_exp_height,

                                            dpi    = if (is.null(input$hub_exp_dpi))   300 else input$hub_exp_dpi,

                                            device = fmt)

    }



    output$hub_exp_download_pdf <- shiny::downloadHandler(

            filename = function() sprintf("GSEAlens_hubgene_%s.pdf", Sys.Date()),

            content  = function(file) .hubgene_render_to_file(file, "pdf")

    )



    output$hub_exp_download_png <- shiny::downloadHandler(

            filename = function() sprintf("GSEAlens_hubgene_%s.png", Sys.Date()),

            content  = function(file) .hubgene_render_to_file(file, "png")

    )



    output$hub_exp_download_other <- shiny::downloadHandler(

            filename = function() sprintf("GSEAlens_hubgene_%s.%s", Sys.Date(),

                                    if (is.null(input$hub_exp_format)) "svg" else input$hub_exp_format),

            content  = function(file) .hubgene_render_to_file(file, input$hub_exp_format)

    )



    shiny::observeEvent(input$hub_exp_copy_code, {

            code <- .hubgene_export_code()

            if (!nzchar(code)) {

        shiny::showNotification("Nothing to copy.", type = "warning"); return()

            }

            ok <- tryCatch({ clipr::write_clip(code); TRUE }, error = function(e) FALSE)

            if (isTRUE(ok)) {

        shiny::showNotification("R code copied to clipboard.", type = "message")

            } else {

        shiny::showModal(shiny::modalDialog(

                    title = "Reproducible R Code (copy manually)",

                    size = "l", easyClose = TRUE,

                    shiny::pre(style = "max-height: 60vh; overflow-y: auto; font-size: 11px;", code),

                    footer = shiny::modalButton("Close")

        ))

            }

    })



    return(list(final_pathways = final_pathways))

    })

}

#' @title Pathway Relationship Exploration Module UI

#' @description Network visualization module with dual-mode pathway selection

#' @keywords internal



mod_pathway_relation_ui <- function(id) {

    ns <- shiny::NS(id)

    shiny::tagList(

    shiny::fluidRow(

            # ===== Left Control Panel =====

            shiny::column(

        2,

        shiny::div(

                    class = "well",

                    style = "padding: 15px;",



                    # ---- Mode Selector ----

                    shiny::h4("Select Analysis Mode"),

                    shiny::radioButtons(

            ns("network_mode"),

            label = NULL,

            choices = c(

                            "Top N from Current Set" = "mode_topN",

                            "Selected from Main Table" = "mode_select"

            ),

            selected = "mode_topN",

            width = "100%"

                    ),

                    shiny::hr(),



                    # ---- Shared Parameters Panel ----

                    shiny::h4("Shared Parameters"),

                    shiny::numericInput(

            ns("fdr_threshold"),

            label = "FDR Threshold:",

            min = 0,

            max = 1.0,

            value = 0.25,

            step = 0.01

                    ),

                    shiny::numericInput(

            ns("min_shared"),

            label = "Min Shared Core Genes:",

            value = 3,

            min = 1,

            max = 9999,

            step = 1

                    ),

                    shiny::sliderInput(

            ns("max_nodes"),

            label = "Max Nodes:",

            min = 5,

            max = 100,

            value = 30,

            step = 5

                    ),

                    shiny::selectInput(

            ns("network_layout"),

            label = "Layout Algorithm:",

            choices = c(

                            "Fruchterman-Reingold" = "fr",

                            "Kamada-Kawai" = "kk",

                            "Circle" = "circle"

            ),

            selected = "fr"

                    ),

                    shiny::numericInput(

            ns("seed"),

            label = "Seed (for reproducibility):",

            value = 42,

            min = 1,

            max = 9999,

            step = 1

                    ),

                    shiny::helpText(

            style = "color: #666; font-size: 11px;",

            "Seed ensures reproducible layout when parameters change"

                    ),

                    shiny::hr(),



                    # ---- Hover Display Settings ----

                    shiny::h4("Hover Display Settings"),

                    shiny::sliderInput(

            ns("hover_max_genes"),

            label = "Max Genes in Hover:",

            min = 3,

            max = 30,

            value = 5,

            step = 1

                    ),

                    shiny::helpText(

            style = "color: #666; font-size: 11px;",

            "Number of shared genes to display in edge hover tooltip"

                    ),

                    shiny::hr(),

                    shiny::conditionalPanel(

            condition = sprintf("input['%s'] == 'mode_topN'", ns("network_mode")),

            shiny::div(

                            style = "background: #e3f2fd; padding: 10px; border-radius: 5px;",

                            shiny::h5("Top N Configuration"),

                            shiny::numericInput(

                ns("topN_count"),

                label = "Top N Count:",

                value = 20,

                min = 1,

                max = 9999,

                step = 1

                            )

            )

                    ),

                    shiny::conditionalPanel(

            condition = sprintf("input['%s'] == 'mode_select'", ns("network_mode")),

            shiny::div(

                            style = "background: #f3e5f5; padding: 10px; border-radius: 5px;",

                            shiny::h5("Main Table Selection"),

                            shiny::helpText(

                style = "color: #6a1b9a; font-size: 11px;",

                "Check pathways in the main table 'Joint Plot' column"

                            )

            )

                    ),

                    shiny::hr(),

                    shiny::h4("Pathways to Plot"),

                    shiny::uiOutput(ns("pathway_preview_list"))

        )

            ),



            # ===== Right Panel =====

            shiny::column(

        10,

        shiny::tabsetPanel(

                    id = ns("active_tab"),

                    type = "tabs",

                    shiny::tabPanel(

            title = "DotPlot",

            value = "dotplot",

            shiny::div(

                            class = "white-box",

                            style = "padding: 15px; margin-top: 15px;",

                            shiny::uiOutput(ns("dotplot_status")),

                            shiny::fluidRow(

                shiny::column(4,

                                    shiny::selectInput(

                    ns("dotplot_color_mode"),

                    label = "Color by:",

                    choices = c(

                                            "-log10(FDR)" = "padj",

                                            "-log10(P-value)" = "pval",

                                            "NES" = "nes"

                    ),

                    selected = "padj"

                                    )

                ),

                shiny::column(4,

                                    shiny::selectInput(

                    ns("dotplot_size_mode"),

                    label = "Size by:",

                    choices = c(

                                            "Core Genes Count" = "core_size",

                                            "Set Size" = "setsize"

                    ),

                    selected = "core_size"

                                    )

                ),

                shiny::column(4,

                                    shiny::div(style = "margin-top: 22px;",

                    shiny::actionButton(

                                            ns("open_dotplot_export"),

                                            label = "Export Publication Plot",

                                            class = "btn-success",

                                            style = "width: 100%;"

                    )

                                    )

                )

                            ),

                            plotly::plotlyOutput(ns("plot_dotplot"), height = "800px") |>

                shinycssloaders::withSpinner(type = 6, color = "#28a745")

            )

                    ),

                    shiny::tabPanel(

            title = "Network",

            value = "network",

            shiny::div(

                            class = "white-box",

                            style = "padding: 15px; margin-top: 15px;",

                            shiny::uiOutput(ns("network_status")),

                            shiny::fluidRow(

                shiny::column(4,

                                    shiny::selectInput(

                    ns("network_edge_width_mode"),

                    label = "Edge Width Mode:",

                    choices = c(

                                            "Weight-based (emapplot style)" = "weight",

                                            "Rank-based (uniform spacing)"  = "rank"

                    ),

                    selected = "weight"

                                    )

                ),

                shiny::column(4,

                                    shiny::div(style = "margin-top: 22px;",

                    shiny::actionButton(

                                            ns("open_network_export"),

                                            label = "Export Publication Plot",

                                            class = "btn-success",

                                            style = "width: 100%;"

                    )

                                    )

                ),

                shiny::column(4,

                                    shiny::helpText(

                    style = "margin-top: 22px; color: #666; font-size: 11px;",

                    "Weight-based: faithful to Jaccard values",

                    shiny::br(),

                    "Rank-based: uniform visual spacing"

                                    )

                )

                            ),

                            shiny::div(

                id = ns("selection_panel"),

                style = "background: #e8f4fd; padding: 12px; border-radius: 8px; margin-bottom: 15px;",

                shiny::fluidRow(

                                    shiny::column(

                    12,

                    shiny::uiOutput(ns("selection_display"))

                                    )

                ),

                shiny::fluidRow(

                                    shiny::column(

                    6,

                    shiny::actionButton(

                                            ns("show_edge_detail"),

                                            label = "Show Edge Detail",

                                            class = "btn-primary",

                                            style = "width: 100%;",

                                            disabled = NA

                    )

                                    ),

                                    shiny::column(

                    6,

                    shiny::actionButton(

                                            ns("clear_selection"),

                                            label = "Clear Selection",

                                            class = "btn-warning",

                                            style = "width: 100%;"

                    )

                                    )

                ),

                shiny::div(

                                    style = "margin-top: 8px; font-size: 11px; color: #666;",

                                    shiny::HTML("<b>Instruction:</b> Click two nodes to select, then click 'Show Edge Detail'")

                )

                            ),

                            plotly::plotlyOutput(ns("plot_network"), height = "1200px") |>

                shinycssloaders::withSpinner(type = 6, color = "#28a745")

            )

                    )

        )

            )

    )

    )

}





#' @title Pathway Relationship Exploration Module Server

#' @description Dual-mode pathway network visualization with defensive rendering

#' @keywords internal



mod_pathway_relation_server <- function(id, data_prep_list, gsea_res, table_result = NULL) {

    shiny::moduleServer(id, function(input, output, session) {

    ns <- session$ns



    # ============================================================

    # 1. Mode State Management

    # ============================================================



    network_mode <- shiny::reactiveVal("mode_topN")



    shiny::observeEvent(input$network_mode, {

            new_mode <- input$network_mode

            if (new_mode != network_mode()) {

        network_mode(new_mode)

            }

    })



    # ============================================================

    # 2. Data Source Reactive

    # ============================================================



    topN_candidates <- shiny::reactive({

            if (network_mode() != "mode_topN") {

        return(character(0))

            }

            data_list <- data_prep_list$data()

            shiny::req(data_list)

            df <- data_list$df

            shiny::req(nrow(df) > 0)

            top_n <- input$topN_count

            if (is.null(top_n)) top_n <- 50

            top_n <- max(1, min(top_n, nrow(df)))

            df[seq_len(top_n), "ID"]

    })



    select_candidates <- shiny::reactive({

            if (network_mode() != "mode_select") {

        return(character(0))

            }

            if (is.null(table_result) || is.null(table_result$selected_pathways)) {

        return(character(0))

            }

            selected <- table_result$selected_pathways()

            if (is.null(selected)) {

        return(character(0))

            }

            return(selected)

    })



    candidate_raw <- shiny::reactive({

            switch(network_mode(),

        "mode_topN" = topN_candidates(),

        "mode_select" = select_candidates(),

        character(0)

            )

    })



    candidate_filtered <- shiny::reactive({

            pathways <- candidate_raw()

            if (length(pathways) == 0) {

        return(character(0))

            }

            data_list <- data_prep_list$data()

            shiny::req(data_list)

            df <- data_list$df

            # IRON FIX (2026-07-14): mode_select = user picked from Main Table.

            # User intent takes priority - do NOT apply FDR filter again (would

            # silently drop pathways with p.adjust >= threshold, e.g.

            # KOHN_EMT_MESENCHYMAL p.adjust=0.30 > 0.25 in RZ_vs_RT contrast).

            # FDR filter only applies to mode_topN (automatic selection).

            if (network_mode() == "mode_select") {

        return(pathways[pathways %in% df$ID])

            }

            fdr_thresh <- input$fdr_threshold

            if (is.null(fdr_thresh)) fdr_thresh <- 0.25

            fdr_vec <- df$p.adjust[match(pathways, df$ID)]

            names(fdr_vec) <- pathways

            filtered <- pathways[!is.na(fdr_vec) & fdr_vec < fdr_thresh]

            return(filtered)

    })



    # ============================================================

    # 3. Final Pathway List Management

    # ============================================================



    final_pathways <- shiny::reactiveVal(character(0))



    shiny::observeEvent(candidate_filtered(), {

            new_candidates <- candidate_filtered()

            if (length(new_candidates) > 0) {

        max_n <- input$max_nodes

        if (is.null(max_n)) max_n <- 999

        final <- new_candidates[seq_len(min(length(new_candidates), max_n))]

        final_pathways(final)

            } else {

        final_pathways(character(0))

            }

    })



    # ============================================================

    # 4. Pathway Preview List UI

    # ============================================================



    output$pathway_preview_list <- shiny::renderUI({

            pathways <- final_pathways()

            if (length(pathways) == 0) {

        return(shiny::div(

                    style = "background: #fff3cd; padding: 10px; border-radius: 5px; color: #856404;",

                    shiny::strong("No pathways available"),

                    shiny::br(),

                    htmltools::tags$small("Adjust parameters or select pathways in Main Table")

        ))

            }

            data_list <- data_prep_list$data()

            df <- data_list$df

            pathway_info <- lapply(pathways, function(pid) {

        row_idx <- which(df$ID == pid)

        if (length(row_idx) == 0) {

                    return(NULL)

        }

        row <- df[row_idx[1], ]

        fdr <- row$p.adjust

        nes <- row$NES

        fdr_str <- if (!is.na(fdr)) sprintf("%.2e", fdr) else "N/A"

        nes_str <- if (!is.na(nes)) sprintf("%.2f", nes) else "N/A"

        shiny::div(

                    style = "background: #f8f9fa; padding: 8px; margin-bottom: 5px; border-radius: 4px; border-left: 3px solid #007bff;",

                    shiny::div(style = "font-size: 12px; overflow: hidden; text-overflow: ellipsis; white-space: nowrap;", shiny::strong(pid)),

                    shiny::div(style = "font-size: 11px; color: #666;", sprintf("FDR: %s | NES: %s", fdr_str, nes_str))

        )

            })

            shiny::tagList(pathway_info)

    })



    # ============================================================

    # 5. DotPlot Status Message

    # ============================================================



    output$dotplot_status <- shiny::renderUI({

            pathways <- final_pathways()

            status_parts <- c(

        sprintf("Mode: %s", ifelse(network_mode() == "mode_topN", "TopN", "Select")),

        sprintf("Pathways: %d", length(pathways)),

        if (network_mode() == "mode_select") "no FDR filter" else sprintf("FDR < %.2f", input$fdr_threshold)

            )

            if (length(pathways) == 0) {

        div_style <- "background: #fff3cd; padding: 10px; border-radius: 5px; margin-bottom: 10px; color: #856404;"

            } else {

        div_style <- "background: #d4edda; padding: 10px; border-radius: 5px; margin-bottom: 10px; color: #155724;"

            }

            shiny::div(style = div_style, shiny::HTML(paste(status_parts, collapse = " | ")))

    })



    # ============================================================

    # 6. Network Status Message

    # ============================================================



    output$network_status <- shiny::renderUI({

            pathways <- final_pathways()

            status_parts <- c(

        sprintf("Mode: %s", ifelse(network_mode() == "mode_topN", "TopN", "Select")),

        sprintf("Pathways: %d", length(pathways)),

        if (network_mode() == "mode_select") "no FDR filter" else sprintf("FDR < %.2f", input$fdr_threshold),

        sprintf("min_shared: %d", input$min_shared),

        sprintf("seed: %d", input$seed)

            )

            if (length(pathways) == 0) {

        div_style <- "background: #fff3cd; padding: 10px; border-radius: 5px; margin-bottom: 10px; color: #856404;"

            } else {

        div_style <- "background: #d4edda; padding: 10px; border-radius: 5px; margin-bottom: 10px; color: #155724;"

            }

            shiny::div(style = div_style, shiny::HTML(paste(status_parts, collapse = " | ")))

    })



    # ============================================================

    # 7. DotPlot Rendering Function

    # ============================================================



    output$plot_dotplot <- plotly::renderPlotly({

            pathways <- final_pathways()

            if (length(pathways) == 0) {

        return(plotly::plot_ly() |> plotly::layout(

                    title = list(text = "No pathways to display", font = list(size = 14), x = 0.5),

                    xaxis = list(showgrid = FALSE), yaxis = list(showgrid = FALSE)

        ))

            }

            data_list <- data_prep_list$data()

            shiny::req(data_list)

            df <- data_list$df

            plot_df <- df[df$ID %in% pathways, ]

            if (nrow(plot_df) == 0) {

        return(plotly::plot_ly() |> plotly::layout(title = list(text = "No matching pathways", font = list(size = 14))))

            }

            task <- list(

        gsea_res = data_list$gsea_res,

        meta = list(

                    left_group = data_list$left_group,

                    right_group = data_list$right_group,

                    contrast_id = data_list$contrast_id

        )

            )

            class(task) <- "GseaTask"

            core_list <- tryCatch(

        {

                    get_core_genes_list(task, plot_df$ID)

        },

        error = function(e) NULL

            )

            plot_df$CoreCount <- vapply(plot_df$ID, function(pid) {

        if (is.null(core_list) || is.null(core_list[[pid]])) {

                    return(0L)

        }

        length(core_list[[pid]])

            }, integer(1))

            # IRON FIX (2026-06-30): coerce key columns to numeric BEFORE any

            # arithmetic. Upstream GSEA result objects sometimes return p.adjust /

            # pvalue / NES / setSize as character or factor (especially after

            # dplyr subsetting or tibble round-trips). Without this, -log10() or

            # sqrt(pmax()) silently receive character vectors and raise either

            # "non-numeric argument to mathematical function" or

            # "non-numeric argument to binary operator" depending on which operator

            # fires first.

            plot_df$NES       <- suppressWarnings(as.numeric(plot_df$NES))

            plot_df$p.adjust  <- suppressWarnings(as.numeric(plot_df$p.adjust))

            plot_df$pvalue    <- suppressWarnings(as.numeric(plot_df$pvalue))

            plot_df$setSize   <- suppressWarnings(as.numeric(plot_df$setSize))

            plot_df$CoreCount <- suppressWarnings(as.integer(plot_df$CoreCount))

            # Guard against NA values that would propagate to NaN/Inf in downstream

            # log/sqrt operations (these render as blank plotly dots).

            plot_df$NES[is.na(plot_df$NES)]       <- 0

            plot_df$p.adjust[is.na(plot_df$p.adjust)] <- 1

            plot_df$pvalue[is.na(plot_df$pvalue)]     <- 1

            plot_df$setSize[is.na(plot_df$setSize)]   <- 0

            plot_df$CoreCount[is.na(plot_df$CoreCount)] <- 0



            color_mode <- input$dotplot_color_mode

            color_vals <- switch(color_mode,

        "padj" = -log10(plot_df$p.adjust),

        "pval" = -log10(plot_df$pvalue),

        "nes" = abs(plot_df$NES),

        -log10(plot_df$p.adjust)

            )

            color_title <- switch(color_mode,

        "padj" = "-log10(FDR)",

        "pval" = "-log10(P-value)",

        "nes" = "|NES|"

            )

            size_mode <- input$dotplot_size_mode

            size_vals <- switch(size_mode,

        "core_size" = plot_df$CoreCount,

        "setsize" = plot_df$setSize,

        plot_df$CoreCount

            )

            # ----- enrichplot-style size mapping (IRON FIX) -----

            # Previous implementation used per-render min-max normalization, which

            # (a) pinned the smallest/largest pathways to fixed pixel sizes every

            #     render, hiding absolute magnitude differences between CoreCount

            #     (typical 5-50) and setSize (typical 50-500), so switching "Size by"

            #     produced visually identical dot patterns;

            # (b) made dot sizes incomparable across FDR/TopN parameter changes.

            # Fix: use a FIXED domain per variable + sqrt perceptual scale, mirroring

            # ggplot2::scale_size_continuous(range=...) semantics used by

            # clusterProfiler::dotplot / enrichplot::dotplot.

            size_range  <- c(5, 25)

            size_domain <- switch(size_mode,

        "core_size" = c(0, 50),   # CoreCount (leading edge) typical range

        "setsize"   = c(0, 500),  # MSigDB gene-set total size typical range

        c(0, 50)

            )

            sqrt_lo <- sqrt(size_domain[1])

            sqrt_hi <- sqrt(size_domain[2])

            size_scaled <- size_range[1] +

        (sqrt(pmax(size_vals, 0)) - sqrt_lo) / (sqrt_hi - sqrt_lo) *

        (size_range[2] - size_range[1])

            size_scaled <- pmin(pmax(size_scaled, size_range[1]), size_range[2])

            # IRON FIX (2026-06-29): compute sort_idx ONCE on the ORIGINAL plot_df,

            # then reorder plot_df / size_scaled / color_vals by the SAME index.

            #

            # Previous bug:

            #   plot_df <- plot_df[order(plot_df$NES, decreasing = TRUE), ]

            #   size_scaled <- size_scaled[order(plot_df$NES, decreasing = TRUE)]  # WRONG

            #   color_vals  <- color_vals[order(plot_df$NES, decreasing = TRUE)]   # WRONG

            # The second call to order() ran on the ALREADY-SORTED plot_df$NES, so

            # when the source df was not pre-sorted by NES, size_scaled/color_vals

            # got permuted inconsistent with the new plot_df row order. Symptom:

            # "Color by switch has no visible effect" because the misalignment was

            # invariant across color_mode changes.

            sort_idx <- order(plot_df$NES, decreasing = TRUE)

            plot_df    <- plot_df[sort_idx, ]

            size_scaled <- size_scaled[sort_idx]

            color_vals  <- color_vals[sort_idx]

            # Hover shows BOTH size dimensions; bold the one currently driving

            # dot size so the user can correlate visual size with the value.

            core_fmt <- if (size_mode == "core_size") "<b>Core Genes: %d</b>" else "Core Genes: %d"

            set_fmt  <- if (size_mode == "setsize")  "<b>Set Size: %d</b>"   else "Set Size: %d"

            hover_text <- sprintf(

        paste0(

                    "<b>%s</b><br>FDR: %.2e<br>P-value: %.2e<br>NES: %.2f<br>",

                    core_fmt, " | ", set_fmt

        ),

        plot_df$ID, plot_df$p.adjust, plot_df$pvalue, plot_df$NES,

        plot_df$CoreCount, plot_df$setSize

            )

            plotly::plot_ly(

        data = plot_df,

        x = plot_df$NES,

        y = seq_len(nrow(plot_df)),

        type = "scatter",

        mode = "markers",

        marker = list(

                    size = size_scaled,

                    color = color_vals,

                    # IRON FIX (2026-06-29, updated 2026-07-21): unify with the
                    # PDF/ggplot2 export, which uses scale_fill_viridis_c(option="D",
                    # direction=-1). The colorscale below is the reversed viridis-D
                    # palette (high value = dark purple, low value = bright yellow),
                    # matching ggplot2's `direction = -1` semantics so that the
                    # interactive preview and the exported PDF render identically.
                    # Viridis-D reversed is perceptually uniform and colorblind-safe
                    # (publication-grade).

                    colorscale = list(

            list(0.0, "#FDE725"), list(0.25, "#5EC962"),

            list(0.5,  "#21918C"), list(0.75, "#3B528B"),

            list(1.0,  "#440154")

                    ),

                    cauto = TRUE,

                    showscale = TRUE,

                    colorbar = list(title = list(text = color_title, font = list(size = 12))),

                    line = list(color = "black", width = 1)

        ),

        text = hover_text,

        hoverinfo = "text",

        hovertemplate = "%{text}<extra></extra>"

            ) |>

        plotly::layout(

                    title = list(

            text = sprintf(

                            "<b>Pathway DotPlot: %s vs %s</b><br><sub style='font-size:10px; text-align: center;'><span style='color:#3B6EA5'>Enriched in %s</span> | <span style='color:#C0392B'>Enriched in %s</span></sub>",

                            data_list$left_group, data_list$right_group,

                            data_list$right_group, data_list$left_group),

            font = list(size = 12), x = 0.5, xanchor = "center"

                    ),

                    xaxis = list(title = "NES", zeroline = TRUE),

                    yaxis = list(title = "", tickmode = "array", tickvals = seq_len(nrow(plot_df)), ticktext = plot_df$ID, tickfont = list(size = 9)),

                    margin = list(l = 250, r = 50, t = 80, b = 50),

                    showlegend = FALSE

        ) |>

        plotly::config(displayModeBar = TRUE, displaylogo = FALSE)

    })



    # ============================================================

    # 8. Node Selection State Management

    # ============================================================



    selected_nodes <- shiny::reactiveVal(character(0))



    output$selection_display <- shiny::renderUI({

            sel <- selected_nodes()

            if (length(sel) == 0) {

        shiny::div(style = "color: #666; font-style: italic; text-align: center;", "No nodes selected. Click on network nodes.")

            } else if (length(sel) == 1) {

        shiny::div(style = "color: #007bff; font-weight: bold; text-align: center;", sprintf("Selected: %s", sel[1]))

            } else {

        shiny::div(style = "color: #28a745; font-weight: bold; text-align: center;", sprintf("Selected: %s with %s", sel[1], sel[2]))

            }

    })



    shiny::observe({

            sel <- selected_nodes()

            if (length(sel) == 2) {

        shinyjs::enable("show_edge_detail")

            } else {

        shinyjs::disable("show_edge_detail")

            }

    })



    shiny::observeEvent(input$clear_selection, {

            selected_nodes(character(0))

    })



    shiny::observeEvent(input$show_edge_detail, {

            sel <- selected_nodes()

            if (length(sel) != 2) {

        shiny::showNotification("Please select exactly 2 nodes first", type = "warning")

        return()

            }



            # Read the current edge list from the reactiveVal. This establishes a

            # proper reactive dependency on plot_network's computation. If the user

            # clicks before the network has finished rendering, we degrade gracefully

            # instead of crashing on `NULL$from`.

            edge_list <- edge_list_rv()

            if (is.null(edge_list)) {

        shiny::showNotification(

                    "Network is still being computed, please wait a moment and try again.",

                    type = "warning",

                    duration = 3

        )

        return()

            }



            from_pw <- sel[1]

            to_pw <- sel[2]



            edge_idx <- which(

        (edge_list$from == from_pw & edge_list$to == to_pw) |

                    (edge_list$from == to_pw & edge_list$to == from_pw)

            )



            if (length(edge_idx) == 0) {

        shiny::showNotification(

                    "No edge found between selected nodes. Try reducing min_shared threshold.",

                    type = "warning",

                    duration = 5

        )

        return()

            }



            i <- edge_idx[1]

            shared_count <- edge_list$shared[i]

            jaccard <- edge_list$weight[i]

            overlap_coef <- edge_list$overlap_coef[i]

            dice_coef <- edge_list$dice_coef[i]

            shared_genes <- unlist(edge_list$shared_genes[[i]])



            pathway_a_genes <- core_list[[from_pw]]

            pathway_b_genes <- core_list[[to_pw]]



            # ============ Modify start: display all genes in showdetail ============

            # In showdetail, do not limit the number of genes; display all shared genes

            # Use comma and space to separate genes



            # Format all genes for display

            gene_buttons <- vapply(shared_genes, function(g) {

        sprintf(

                    '<span style="display:inline-block;background:#e3f2fd;padding:4px 8px;margin:2px;border-radius:4px;font-size:12px;">%s</span>',

                    g

        )

            }, character(1))



            # No longer show "+N more" hint

            gene_display <- paste(gene_buttons, collapse = ",")



            # Modal UI

            shiny::showModal(shiny::modalDialog(

        title = HTML(paste0("<b>Edge Detail:</b> ", from_pw, " with ", to_pw)),

        size = "l",

        easyClose = TRUE,

        footer = shiny::modalButton("Close"),



        # Similarity metrics

        HTML("<div style='background:#f8f9fa;padding:15px;border-radius:8px;margin-bottom:15px;'>"),

        HTML("<h4 style='margin-top:0;'>Similarity Metrics</h4>"),

        HTML("<table style='width:100%;text-align:center;'>"),

        HTML("<tr><td><h3>", sprintf("%.3f", jaccard), "</h3><small>Jaccard</small></td>"),

        HTML("<td><h3>", sprintf("%.3f", overlap_coef), "</h3><small>Overlap</small></td>"),

        HTML("<td><h3>", sprintf("%.3f", dice_coef), "</h3><small>Dice</small></td>"),

        HTML("<td><h3>", shared_count, "</h3><small>Shared</small></td></tr>"),

        HTML("</table></div>"),



        # Shared genes

        HTML("<div style='background:#fff;padding:15px;border-radius:8px;border:1px solid #dee2e6;'>"),

        HTML("<h4 style='margin-top:0;'>Shared Core Genes (", shared_count, ")</h4>"),

        HTML(gene_display),

        HTML("</div>")

            ))

    })



    # ============================================================

    # 9. Network Rendering Function

    # ============================================================

    #

    # edge_list and node_df are computed inside output$plot_network and read

    # by observeEvent(input$show_edge_detail). Previously they used `<<-` to

    # mutate module-scope variables, but this had two problems:

    #   1. Bioconductor review: `<<-` is discouraged.

    #   2. Functional bug: `<<-` into a non-reactive variable does NOT trigger

    #      reactive dependency, so observeEvent could read a stale/NULL value

    #      during async re-rendering of plot_network.

    # Using reactiveVal() establishes a proper reactive dependency and lets

    # observeEvent gracefully degrade when the network is still being computed.

    edge_list_rv <- shiny::reactiveVal(NULL)

    node_df_rv <- shiny::reactiveVal(NULL)

    core_list <- NULL



    output$plot_network <- plotly::renderPlotly({

            pathways <- final_pathways()



            if (length(pathways) == 0) {

        return(plotly::plot_ly() |> plotly::layout(

                    title = list(text = "No pathways to display", font = list(size = 14), x = 0.5),

                    xaxis = list(showgrid = FALSE, showticklabels = FALSE),

                    yaxis = list(showgrid = FALSE, showticklabels = FALSE)

        ))

            }



            data_list <- data_prep_list$data()

            shiny::req(data_list)



            task <- list(

        gsea_res = data_list$gsea_res,

        meta = list(

                    left_group = data_list$left_group,

                    right_group = data_list$right_group,

                    contrast_id = data_list$contrast_id

        )

            )

            class(task) <- "GseaTask"



            core_list <- tryCatch(

        {

                    get_core_genes_list(task, pathways)

        },

        error = function(e) NULL

            )


            if (is.null(core_list) || length(core_list) == 0) {

        return(plotly::plot_ly() |> plotly::layout(

                    title = list(text = "No core genes found", font = list(size = 14)),

                    xaxis = list(showgrid = FALSE), yaxis = list(showgrid = FALSE)

        ))

            }



            valid_pathways <- names(core_list)[vapply(core_list, function(x) length(x) > 0, logical(1))]

            if (length(valid_pathways) == 0) {

        return(plotly::plot_ly() |> plotly::layout(

                    title = list(text = "All pathways have empty core genes", font = list(size = 14)),

                    xaxis = list(showgrid = FALSE), yaxis = list(showgrid = FALSE)

        ))

            }



            min_shared <- input$min_shared

            if (is.null(min_shared)) min_shared <- 3



            # Get hover max genes limit

            hover_max_genes <- input$hover_max_genes

            if (is.null(hover_max_genes)) hover_max_genes <- 10



            # Build edge list (now includes jaccard, overlap_coef, dice_coef).

            # Use a local object for downstream code in this renderPlotly scope,

            # and push it to the module-scope reactiveVal so observeEvent can read

            # the latest value with a proper reactive dependency.

            edge_list <- tryCatch(

        {

                    build_edge_list_safely(core_list[valid_pathways], min_shared_genes = min_shared)

        },

        error = function(e) NULL

            )

            edge_list_rv(edge_list)



            if (is.null(edge_list) || nrow(edge_list) == 0) {

        return(plotly::plot_ly() |> plotly::layout(

                    title = list(text = paste0("No edges found (min_shared=", min_shared, ")"), font = list(size = 14), x = 0.5),

                    xaxis = list(showgrid = FALSE, showticklabels = FALSE),

                    yaxis = list(showgrid = FALSE, showticklabels = FALSE)

        ))

            }



            # ============================================================

            # Edge width mapping (user-selectable via input$network_edge_width_mode)

            # ------------------------------------------------------------

            # Two encoding strategies are exposed because they serve different

            # analytical goals:

            #   "weight" (default)  -> edge width proportional to Jaccard value.

            #                          Faithful to the underlying similarity

            #                          magnitude (matches emapplot / ggraph

            #                          conventions). Recommended for publication.

            #   "rank"              -> edge width assigned by Jaccard rank order.

            #                          Guarantees uniform visual spacing across

            #                          edges regardless of absolute weight. Useful

            #                          when the goal is "make every edge visible"

            #                          (e.g. dense networks with small weight

            #                          variance). Original behavior.

            # ============================================================



            # Sort by Jaccard (weight) in descending order

            edge_list <- edge_list[order(edge_list$weight, decreasing = TRUE), ]



            width_mode <- if (is.null(input$network_edge_width_mode)) "weight"

                    else input$network_edge_width_mode



            n_edges <- nrow(edge_list)

            if (width_mode == "rank") {

        # Rank-based: assign 1-5 pixels based on rank (original behavior)

        edge_list$width_rank <- vapply(seq_len(n_edges), function(i) {

                    width <- 5 - (i - 1) * (4 / max(1, n_edges - 1))

                    max(1, min(5, round(width)))

        }, numeric(1))

            } else {

        # Weight-based: linear mapping of Jaccard in [0,1] to [1,8] pixels

        edge_list$width_rank <- 1 + edge_list$weight * 7

            }



            # Normal edge width range: rank->1-5, weight->1-8 pixels

            edge_list$edge_width_normal <- edge_list$width_rank



            # Selected edge fixed width: 7-8 pixels thicker than normal thickest (5 + 8 = 13)

            edge_list$edge_width_selected <- 13



            # Build igraph object

            g <- tryCatch(

        {

                    igraph::graph_from_data_frame(edge_list, directed = FALSE, vertices = valid_pathways)

        },

        error = function(e) NULL

            )



            if (is.null(g)) {

        return(plotly::plot_ly() |> plotly::layout(

                    title = list(text = "Graph construction failed", font = list(size = 14)),

                    xaxis = list(showgrid = FALSE), yaxis = list(showgrid = FALSE)

        ))

            }



            layout_algo <- input$network_layout

            # Defensive default: if input$seed is NULL (e.g. during fast reactive

            # invalidation before the numericInput has settled), fall back to 42.

            # This guarantees withr::with_seed() always receives an integer.

            seed_val <- if (is.null(input$seed)) 42L else as.integer(input$seed)



            layout_coords <- tryCatch(

        {

                    switch(layout_algo,

            "fr" = {

                            # Local seed scope: ensures reproducible FR layout across renders

                            # without polluting the caller's RNG state.

                            # seed_val is controlled by the user via input$seed (Shiny numericInput).

                            withr::with_seed(seed_val, igraph::layout_with_fr(g))

            },

            "kk" = igraph::layout_with_kk(g),

            "circle" = igraph::layout_in_circle(g),

            {

                            # Default: Fruchterman-Reingold; local seed scope as above.

                            withr::with_seed(seed_val, igraph::layout_with_fr(g))

            }

                    )

        },

        error = function(e) NULL

            )



            if (is.null(layout_coords)) {

        return(plotly::plot_ly() |> plotly::layout(

                    title = list(text = "Layout calculation failed", font = list(size = 14)),

                    xaxis = list(showgrid = FALSE), yaxis = list(showgrid = FALSE)

        ))

            }



            node_df <- data.frame(

        name = igraph::V(g)$name,

        x = layout_coords[, 1],

        y = layout_coords[, 2],

        stringsAsFactors = FALSE

            )

            node_df_rv(node_df)



            res_df <- as.data.frame(task$gsea_res@result)

            node_info <- res_df[match(node_df$name, res_df$ID), ]

            node_df$NES <- node_info$NES

            node_df$FDR <- node_info$p.adjust

            node_df$CoreCount <- vapply(node_df$name, function(n) {

        if (is.null(core_list[[n]])) {

                    return(0L)

        }

        length(core_list[[n]])

            }, integer(1))

            # IRON FIX (2026-06-30): coerce to numeric to prevent

            # "non-numeric argument to binary operator" when NES/FDR arrive as

            # character from the upstream GSEA result data.frame.

            node_df$NES <- suppressWarnings(as.numeric(node_df$NES))

            node_df$FDR <- suppressWarnings(as.numeric(node_df$FDR))

            node_df$NES[is.na(node_df$NES)] <- 0

            node_df$FDR[is.na(node_df$FDR)] <- 1

            node_df$color_val <- ifelse(is.na(node_df$NES), 0, node_df$NES)



            sel_nodes <- selected_nodes()



            node_hover_text <- sprintf(

        "<b>%s</b><br>NES: %.2f<br>FDR: %.2e<br>Core Genes: %d",

        node_df$name, node_df$NES, node_df$FDR, node_df$CoreCount

            )



            title_text <- sprintf(

        "Pathway Network: %s vs %s<br><sub>%d nodes, %d edges | Width: Jaccard rank (1-5px)</sub>",

        task$meta$left_group, task$meta$right_group,

        igraph::vcount(g), igraph::ecount(g)

            )



            p <- plotly::plot_ly(source = ns("network_plot"))



            # ============================================================

            # Add edges (using Jaccard ranking-mapped thickness)

            # ============================================================



            for (i in seq_len(nrow(edge_list))) {

        from_node <- node_df[node_df$name == edge_list$from[i], ]

        to_node <- node_df[node_df$name == edge_list$to[i], ]

        is_selected <- (edge_list$from[i] %in% sel_nodes) && (edge_list$to[i] %in% sel_nodes)



        if (is_selected) {

                    # Selected edge: orange + fixed thickness 13px

                    edge_color <- "#FF6600"

                    edge_width <- edge_list$edge_width_selected[i]

        } else {

                    # Normal edge: gray + Jaccard ranking thickness 1-5px

                    edge_color <- "rgba(150, 150, 150, 0.5)"

                    edge_width <- edge_list$edge_width_normal[i]

        }



        p <- p |> plotly::add_trace(

                    type = "scatter", mode = "lines",

                    x = c(from_node$x, to_node$x, NA),

                    y = c(from_node$y, to_node$y, NA),

                    line = list(color = edge_color, width = edge_width),

                    hoverinfo = "skip", showlegend = FALSE, inherit = TRUE

        )

            }



            # ============================================================

            # Add edge hover (enhanced: display shared gene list)

            # ============================================================



            for (i in seq_len(nrow(edge_list))) {

        from_node <- node_df[node_df$name == edge_list$from[i], ]

        to_node <- node_df[node_df$name == edge_list$to[i], ]



        is_selected <- (edge_list$from[i] %in% sel_nodes) && (edge_list$to[i] %in% sel_nodes)



        shared_genes_vec <- edge_list$shared_genes[[i]]

        shared_count <- edge_list$shared[i]



        if (length(shared_genes_vec) > 0) {

                    display_genes <- shared_genes_vec[seq_len(min(length(shared_genes_vec), hover_max_genes))]

                    genes_display <- paste(display_genes, collapse = ", ")

                    if (length(shared_genes_vec) > hover_max_genes) {

            remaining <- length(shared_genes_vec) - hover_max_genes

            genes_display <- paste0(

                            genes_display,

                            sprintf(" <span style='color:#ccc;'>(+%d more)</span>", remaining)

            )

                    }

        } else {

                    genes_display <- "(none)"

        }



        # Simplified format to reduce line breaks

        edge_hover <- sprintf(

                    "<b>%s with %s</b><br>Shared Genes (%d): %s<br>Jaccard: %.4f | Overlap: %.4f | Dice: %.4f",

                    edge_list$from[i], edge_list$to[i], shared_count,

                    genes_display,

                    edge_list$weight[i], edge_list$overlap_coef[i], edge_list$dice_coef[i]

        )



        hover_bg <- if (is_selected) "#FF8C00" else "#333"



        p <- p |> plotly::add_trace(

                    type = "scatter", mode = "markers",

                    x = c(mean(c(from_node$x, to_node$x))),

                    y = c(mean(c(from_node$y, to_node$y))),

                    marker = list(size = 12, opacity = 0, color = "transparent"),

                    text = edge_hover,

                    hoverinfo = "text",

                    hoverlabel = list(

            bgcolor = hover_bg,

            font = list(color = "white", size = 12),

            align = "left", # <- left-aligned

            bordercolor = hover_bg

                    ),

                    showlegend = FALSE,

                    inherit = TRUE

        )

            }



            # ============================================================

            # Add nodes

            # ============================================================



            node_colors <- ifelse(node_df$name %in% sel_nodes, "#FFD700", node_df$color_val)

            node_sizes <- ifelse(node_df$name %in% sel_nodes, 25, 15)

            node_border <- ifelse(node_df$name %in% sel_nodes, 4, 1.5)



            p <- p |> plotly::add_trace(

        type = "scatter", mode = "markers",

        x = node_df$x, y = node_df$y,

        marker = list(

                    size = node_sizes,

                    color = node_colors,

                    colorscale = list(list(0, "#377EB8"), list(0.5, "white"), list(1, "#E41A1C")),

                    cauto = FALSE, cmin = -3, cmax = 3, showscale = TRUE,

                    colorbar = list(title = list(text = "NES", font = list(size = 12)), len = 0.5, y = 0.5),

                    line = list(color = "black", width = node_border)

        ),

        text = node_df$name,

        hovertemplate = paste(node_hover_text, "<extra></extra>"),

        showlegend = FALSE, key = node_df$name, inherit = TRUE

            )



            # ============================================================

            # Add labels (selected nodes are larger and bolder)

            # ------------------------------------------------------------

            # IRON FIX (2026-06-29): Network text rendering improvements.

            #   * Default label size bumped 9 -> 11 (legibility at small zoom).

            #   * White "halo" shadow drawn UNDER the label so dark text remains

            #     readable against clustered nodes / overlapping edges. We achieve

            #     this by drawing TWO text traces: first a white shadow, then the

            #     colored label on top. plotly has no native stroke/fill, so the

            #     two-trace trick is the canonical workaround.

            #   * Vertical offset 0.12 -> 0.18 to avoid covering the node circle.

            #   * Display the pathway short name (strip "HALLMARK_" / dataset

            #     prefix) to reduce horizontal clutter; hover still shows the

            #     full ID.

            # ============================================================



            label_size     <- ifelse(node_df$name %in% sel_nodes, 16, 11)

            label_color    <- ifelse(node_df$name %in% sel_nodes, "#FF6600", "#222")

            label_bold     <- ifelse(node_df$name %in% sel_nodes, "bold",   "normal")

            label_short    <- sub("^[^_]*_", "", node_df$name)

            label_offset   <- 0.18



            # White shadow trace (drawn first -> underneath the colored label)

            p <- p |> plotly::add_trace(

        type = "scatter", mode = "text",

        x = node_df$x, y = node_df$y + label_offset,

        text = label_short,

        textposition = "top center",

        textfont = list(size = label_size + 1, color = "#FFFFFF",

                        family = "Arial Black, sans-serif"),

        hoverinfo = "skip", showlegend = FALSE, inherit = TRUE

            )



            # Colored label trace on top

            p <- p |> plotly::add_trace(

        type = "scatter", mode = "text",

        x = node_df$x, y = node_df$y + label_offset,

        text = label_short,

        textposition = "top center",

        textfont = list(size = label_size, color = label_color,

                        family = "Arial, sans-serif", font = label_bold),

        hoverinfo = "skip", showlegend = FALSE, inherit = TRUE

            )



            x_range <- c(min(layout_coords[, 1]) - 0.5, max(layout_coords[, 1]) + 0.5)

            y_range <- c(min(layout_coords[, 2]) - 0.5, max(layout_coords[, 2]) + 0.5)



            p |>

        plotly::layout(

                    title = list(text = title_text, font = list(size = 12), x = 0.5, xanchor = "center"),

                    xaxis = list(

            title = "",

            showgrid = FALSE,

            showticklabels = FALSE,

            zeroline = FALSE,

            range = x_range,

            scaleanchor = "x", # Maintain aspect ratio

            scaleratio = 1

                    ),

                    yaxis = list(

            title = "",

            showgrid = FALSE,

            showticklabels = FALSE,

            zeroline = FALSE,

            range = y_range,

            scaleanchor = "x", # Maintain aspect ratio

            scaleratio = 1

                    ),

                    hovermode = "closest",

                    dragmode = "pan",

                    showlegend = FALSE,

                    margin = list(l = 50, r = 50, t = 80, b = 50),

                    paper_bgcolor = "rgba(0,0,0,0)", # Transparent background

                    plot_bgcolor = "rgba(0,0,0,0)", # Transparent canvas

                    autosize = TRUE # <- Key: auto-resize

        ) |>

        plotly::config(

                    displayModeBar = TRUE,

                    displaylogo = FALSE,

                    modeBarButtonsToRemove = c("lasso2d", "select2d"),

                    responsive = TRUE # <- Key: responsive

        )

    })



    # ============================================================

    # 10. Node Click Event Listener

    # ============================================================



    shiny::observeEvent(plotly::event_data("plotly_click", source = ns("network_plot")), {

            click_data <- plotly::event_data("plotly_click", source = ns("network_plot"))

            if (is.null(click_data) || is.null(click_data$pointNumber)) {

        return()

            }



            n_edges <- nrow(edge_list)

            point_idx <- click_data$pointNumber + 1



            if (point_idx > nrow(node_df) || point_idx < 1) {

        return()

            }



            node_name <- as.character(node_df$name[point_idx])

            current_sel <- selected_nodes()



            if (length(current_sel) < 2) {

        if (!(node_name %in% current_sel)) {

                    selected_nodes(c(current_sel, node_name))

        }

            } else {

        selected_nodes(node_name)

            }

    })



    # ============================================================

    # 12. Export Center (DotPlot + Network)

    # ------------------------------------------------------------

    # Provides a unified modal for downloading the current view as a

    # static ggplot2 publication figure (PDF/PNG/SVG/TIFF) and for

    # copying the corresponding reproducible R code.

    #

    # Design decisions:

    #   * Static rendering uses ggplot2 + ggsave (zero external deps,

    #     unlike plotly::save_image which requires kaleido/orca).

    #   * Code is generated via generate_dotplot_code() /

    #     generate_network_code() in R/15_code_generator.R.

    #   * Clipboard copy uses clipr (already in Imports).

    # ============================================================



    # ----- Shared export modal UI builder -----

    # ------------------------------------------------------------

    # Redesign (2026-06-29):

    #   * Live preview pane reflects the current palette/size/width/height.

    #   * Download buttons split into "PDF" and "PNG" (the two formats users

    #     actually use most); SVG/TIFF remain available via the Format

    #     dropdown for advanced users.

    #   * Reactive preview re-runs the generator code in a sandboxed env

    #     (same parent=globalenv() fix used by the download handler).

    # ------------------------------------------------------------

    .build_export_modal <- function(plot_title, show_aesthetics = TRUE) {

            aesthetics_block <- if (isTRUE(show_aesthetics)) {

        shiny::tagList(

                    shiny::hr(),

                    shiny::h5("Aesthetics"),

                    shiny::selectInput(ns("exp_palette"), "Color palette",

            choices = c("Viridis-D (default)" = "D",

                        "Viridis-C"          = "C",

                        "Magma"               = "A",

                        "Inferno"             = "B"),

            selected = "D"),

                    shiny::sliderInput(ns("exp_point_range"), "Point size range",

            min = 1, max = 15, value = c(3, 8))

        )

            } else {

        # For Network exports: palette / point range have no effect because

        # the network generator uses a fixed Up/Down color scheme and node

        # sizes are derived from -log10(FDR) internally. Hide these inputs

        # so the UI does not mislead the user.

        shiny::tagList(

                    shiny::hr(),

                    shiny::div(

            style = "background:#f8f9fa; padding:8px 10px; border-radius:4px; color:#666; font-size:12px;",

            shiny::icon("info-circle"),

            " Network coloring uses a fixed Up (red) / Down (blue) scheme.",

            " Adjust edge width and layout seed in the main panel instead."

                    )

        )

            }



            shiny::modalDialog(

        title = sprintf("Export Publication Plot - %s", plot_title),

        size = "l",

        footer = NULL,

        easyClose = TRUE,

        shiny::fluidRow(

                    # Left column: parameters

                    shiny::column(5,

            shiny::h5("Dimensions"),

            shiny::fluidRow(

                            shiny::column(6,

                shiny::numericInput(ns("exp_width"),  "Width (inch)",  value = 9, min = 2, max = 24)

                            ),

                            shiny::column(6,

                shiny::numericInput(ns("exp_height"), "Height (inch)", value = 7, min = 2, max = 24)

                            )

            ),

            shiny::numericInput(ns("exp_dpi"), "DPI", value = 300, min = 72, max = 600),



            shiny::h6("Canvas Margin (pt)"),

            shiny::fluidRow(

                            shiny::column(3, shiny::numericInput(ns("exp_margin_top"),    "Top",    value = 18, min = 0, max = 80)),

                            shiny::column(3, shiny::numericInput(ns("exp_margin_bottom"), "Bottom", value = 18, min = 0, max = 80)),

                            shiny::column(3, shiny::numericInput(ns("exp_margin_left"),   "Left",   value = 18, min = 0, max = 80)),

                            shiny::column(3, shiny::numericInput(ns("exp_margin_right"),  "Right",  value = 18, min = 0, max = 80))

            ),

            aesthetics_block,

            shiny::hr(),

            shiny::h5("Download"),

            shiny::fluidRow(

                            shiny::column(6,

                shiny::downloadButton(ns("exp_download_pdf"),

                                    "Download PDF",

                                    class = "btn-danger btn-block",

                                    icon  = shiny::icon("file-pdf"))

                            ),

                            shiny::column(6,

                shiny::downloadButton(ns("exp_download_png"),

                                    "Download PNG",

                                    class = "btn-primary btn-block",

                                    icon  = shiny::icon("file-image"))

                            )

            ),

            shiny::hr(),

            shiny::fluidRow(

                            shiny::column(6,

                shiny::selectInput(ns("exp_format"), "Other format",

                                    choices = c("SVG" = "svg", "TIFF" = "tiff"),

                                    selected = "svg")

                            ),

                            shiny::column(6,

                shiny::div(style = "margin-top: 22px;",

                                    shiny::downloadButton(ns("exp_download_other"),

                    "Download",

                    class = "btn-default btn-block")

                )

                            )

            ),

            shiny::hr(),

            shiny::fluidRow(

                            shiny::column(6,

                shiny::actionButton(ns("exp_copy_code"),

                                    "Copy R Code",

                                    class = "btn-info btn-block",

                                    icon = shiny::icon("clipboard"))

                            ),

                            shiny::column(6,

                shiny::actionButton(ns("exp_dismiss"),

                                    "Close",

                                    class = "btn-default btn-block")

                            )

            )

                    ),

                    # Right column: live preview / external canvas

                    shiny::column(7,

            shiny::h5("Live Preview"),

            shiny::div(style = "background:#f5f5f5; border:1px solid #ddd; border-radius:4px; padding:8px; width:100%; height:520px; display:flex; align-items:center; justify-content:center; overflow:auto;",

                            shiny::plotOutput(ns("exp_preview")) |>

                shinycssloaders::withSpinner(type = 6, color = "#28a745")

            ),

            shiny::tags$small(style = "color: #666; display:block; margin-top:6px;",

                            "Preview reflects current Width / Height and is auto-scaled to fit while keeping the export aspect ratio.")

                    )

        )

            )

    }



    # ----- Reactive: live preview ggplot object -----

    # IRON FIX (2026-06-30): the generated R code includes a `print(p)` line

    # so the clipboard-copy path renders the plot when the user pastes it

    # into their own R console. But when we eval() the same code inside a

    # Shiny reactive for the live preview, `print(p)` opens a separate

    # graphics device (windows()/pdf()/quartz()) OUTSIDE Shiny's device

    # chain. On ggplot2 >= 4.0 with an active popup device, the subsequent

    # renderPlot print can surface "non-numeric argument to binary operator"

    # from unit/arithmetic ops occurring in the now-stale device context.

    # Fix: override `print` in eval_env to be a no-op so the popup never

    # opens; renderPlot does its own print on its own device.

    .exp_preview_plot <- shiny::reactive({

            tgt <- current_export_target()

            if (is.null(tgt)) return(NULL)

            # Wrap code generation in tryCatch so errors here surface as a Shiny

            # notification (and a clean NULL return) rather than propagating into

            # renderPlot and showing as a grey error pane.

            code <- tryCatch(

        .current_export_code(),

        error = function(e) {

                    shiny::showNotification(

            sprintf("[preview] code generation failed: %s", e$message),

            type = "error", duration = 8)

                    ""

        })

            if (!nzchar(code)) return(NULL)

            eval_env <- new.env(parent = globalenv())

            eval_env$p <- NULL

            eval_env$print <- base::invisible   # suppress popup device from print(p)
            eval_env$gsea_res <- gsea_res

            tryCatch(

        eval(parse(text = code), envir = eval_env),

        error = function(e) {

                    shiny::showNotification(

            sprintf("[preview] %s", e$message),

            type = "error", duration = 8)

                    NULL

        }

            )

            eval_env$p

    })



    # Preview plotOutput scales so the on-screen aspect ratio ALWAYS matches

    # the eventual saved figure, with the longer side capped to a max pixel

    # dimension so ultra-wide or ultra-tall specs still fit on screen.

    #

    # Algorithm: take the user's chosen width_in / height_in (inches), find

    # which is the longer side, pin it to max_dim (720 px), and scale the

    # shorter side by the same ratio. This preserves aspect ratio exactly

    # while bounding the preview to a sensible size:

    #   * 16x4 banner  -> 720x180 (wide-and-short)

    #   * 9x7 default  -> 720x560 (matches saved file proportions)

    #   * 4x16 column  -> 180x720 (tall-and-narrow, container scrolls)

    #

    # IRON FIX (2026-06-30): previously width/height were capped

    # INDEPENDENTLY at 720x540, which distorted the aspect ratio when

    # width > 7.2 inch (after that, width was clipped and the preview

    # stopped growing horizontally, showing a squished image).

    # Preview device matching export physical size (2026-07-06 v3):
    # Shiny renderPlot does NOT accept res as a function, so res is fixed
    # at 72 and pixel dims are computed as inch * 72 (with a pixel cap).
    # Physical size = width_px / 72 = w_in inches, matching the export.
    .preview_dims <- function(w_in, h_in, res = 72, max_px = 1600) {

            w_in <- suppressWarnings(as.numeric(w_in[1]))

            h_in <- suppressWarnings(as.numeric(h_in[1]))

            if (length(w_in) == 0 || is.na(w_in) || w_in <= 0) w_in <- 9

            if (length(h_in) == 0 || is.na(h_in) || h_in <= 0) h_in <- 7

            scale <- min(1, max_px / (max(w_in, h_in) * res))

            list(width  = round(w_in * res * scale),

                        height = round(h_in * res * scale))

    }



    output$exp_preview <- shiny::renderPlot(

            {

        p <- .exp_preview_plot()

        shiny::req(p)

        p

            },

            res  = 72,

            width  = function() {

        d <- .preview_dims(input$exp_width, input$exp_height); d$width

            },

            height = function() {

        d <- .preview_dims(input$exp_width, input$exp_height); d$height

            }

    )



    # ----- Reactive: current DotPlot data slice -----

    dotplot_export_df <- shiny::reactive({

            pathways <- final_pathways()

            if (length(pathways) == 0) return(NULL)

            data_list <- data_prep_list$data()

            shiny::req(data_list)

            df <- data_list$df

            plot_df <- df[df$ID %in% pathways, ]

            if (nrow(plot_df) == 0) return(NULL)



            task <- list(

        gsea_res = data_list$gsea_res,

        meta = list(left_group = data_list$left_group,

                    right_group = data_list$right_group,

                    contrast_id = data_list$contrast_id)

            )

            class(task) <- "GseaTask"

            core_list <- tryCatch(get_core_genes_list(task, plot_df$ID), error = function(e) NULL)

            plot_df$CoreCount <- vapply(plot_df$ID, function(pid) {

        if (is.null(core_list) || is.null(core_list[[pid]])) 0L else length(core_list[[pid]])

            }, integer(1))

            plot_df

    })



    # ----- Reactive: current Network edges + nodes -----

    network_export_data <- shiny::reactive({

            el <- edge_list_rv(); nd <- node_df_rv()

            if (is.null(el) || is.null(nd) || nrow(el) == 0) return(NULL)

            list(edge_df = el[, c("from", "to", "weight", "shared")],

                        node_df = nd)

    })



    # ----- Reactive: ggplot object + code string for the OPEN modal -----

    current_export_target <- shiny::reactiveVal(NULL)



    shiny::observeEvent(input$open_dotplot_export, {

            if (is.null(dotplot_export_df())) {

        shiny::showNotification("No pathways to export.", type = "warning"); return()

            }

            current_export_target("dotplot")

            shiny::showModal(.build_export_modal("Pathway DotPlot"))

    })



    shiny::observeEvent(input$open_network_export, {

            if (is.null(network_export_data())) {

        shiny::showNotification("No network to export.", type = "warning"); return()

            }

            current_export_target("network")

            # Network exports ignore palette/point_range (they use a fixed Up/Down

            # scheme), so hide the Aesthetics block to avoid misleading the user.

            shiny::showModal(.build_export_modal("Pathway Network", show_aesthetics = FALSE))

    })



    shiny::observeEvent(input$exp_dismiss, shiny::removeModal())



    # ----- Shared code builder (depends on current target) -----

    .current_export_code <- function() {

            pal <- if (is.null(input$exp_palette)) "D" else input$exp_palette

            pr  <- if (is.null(input$exp_point_range)) c(3, 8) else input$exp_point_range

            data_list <- data_prep_list$data()

            lg <- if (is.null(data_list$left_group))  "Left"  else data_list$left_group

            rg <- if (is.null(data_list$right_group)) "Right" else data_list$right_group



            tgt <- current_export_target()

            if (is.null(tgt)) return("")



            if (tgt == "dotplot") {

        generate_dotplot_code(

                    gsea_res_var = "gsea_res",

                    contrast_id  = data_list$contrast_id,

                    pathways     = final_pathways(),

                    color_mode   = input$dotplot_color_mode,

                    size_mode    = input$dotplot_size_mode,

                    left_group   = lg, right_group = rg,

                    palette      = pal, point_range = pr,

                    target_collection = if (is.null(data_list$collections)) "ALL" else data_list$collections,

                    margin_top    = if (is.null(input$exp_margin_top)    || is.na(input$exp_margin_top))    18 else input$exp_margin_top,
                    margin_bottom = if (is.null(input$exp_margin_bottom) || is.na(input$exp_margin_bottom)) 18 else input$exp_margin_bottom,
                    margin_left   = if (is.null(input$exp_margin_left)   || is.na(input$exp_margin_left))   18 else input$exp_margin_left,
                    margin_right  = if (is.null(input$exp_margin_right)  || is.na(input$exp_margin_right))  18 else input$exp_margin_right,

                    fig_width     = if (is.null(input$exp_width)         || is.na(input$exp_width))          9 else input$exp_width,

                    fig_height    = if (is.null(input$exp_height)        || is.na(input$exp_height))         7 else input$exp_height,
                    fig_dpi    = if (is.null(input$exp_dpi)   || is.na(input$exp_dpi))   300 else input$exp_dpi)

            } else if (tgt == "network") {

        seed_val <- if (is.null(input$seed)) 42L else as.integer(input$seed)

        data_list <- data_prep_list$data()

        generate_network_code(

                    gsea_res_var = "gsea_res",

                    contrast_id  = data_list$contrast_id,

                    pathways     = final_pathways(),

                    min_shared_genes = if (is.null(input$min_shared)) 3 else input$min_shared,

                    width_mode   = if (is.null(input$network_edge_width_mode)) "weight"

                                                    else input$network_edge_width_mode,

                    seed = seed_val,

                    target_collection = if (is.null(data_list$collections)) "ALL" else data_list$collections,

                    margin_top    = if (is.null(input$exp_margin_top)    || is.na(input$exp_margin_top))    18 else input$exp_margin_top,

                    margin_bottom = if (is.null(input$exp_margin_bottom) || is.na(input$exp_margin_bottom)) 18 else input$exp_margin_bottom,

                    margin_left   = if (is.null(input$exp_margin_left)   || is.na(input$exp_margin_left))   18 else input$exp_margin_left,

                    margin_right  = if (is.null(input$exp_margin_right)  || is.na(input$exp_margin_right))  18 else input$exp_margin_right,

                    fig_width     = if (is.null(input$exp_width)         || is.na(input$exp_width))         10 else input$exp_width,

                    fig_height    = if (is.null(input$exp_height)        || is.na(input$exp_height))         8 else input$exp_height,
                    fig_dpi    = if (is.null(input$exp_dpi)   || is.na(input$exp_dpi))   300 else input$exp_dpi)

            } else ""

    }



    # ----- Internal helper: render to a specific format -----

    # ------------------------------------------------------------

    # Three download handlers (exp_download_pdf / _png / _other) below all

    # delegate to this helper so behavior is identical modulo format.

    # ------------------------------------------------------------

    .export_render_to_file <- function(file, fmt) {

            code <- .current_export_code()

            if (!nzchar(code)) {

        shiny::showNotification("Nothing to export.", type = "warning"); return()

            }

            # parent=globalenv() so lexical lookup resolves attached packages

            # (see IRON FIX comment in v0.99.11 commit). baseenv() reproduces

            # "could not find function 'ggplot'" and the .htm fallback symptom.

            eval_env <- new.env(parent = globalenv())

            eval_env$p <- NULL

            eval_env$print <- base::invisible   # suppress popup device; ggsave() draws itself
            eval_env$gsea_res <- gsea_res

            tryCatch({

        eval(parse(text = code), envir = eval_env)

            }, error = function(e) {

        shiny::showNotification(sprintf("Render failed: %s", e$message),

                                type = "error", duration = 8)

            })

            if (is.null(eval_env$p)) return()

            ggplot2::ggsave(

        file, eval_env$p,

        width  = if (is.null(input$exp_width))  9 else input$exp_width,

        height = if (is.null(input$exp_height)) 7 else input$exp_height,

        dpi    = if (is.null(input$exp_dpi))   300 else input$exp_dpi,

        device = fmt

            )

    }



    output$exp_download_pdf <- shiny::downloadHandler(

            filename = function() sprintf("GSEAlens_%s_%s.pdf",

                                    current_export_target() %||% "plot", Sys.Date()),

            content  = function(file) .export_render_to_file(file, "pdf")

    )



    output$exp_download_png <- shiny::downloadHandler(

            filename = function() sprintf("GSEAlens_%s_%s.png",

                                    current_export_target() %||% "plot", Sys.Date()),

            content  = function(file) .export_render_to_file(file, "png")

    )



    output$exp_download_other <- shiny::downloadHandler(

            filename = function() sprintf("GSEAlens_%s_%s.%s",

                                    current_export_target() %||% "plot", Sys.Date(),

                                    if (is.null(input$exp_format)) "svg" else input$exp_format),

            content  = function(file) .export_render_to_file(file, input$exp_format)

    )



    # ----- Copy code to clipboard -----

    shiny::observeEvent(input$exp_copy_code, {

            code <- .current_export_code()

            if (!nzchar(code)) {

        shiny::showNotification("Nothing to copy.", type = "warning"); return()

            }

            ok <- tryCatch({ clipr::write_clip(code); TRUE }, error = function(e) FALSE)

            if (isTRUE(ok)) {

        shiny::showNotification("R code copied to clipboard.", type = "message")

            } else {

        # Fallback: show code in a new modal so the user can manually copy

        shiny::showModal(shiny::modalDialog(

                    title = "Reproducible R Code (copy manually)",

                    size = "l", easyClose = TRUE,

                    shiny::pre(style = "max-height: 60vh; overflow-y: auto; font-size: 11px;",

                                            code),

                    footer = shiny::modalButton("Close")

        ))

            }

    })



    # ============================================================

    # 13. Return Values

    # ============================================================



    return(list(

            final_pathways = final_pathways,

            network_mode = network_mode

    ))

    })

}

#' @title Generate Pathway Plotting Code

#' @param GSEAlens_res GseaRes object

#' @param contrast_id Character. Contrast ID

#' @param target_pathways Character vector. Pathway IDs

#' @param user_genes Character vector. Genes of interest

#' @param pathway_genes Character vector. Pathway genes (for coloring)

#' @param expr_type Character. Expression type

#' @return A character string containing executable R code

#' @export



#' @examples

#' # Generate plotting code for the top enriched pathway in a pre-computed task

#' data(precomputed_gseares, package = "GSEAlens")
#'
#' gsea_res <- precomputed_gseares

#' task <- extract_gsea_task(gsea_res, contrast_id = "untrt_vs_trt")

#' res_df <- as.data.frame(task$gsea_res)

#' top_pathway <- res_df$ID[1]

#'

#' code <- generate_pathway_plot_code(

#'   GSEAlens_res   = gsea_res,

#'   contrast_id    = "untrt_vs_trt",

#'   target_pathways = top_pathway

#' )

#' cat(code)

generate_pathway_plot_code <- function(GSEAlens_res,

                                                                                contrast_id,

                                                                                target_pathways,

                                                                                user_genes = character(0),

                                                                                pathway_genes = character(0),

                                                                                expr_type = "logcpm") {

    # Extract necessary information

    GSEAlens_task <- tryCatch(

    {

            GSEAlens::extract_gsea_task(GSEAlens_res, contrast_id)

    },

    error = function(e) {

            stop("Failed to extract task for contrast '", contrast_id, "': ", e$message)

    }

    )



    left_group <- GSEAlens_task$meta$left_group

    right_group <- GSEAlens_task$meta$right_group



    # Color palette

    colors <- c(

    "#E41A1C", "#377EB8", "#4DAF4A", "#984EA3",

    "#FF7F00", "#A65628", "#F781BF", "#999999"

    )

    color_str <- paste(colors[seq_len(min(4, length(target_pathways)))], collapse = '", "')



    # Format pathway vector

    pathways_str <- paste(sprintf('    "%s"', target_pathways), collapse = ",\n")



    # Format user genes vector (no newline)

    if (length(user_genes) > 0) {

    genes_str <- paste0('GSEAlens_genes <- c("', paste(user_genes, collapse = '", "'), '")')

    } else {

    genes_str <- "GSEAlens_genes <- character(0)  # No genes of interest specified"

    }



    # Format pathway genes vector (no newline)

    if (length(pathway_genes) > 0) {

    pathway_str <- paste0('GSEAlens_pathway_genes <- c("', paste(pathway_genes, collapse = '", "'), '")')

    } else {

    pathway_str <- "GSEAlens_pathway_genes <- character(0)  # No pathway genes specified"

    }



    # Build code using paste0

    code <- paste0(

    '# =============================================================================

# GSEAlens Static Plotting Code

# Generated by GSEAlens Shiny App

# =============================================================================



# Step 1: Load your GseaRes object

GSEAlens_res <- YOUR_GSEA_RES_OBJECT



# Step 2: Extract task for the specified contrast

GSEAlens_task <- GSEAlens::extract_gsea_task(GSEAlens_res, contrast_id = "', contrast_id, '")

GSEAlens_left_group <- "', left_group, '"

GSEAlens_right_group <- "', right_group, '"



# Step 3: Define pathways to plot

GSEAlens_pathways <- c(

', pathways_str, "

)



# Step 4: Define genes of interest (optional)

", genes_str, "



# Step 5: Define pathway genes (for coloring)

", pathway_str, '



# Step 6: Load required libraries


library(ggplot2)

library(patchwork)

library(ggrepel)



# -----------------------------------------------------------------------------

# 6.1 Combined GSEA Enrichment Plot

# -----------------------------------------------------------------------------

GSEAlens_combined_plot <- plot_directional_gsea(

    directional_gsea_obj = GSEAlens_task,

    target_pathways = GSEAlens_pathways,

    subPlot = 3,

    curveCol = c("', color_str, '"),

    main_title = paste0("Combined: ", length(GSEAlens_pathways), " Pathways")

)

print(GSEAlens_combined_plot)



# -----------------------------------------------------------------------------

# 6.2 DE Volcano Plot with ggrepel Labels (Six-Color Aesthetics)

# -----------------------------------------------------------------------------

GSEAlens_de_df <- GSEAlens::get_de_table(GSEAlens_res, contrast_id = "', contrast_id, '")



# Thresholds

GSEAlens_logfc_thresh <- 1

GSEAlens_pval_thresh <- 0.05



# Color definitions

GSEAlens_color_map <- c(

    "both" = "#9C27B0",    # Purple: User & Pathway

    "user" = "#4DAF4A",    # Green: User only

    "pathway" = "#FF9800",  # Orange: Pathway only

    "up" = "#E41A1C",      # Red: Up-regulated

    "down" = "#377EB8",     # Blue: Down-regulated

    "ns" = "#C0C0C0"       # Gray: Not significant

)



# Classify genes

GSEAlens_user_upper <- toupper(GSEAlens_genes)

GSEAlens_pathway_upper <- toupper(GSEAlens_pathway_genes)

GSEAlens_de_df$GSEAlens_is_user <- toupper(GSEAlens_de_df$gene_symbol) %in% GSEAlens_user_upper

GSEAlens_de_df$GSEAlens_is_pathway <- toupper(GSEAlens_de_df$gene_symbol) %in% GSEAlens_pathway_upper

GSEAlens_de_df$GSEAlens_is_significant <- abs(GSEAlens_de_df$logFC) > GSEAlens_logfc_thresh & GSEAlens_de_df$pvalue < GSEAlens_pval_thresh



# Assign category colors

GSEAlens_de_df$GSEAlens_category <- dplyr::case_when(

    GSEAlens_de_df$GSEAlens_is_user & GSEAlens_de_df$GSEAlens_is_pathway ~ "both",

    GSEAlens_de_df$GSEAlens_is_user ~ "user",

    GSEAlens_de_df$GSEAlens_is_pathway ~ "pathway",

    GSEAlens_de_df$GSEAlens_is_significant & GSEAlens_de_df$logFC > 0 ~ "up",

    GSEAlens_de_df$GSEAlens_is_significant & GSEAlens_de_df$logFC < 0 ~ "down",

    TRUE ~ "ns"

)



# Assign plot order (higher = on top layer)

GSEAlens_de_df$GSEAlens_plot_order <- dplyr::case_when(

    GSEAlens_de_df$GSEAlens_is_user & GSEAlens_de_df$GSEAlens_is_pathway ~ 5,  # both - top layer

    GSEAlens_de_df$GSEAlens_is_user ~ 4,  # user

    GSEAlens_de_df$GSEAlens_is_pathway ~ 3,  # pathway

    GSEAlens_de_df$GSEAlens_is_significant ~ 2,  # up/down

    TRUE ~ 1  # ns - bottom layer

)



# Sort by plot order (draw bottom layer first, top layer last)

GSEAlens_de_df <- GSEAlens_de_df[order(GSEAlens_de_df$GSEAlens_plot_order), ]



# Statistics

GSEAlens_n_up <- sum(GSEAlens_de_df$GSEAlens_is_significant & GSEAlens_de_df$logFC > 0, na.rm = TRUE)

GSEAlens_n_down <- sum(GSEAlens_de_df$GSEAlens_is_significant & GSEAlens_de_df$logFC < 0, na.rm = TRUE)

GSEAlens_n_ns <- sum(!GSEAlens_de_df$GSEAlens_is_significant, na.rm = TRUE)

GSEAlens_n_user <- sum(GSEAlens_de_df$GSEAlens_is_user, na.rm = TRUE)

GSEAlens_n_pathway <- sum(GSEAlens_de_df$GSEAlens_is_pathway, na.rm = TRUE)

GSEAlens_n_both <- sum(GSEAlens_de_df$GSEAlens_is_user & GSEAlens_de_df$GSEAlens_is_pathway, na.rm = TRUE)



# Create volcano plot

GSEAlens_volcano_plot <- ggplot(GSEAlens_de_df, aes(x = logFC, y = -log10(pvalue))) +

    geom_point(aes(color = GSEAlens_category), alpha = 0.6, size = 3) +

    # Add ggrepel labels for both and user genes

    ggrepel::geom_text_repel(

    data = GSEAlens_de_df[GSEAlens_de_df$GSEAlens_is_user, ],

    aes(label = gene_symbol),

    size = 3,

    max.overlaps = 50,

    box.padding = 0.3,

    point.padding = 0.2,

    segment.alpha = 0.5,

    fontface = "bold",

    # Color based on is_pathway: both=purple, user=green

    color = ifelse(GSEAlens_de_df$GSEAlens_is_user[GSEAlens_de_df$GSEAlens_is_user] & GSEAlens_de_df$GSEAlens_is_pathway[GSEAlens_de_df$GSEAlens_is_user], "#9C27B0", "#4DAF4A")

    ) +

    scale_color_manual(

    values = GSEAlens_color_map,

    breaks = c("both", "user", "pathway", "up", "down", "ns"),

    labels = c("Both", "Selected", "Pathway", "Up", "Down", "NS")

    ) +

    labs(

    title = paste0(GSEAlens_left_group, " vs ", GSEAlens_right_group, " - DE Volcano"),

    subtitle = paste0(

    "Both: ", GSEAlens_n_both, " | Selected: ", GSEAlens_n_user, " | Pathway: ", GSEAlens_n_pathway, " | Up: ", GSEAlens_n_up, " | Down: ", GSEAlens_n_down, " | NS: ", GSEAlens_n_ns,

    "\n",

    tail(GSEAlens_pathways, 1)

),

    x = "log2 Fold Change",

    y = "-log10(P-value)"

    ) +

    theme_bw() +

    theme(

    legend.position = "bottom",

    legend.title = element_blank(),

    plot.title = element_text(hjust = 0.5),

    plot.subtitle = element_text(hjust = 0.5, color = "gray40", lineheight = 0.8)

    ) +

    geom_vline(xintercept = c(-GSEAlens_logfc_thresh, GSEAlens_logfc_thresh),

                            linetype = "dashed", color = "gray50", alpha = 0.7) +

    geom_hline(yintercept = -log10(GSEAlens_pval_thresh),

                            linetype = "dashed", color = "gray50", alpha = 0.7)



print(GSEAlens_volcano_plot)



# -----------------------------------------------------------------------------

# 6.3 Save plots to file (optional)

# --- Save to file (uncomment one) --------------------------------------------
# ggsave("GSEAlens_combined_plot.pdf", GSEAlens_combined_plot, width = 12, height = 8, device = cairo_pdf)
# ggsave("GSEAlens_combined_plot.png", GSEAlens_combined_plot, width = 12, height = 8, dpi = 300)
# ggsave("GSEAlens_combined_plot.svg", GSEAlens_combined_plot, width = 12, height = 8)
# ggsave("GSEAlens_volcano_plot.pdf", GSEAlens_volcano_plot, width = 10, height = 7, device = cairo_pdf)
# ggsave("GSEAlens_volcano_plot.png", GSEAlens_volcano_plot, width = 10, height = 7, dpi = 300)
# ggsave("GSEAlens_volcano_plot.svg", GSEAlens_volcano_plot, width = 10, height = 7)

'

    )



    return(code)

}





#' @title Generate Joint Canvas Code

#' @param GSEAlens_res GseaRes object

#' @param contrast_ids Character vector. Multiple contrast IDs

#' @param target_pathways Character vector. Pathway IDs

#' @param ncol Integer. Number of columns

#' @return A character string containing executable R code

#' @export



#' @examples
#' \donttest{
#' data(precomputed_gseares, package = "GSEAlens")
#' task <- extract_gsea_task(precomputed_gseares, contrast_id = "untrt_vs_trt")
#' target <- as.data.frame(task$gsea_res)$ID[1]
#' code <- generate_joint_canvas_code(GSEAlens_res = precomputed_gseares,
#'                                    contrast_ids = "untrt_vs_trt",
#'                                    target_pathways = target)
#' cat(code)
#' }
generate_joint_canvas_code <- function(GSEAlens_res,

                                                                                contrast_ids,

                                                                                target_pathways,

                                                                                ncol = 2) {

    # Format contrast IDs

    contrasts_str <- paste(sprintf('    "%s"', contrast_ids), collapse = ",\n")



    # Format pathways

    pathways_str <- paste(sprintf('    "%s"', target_pathways), collapse = ",\n")



    # Color palette

    colors <- c(

    "#E41A1C", "#377EB8", "#4DAF4A", "#984EA3",

    "#FF7F00", "#A65628", "#F781BF", "#999999"

    )

    color_str <- paste(colors[seq_len(min(4, length(target_pathways)))], collapse = '", "')



    # Build code

    code <- paste0(

    "# =============================================================================

# GSEAlens Joint Canvas Code

# Generated by GSEAlens Shiny App

# =============================================================================



# Step 1: Load your GseaRes object

GSEAlens_res <- YOUR_GSEA_RES_OBJECT



# Step 2: Define parameters

GSEAlens_contrasts <- c(

", contrasts_str, "

)



GSEAlens_pathways <- c(

", pathways_str, "

)



GSEAlens_ncol <- ", ncol, '



# Step 3: Load required libraries


library(ggplot2)

library(patchwork)



# Step 4: Generate plots for each contrast

GSEAlens_plot_list <- list()



for (GSEAlens_i in seq_along(GSEAlens_contrasts)) {

    GSEAlens_contrast_id <- GSEAlens_contrasts[GSEAlens_i]



    GSEAlens_task <- GSEAlens::extract_gsea_task(GSEAlens_res, contrast_id = GSEAlens_contrast_id)



    GSEAlens_p <- plot_directional_gsea(

    directional_gsea_obj = GSEAlens_task,

    target_pathways = GSEAlens_pathways,

    subPlot = 3,

    curveCol = c("', color_str, '"),

    main_title = paste0(GSEAlens_contrast_id, " [", length(GSEAlens_pathways), " pathways]"),

    add_pval = FALSE,

    show_contrast_in_axis = TRUE

    )



    GSEAlens_plot_list[[GSEAlens_i]] <- GSEAlens_p

}



# Step 5: Combine plots using patchwork

GSEAlens_combined <- patchwork::wrap_plots(

    GSEAlens_plot_list,

    ncol = GSEAlens_ncol,

    nrow = ceiling(length(GSEAlens_plot_list) / GSEAlens_ncol),

    byrow = TRUE,

    guides = "collect"

) + patchwork::plot_annotation(

    title = paste0("Joint GSEA Canvas: ", length(GSEAlens_contrasts), " contrasts x ", length(GSEAlens_pathways), " pathways"),

    subtitle = paste0("Arrangement: ", paste(GSEAlens_contrasts, collapse = " -> ")),

    theme = theme(

    plot.title = element_text(size = 14, face = "bold", hjust = 0.5),

    plot.subtitle = element_text(size = 10, color = "gray50", hjust = 0.5)

    )

)



# Step 6: Wrap the canvas so outer margin (canvas breathing room) does NOT

# leak into the per-contrast 3-panel GSEA plot. Setting plot.margin on the

# bare canvas via the and-operator would override the per-panel margin = 0

# configured in plot_directional_gsea and .gsea_nb_core, re-inflating the

# gaps between Panel 1 (RES curve), Panel 2 (hit strips) and Panel 3

# (ranked list). wrap_elements(full = ...) keeps the title/subtitle

# while confining the outer margin to the canvas border.

GSEAlens_combined <- patchwork::wrap_elements(full = GSEAlens_combined) +

    theme(plot.margin = margin(10, 10, 10, 10))



# Step 7: Display combined plot

print(GSEAlens_combined)



# Step 8: Save to file (uncomment one)

# ggsave("GSEAlens_joint_canvas.pdf", GSEAlens_combined, width = 14, height = 10, device = cairo_pdf)

# ggsave("GSEAlens_joint_canvas.png", GSEAlens_combined, width = 14, height = 10, dpi = 300)

# ggsave("GSEAlens_joint_canvas.svg", GSEAlens_combined, width = 14, height = 10)

'

    )



    return(code)
}






#' @title Generate Combined Pathway Plotting Code
#' @description Return an executable R script reproducing the Combined Pathway
#'   Plotting module (mod_multi_plot / module 12) as a static figure. Mirrors
#'   the interactive \code{plot_directional_gsea()} call the app performs when
#'   the user selects pathways in the main table's "Combined Display" column.
#' @param gsea_res_var Character. Name of the GseaRes variable the generated
#'   code should reference (default \code{"gsea_res"}). The user replaces the
#'   placeholder assignment with their own object.
#' @param contrast_id Character. Contrast ID passed to
#'   \code{extract_gsea_task()}.
#' @param target_pathways Character vector. Pathway IDs to plot.
#' @param subPlot Integer (1-3). \code{plot_directional_gsea} subPlot argument
#'   (1 = running score, 2 = rank position, 3 = both). Default 3.
#' @param curve_colors Character vector of color hex codes for pathway curves.
#'   If NULL, falls back to a sensible default palette.
#' @param left_group Character. Left group label for axis annotation.
#' @param right_group Character. Right group label.
#' @return Character scalar of executable R code.
#' @examples
#' code <- generate_combined_plot_code(
#'   contrast_id    = "untrt_vs_trt",
#'   target_pathways = c("HALLMARK_TNFA_SIGNALING_VIA_NFKB")
#' )
#' cat(code)
#' @export
generate_combined_plot_code <- function(gsea_res_var = "gsea_res",
                                                                                    contrast_id,
                                                                                    target_pathways,
                                                                                    subPlot = 3,
                                                                                    curve_colors = NULL,
                                                                                    left_group = "Left",
                                                                                    right_group = "Right") {
    if (length(target_pathways) == 0) {
    return("# No pathways selected; nothing to plot.")
    }

    pathways_str <- paste(sprintf('    "%s"', target_pathways), collapse = ",\n")

    if (is.null(curve_colors) || length(curve_colors) == 0) {
    curve_colors <- c("#E41A1C", "#377EB8", "#4DAF4A", "#984EA3",
                                            "#FF7F00", "#A65628", "#F781BF", "#999999")
    }
    color_str <- paste(curve_colors, collapse = '", "')

    sprintf(
'# =============================================================================
# GSEAlens Combined Pathway Plotting Code
# Generated by GSEAlens Shiny App
# =============================================================================

# Step 1: Load your GseaRes object
%s <- YOUR_GSEA_RES_OBJECT

# Step 2: Define parameters
GSEAlens_contrast_id <- "%s"
GSEAlens_pathways <- c(
%s
)
GSEAlens_subPlot   <- %d
GSEAlens_left_grp  <- "%s"
GSEAlens_right_grp <- "%s"

# Step 3: Load required libraries
library(ggplot2)

# Step 4: Extract the task for the specified contrast
# NOTE: target_collection = "ALL" merges cross-collection results when applicable.
GSEAlens_task <- GSEAlens::extract_gsea_task(%s, contrast_id = GSEAlens_contrast_id, target_collection = "ALL")

# Step 5: Build the combined directional GSEA plot
GSEAlens_p <- plot_directional_gsea(
    directional_gsea_obj = GSEAlens_task,
    target_pathways = GSEAlens_pathways,
    subPlot = GSEAlens_subPlot,
    curveCol = c("%s")
)

# Step 6: Display
print(GSEAlens_p)

# Step 7: Save to file (uncomment one)
# ggsave("GSEAlens_combined_plot.pdf", GSEAlens_p, width = 12, height = 10, device = cairo_pdf)
# ggsave("GSEAlens_combined_plot.png", GSEAlens_p, width = 12, height = 10, dpi = 300)
# ggsave("GSEAlens_combined_plot.svg", GSEAlens_p, width = 12, height = 10)
',
    gsea_res_var,
    contrast_id,
    pathways_str,
    as.integer(subPlot),
    left_group,
    right_group,
    gsea_res_var,
    color_str
    )
}




#' @title Generate Boxplot Expression-Data Extraction Code

#' @description Produce a self-contained, reproducible R script that extracts

#'   the per-sample expression values (Sample / Group / Expression) of a

#'   single gene from a \code{GseaRes} object. The returned data frame

#'   matches exactly what is shown in the Quadrant module's expression

#'   boxplot (panel 4), so the user can paste it into other software

#'   (e.g. GraphPad Prism, Excel, R) for downstream plotting.

#' @param GSEAlens_res A \code{GseaRes} object.

#' @param contrast_id Character. Contrast ID (e.g. "Treat_vs_Control").

#' @param gene_symbol Character. Gene symbol as shown in the boxplot title.

#' @param expr_type Character. Expression matrix type passed to

#'   \code{get_expr_matrix()} (e.g. "logcpm", "vst", "count"). Default "logcpm".

#' @param custom_order Character or NULL. Custom group ordering string

#'   (e.g. "GroupA -> GroupB") or "default" / NULL to use natural order.

#'   When non-default, the generated code will re-order the \code{Group}

#'   factor accordingly.

#' @return A character string of executable R code.

#' @export

#'
#' @examples
#' \donttest{
#' data(precomputed_gseares, package = "GSEAlens")
#' code <- generate_boxplot_data_code(
#'   GSEAlens_res = precomputed_gseares,
#'   contrast_id  = "untrt_vs_trt",
#'   gene_symbol  = "GAPDH",
#'   expr_type    = "logcpm"
#' )
#' cat(code)
#' }
generate_boxplot_data_code <- function(GSEAlens_res,

                                                                                contrast_id,

                                                                                gene_symbol,

                                                                                expr_type = "logcpm",

                                                                                custom_order = NULL) {

    # ---- 1. Pull side-info from the live GseaRes (so the generated code is

    #         informative even if the user just copy-pastes it). ----

    # Prefer GSEAlens::extract_gsea_task() (same source as generate_pathway_plot_code)

    # for consistency; fall back to a heuristic split of the contrast ID.

    left_group  <- NA_character_

    right_group <- NA_character_

    gtask <- tryCatch(GSEAlens::extract_gsea_task(GSEAlens_res, contrast_id), error = function(e) NULL)

    if (!is.null(gtask) && !is.null(gtask$meta)) {

    left_group  <- gtask$meta$left_group  %||% NA_character_

    right_group <- gtask$meta$right_group %||% NA_character_

    }

    if (is.na(left_group) || is.na(right_group)) {

    if (grepl("_vs_", contrast_id, fixed = TRUE)) {

            parts <- strsplit(contrast_id, "_vs_", fixed = TRUE)[[1]]

            left_group  <- parts[1]

            right_group <- parts[2]

    }

    }



    # ---- 2. Determine whether to include the optional ordering block. ----

    has_custom_order <- !is.null(custom_order) &&

                                            length(custom_order) == 1 &&

                                            nzchar(custom_order) &&

                                            !identical(custom_order, "default")



    # ---- 3. Build the R script as a single string. ----

    # Use single quotes around user-provided strings; escape any embedded '.

    safe_gene <- gsub("'", "''", gene_symbol %||% "")

    safe_contrast <- gsub("'", "''", contrast_id %||% "")



    ordering_block <- if (has_custom_order) {

    safe_order <- gsub("'", "''", custom_order)

    paste0(

            "\n",

            "# Optional: apply a custom group ordering (matches the Shiny boxplot)\n",

            "GSEAlens_custom_order <- '", safe_order, "'\n",

            "if (!is.null(GSEAlens_custom_order) &&\n",

            "    nzchar(GSEAlens_custom_order) &&\n",

            "    !identical(GSEAlens_custom_order, 'default')) {\n",

            "  GSEAlens_sep <- if (grepl('->', GSEAlens_custom_order, fixed = TRUE)) '->' else ','\n",

            "  GSEAlens_order_parts <- trimws(strsplit(GSEAlens_custom_order, GSEAlens_sep)[[1]])\n",

            "  GSEAlens_actual_groups <- unique(as.character(GSEAlens_box_data$Group))\n",

            "  GSEAlens_valid_parts <- GSEAlens_order_parts[GSEAlens_order_parts %in% GSEAlens_actual_groups]\n",

            "  GSEAlens_x_categories <- c(GSEAlens_valid_parts,\n",

            "                              setdiff(GSEAlens_actual_groups, GSEAlens_valid_parts))\n",

            "  GSEAlens_box_data <- GSEAlens_box_data[GSEAlens_box_data$Group %in% GSEAlens_x_categories, ]\n",

            "  GSEAlens_box_data$Group <- factor(GSEAlens_box_data$Group,\n",

            "                                     levels = GSEAlens_x_categories,\n",

            "                                     ordered = TRUE)\n",

            "} else {\n",

            "  GSEAlens_box_data$Group <- factor(GSEAlens_box_data$Group)\n",

            "}\n"

    )

    } else {

    paste0(

            "\n",

            "# Optional: order the Group factor (alphabetical by default)\n",

            "# GSEAlens_box_data$Group <- factor(GSEAlens_box_data$Group)\n"

    )

    }



    code <- paste0(

    '# =============================================================================

# GSEAlens Boxplot Data Extraction Code

# Generated by GSEAlens Shiny App

# =============================================================================

#

# What this script does

# ----------------------

# Extracts the per-sample expression values of a single gene from a GseaRes

# object and returns a tidy data frame (columns: Sample, Group, Expression)

# that exactly matches the values shown in the Quadrant module boxplot

# (panel 4 "Full Expression Distribution").

#

# You can then:

#   * write.csv(GSEAlens_box_data, "my_gene_box.csv", row.names = FALSE)

#   * copy GSEAlens_box_data to GraphPad Prism / Excel for further plotting

#   * ggplot2::ggplot(GSEAlens_box_data, aes(Group, Expression)) +

#       ggplot2::geom_boxplot() + ggplot2::geom_jitter(width = 0.2)



# Step 1: Load your GseaRes object

# --------------------------------

GSEAlens_res <- YOUR_GSEA_RES_OBJECT



GSEAlens_contrast_id <- "', safe_contrast, '"

GSEAlens_gene_symbol <- "', safe_gene, '"

GSEAlens_expr_type   <- "', expr_type, '"

', if (!is.na(left_group) && !is.na(right_group)) {

    paste0('GSEAlens_left_group  <- "', left_group, '"\n',

                    'GSEAlens_right_group <- "', right_group, '"\n')

} else {

    paste0('# (Left/right group labels could not be inferred from the GseaRes.)\n')

},



    '\n

# Step 2: Pull the expression matrix and sample metadata

# ------------------------------------------------------

GSEAlens_expr_mat    <- GSEAlens::get_expr_matrix(GSEAlens_res, type = GSEAlens_expr_type)

GSEAlens_sample_meta <- GSEAlens::get_sample_meta(GSEAlens_res)



# Step 3: Locate the gene (handles Ensembl IDs, SYMBOL lookup, case-insensitive)

# -------------------------------------------------------------------------------

GSEAlens_target_upper   <- toupper(GSEAlens_gene_symbol)

GSEAlens_rownames_upper <- toupper(rownames(GSEAlens_expr_mat))

GSEAlens_match_idx <- which(GSEAlens_rownames_upper == GSEAlens_target_upper)



if (length(GSEAlens_match_idx) == 0 &&

    !is.null(GSEAlens_res$expr_bundle$gene_meta)) {

    GSEAlens_gene_meta <- GSEAlens_res$expr_bundle$gene_meta

    if (is.null(rownames(GSEAlens_gene_meta))) {

    rownames(GSEAlens_gene_meta) <- rownames(GSEAlens_expr_mat)

    }

    GSEAlens_symbol_col <- intersect(

    c("SYMBOL", "symbol", "Gene", "gene_name", "gene_symbol"),

    colnames(GSEAlens_gene_meta)

    )[1]

    if (!is.na(GSEAlens_symbol_col)) {

    GSEAlens_meta_upper <- toupper(as.character(GSEAlens_gene_meta[[GSEAlens_symbol_col]]))

    GSEAlens_meta_idx   <- which(GSEAlens_meta_upper == GSEAlens_target_upper)

    if (length(GSEAlens_meta_idx) > 0) {

            GSEAlens_ensembl_id <- rownames(GSEAlens_gene_meta)[GSEAlens_meta_idx[1]]

            GSEAlens_match_idx  <- which(rownames(GSEAlens_expr_mat) == GSEAlens_ensembl_id)

    }

    }

}



if (length(GSEAlens_match_idx) == 0) {

    stop(sprintf("Gene \'%s\' was not found in the expression matrix.", GSEAlens_gene_symbol))

}



# Step 4: Build the per-sample data frame

# ---------------------------------------

GSEAlens_actual_gene  <- rownames(GSEAlens_expr_mat)[GSEAlens_match_idx[1]]

GSEAlens_expr_values  <- GSEAlens_expr_mat[GSEAlens_actual_gene, ]

GSEAlens_sample_names <- names(GSEAlens_expr_values)

GSEAlens_group_info   <- GSEAlens_sample_meta$group[

    match(GSEAlens_sample_names, rownames(GSEAlens_sample_meta))

]



GSEAlens_box_data <- data.frame(

    Sample     = GSEAlens_sample_names,

    Group      = as.character(GSEAlens_group_info),

    Expression = as.numeric(GSEAlens_expr_values),

    stringsAsFactors = FALSE

)

GSEAlens_box_data <- GSEAlens_box_data[!is.na(GSEAlens_box_data$Group), ]

', ordering_block, '

# Step 5: Preview / save

# ----------------------

print(GSEAlens_box_data)



# Uncomment to save to CSV (Excel / GraphPad friendly):

# write.csv(GSEAlens_box_data, "GSEAlens_', safe_gene, '_', safe_contrast, '_box.csv", row.names = FALSE)



# Optional: make a boxplot with ggplot2 (matches the Shiny look-and-feel)

# GSEAlens_p <- ggplot2::ggplot(GSEAlens_box_data,

#                               ggplot2::aes(x = Group, y = Expression, fill = Group)) +

#   ggplot2::geom_boxplot(alpha = 0.7, outlier.shape = NA) +

#   ggplot2::geom_jitter(width = 0.2, size = 3, alpha = 0.6) +

#   ggplot2::theme_bw(base_size = 12) +

#   ggplot2::labs(title = GSEAlens_gene_symbol, y = GSEAlens_expr_type, x = NULL) +

#   ggplot2::theme(legend.position = "none",

#                  axis.text.x = ggplot2::element_text(angle = 45, hjust = 1))

# print(GSEAlens_p)

'

    )



    return(code)

}







#' @title Extract Boxplot Data (Long and Wide)

#' @description Internal helper used by the Quadrant module's "Export Boxplot

#'   Data" button. Computes the exact per-sample expression data frame that

#'   the Shiny boxplot (panel 4) is built on, and returns it in two formats:

#' \describe{

#'   \item{long}{Tidy data frame with columns \code{Sample}, \code{Group},

#'     \code{Expression}. One row per sample. Suitable for GraphPad Prism

#'     ("Grouped" data table) and Excel.}

#'   \item{wide}{A one-row data frame. The first column holds the gene

#'     symbol (column name \code{Gene}); the remaining columns hold the

#'     expression value of that gene in each sample (column names are the

#'     sample IDs, in the expression matrix's original column order).

#'     Suitable for direct copy-paste into Prism ("Column" data table) or

#'     any spreadsheet.}

#' }

#' @param GSEAlens_res A \code{GseaRes} object.

#' @param contrast_id Character. Contrast ID (e.g. \code{"Treat_vs_Control"}).

#' @param gene_symbol Character. Gene symbol as shown in the boxplot title.

#' @param expr_type Character. Expression matrix type passed to

#'   \code{get_expr_matrix()}. Default \code{"logcpm"}.

#' @return A named list with elements \code{long}, \code{wide}, \code{gene},

#'   \code{contrast_id}, \code{expr_type}, \code{left_group},

#'   \code{right_group}, \code{sample_order}.

#' @keywords internal

extract_boxplot_data <- function(GSEAlens_res,

                                                                    contrast_id,

                                                                    gene_symbol,

                                                                    expr_type = "logcpm") {

    # ---- 1. Pull side-info (left/right group labels, with safe fallback) ----

    left_group  <- NA_character_

    right_group <- NA_character_

    gtask <- tryCatch(GSEAlens::extract_gsea_task(GSEAlens_res, contrast_id),

                    error = function(e) NULL)

    if (!is.null(gtask) && !is.null(gtask$meta)) {

    left_group  <- gtask$meta$left_group  %||% NA_character_

    right_group <- gtask$meta$right_group %||% NA_character_

    }

    if (is.na(left_group) || is.na(right_group)) {

    if (grepl("_vs_", contrast_id, fixed = TRUE)) {

            parts <- strsplit(contrast_id, "_vs_", fixed = TRUE)[[1]]

            left_group  <- parts[1]

            right_group <- parts[2]

    }

    }



    # ---- 2. Expression matrix and sample metadata ----

    expr_mat    <- get_expr_matrix(GSEAlens_res, type = expr_type)

    sample_meta <- get_sample_meta(GSEAlens_res)



    if (is.null(expr_mat)) {

    stop(sprintf("Expression matrix (type = '%s') is not available.", expr_type))

    }

    if (is.null(sample_meta)) {

    stop("Sample metadata is not available in the GseaRes object.")

    }



    # ---- 3. Locate the gene (case-insensitive, with SYMBOL/Ensembl fallback) ----

    target_upper   <- toupper(gene_symbol)

    rownames_upper <- toupper(rownames(expr_mat))

    match_idx <- which(rownames_upper == target_upper)



    if (length(match_idx) == 0 && !is.null(GSEAlens_res$expr_bundle$gene_meta)) {

    gene_meta <- GSEAlens_res$expr_bundle$gene_meta

    if (is.null(rownames(gene_meta))) {

            rownames(gene_meta) <- rownames(expr_mat)

    }

    symbol_col <- intersect(

            c("SYMBOL", "symbol", "Gene", "gene_name", "gene_symbol"),

            colnames(gene_meta)

    )[1]

    if (!is.na(symbol_col)) {

            meta_upper <- toupper(as.character(gene_meta[[symbol_col]]))

            meta_idx   <- which(meta_upper == target_upper)

            if (length(meta_idx) > 0) {

        ensembl_id <- rownames(gene_meta)[meta_idx[1]]

        match_idx  <- which(rownames(expr_mat) == ensembl_id)

            }

    }

    }



    if (length(match_idx) == 0) {

    stop(sprintf("Gene '%s' was not found in the expression matrix.", gene_symbol))

    }



    # ---- 4. Build the per-sample data frame (preserves expr_mat column order) ----

    actual_gene  <- rownames(expr_mat)[match_idx[1]]

    expr_values  <- expr_mat[actual_gene, ]

    sample_names <- names(expr_values)

    group_info   <- sample_meta$group[match(sample_names, rownames(sample_meta))]



    long_df <- data.frame(

    Sample     = sample_names,

    Group      = as.character(group_info),

    Expression = as.numeric(expr_values),

    stringsAsFactors = FALSE

    )

    # Drop samples without a group assignment

    long_df <- long_df[!is.na(long_df$Group), , drop = FALSE]



    # ---- 5. Build the wide data frame: first row = gene, columns = samples ----

    wide_df <- data.frame(

    Gene = gene_symbol,

    as.list(setNames(long_df$Expression, long_df$Sample)),

    check.names = FALSE,

    stringsAsFactors = FALSE

    )



    # ---- 6. Return ----

    list(

    long         = long_df,

    wide         = wide_df,

    gene         = gene_symbol,

    contrast_id  = contrast_id,

    expr_type    = expr_type,

    left_group   = left_group,

    right_group  = right_group,

    sample_order = long_df$Sample

    )

}





# =============================================================================

# Publication-ready static plot code generators

# =============================================================================

# These functions return self-contained R script strings that re-create the

# interactive plotly/visNetwork figures as static ggplot2 graphics for

# publication. The returned code is displayed in the Shiny app's export modal

# and can be copy-pasted into an R session.

# All generators accept a `data.frame` slice (the exact subset shown in the

# interactive view) plus user-tunable style parameters, so the generated code

# is both reproducible and customizable.

# =============================================================================



#' @title Generate Publication DotPlot Code
#' @description Return an executable ggplot2 script reproducing the Pathway
#'   Relationship module's dotplot as a static, publication-ready figure.
#'   The generated script assumes a \code{GseaRes} object is already loaded
#'   in the R environment and derives plot data via \code{extract_gsea_task()}
#'   + \code{get_core_genes_list()}, producing concise reproducible code
#'   without embedding raw data.
#' @param gsea_res_var Character. Name of the GseaRes variable in the user's
#'   environment (default \code{"gsea_res"}).
#' @param contrast_id Character. Contrast ID to extract.
#' @param pathways Character vector. Pathway IDs to display.
#' @param color_mode Character. One of "padj", "pval", "nes".
#' @param size_mode  Character. One of "core_size", "setsize".
#' @param left_group Character. Left contrast label.
#' @param right_group Character. Right contrast label.
#' @param palette Character. Viridis option letter ("A"-"H").
#' @param point_range Numeric length 2. Min/max point size in mm.
#' @param target_collection Character. Gene set collection(s) to extract
#'   (e.g. \code{"H"}, \code{c("C2", "C2:CP")}). Default \code{"ALL"}.
#' @param base_size Numeric. theme_bw base font size.
#' @param margin_top Numeric. Top plot margin (mm).
#' @param margin_bottom Numeric. Bottom plot margin (mm).
#' @param margin_left Numeric. Left plot margin (mm).
#' @param margin_right Numeric. Right plot margin (mm).
#' @param fig_width Numeric. Figure width in inches for ggsave.
#' @param fig_height Numeric. Figure height in inches for ggsave.
#' @param fig_dpi Numeric. DPI for PNG raster output (default 300).
#' @return Character scalar of executable R code.
#' @examples
#' code <- generate_dotplot_code(
#'   gsea_res_var = "gsea_res", contrast_id = "untrt_vs_trt",
#'   pathways = c("HALLMARK_TNFA_SIGNALING_VIA_NFKB"))
#' cat(code)
#' @export
generate_dotplot_code <- function(gsea_res_var = "gsea_res",
                                                                    contrast_id,
                                                                    pathways,
                                                                    color_mode = "padj",
                                                                    size_mode = "core_size",
                                                                    left_group = "Left",
                                                                    right_group = "Right",
                                                                    palette = "D",
                                                                    point_range = c(3, 8),
                                                                    target_collection = "ALL",
                                                                    base_size = 12,
                                                                    margin_top = 18,
                                                                    margin_bottom = 18,
                                                                    margin_left = 18,
                                                                    margin_right = 18,
                                                                    fig_width = 9,
                                                                    fig_height = 7,
                                                                    fig_dpi = 300) {

    if (length(pathways) == 0) {
    return("# No pathways selected; nothing to plot.")
    }

    pathways_str <- paste(sprintf('    "%s"', pathways), collapse = ",\n")

    # Format target_collection for R code output
    tc_str <- if (length(target_collection) == 1 && toupper(target_collection) == "ALL") {
    '"ALL"'
    } else {
    paste0('c(', paste0('"', target_collection, '"', collapse = ", "), ')')
    }

    color_field <- switch(color_mode,
    "padj" = "-log10(p.adjust)",
    "pval" = "-log10(pvalue)",
    "nes"  = "abs(NES)",
    "-log10(p.adjust)")
    color_title <- switch(color_mode,
    "padj" = "-log10(FDR)",
    "pval" = "-log10(P-value)",
    "nes"  = "|NES|",
    "-log10(FDR)")

    size_field <- switch(size_mode,
    "core_size" = "CoreCount",
    "setsize"   = "setSize",
    "CoreCount")
    size_title <- switch(size_mode,
    "core_size" = "Core Genes",
    "setsize"   = "Set Size",
    "Core Genes")

    sprintf(
'# =============================================================================
# GSEAlens Publication DotPlot Code
# =============================================================================

library(ggplot2)

# --- Assume gsea_res (GseaRes object) is loaded in your environment ---------
# Uncomment to load from RDS:
# %s <- readRDS("your_capsule.rds")

GSEAlens_contrast_id <- "%s"
GSEAlens_pathways <- c(
%s
)

# --- Extract task and build plot data ----------------------------------------
GSEAlens_task <- GSEAlens::extract_gsea_task(%s,
    contrast_id = GSEAlens_contrast_id, target_collection = %s)
df <- as.data.frame(GSEAlens_task$gsea_res@result)
df <- df[df$ID %%in%% GSEAlens_pathways, ]

# CoreCount via leading-edge (core) genes
GSEAlens_core <- GSEAlens::get_core_genes_list(GSEAlens_task, GSEAlens_pathways)
df$CoreCount <- vapply(GSEAlens_pathways, function(pid) {
    if (is.null(GSEAlens_core[[pid]])) 0L else length(GSEAlens_core[[pid]])
}, integer(1))

# --- Type coercion (defensive) -----------------------------------------------
df$NES       <- suppressWarnings(as.numeric(df$NES))
df$pvalue    <- suppressWarnings(as.numeric(df$pvalue))
df$p.adjust  <- suppressWarnings(as.numeric(df$p.adjust))
df$setSize   <- suppressWarnings(as.numeric(df$setSize))
df$CoreCount <- suppressWarnings(as.integer(df$CoreCount))
df$NES[is.na(df$NES)]             <- 0
df$pvalue[is.na(df$pvalue)]       <- 1
df$p.adjust[is.na(df$p.adjust)]   <- 1
df$setSize[is.na(df$setSize)]     <- 0
df$CoreCount[is.na(df$CoreCount)] <- 0L

# --- Aesthetic config --------------------------------------------------------
color_field <- "%s"
color_title <- "%s"
size_field  <- "%s"
size_title  <- "%s"

# Size column clamp (prevents ggplot2 censoring large values)
size_lim_hi <- if (size_field == "CoreCount") 50 else 500
df[[size_field]] <- pmin(pmax(df[[size_field]], 0), size_lim_hi)

p <- ggplot(df, aes(
    x = NES,
    y = reorder(ID, NES),
    size  = .data[[size_field]],
    fill  = if (color_field == "abs(NES)") abs(NES)
                    else if (color_field == "-log10(p.adjust)") -log10(p.adjust)
                    else -log10(pvalue)
)) +
    geom_point(shape = 21, color = "black", stroke = 0.3, alpha = 0.85) +
    scale_size_continuous(name = size_title, range = c(%g, %g)) +
    scale_fill_viridis_c(name = color_title, option = "%s", direction = -1) +
    geom_vline(xintercept = 0, linetype = "dashed", color = "grey50") +
    theme_bw(base_size = %g) +
    labs(
    title    = sprintf("Pathway DotPlot: %%s vs %%s", "%s", "%s"),
    subtitle = sprintf("Color: %%s  |  Size: %%s", color_title, size_title),
    x = "NES (Normalized Enrichment Score)",
    y = NULL
    ) +
    coord_cartesian(clip = "off") +
    theme(
    plot.title    = element_text(face = "bold", hjust = 0.5),
    plot.subtitle = element_text(color = "gray40", hjust = 0.5, margin = margin(b = 20)),
    axis.text.y   = element_text(size = rel(0.85)),
    legend.position = "right",
    plot.margin     = margin(%d, %d, %d, %d)
    )

# --- Direction annotation: colored enriched-in labels above the plot ---------
# Single-direction aware: if only one NES sign is present, show one label only.
# y=1.01 + just="bottom": text sits just outside the top border, growing upward.
has_neg <- any(df$NES < 0, na.rm = TRUE)
has_pos <- any(df$NES > 0, na.rm = TRUE)
if (has_neg) {
    p <- p + annotation_custom(
    grid::textGrob("Enriched in %s", x = 0.02, y = 1.01, just = c("left", "bottom"),
                                        gp = grid::gpar(col = "#3B6EA5", fontface = "bold", fontsize = %g)),
    xmin = -Inf, xmax = Inf, ymin = -Inf, ymax = Inf)
}
if (has_pos) {
    p <- p + annotation_custom(
    grid::textGrob("Enriched in %s", x = 0.98, y = 1.01, just = c("right", "bottom"),
                                        gp = grid::gpar(col = "#C0392B", fontface = "bold", fontsize = %g)),
    xmin = -Inf, xmax = Inf, ymin = -Inf, ymax = Inf)
}

print(p)

# --- Save to file (uncomment one) --------------------------------------------
# ggsave("dotplot.pdf", p, width = %g, height = %g, device = cairo_pdf)
# ggsave("dotplot.png", p, width = %g, height = %g, dpi = %g)
# ggsave("dotplot.svg", p, width = %g, height = %g)
',
    gsea_res_var,
    contrast_id,
    pathways_str,
    gsea_res_var,
    tc_str,
    color_field, color_title,
    size_field,  size_title,
    point_range[1], point_range[2],
    palette,
    base_size,
    left_group, right_group,
    margin_top, margin_right, margin_bottom, margin_left,
    right_group,
    base_size,
    left_group,
    base_size,
    fig_width, fig_height,
    fig_width, fig_height, fig_dpi,
    fig_width, fig_height
    )
}


#' @title Generate Publication Volcano Code
#' @description Return an executable ggplot2 script reproducing the Quadrant
#'   module's pathway volcano plot as a static, publication-ready figure.
#'   Assumes a \code{GseaRes} object is already loaded in the R environment.
#' @param gsea_res_var Character. Name of the GseaRes variable.
#' @param contrast_id Character. Contrast ID to extract.
#' @param left_group Character.
#' @param right_group Character.
#' @param base_size Numeric.
#' @return Character scalar of executable R code.
#' @param n_selected Integer. Number of selected pathways (subtitle).
#' @param selected_ids Character vector. Pathway IDs for ggrepel annotation.
#' @param show_annotations Logical. Add ggrepel labels for selected pathways.
#' @param target_collection Character. Gene set collection(s) to extract.
#'   Default \code{"ALL"}.
#' @param margin_top Numeric. Top plot margin (mm).
#' @param margin_bottom Numeric. Bottom plot margin (mm).
#' @param margin_left Numeric. Left plot margin (mm).
#' @param margin_right Numeric. Right plot margin (mm).
#' @param fig_width Numeric. Figure width in inches for ggsave.
#' @param fig_height Numeric. Figure height in inches for ggsave.
#' @param fig_dpi Numeric. DPI for PNG raster output (default 300).
#' @examples
#' code <- generate_volcano_code(
#'   gsea_res_var = "gsea_res", contrast_id = "untrt_vs_trt",
#'   left_group = "Ctrl", right_group = "Trt")
#' cat(code)
#' @export
generate_volcano_code <- function(gsea_res_var = "gsea_res",
                                                                    contrast_id,
                                                                    left_group = "Left",
                                                                    right_group = "Right",
                                                                    n_selected = 0L,
                                                                    selected_ids = character(0),
                                                                    show_annotations = FALSE,
                                                                    target_collection = "ALL",
                                                                    base_size = 12,
                                                                    margin_top = 15, margin_bottom = 15,
                                                                    margin_left = 18, margin_right = 18,
                                                                    fig_width = 8,
                                                                    fig_height = 6,
                                                                    fig_dpi = 300) {

    selected_str <- if (length(selected_ids) > 0) {
    paste0('c(', paste0('"', selected_ids, '"', collapse = ", "), ')')
    } else "character(0)"

    # Format target_collection for R code output
    tc_str <- if (length(target_collection) == 1 && toupper(target_collection) == "ALL") {
    '"ALL"'
    } else {
    paste0('c(', paste0('"', target_collection, '"', collapse = ", "), ')')
    }

    labs_block <- if (show_annotations) {
    paste0(
'  labs(\n',
'    title = sprintf("Pathway Volcano: %s vs %s", "', left_group, '", "', right_group, '"),\n',
'    subtitle = sprintf("%d pathways | %d selected | %d significant (P<0.25)",\n',
'                       nrow(df), ', n_selected, ', sum(df$p.adjust < 0.25, na.rm = TRUE)),\n',
'    caption = sprintf("Left (NES<0): Enriched in %s  |  Right (NES>0): Enriched in %s",\n',
'                      "', right_group, '", "', left_group, '"),\n',
'    x = "NES",\n',
'    y = "-log10(P-value)"\n',
'  ) +'
    )
    } else {
    paste0(
'  labs(\n',
'    title = sprintf("Pathway Volcano: %s vs %s", "', left_group, '", "', right_group, '"),\n',
'    caption = sprintf("Left (NES<0): Enriched in %s  |  Right (NES>0): Enriched in %s",\n',
'                      "', right_group, '", "', left_group, '"),\n',
'    x = "NES",\n',
'    y = "-log10(P-value)"\n',
'  ) +'
    )
    }

    sprintf(
'# =============================================================================
# GSEAlens Publication Pathway Volcano Code
# =============================================================================

library(ggplot2)

# --- Assume gsea_res (GseaRes object) is loaded in your environment ---------
# Uncomment to load from RDS:
# %s <- readRDS("your_capsule.rds")

GSEAlens_contrast_id <- "%s"

# --- Extract task and build plot data ----------------------------------------
GSEAlens_task <- GSEAlens::extract_gsea_task(%s,
    contrast_id = GSEAlens_contrast_id, target_collection = %s)
df <- as.data.frame(GSEAlens_task$gsea_res@result)

# --- Selected pathway IDs (for ggrepel labels) -------------------------------
selected_ids <- %s
n_selected   <- %d

# --- Type coercion (defensive) -----------------------------------------------
df$NES      <- suppressWarnings(as.numeric(df$NES))
df$pvalue   <- suppressWarnings(as.numeric(df$pvalue))
df$p.adjust <- suppressWarnings(as.numeric(df$p.adjust))
df$NES[is.na(df$NES)]             <- 0
df$pvalue[is.na(df$pvalue)]       <- 1
df$p.adjust[is.na(df$p.adjust)]   <- 1

# --- Direction: NES > 0 = enriched in left_group; NES < 0 = right_group ------
df$direction <- with(df, ifelse(NES > 0,
    "Enriched in %s", "Enriched in %s"))
df$sig_level <- with(df,
    cut(pvalue,
            breaks = c(-Inf, 1e-10, 1e-5, 1e-2, 0.05, Inf),
            labels = c("P<1e-10", "P<1e-5", "P<1e-2", "P<0.05", "NS"),
            right  = FALSE))

p <- ggplot(df, aes(x = NES, y = -log10(pvalue),
                    color = direction, alpha = sig_level)) +
    geom_point(size = 2.5) +
    scale_color_manual(
    values = c("Enriched in %s" = "#E41A1C", "Enriched in %s" = "#377EB8"),
    name = "Direction") +
    scale_alpha_manual(
    values = c("P<1e-10" = 1.0, "P<1e-5" = 0.9, "P<1e-2" = 0.7,
                                "P<0.05" = 0.5, "NS" = 0.25),
    name = "Significance", drop = FALSE) +
    geom_hline(yintercept = -log10(0.05), linetype = "dashed", color = "grey50") +
    geom_vline(xintercept = 0, linetype = "dashed", color = "grey50") +
    theme_bw(base_size = %g) +
%s
    theme(
    plot.title   = element_text(face = "bold", hjust = 0.5),
    plot.caption = element_text(color = "gray40", hjust = 0.5, size = rel(0.85)),
    legend.position = "right",
    plot.margin  = margin(%d, %d, %d, %d)
    )

# --- ggrepel labels for selected pathways ------------------------------------
if (length(selected_ids) > 0) {
    df_label <- df[df$ID %%in%% selected_ids, ]
    if (nrow(df_label) > 0) {
    library(ggrepel)
    p <- p + ggrepel::geom_label_repel(
            data = df_label,
            aes(label = ID),
            size = 3, fontface = "bold",
            fill = "white", color = "#222", alpha = 0.9,
            box.padding = 0.5, label.padding = 0.18, linewidth = 0.2,
            min.segment.length = 0,
            segment.color = "grey55", segment.size = 0.3, segment.alpha = 0.7,
            max.overlaps = 20, max.iter = 5000,
            nudge_y = 0.3, direction = "both", force = 2
    )
    }
}

print(p)

# --- Save to file (uncomment one) --------------------------------------------
# ggsave("volcano.pdf", p, width = %g, height = %g, device = cairo_pdf)
# ggsave("volcano.png", p, width = %g, height = %g, dpi = %g)
# ggsave("volcano.svg", p, width = %g, height = %g)
',
    gsea_res_var,
    contrast_id,
    gsea_res_var,
    tc_str,
    selected_str,
    n_selected,
    left_group, right_group,
    left_group, right_group,
    base_size,
    labs_block,
    margin_top, margin_right, margin_bottom, margin_left,
    fig_width, fig_height,
    fig_width, fig_height, fig_dpi,
    fig_width, fig_height
    )
}


#' @title Generate Publication DE Volcano Code
#' @description Return an executable ggplot2 script reproducing the Quadrant
#'   module's gene-level Differential Expression Volcano as a static figure.
#'   Assumes a \code{GseaRes} object is already loaded. Six-color classification
#'   (both / user / pathway / up / down / ns) mirrors the interactive view.
#' @param gsea_res_var Character. Name of the GseaRes variable.
#' @param contrast_id Character. Contrast ID for DE table extraction.
#' @param user_genes Character vector of user-selected genes (upper-cased).
#' @param pathway_genes Character vector of pathway genes (upper-cased).
#' @param logfc_thresh Numeric. Default 1.
#' @param pval_thresh Numeric. Default 0.05.
#' @param left_group Character.
#' @param right_group Character.
#' @param base_size Numeric.
#' @return Character scalar of executable R code.
#' @param show_annotations Logical. Add ggrepel labels.
#' @param margin_top Numeric. Top plot margin (mm).
#' @param margin_bottom Numeric. Bottom plot margin (mm).
#' @param margin_left Numeric. Left plot margin (mm).
#' @param margin_right Numeric. Right plot margin (mm).
#' @param fig_width Numeric. Figure width in inches for ggsave.
#' @param fig_height Numeric. Figure height in inches for ggsave.
#' @param fig_dpi Numeric. DPI for PNG raster output (default 300).
#' @examples
#' code <- generate_de_volcano_code(
#'   gsea_res_var = "gsea_res", contrast_id = "untrt_vs_trt",
#'   left_group = "Ctrl", right_group = "Trt")
#' cat(code)
#' @export
generate_de_volcano_code <- function(gsea_res_var = "gsea_res",
                                                                            contrast_id,
                                                                            user_genes = character(0),
                                                                            pathway_genes = character(0),
                                                                            logfc_thresh = 1,
                                                                            pval_thresh = 0.05,
                                                                            left_group = "Left",
                                                                            right_group = "Right",
                                                                            show_annotations = FALSE,
                                                                            base_size = 12,
                                                                            margin_top = 15, margin_bottom = 15,
                                                                            margin_left = 18, margin_right = 18,
                                                                            fig_width = 8,
                                                                            fig_height = 6,
                                                                            fig_dpi = 300) {

    user_str <- if (length(user_genes) > 0) {
    paste0('c(', paste0('"', user_genes, '"', collapse = ", "), ')')
    } else "character(0)"
    pathway_str <- if (length(pathway_genes) > 0) {
    paste0('c(', paste0('"', pathway_genes, '"', collapse = ", "), ')')
    } else "character(0)"

    labs_annot_block <- if (show_annotations) {
    paste0(
'  labs(\n',
'    title = sprintf("DE Volcano: %s vs %s", "', left_group, '", "', right_group, '"),\n',
'    subtitle = sprintf(\n',
'      "Up %d | Down %d | NS %d | Selected %d | Pway %d | Both %d",\n',
'      sum(de_df$category == "up"),   sum(de_df$category == "down"),\n',
'      sum(de_df$category == "ns"),   sum(de_df$category == "user"),\n',
'      sum(de_df$category == "pathway"), sum(de_df$category == "both")),\n',
'    x = "log2 Fold Change",\n',
'    y = "-log10(P-value)"\n',
'  ) +\n',
'  annotate("text", x = Inf, y = Inf, color = "#E41A1C",\n',
'           label = sprintf("High in %s", "', left_group, '"),\n',
'           hjust = 1.08, vjust = 1.6, fontface = "bold", size = 4.5) +\n',
'  annotate("text", x = -Inf, y = Inf, color = "#377EB8",\n',
'           label = sprintf("High in %s", "', right_group, '"),\n',
'           hjust = -0.08, vjust = 1.6, fontface = "bold", size = 4.5) +\n',
'  coord_cartesian(clip = "off") +\n',
'  theme(\n',
'    plot.title    = element_text(face = "bold", hjust = 0.5),\n',
'    plot.subtitle = element_text(color = "gray30", hjust = 0.5),\n',
    sprintf('    plot.margin   = margin(%d, %d, %d, %d),\n', margin_top, margin_right, margin_bottom, margin_left),
'    legend.position = "right"\n',
'  )'
    )
    } else {
    paste0(
'  labs(\n',
'    title = sprintf("DE Volcano: %s vs %s", "', left_group, '", "', right_group, '"),\n',
'    x = "log2 Fold Change",\n',
'    y = "-log10(P-value)"\n',
'  ) +\n',
'  theme(\n',
'    plot.title    = element_text(face = "bold", hjust = 0.5),\n',
'    legend.position = "right",\n',
    sprintf('    plot.margin = margin(%d, %d, %d, %d)\n', margin_top, margin_right, margin_bottom, margin_left),
'  )'
    )
    }

    sprintf(
'# =============================================================================
# GSEAlens Publication DE Volcano Code
# =============================================================================

library(ggplot2)

# --- Assume gsea_res (GseaRes object) is loaded in your environment ---------
# Uncomment to load from RDS:
# %s <- readRDS("your_capsule.rds")

GSEAlens_contrast_id <- "%s"

# --- Extract DE table from GseaRes -------------------------------------------
de_df <- GSEAlens::get_de_table(%s, GSEAlens_contrast_id)
if (!"logFC"  %%in%% names(de_df)) de_df$logFC  <- de_df$log2FoldChange
if (!"pvalue" %%in%% names(de_df)) de_df$pvalue <- de_df$p.value

user_genes    <- toupper(%s)
pathway_genes <- toupper(%s)

# --- Type coercion (defensive) -----------------------------------------------
de_df$logFC  <- suppressWarnings(as.numeric(de_df$logFC))
de_df$pvalue <- suppressWarnings(as.numeric(de_df$pvalue))
de_df$logFC[is.na(de_df$logFC)]   <- 0
de_df$pvalue[is.na(de_df$pvalue)] <- 1

# --- Thresholds --------------------------------------------------------------
logfc_thresh <- %g
pval_thresh  <- %g

# --- Classify ----------------------------------------------------------------
de_df$gene_upper <- toupper(de_df$gene_symbol)
de_df$is_user     <- de_df$gene_upper %%in%% user_genes
de_df$is_pathway  <- de_df$gene_upper %%in%% pathway_genes
de_df$is_sig      <- abs(de_df$logFC) > logfc_thresh & de_df$pvalue < pval_thresh

de_df$category <- dplyr::case_when(
    de_df$is_user & de_df$is_pathway ~ "both",
    de_df$is_user                    ~ "user",
    de_df$is_pathway                 ~ "pathway",
    de_df$is_sig & de_df$logFC > 0   ~ "up",
    de_df$is_sig & de_df$logFC < 0   ~ "down",
    TRUE                             ~ "ns"
)

color_map <- c(
    both = "#9C27B0", user = "#4DAF4A", pathway = "#FF9800",
    up = "#E41A1C",   down = "#377EB8", ns = "#C0C0C0"
)
de_df$category <- factor(de_df$category,
    levels = c("ns", "down", "up", "pathway", "user", "both"))

# --- Layer ordering (mirrors interactive plotly) -----------------------------
# Sort so special categories (pathway/user/both) are drawn LAST and appear
# on top of the significant and NS points, making them easy to spot.
de_df$plot_order <- dplyr::case_when(
    de_df$category == "both"    ~ 6,
    de_df$category == "user"    ~ 5,
    de_df$category == "pathway" ~ 4,
    de_df$category %%in%% c("up", "down") ~ 2,
    TRUE ~ 1
)
de_df <- de_df[order(de_df$plot_order), ]

p <- ggplot(de_df, aes(x = logFC, y = -log10(pvalue), color = category)) +
    geom_point(aes(size = category, alpha = category)) +
    scale_color_manual(values = color_map, name = "Category",
    labels = c(ns = "NS", down = "Down", up = "Up",
                                pathway = "Pathway", user = "Selected", both = "Both")) +
    scale_size_manual(values = c(ns = 1, down = 2, up = 2,
                                                                pathway = 3.5, user = 3.5, both = 4), guide = "none") +
    scale_alpha_manual(values = c(ns = 0.4, down = 0.7, up = 0.7,
                                pathway = 0.9, user = 0.9, both = 1.0), guide = "none") +
    geom_vline(xintercept = c(-logfc_thresh, logfc_thresh),
                            linetype = "dashed", color = "grey60") +
    geom_hline(yintercept = -log10(pval_thresh),
                            linetype = "dashed", color = "grey60") +
    theme_bw(base_size = %g) +
%s

# --- ggrepel labels for Confirmed Gene Markers ONLY --------------------------
de_df_label <- de_df[de_df$category %%in%% c("user", "both"), ]
if (nrow(de_df_label) > 0) {
    label_color_map <- c(user = "#4DAF4A", both = "#9C27B0")
    library(ggrepel)
    p <- p + ggrepel::geom_label_repel(
    data = de_df_label,
    aes(label = gene_symbol, fill = category),
    color = "white", size = 3, fontface = "bold",
    box.padding = 0.45, label.padding = 0.2,
    min.segment.length = 0, segment.color = "grey55",
    max.overlaps = Inf, max.iter = 5000,
    show.legend = FALSE
    ) +
    scale_fill_manual(values = label_color_map, guide = "none")
}

print(p)

# --- Save to file (uncomment one) --------------------------------------------
# ggsave("de_volcano.pdf", p, width = %g, height = %g, device = cairo_pdf)
# ggsave("de_volcano.png", p, width = %g, height = %g, dpi = %g)
# ggsave("de_volcano.svg", p, width = %g, height = %g)
',
    gsea_res_var,
    contrast_id,
    gsea_res_var,
    user_str, pathway_str,
    logfc_thresh, pval_thresh,
    base_size,
    labs_annot_block,
    fig_width, fig_height,
    fig_width, fig_height, fig_dpi,
    fig_width, fig_height
    )
}


#' @title Generate Publication Pathway Network Code
#' @description Return an executable R script reproducing the Pathway
#'   Relationship module's network view as a static ggplot2 figure.
#'   Assumes a \code{GseaRes} object is already loaded. Uses igraph layout +
#'   geom_segment edges + geom_point nodes (no ggraph dependency).
#' @param gsea_res_var Character. Name of the GseaRes variable.
#' @param contrast_id Character. Contrast ID to extract.
#' @param pathways Character vector. Pathway IDs to include in the network.
#' @param min_shared_genes Integer. Minimum shared core genes for an edge.
#' @param width_mode Character. "rank" or "weight".
#' @param seed Integer. Layout seed.
#' @param target_collection Character. Gene set collection(s) to extract.
#'   Default \code{"ALL"}.
#' @param base_size Numeric.
#' @return Character scalar of executable R code.
#' @param margin_top Numeric. Top plot margin (mm).
#' @param margin_bottom Numeric. Bottom plot margin (mm).
#' @param margin_left Numeric. Left plot margin (mm).
#' @param margin_right Numeric. Right plot margin (mm).
#' @param fig_width Numeric. Figure width in inches for ggsave.
#' @param fig_height Numeric. Figure height in inches for ggsave.
#' @param fig_dpi Numeric. DPI for PNG raster output (default 300).
#' @examples
#' code <- generate_network_code(
#'   gsea_res_var = "gsea_res", contrast_id = "untrt_vs_trt",
#'   pathways = c("HALLMARK_TNFA_SIGNALING_VIA_NFKB", "HALLMARK_HYPOXIA"))
#' cat(code)
#' @export
generate_network_code <- function(gsea_res_var = "gsea_res",
                                                                    contrast_id,
                                                                    pathways,
                                                                    min_shared_genes = 2,
                                                                    width_mode = "weight",
                                                                    seed = 42L,
                                                                    target_collection = "ALL",
                                                                    base_size = 12,
                                                                    margin_top = 18, margin_bottom = 18,
                                                                    margin_left = 18, margin_right = 18,
                                                                    fig_width = 10,
                                                                    fig_height = 8,
                                                                    fig_dpi = 300) {

    if (length(pathways) == 0) {
    return("# No pathways selected; nothing to plot.")
    }

    pathways_str <- paste(sprintf('    "%s"', pathways), collapse = ",\n")

    # Format target_collection for R code output
    tc_str <- if (length(target_collection) == 1 && toupper(target_collection) == "ALL") {
    '"ALL"'
    } else {
    paste0('c(', paste0('"', target_collection, '"', collapse = ", "), ')')
    }

    width_block <- if (width_mode == "rank") {
'  # rank-based: assign edge width by Jaccard rank (1..n -> 1..5 px)
    edge_df <- edge_df[order(edge_df$weight, decreasing = TRUE), ]
    n <- nrow(edge_df)
    edge_df$width <- 5 - (seq_len(n) - 1) * 4 / max(1, n - 1)
    edge_df$width <- pmax(1, pmin(5, edge_df$width))'
    } else {
'  # weight-based: edge width proportional to Jaccard value (emapplot style)
    edge_df$width <- 1 + edge_df$weight * 7   # Jaccard in [0,1] -> [1,8] px'
    }

    sprintf(
'# =============================================================================
# GSEAlens Publication Pathway Network Code
# =============================================================================

library(igraph)
library(ggplot2)
library(ggrepel)

# --- Assume gsea_res (GseaRes object) is loaded in your environment ---------
# Uncomment to load from RDS:
# %s <- readRDS("your_capsule.rds")

GSEAlens_contrast_id <- "%s"
GSEAlens_pathways <- c(
%s
)

# --- Extract task and build network data -------------------------------------
GSEAlens_task <- GSEAlens::extract_gsea_task(%s,
    contrast_id = GSEAlens_contrast_id, target_collection = %s)
df <- as.data.frame(GSEAlens_task$gsea_res@result)
df <- df[df$ID %%in%% GSEAlens_pathways, ]

# Build edge list from shared leading-edge (core) genes
GSEAlens_core <- GSEAlens::get_core_genes_list(GSEAlens_task, GSEAlens_pathways)
edge_df <- GSEAlens::build_edge_list_safely(GSEAlens_core, min_shared_genes = %d)
node_df <- df[, c("ID", "NES", "p.adjust")]
names(node_df)[1] <- "name"

# --- Type coercion (defensive) -----------------------------------------------
node_df$NES      <- suppressWarnings(as.numeric(node_df$NES))
node_df$p.adjust <- suppressWarnings(as.numeric(node_df$p.adjust))
edge_df$weight   <- suppressWarnings(as.numeric(edge_df$weight))
node_df$NES[is.na(node_df$NES)]           <- 0
node_df$p.adjust[is.na(node_df$p.adjust)] <- 1
edge_df$weight[is.na(edge_df$weight)]     <- 0

# --- Edge width mapping ------------------------------------------------------
%s

# --- Build graph and compute FR layout ---------------------------------------
g <- graph_from_data_frame(edge_df[, c("from", "to")],
                                                        directed = FALSE,
                                                        vertices = node_df$name)
set.seed(%d)
layout_xy <- layout_with_fr(g)
node_df$x <- layout_xy[, 1]
node_df$y <- layout_xy[, 2]

# Edge segments need explicit x/y endpoints
edge_plot <- merge(edge_df, node_df, by.x = "from", by.y = "name")
names(edge_plot)[names(edge_plot) == "x"] <- "x_from"
names(edge_plot)[names(edge_plot) == "y"] <- "y_from"
edge_plot <- merge(edge_plot, node_df, by.x = "to", by.y = "name")
names(edge_plot)[names(edge_plot) == "x"] <- "x_to"
names(edge_plot)[names(edge_plot) == "y"] <- "y_to"

node_df$direction <- ifelse(node_df$NES > 0, "Up", "Down")

p <- ggplot() +
    geom_segment(data = edge_plot,
                                aes(x = x_from, y = y_from, xend = x_to, yend = y_to,
                                        linewidth = width),
                                color = "grey70", alpha = 0.6) +
    scale_linewidth_continuous(range = c(0.3, 2.5), name = "Edge width") +
    geom_point(data = node_df,
                            aes(x = x, y = y, fill = direction, size = -log10(p.adjust)),
                            shape = 21, color = "black", stroke = 0.4, alpha = 0.85) +
    scale_fill_manual(values = c("Up" = "#F28F8F", "Down" = "#8FB7E8"),
                    name = "Direction") +
    scale_size_continuous(range = c(3, 9), name = "-log10(FDR)") +
    ggrepel::geom_label_repel(
    data = node_df,
    aes(x = x, y = y, label = sub("^[^_]*_", "", name)),
    size = 3.2, fontface = "bold",
    fill = "white", color = "#222", alpha = 0.95,
    box.padding = 0.45, label.padding = 0.18, linewidth = 0.2,
    min.segment.length = 0,
    segment.color = "grey55", segment.size = 0.3, segment.alpha = 0.7,
    max.overlaps = Inf, nudge_y = 0.18, direction = "both"
    ) +
    guides(
    fill = guide_legend(override.aes = list(size = 7, shape = 21, stroke = 1)),
    size = guide_legend(override.aes = list(shape = 21, fill = "grey50", stroke = 0.5))
    ) +
    theme_void(base_size = %g) +
    theme(legend.position = "right",
        plot.margin = margin(%d, %d, %d, %d)) +
    coord_cartesian(clip = "off") +
    labs(title = "Pathway Relationship Network")

print(p)

# --- Save to file (uncomment one) --------------------------------------------
# ggsave("network.pdf", p, width = %g, height = %g, device = cairo_pdf)
# ggsave("network.png", p, width = %g, height = %g, dpi = %g)
# ggsave("network.svg", p, width = %g, height = %g)
',
    gsea_res_var,
    contrast_id,
    pathways_str,
    gsea_res_var,
    tc_str,
    min_shared_genes,
    width_block,
    seed, base_size,
    margin_top, margin_right, margin_bottom, margin_left,
    fig_width, fig_height,
    fig_width, fig_height, fig_dpi,
    fig_width, fig_height
    )
}


#' @title Generate Publication HubGene Network Code
#' @description Return an executable R script reproducing the HubGene module's
#'   visNetwork view as a static ggplot2 figure. Assumes a \code{GseaRes}
#'   object is already loaded. Node size is controlled by the same three-mode
#'   logic (fixed / fdr / setsize) as the interactive view.
#' @param gsea_res_var Character. Name of the GseaRes variable.
#' @param contrast_id Character. Contrast ID to extract.
#' @param pathways Character vector. Pathway IDs to include.
#' @param min_hub_degree Integer. Minimum hub degree for gene inclusion.
#' @param pw_size_mode Character. One of "fixed", "fdr", "setsize".
#' @param seed Integer.
#' @param target_collection Character. Gene set collection(s) to extract.
#'   Default \code{"ALL"}.
#' @param base_size Numeric.
#' @return Character scalar of executable R code.
#' @param margin_top Numeric. Top plot margin (mm).
#' @param margin_bottom Numeric. Bottom plot margin (mm).
#' @param margin_left Numeric. Left plot margin (mm).
#' @param margin_right Numeric. Right plot margin (mm).
#' @param fig_width Numeric. Figure width in inches for ggsave.
#' @param fig_height Numeric. Figure height in inches for ggsave.
#' @param fig_dpi Numeric. DPI for PNG raster output (default 300).
#' @examples
#' code <- generate_hubgene_code(
#'   gsea_res_var = "gsea_res", contrast_id = "untrt_vs_trt",
#'   pathways = c("HALLMARK_TNFA_SIGNALING_VIA_NFKB"))
#' cat(code)
#' @export
generate_hubgene_code <- function(gsea_res_var = "gsea_res",
                                                                    contrast_id,
                                                                    pathways,
                                                                    min_hub_degree = 2,
                                                                    pw_size_mode = "setsize",
                                                                    seed = 42L,
                                                                    target_collection = "ALL",
                                                                    base_size = 12,
                                                                    margin_top = 18, margin_bottom = 18,
                                                                    margin_left = 18, margin_right = 18,
                                                                    fig_width = 10,
                                                                    fig_height = 8,
                                                                    fig_dpi = 300) {

    if (length(pathways) == 0) {
    return("# No pathways selected; nothing to plot.")
    }

    pathways_str <- paste(sprintf('    "%s"', pathways), collapse = ",\n")

    # Format target_collection for R code output
    tc_str <- if (length(target_collection) == 1 && toupper(target_collection) == "ALL") {
    '"ALL"'
    } else {
    paste0('c(', paste0('"', target_collection, '"', collapse = ", "), ')')
    }

    size_block <- switch(pw_size_mode,
    "fdr" =
'  # FDR-driven sizing
    pw$fdr_neglog <- -log10(pmax(pw$FDR, 1e-300))
    r <- range(pw$fdr_neglog); if (diff(r) < 1e-6) r <- c(0, 10)
    pw$size <- 30 * (0.6 + 0.8 * (pw$fdr_neglog - r[1]) / diff(r))',
    "setsize" =
'  # setSize-driven sizing (cnetplot classic)
    r <- range(pw$setSize); if (diff(r) < 1) r <- c(0, 200)
    pw$size <- 30 * (0.6 + 0.8 * (sqrt(pw$setSize) - sqrt(r[1])) /
                            (sqrt(r[2]) - sqrt(r[1])))',
'  # Fixed size (user-controlled constant)
    pw$size <- 30'
    )

    sprintf(
'# =============================================================================
# GSEAlens Publication HubGene Network Code
# =============================================================================

library(igraph)
library(ggplot2)
library(ggrepel)

# --- Assume gsea_res (GseaRes object) is loaded in your environment ---------
# Uncomment to load from RDS:
# %s <- readRDS("your_capsule.rds")

GSEAlens_contrast_id <- "%s"
GSEAlens_pathways <- c(
%s
)

# --- Extract task and build HubGene network ----------------------------------
GSEAlens_task <- GSEAlens::extract_gsea_task(%s,
    contrast_id = GSEAlens_contrast_id, target_collection = %s)

GSEAlens_net <- GSEAlens::build_hubgene_network(
    GSEAlens_task,
    pathway_ids    = GSEAlens_pathways,
    min_hub_degree = %d
)

pw    <- GSEAlens_net$nodes$pathway
genes <- GSEAlens_net$nodes$gene
edges_raw <- GSEAlens_net$edges

edges <- if (!is.null(edges_raw) && nrow(edges_raw) > 0) {
    data.frame(from = as.character(edges_raw$source),
                            to   = as.character(edges_raw$target),
                            stringsAsFactors = FALSE)
} else {
    data.frame(from = character(0), to = character(0), stringsAsFactors = FALSE)
}

# --- Type coercion (defensive) -----------------------------------------------
if (!"setSize" %%in%% names(pw))    pw$setSize <- pw$n_total
if (!"degree"  %%in%% names(genes)) genes$degree <- 1L
pw$NES     <- suppressWarnings(as.numeric(pw$NES))
pw$FDR     <- suppressWarnings(as.numeric(pw$FDR))
pw$setSize <- suppressWarnings(as.numeric(pw$setSize))
genes$stat   <- suppressWarnings(as.numeric(genes$stat))
genes$degree <- suppressWarnings(as.integer(genes$degree))
pw$NES[is.na(pw$NES)]         <- 0
pw$FDR[is.na(pw$FDR)]         <- 1
pw$setSize[is.na(pw$setSize)] <- 100
genes$stat[is.na(genes$stat)]       <- 0
genes$degree[is.na(genes$degree)]   <- 1

# --- Pathway node sizing -----------------------------------------------------
%s

# --- Graph + layout ----------------------------------------------------------
pw$vname    <- paste0("pw_",   pw$id)
genes$vname <- paste0("gene_", genes$id)
edges$from  <- paste0("gene_", edges$from)
edges$to    <- paste0("pw_",   edges$to)

all_verts <- data.frame(
    name = c(pw$vname, genes$vname), stringsAsFactors = FALSE)
g <- graph_from_data_frame(edges[, c("from", "to")],
                                                        directed = FALSE, vertices = all_verts)
set.seed(%d)
layout_xy <- layout_with_fr(g)
all_verts$x <- layout_xy[, 1]
all_verts$y <- layout_xy[, 2]
pw    <- cbind(pw,    all_verts[match(pw$vname,    all_verts$name), c("x", "y")])
genes <- cbind(genes, all_verts[match(genes$vname, all_verts$name), c("x", "y")])

pw$direction <- ifelse(pw$NES > 0, "Up", "Down")
genes$direction <- ifelse(genes$stat > 0, "Up",
                                        ifelse(genes$stat < 0, "Down", "Neutral"))

edge_plot <- merge(edges, all_verts, by.x = "from", by.y = "name")
names(edge_plot)[names(edge_plot) == "x"] <- "x_from"
names(edge_plot)[names(edge_plot) == "y"] <- "y_from"
edge_plot <- merge(edge_plot, all_verts, by.x = "to", by.y = "name")
names(edge_plot)[names(edge_plot) == "x"] <- "x_to"
names(edge_plot)[names(edge_plot) == "y"] <- "y_to"

p <- ggplot() +
    geom_segment(data = edge_plot,
                                aes(x = x_from, y = y_from, xend = x_to, yend = y_to),
                                color = "grey75", alpha = 0.5, linewidth = 0.4) +
    geom_point(data = genes,
                            aes(x = x, y = y, color = direction, size = degree),
                            alpha = 0.85) +
    scale_color_manual(
    values = c("Up" = "#F28F8F", "Down" = "#8FB7E8", "Neutral" = "#BDBDBD"),
    name = "Direction") +
    scale_size_continuous(range = c(2, 8), name = "Hub degree") +
    geom_point(data = pw,
                            aes(x = x, y = y, size = size),
                            shape = 23, fill = "#FFC107", color = "black", stroke = 0.5) +
    ggrepel::geom_label_repel(
    data = pw,
    aes(x = x, y = y, label = sub("^[^_]*_", "", id)),
    size = 3.2, fontface = "bold",
    fill = "white", color = "#222", alpha = 0.95,
    box.padding = 0.45, label.padding = 0.18, linewidth = 0.2,
    min.segment.length = 0,
    segment.color = "grey55", segment.size = 0.3, segment.alpha = 0.7,
    max.overlaps = Inf, nudge_y = 0.18, direction = "both"
    ) +
    guides(
    color = guide_legend(override.aes = list(size = 7)),
    size  = guide_legend()
    ) +
    theme_void(base_size = %g) +
    theme(legend.position = "right",
        plot.margin = margin(%d, %d, %d, %d)) +
    coord_cartesian(clip = "off") +
    labs(title = "HubGene Pathway-Gene Network")

print(p)

# --- Save to file (uncomment one) --------------------------------------------
# ggsave("hubgene.pdf", p, width = %g, height = %g, device = cairo_pdf)
# ggsave("hubgene.png", p, width = %g, height = %g, dpi = %g)
# ggsave("hubgene.svg", p, width = %g, height = %g)
',
    gsea_res_var,
    contrast_id,
    pathways_str,
    gsea_res_var,
    tc_str,
    min_hub_degree,
    size_block,
    seed, base_size,
    margin_top, margin_right, margin_bottom, margin_left,
    fig_width, fig_height,
    fig_width, fig_height, fig_dpi,
    fig_width, fig_height
    )
}


#' @title Serialize a Data Frame to TSV

#' @description Internal helper used by the Quadrant module's "Export Boxplot

#'   Data" modal. Writes a \code{data.frame} as tab-separated values with a

#'   header row and \code{NA} rendered as empty strings. Values are quoted

#'   only when they contain tab / newline / double-quote characters, so

#'   clean sample IDs stay unquoted.

#' @param df A \code{data.frame}.

#' @return A character scalar containing the TSV (no trailing newline).

#' @keywords internal

.serialize_df_tsv <- function(df) {

    if (!is.data.frame(df) || nrow(df) == 0) {

    return("")

    }



    needs_quote <- function(s) {

    !is.na(s) && nzchar(s) && grepl("[\t\n\"]", s)

    }



    encode_cell <- function(val) {

    if (is.na(val)) return("")

    if (is.numeric(val)) return(format(val, trim = TRUE, scientific = FALSE))

    s <- as.character(val)

    if (needs_quote(s)) shQuote(s, type = "sh") else s

    }



    header <- paste(vapply(colnames(df), encode_cell, character(1)), collapse = "\t")

    body_lines <- vapply(seq_len(nrow(df)), function(i) {

    paste(vapply(df, function(col) encode_cell(col[i]), character(1)),

                    collapse = "\t")

    }, character(1))



    paste(c(header, body_lines), collapse = "\n")

}





#' @title Generate Publication Boxplot Image Code

#' @description Return an executable ggplot2 script reproducing the Quadrant

#'   module's per-gene expression boxplot (panel 4) as a static,

#'   publication-ready figure. Mirrors the interactive plotly: boxplot +

#'   jittered points, group colors, optional zero baseline, optional custom

#'   group order.

#' @param box_data_long data.frame with columns: Sample, Group, Expression.

#'   (Same long format returned by \code{extract_boxplot_data()$long}.)

#' @param gene_symbol Character. Gene name for the plot title.

#' @param expr_type Character. Y-axis label (e.g. "logcpm", "log2FC").

#' @param left_group Character. First group name (gets red fill).

#' @param right_group Character. Second group name (gets blue fill).

#' @param use_zero_baseline Logical. Add dashed red line at y = 0.

#' @param custom_order Character scalar. Custom group ordering

#'   (e.g. "A->B,C") or "default".

#' @param base_size Numeric.

#' @param margin_top Numeric. Top plot margin (mm).

#' @param margin_bottom Numeric. Bottom plot margin (mm).

#' @param margin_left Numeric. Left plot margin (mm).

#' @param margin_right Numeric. Right plot margin (mm).

#' @param fig_width Numeric. Figure width in inches for ggsave.

#' @param fig_height Numeric. Figure height in inches for ggsave.

#' @param fig_dpi Numeric. DPI for PNG raster output (default 300).

#' @return Character scalar of executable R code.

#' @export

#'
#' @examples
#' \donttest{
#' # Build a minimal long-format expression data frame (self-contained)
#' box_long <- data.frame(
#'   Sample     = c("S1", "S2", "S3", "S4"),
#'   Group      = factor(c("untrt", "untrt", "trt", "trt"),
#'                       levels = c("untrt", "trt")),
#'   Expression = c(8.1, 8.3, 9.5, 9.7)
#' )
#' code <- generate_boxplot_image_code(
#'   box_data_long = box_long,
#'   gene_symbol   = "GAPDH",
#'   expr_type     = "logcpm",
#'   left_group    = "untrt",
#'   right_group   = "trt"
#' )
#' cat(code)
#' }
generate_boxplot_image_code <- function(box_data_long,

                                        gene_symbol = "Gene",

                                        expr_type = "Expression",

                                        left_group = NA_character_,

                                        right_group = NA_character_,

                                        use_zero_baseline = TRUE,

                                        custom_order = "default",

                                        base_size = 12,

                                        margin_top = 15, margin_bottom = 15,

                                        margin_left = 18, margin_right = 18, fig_width = 5, fig_height = 5, fig_dpi = 300) {

    # Subset to the three essential columns + drop NA groups, mirroring the

    # interactive plotly rendering path.

    keep_cols <- intersect(c("Sample", "Group", "Expression"),

                                                    colnames(box_data_long))

    box_subset <- box_data_long[, keep_cols, drop = FALSE]

    box_subset <- box_subset[!is.na(box_subset$Group), , drop = FALSE]

    data_literal <- paste(utils::capture.output(dput(box_subset)), collapse = "\n")



    safe_gene <- gsub("'", "''", gene_symbol %||% "")

    safe_expr <- gsub("'", "''", expr_type %||% "Expression")



    # Build the optional custom-ordering block (mirrors

    # generate_boxplot_data_code's logic, inlined for a self-contained script).

    has_custom_order <- !is.null(custom_order) &&

                                            length(custom_order) == 1 &&

                                            nzchar(custom_order) &&

                                            !identical(custom_order, "default")

    ordering_block <- if (has_custom_order) {

    safe_order <- gsub("'", "''", custom_order)

    paste0(

'# --- Custom group ordering (matches the Shiny boxplot) ----------------------\n',

'custom_order <- \'', safe_order, '\'\n',

'sep <- if (grepl("->", custom_order, fixed = TRUE)) "->" else ","\n',

'order_parts <- trimws(strsplit(custom_order, sep)[[1]])\n',

'actual_groups <- unique(as.character(df$Group))\n',

'valid_parts <- order_parts[order_parts %in% actual_groups]\n',

'x_categories <- c(valid_parts, setdiff(actual_groups, valid_parts))\n',

'df <- df[df$Group %in% x_categories, ]\n',

'df$Group <- factor(df$Group, levels = x_categories, ordered = TRUE)\n'

    )

    } else {

'# --- Group ordering (alphabetical by default) -------------------------------\n# df$Group <- factor(df$Group)\nx_categories <- unique(as.character(df$Group))\ndf$Group <- factor(df$Group, levels = x_categories)\n'

    }



    # Zero-baseline line block

    baseline_block <- if (isTRUE(use_zero_baseline)) {

'  geom_hline(yintercept = 0, linetype = "dashed",

                            color = "red", alpha = 0.7, linewidth = 0.8) +'

    } else {

'  # (zero baseline line disabled) #'

    }



    sprintf(

'# =============================================================================

# GSEAlens Publication Boxplot Code

# Generated by GSEAlens Shiny App

# =============================================================================

# Per-gene expression boxplot with jittered points, mirroring the

# interactive plotly view in the Quadrant module (panel 4).



library(ggplot2)



# --- Data --------------------------------------------------------------------

df <- %s



# --- Type coercion (defensive) -----------------------------------------------

if (!"Sample"     %%in%% names(df)) df$Sample     <- NA_character_

if (!"Group"      %%in%% names(df)) df$Group      <- NA_character_

if (!"Expression" %%in%% names(df)) df$Expression <- NA_real_

df$Sample     <- as.character(df$Sample)

df$Group      <- as.character(df$Group)

df$Expression <- suppressWarnings(as.numeric(df$Expression))

df <- df[!is.na(df$Group) & !is.na(df$Expression), ]



%s



# --- Group color palette -----------------------------------------------------

# Two-group case uses the same red/blue scheme as the DE volcano; otherwise

# a qualitative palette. Matches the interactive plotly fill mapping.

unique_groups <- levels(df$Group)

if (length(unique_groups) == 0) unique_groups <- unique(as.character(df$Group))

group_colors <- if (length(unique_groups) == 2) {

    setNames(c("#E41A1C", "#377EB8"), unique_groups[1:2])

} else {

    cols <- c("#E41A1C", "#377EB8", "#4DAF4A", "#984EA3",

            "#FF7F00", "#A65628", "#F781BF", "#999999")[seq_along(unique_groups)]

    setNames(cols, unique_groups)

}



# --- Y-axis range with headroom (mirrors interactive plotly) -----------------

y_min <- min(df$Expression, na.rm = TRUE)

y_max <- max(df$Expression, na.rm = TRUE)

%s

y_range <- y_max - y_min

y_min <- y_min - y_range * 0.1

y_max <- y_max + y_range * 0.1



p <- ggplot(df, aes(x = Group, y = Expression, fill = Group)) +

    geom_boxplot(alpha = 0.7, outlier.shape = NA) +

    geom_jitter(width = 0.2, size = 2, alpha = 0.6) +

    scale_fill_manual(values = group_colors) +

    scale_x_discrete(limits = levels(df$Group), drop = FALSE) +

%s

    coord_cartesian(ylim = c(y_min, y_max)) +

    theme_bw(base_size = %g) +

    labs(

    title = "%s",

    y = "%s",

    x = NULL

    ) +

    theme(

    legend.position = "none",

    axis.text.x = element_text(angle = 45, hjust = 1),

    plot.title = element_text(face = "bold", hjust = 0.5),

    plot.margin = margin(%d, %d, %d, %d)

    )



print(p)



# --- Save to file (uncomment one) --------------------------------------------

# ggsave("boxplot.pdf", p, width = %g, height = %g, device = cairo_pdf)

# ggsave("boxplot.png", p, width = %g, height = %g, dpi = %g)

# ggsave("boxplot.svg", p, width = %g, height = %g)

',

    data_literal,

    ordering_block,

    if (isTRUE(use_zero_baseline)) 'y_min <- min(y_min, 0)\ny_max <- max(y_max, 0)' else '# (zero baseline disabled)',

    baseline_block,

    base_size,

    safe_gene,

    safe_expr,

    margin_top, margin_right, margin_bottom, margin_left,

    fig_width, fig_height,

    fig_width, fig_height, fig_dpi,

    fig_width, fig_height

    )

}

#!/usr/bin/env Rscript

# Description -------------------------------------------------------------

# Pre-compute a lightweight `build_gsea_pathways()` result for vignette use.

#

# Rationale: the full vignette call uses `auto_select = c(17,18,19,20,26)`

# which loads 5 MSigDB collections (~8000 pathways, ~30-90 s on the

# Bioconductor build machine). To keep vignette compilation fast and avoid

# the "vignette rebuilding took > 60s" NOTE, we ship a pre-computed

# lightweight object in data/.

#

# Run this script manually whenever the msigdbr version bumps or the

# build_gsea_pathways() return structure changes:

#

#   Rscript inst/scripts/make_gsea_pathwaysets_toy.R

#

# Output files (written to data/):

#   - gsea_pathwaysets_toy_hallmark.rds  (~80 KB, H only, 50 pathways)

#   - gsea_pathwaysets_toy.rds           (~500 KB, H + C2:CP:KEGG, ~300 pathways)

# -------------------------------------------------------------------------



# Use the installed GSEAlens (>= 0.99.9) so the return structure matches

# exactly what users get.

library(GSEAlens)



out_dir <- file.path("data")

dir.create(out_dir, showWarnings = FALSE, recursive = TRUE)



# NOTE on `auto_select` form:

# Numeric indices (e.g. c(17, 19)) depend on the row order of

# msigdbr::msigdbr_collections(), which can drift across msigdbr releases.

# Prefer the **string** form ("H", "C2:CP:KEGG") for reproducibility.



# ---- Hallmark only (smallest, ~50 pathways) -----------------------------

toy_hallmark <- build_gsea_pathways(species = "HS", auto_select = "H")

saveRDS(

  toy_hallmark,

  file.path(out_dir, "gsea_pathwaysets_toy_hallmark.rds"),

  compress = "xz"

)



# ---- Hallmark + C2:CP:KEGG_LEGACY (demo set, ~200 pathways) --------------

# KEGG_LEGACY gives downstream GSEA something to enrich on with the airway

# dataset (drug-treatment contrast) while still keeping the file < 1 MB.

# NOTE: msigdbr >= 25 split KEGG into KEGG_LEGACY and KEGG_MEDICUS; we use

# KEGG_LEGACY because its pathway names are the classic KEGG_* identifiers

# users recognise.

toy_demo <- build_gsea_pathways(species = "HS", auto_select = c("H", "C2:CP:KEGG_LEGACY"))

saveRDS(

  toy_demo,

  file.path(out_dir, "gsea_pathwaysets_toy.rds"),

  compress = "xz"

)



# ---- Report -------------------------------------------------------------

sizes <- file.size(file.path(

  out_dir,

  c("gsea_pathwaysets_toy_hallmark.rds", "gsea_pathwaysets_toy.rds")

))

md5 <- tools::md5sum(file.path(

  out_dir,

  c("gsea_pathwaysets_toy_hallmark.rds", "gsea_pathwaysets_toy.rds")

))



cat(sprintf(

  "Wrote:\n  %s  (%.1f KB, %d pathways, %d genes, md5=%s)\n  %s  (%.1f KB, %d pathways, %d genes, md5=%s)\n",

  "gsea_pathwaysets_toy_hallmark.rds", sizes[1] / 1024,

  nrow(toy_hallmark$meta_dict),

  length(unique(toy_hallmark$TERM2GENE$gene_symbol)),

  md5[1],

  "gsea_pathwaysets_toy.rds", sizes[2] / 1024,

  nrow(toy_demo$meta_dict),

  length(unique(toy_demo$TERM2GENE$gene_symbol)),

  md5[2]

))



cat("\nmsigdbr version:", as.character(packageVersion("msigdbr")), "\n")

cat("GSEAlens version:", as.character(packageVersion("GSEAlens")), "\n")

cat("Generation date:", format(Sys.Date(), "%Y-%m-%d"), "\n")

# ============================================================
# File: R/mod_ai_abs_page.R
# Function: AI Prompt Generation Module (with Custom Template Support)
# ============================================================

#' @title AI Interpretation Page UI
#' @param id Module ID
#' @keywords internal

mod_ai_abs_page_ui <- function(id) {
    ns <- shiny::NS(id)

    shiny::tagList(
    shiny::fluidRow(
            # ===== Left Control Panel =====
            shiny::column(
        width = 3,
        shiny::div(
                    class = "well",
                    style = "padding: 15px;",
                    shiny::h4("AI Prompt Generator"),
                    shiny::hr(),

                    # Current comparison group info
                    shiny::h5("Current Comparison"),
                    shiny::uiOutput(ns("current_contrast_info")),
                    shiny::hr(),

                    # Selected pathway statistics
                    shiny::h5("Selected Pathways"),
                    shiny::uiOutput(ns("selection_stats")),
                    shiny::hr(),

                    # ===== Custom Template Area =====
                    shiny::h5("Custom Template"),
                    shiny::div(
            style = "background: #f0f8ff; padding: 10px; border-radius: 5px; margin-bottom: 10px;",
            shiny::checkboxInput(
                            ns("use_custom_template"),
                            label = "Use custom template",
                            value = FALSE
            )
                    ),
                    shiny::conditionalPanel(
            condition = sprintf("input['%s'] == true", ns("use_custom_template")),
            shiny::div(
                            style = "background: #fff; padding: 10px; border: 1px solid #ddd; border-radius: 5px;",
                            shiny::p(shiny::strong("Placeholder Guide:")),
                            htmltools::tags$small(
                "{left_group} - Left group name", shiny::br(),
                "{right_group} - Right group name", shiny::br(),
                "{comparison} - Comparison description", shiny::br(),
                "{total} - Total pathway count", shiny::br(),
                "{high} - High confidence count", shiny::br(),
                "{mod} - Medium confidence count", shiny::br(),
                "{low} - Low confidence count", shiny::br(),
                "{table} - Pathway data table", shiny::br(),
                shiny::hr(),
                "Example: {left_group} shows activation trend in pathway X"
                            ),
                            shiny::hr(),
                            shiny::actionButton(
                ns("import_template"),
                label = "Import Template",
                icon = shiny::icon("upload"),
                class = "btn-outline-primary btn-sm",
                style = "width: 100%;"
                            ),
                            shiny::actionButton(
                ns("export_template"),
                label = "Export Current",
                icon = shiny::icon("download"),
                class = "btn-outline-secondary btn-sm",
                style = "width: 100%; margin-top: 5px;"
                            ),
                            shiny::hr(),
                            shiny::textAreaInput(
                ns("custom_template_text"),
                label = "Edit Template:",
                value = "",
                width = "100%",
                height = "300px"
                            )
            )
                    ),
                    shiny::hr(),

                    # Generate button
                    shiny::actionButton(
            ns("generate_prompt"),
            label = "Generate Prompt",
            icon = shiny::icon("wand-magic-sparkles"),
            class = "btn-primary",
            style = "width: 100%; font-weight: bold;"
                    ),
                    shiny::hr(),

                    # Copy button
                    shiny::actionButton(
            ns("copy_to_clipboard"),
            label = "Copy to Clipboard",
            icon = shiny::icon("copy"),
            class = "btn-success",
            style = "width: 100%;"
                    )
        )
            ),

            # ===== Right Prompt Output =====
            shiny::column(
        width = 9,
        shiny::div(
                    class = "white-box",
                    style = "min-height: 800px; padding: 15px;",
                    shiny::h4("AI Prompt Output"),
                    shiny::hr(),

                    # Prompt output area
                    shiny::div(
            style = "background: #f8f9fa; padding: 15px; border-radius: 5px;",
            shiny::textAreaInput(
                            ns("prompt_output"),
                            label = NULL,
                            value = "Click 'Generate Prompt' button to create AI interpretation prompt.",
                            width = "100%",
                            height = "700px"
            )
                    )
        )
            )
    )
    )
}


#' @title AI Interpretation Page Server
#' @description AI Interpretation Page Server. Called for side effects within a Shiny module.
#' @param id Module ID
#' @param gsea_res GseaRes object
#' @param data_prep_list Data preprocessing list
#' @param table_controller Table controller
#' @return No return value; called for side effects within a Shiny module.
mod_ai_abs_page_server <- function(id, gsea_res, data_prep_list, table_controller) {
    shiny::moduleServer(id, function(input, output, session) {
    ns <- session$ns

    # ============================================================
    # Default prompt template (English optimized version)
    # ============================================================

    default_template <- 'Role Setting: GSEA Enrichment Direction Interpretation Expert

language: English
description: You are an expert in interpreting GSEA results. You have removed the sign from the Normalized Enrichment Score (NES) and focus solely on the absolute value (|NES|) for analysis. Your core task is to objectively describe the enrichment trend of gene sets in the predefined {left_group} group or {right_group} group based on the degree of enrichment, allowing the use of terms such as "upregulated" and "activated", while ensuring all analyses are based on absolute values.

### Gene Set Names and Their Analysis
In the analysis, in addition to using absolute values, please pay attention to interpreting the names of gene sets. For example, the gene set name "FLORIO_NEOCORTEX_BASAL_RADIAL_GLIA_DN" should be understood as this gene set being downregulated in a certain group. Therefore, analysis should combine the meaning of the gene set name with its absolute value.

Core Principles [Must Be Strictly Followed]
- The absolute value of the gene set (|NES|) directly indicates its enrichment intensity in the ranked gene list; the larger the absolute value, the more pronounced the enrichment trend of that gene set.
- When using terms such as "upregulated" or "activated" in the analysis, it must be stated that the sign has been removed and only the absolute value is being used for description.

Expression Rules (Golden Rule):
- The use of expressions such as "inhibited" or "decreased" is strictly prohibited.
- The use of terms such as "activated" or "upregulated" is permitted, based on the absolute value (|NES|) of each gene set to describe the enrichment trend, combined with analysis of the meaning of the gene set name.

Mandatory Objective Description Templates:
- "This gene set shows an activation trend in the [{left_group} group] (or: the members of this gene set have overall higher expression levels in the [{left_group} group], with |NES| = {abs_nes})."
- "This gene set shows an activation trend in the [{right_group} group] (or: the members of this gene set have overall higher expression levels in the [{right_group} group], with |NES| = {abs_nes})."

Constraints and Correction Mechanisms:
- In any interpretation, if language that is hypothetical or inferential is used, it must be stopped immediately and strictly traced back to the Golden Rule for expression correction.
- All descriptions must be based on absolute values and ensure accurate description to avoid ambiguity.

Workflow
1. Confirm Background (Prerequisites):
        - Group A = {left_group} group
        - Group B = {right_group} group
2. Assess Significance: Use the FDR q-val (typically < 0.25) indicator to determine whether the enrichment direction is statistically significant, and judge the strength of enrichment based on the absolute value of |NES|.
3. Classification and Description:
        - Strictly classify and describe significant enrichment results (e.g., FDR < 0.25) as follows:
            - Enriched in {left_group}: Classified as "gene sets activated in the {left_group} group".
            - Enriched in {right_group}: Classified as "gene sets activated in the {right_group} group".
4. Cautious Interpretation: After completing the objective directional description, analyze the biological significance of the two enrichment directions and provide constructive insights and explanations, and provide a possible functional interpretation of gene pathways in combination with specific leading edge genes.

Initialization
As your GSEA enrichment direction interpretation expert, I am ready. Please analyze the following GSEA results.

### Enrichment Analysis Data

**Comparison Group**: {comparison}

**Statistical Summary**: Total {total} pathways | High confidence: {high} | Medium confidence: {mod} | Low confidence: {low}

| # | Pathway ID | |NES| | Enrichment Direction | FDR | Description | Leading Edge Genes |
|:--:|:-----------|:----:|:-----------:|:------:|:--------------|:-------------------|
{table}

**Field Descriptions**:
- **|NES|**: Absolute Normalized Enrichment Score. |NES| >= 1.5 indicates significant enrichment; |NES| >= 2.0 indicates strong enrichment
- **Enrichment Direction**: Enrichment in {left_group} indicates activation in the {left_group} group; enrichment in {right_group} indicates activation in the {right_group} group
- **FDR**: Multiple testing corrected P-value. FDR < 0.25 is the MSigDB standard threshold
- **Leading Edge Genes**: Core contributing genes, key to understanding regulatory mechanisms
- **Numbers may be presented in scientific notation, like 1.45e-03.**'

    # Initialize custom template as empty
    shiny::observe({
            shiny::updateTextAreaInput(
        session,
        "custom_template_text",
        value = default_template
            )
    })

    # ============================================================
    # Helper Functions
    # ============================================================

    #' Extract Leading Edge genes and format as comma-separated
    extract_leading_genes <- function(core_str, max_genes = 20) {
            if (is.null(core_str) || is.na(core_str) || core_str == "") {
        return(list(genes = "N/A", count = 0))
            }
            genes <- unlist(strsplit(as.character(core_str), "/"))
            genes <- trimws(genes)
            genes <- genes[genes != ""]
            count <- length(genes)
            if (count > max_genes) {
        display <- paste(paste(genes[seq_len(max_genes)], collapse = ", "),
                    paste0("(+", count - max_genes, " more)"),
                    sep = " "
        )
            } else {
        display <- paste(genes, collapse = ", ")
            }
            return(list(genes = display, count = count))
    }

    #' Determine confidence level
    get_confidence <- function(nes, fdr) {
            abs_nes <- abs(nes)
            if (abs_nes >= 1.5 && fdr < 0.05) {
        return("High confidence")
            } else if (abs_nes >= 1.0 && fdr < 0.25) {
        return("Medium confidence")
            } else {
        return("Low confidence")
            }
    }

    # ============================================================
    # Current Comparison Group Info
    # ============================================================

    output$current_contrast_info <- shiny::renderUI({
            data_list <- data_prep_list$data()
            shiny::req(data_list)

            left <- data_list$left_group
            right <- data_list$right_group

            shiny::tagList(
        shiny::strong("Left: ", style = "color: #E41A1C;"),
        shiny::code(left), shiny::br(),
        shiny::strong("Right: ", style = "color: #377EB8;"),
        shiny::code(right)
            )
    })

    # ============================================================
    # Selected Pathway Statistics
    # ============================================================

    output$selection_stats <- shiny::renderUI({
            shiny::req(table_controller)
            selected_ids <- table_controller$selected_pathways()
            n_selected <- length(selected_ids)

            if (n_selected == 0) {
        shiny::div(
                    style = "background: #fff3cd; padding: 10px; border-radius: 5px; color: #856404;",
                    shiny::icon("exclamation-triangle"),
                    " No pathways selected",
                    shiny::br(),
                    htmltools::tags$small("Check 'Joint Plot' in main table")
        )
            } else {
        shiny::div(
                    style = "background: #d4edda; padding: 10px; border-radius: 5px; color: #155724;",
                    shiny::icon("check-circle"),
                    sprintf(" %d pathway(s) selected", n_selected)
        )
            }
    })

    # ============================================================
    # Import Template
    # ============================================================

    shiny::observeEvent(input$import_template, {
            shiny::showModal(shiny::modalDialog(
        title = "Import Custom Template",
        size = "l",
        easyClose = TRUE,
        shiny::textAreaInput(
                    ns("import_text"),
                    label = "Paste your template here:",
                    value = "",
                    width = "100%",
                    height = "400px"
        ),
        footer = shiny::tagList(
                    shiny::actionButton(ns("confirm_import"), "Confirm Import", class = "btn-primary"),
                    shiny::modalButton("Cancel")
        )
            ))
    })

    shiny::observeEvent(input$confirm_import, {
            imported_text <- input$import_text
            if (!is.null(imported_text) && imported_text != "") {
        shiny::updateTextAreaInput(
                    session,
                    "custom_template_text",
                    value = imported_text
        )
        shiny::showNotification("Template imported successfully!", type = "message", duration = 3)
            }
            shiny::removeModal()
    })

    # ============================================================
    # Export Template
    # ============================================================

    shiny::observeEvent(input$export_template, {
            current_template <- input$custom_template_text
            if (is.null(current_template) || current_template == "") {
        current_template <- default_template
            }

            # Save to file
            output_path <- file.path(getwd(), "gsea_prompt_template.txt")
            writeLines(current_template, output_path, useBytes = TRUE)

            shiny::showNotification(
        sprintf("Template exported to: %s", output_path),
        type = "message", duration = 5
            )
    })

    # ============================================================
    # Generate Prompt
    # ============================================================

    shiny::observeEvent(input$generate_prompt, {
            shiny::req(data_prep_list$data())

            data_list <- data_prep_list$data()

            # Get selected pathways
            selected_ids <- table_controller$selected_pathways()

            # If none selected, use Top 30
            if (length(selected_ids) == 0) {
        selected_ids <- data_list$df$ID[seq_len(min(30, nrow(data_list$df)))]
        shiny::showNotification(
                    "No pathways selected - using Top 15 by |NES|",
                    type = "warning", duration = 3
        )
            }

            # Filter data
            df_subset <- data_list$df[data_list$df$ID %in% selected_ids, ]
            df_subset <- df_subset[order(-abs(df_subset$NES)), ]

            # Limit count
            max_pathways <- 20
            if (nrow(df_subset) > max_pathways) {
        df_subset <- df_subset[seq_len(max_pathways), ]
            }

            # Basic info
            left_group <- data_list$left_group
            right_group <- data_list$right_group
            comparison <- paste0(left_group, " vs ", right_group)

            # Statistics
            n_total <- nrow(df_subset)
            n_high <- sum(vapply(seq_len(n_total), function(i) {
        get_confidence(df_subset$NES[i], df_subset$p.adjust[i]) == "High confidence"
            }, logical(1)))
            n_mod <- sum(vapply(seq_len(n_total), function(i) {
        get_confidence(df_subset$NES[i], df_subset$p.adjust[i]) == "Medium confidence"
            }, logical(1)))
            n_low <- n_total - n_high - n_mod

            # ============================================================
            # Build pathway table rows
            # ============================================================

            table_rows <- c()
            for (i in seq_len(nrow(df_subset))) {
        row <- df_subset[i, ]

        # Extract Leading Edge genes
        leading <- extract_leading_genes(row$core_enrichment)

        # Determine enrichment direction
        enriched_in <- ifelse(row$NES > 0, left_group, right_group)

        # Confidence level
        conf <- get_confidence(row$NES, row$p.adjust)

        # Truncate Description
        desc <- if (!is.null(row$Description) && !is.na(row$Description)) {
                    substr(as.character(row$Description), 1, 9999)
        } else {
                    row$ID
        }

        # Build table row
        table_rows <- c(table_rows, sprintf(
                    "| %d | `%s` | %.2f | %s | %.2e | %s | %s [%s] |",
                    i,
                    row$ID,
                    abs(row$NES),
                    enriched_in,
                    row$p.adjust,
                    substr(desc, 1, 9999),
                    leading$genes,
                    conf
        ))
            }

            table_body <- paste(table_rows, collapse = "\n")

            # ============================================================
            # Get template (custom or default)
            # ============================================================

            if (isTRUE(input$use_custom_template) &&
        !is.null(input$custom_template_text) &&
        input$custom_template_text != "") {
        template_text <- input$custom_template_text
            } else {
        template_text <- default_template
            }

            # ============================================================
            # Replace placeholders
            # ============================================================

            prompt_text <- template_text

            # Basic placeholder replacement
            prompt_text <- gsub("\\{left_group\\}", left_group, prompt_text, fixed = FALSE)
            prompt_text <- gsub("\\{right_group\\}", right_group, prompt_text, fixed = FALSE)
            prompt_text <- gsub("\\{comparison\\}", comparison, prompt_text, fixed = FALSE)
            prompt_text <- gsub("\\{total\\}", as.character(n_total), prompt_text, fixed = FALSE)
            prompt_text <- gsub("\\{high\\}", as.character(n_high), prompt_text, fixed = FALSE)
            prompt_text <- gsub("\\{mod\\}", as.character(n_mod), prompt_text, fixed = FALSE)
            prompt_text <- gsub("\\{low\\}", as.character(n_low), prompt_text, fixed = FALSE)
            prompt_text <- gsub("\\{table\\}", table_body, prompt_text, fixed = FALSE)

            # Special placeholder: abs_nes (used in description template)
            # Can be extended for more placeholders
            prompt_text <- gsub("\\{abs_nes\\}", sprintf("%.2f", abs(df_subset$NES[1])), prompt_text, fixed = FALSE)

            # Update output
            shiny::updateTextAreaInput(
        session,
        "prompt_output",
        value = prompt_text
            )

            shiny::showNotification(
        sprintf("Prompt generated: %d pathways", n_total),
        type = "message", duration = 3
            )
    })

    # ============================================================
    # Copy to Clipboard
    # ============================================================

    shiny::observeEvent(input$copy_to_clipboard, {
            prompt <- input$prompt_output

            if (is.null(prompt) || prompt == "") {
        shiny::showNotification("No prompt to copy!", type = "warning", duration = 3)
        return()
            }

            # Use clipr or system clipboard
            if (requireNamespace("clipr", quietly = TRUE)) {
        clipr::write_clip(prompt)
        shiny::showNotification("Copied!", type = "message", duration = 2)
            } else {
        writeLines(prompt, con <- file("clipboard", "w"))
        close(con)
        shiny::showNotification("Copied!", type = "message", duration = 2)
            }
    })
    })
}

#!/usr/bin/env Rscript

# Description -------------------------------------------------------------

# Pre-compute the `batch_calc_gsea()` result object ("GseaRes") that ships in

# `data/precomputed_gseares.rds` and is consumed by:

#

#   - 8 accessor examples in R/utils-accessors.R and R/05_extraction.R

#     (get_expr_matrix, get_de_table, get_sample_meta, get_contrast_registry,

#      get_geneset_info, extract_gsea_task, inspect_gsea_res, ...). These

#     examples MUST be runnable (no \dontrun) to satisfy BiocCheck's ">= 80%

#     runnable examples" rule, but running batch_calc_gsea() inside each

#     example would inflate check time. Shipping the pre-computed result

#     keeps each accessor example < 1 s.

#

# Why a separate script (not regenerated on check): batch_calc_gsea() on the

# toy Hallmark + KEGG_LEGACY pathway set takes ~20 s and pulls in clusterProfiler

# + fgsea. Running it at install/check time would (a) bloat `R CMD check`

# runtime and (b) re-introduce the "examples took > 60s" NOTE we just removed.

#

# Run manually (maintainer only) when:

#   - batch_calc_gsea() return structure changes, OR

#   - clusterProfiler / fgsea see a version bump that alters the gseaResult

#     layout, OR

#   - the toy pathway set is regenerated (run make_gsea_pathwaysets_toy.R

#     first)

#

#   Rscript inst/scripts/make_precomputed_gseares.R

#

# Output file (written to data/):

#   - precomputed_gseares.rds  (~1.3 MB, GseaRes list for 2 contrasts)

# -------------------------------------------------------------------------



# Use the in-tree GSEAlens so the return structure matches what users get

# after install. This script must be run from the package root:

#

#   Rscript inst/scripts/make_precomputed_gseares.R

#

pkg_root <- getwd()

if (!dir.exists(file.path(pkg_root, "data"))) {

  stop("Run from the package root: `Rscript inst/scripts/make_precomputed_gseares.R`")

}



if (!nzchar(Sys.getenv("GSEALENS_USE_INSTALLED"))) {

  devtools::load_all(pkg_root)

} else {

  library(GSEAlens)

}



stopifnot(

  requireNamespace("clusterProfiler", quietly = TRUE),

  requireNamespace("fgsea", quietly = TRUE)

)



out_dir <- file.path(pkg_root, "data")

extdata <- out_dir



# ---- Load pre-computed inputs (regenerated by their own scripts) --------

stopifnot(

  file.exists(file.path(extdata, "preprocessed_limma.rds")),

  file.exists(file.path(extdata, "gsea_pathwaysets_toy.rds"))

)



precomp <- readRDS(file.path(extdata, "preprocessed_limma.rds"))

pathways <- readRDS(file.path(extdata, "gsea_pathwaysets_toy.rds"))



gsea_env <- setup_gsea_env(

  fit = precomp$fit,

  pathway_obj = pathways,

  expr_data = precomp$gsea_limma_voom_data

)



# ---- Run batch_calc_gsea (2 workers, scratch output dir) ----------------

scratch <- file.path(tempdir(), "gseares_build")

unlink(scratch, recursive = TRUE)

dir.create(scratch, showWarnings = FALSE, recursive = TRUE)



gsea_res <- batch_calc_gsea(

  gsea_env         = gsea_env,

  custom_series_name = "precomputed",

  output_dir       = scratch,

  workers          = 2,

  use_progress     = FALSE

)



# ---- Save ---------------------------------------------------------------

out_file <- file.path(out_dir, "precomputed_gseares.rds")

saveRDS(gsea_res, out_file, compress = "xz")



# ---- Report -------------------------------------------------------------

size_mb <- file.size(out_file) / 1024^2

md5 <- unname(tools::md5sum(out_file))

n_contrasts <- length(gsea_res)

n_pathways_total <- if (is.list(gsea_res)) {

  sum(vapply(gsea_res, function(t) {

    if (is.null(t$gsea_res)) return(0L)

    nrow(as.data.frame(t$gsea_res))

  }, integer(1)))

} else NA_integer_



cat(sprintf(

  "Wrote:\n  %s  (%.2f MB, %d contrasts, %d total enriched pathways, md5=%s)\n",

  basename(out_file), size_mb, n_contrasts, n_pathways_total, md5

))



cat("\nclusterProfiler version:",

    as.character(packageVersion("clusterProfiler")), "\n")

cat("fgsea version:", as.character(packageVersion("fgsea")), "\n")

cat("GSEAlens version:", as.character(packageVersion("GSEAlens")), "\n")

cat("Generation date:", format(Sys.Date(), "%Y-%m-%d"), "\n")

# Section ----



#' @title Quadrant Linkage Module

#' @description Four-quadrant interactive visualization module for pathway and gene analysis.

#'   Provides synchronized pathway selection, gene ranking, differential expression volcano,

#'   and expression boxplot visualizations with bidirectional table-volcano synchronization.

#' @keywords internal



#' @title Quadrant Linkage UI

#' @description UI components for the four-quadrant interactive visualization

#' @param id Module ID

#' @return Shiny UI tagList

#' @keywords internal



# Subsection ----



mod_quadrant_ui <- function(id) {

    ns <- shiny::NS(id)

    shiny::tagList(

    shiny::fluidRow(

            shiny::column(6, shiny::div(

        class = "white-box",

        shiny::h4("1. Pathway Volcano"),

        shiny::div(

                    style = "position: relative;",

                    plotly::plotlyOutput(ns("volcano_pathway"), height = "400px"),

                    shiny::div(

            style = "position: absolute; top: 10px; left: 10px; background: rgba(255,255,255,0.9); padding: 5px 10px; border-radius: 4px; font-size: 11px; color: #666;",

            "Click pathway to highlight | Check 'Joint Plot' in table"

                    )

        ),

        shiny::uiOutput(ns("selected_pathways_display")),

        shiny::div(

                    style = "margin-top: 10px;",

                    shiny::actionButton(

            ns("open_volcano_export"),

            label = "Export Publication Plot",

            icon = shiny::icon("file-image"),

            class = "btn-success btn-sm",

            style = "width: 100%;"

                    )

        )

            )),

            shiny::column(6, shiny::div(

        class = "white-box",

        shiny::h4("2. Gene Rank Distribution"),

        plotly::plotlyOutput(ns("volcano_gene"), height = "400px")

            ))

    ),

    shiny::fluidRow(

            shiny::column(6, shiny::div(

        class = "white-box",

        shiny::h4("3. Differential Expression Volcano"),

        shiny::div(

                    style = "margin-bottom: 10px;",

                    shiny::checkboxInput(ns("toggle_volcano_settings"), label = "Settings", value = FALSE)

        ),

        shiny::conditionalPanel(

                    condition = sprintf("input['%s']", ns("toggle_volcano_settings")),

                    shiny::div(

            style = "background: #f8f9fa; padding: 10px; border-radius: 5px; margin-bottom: 10px;",

            shiny::numericInput(ns("volcano_logfc_thresh"), "logFC Threshold:", value = 1, min = 0, max = 22, step = 0.5),

            shiny::numericInput(ns("volcano_pval_thresh"), "P-value Threshold:", value = 0.05, min = 0.001, max = 1, step = 0.01)

                    )

        ),

        plotly::plotlyOutput(ns("de_volcano"), height = "400px"),

        shiny::div(

                    style = "margin-top: 10px;",

                    shiny::actionButton(

            ns("open_de_volcano_export"),

            label = "Export Publication Plot",

            icon = shiny::icon("file-image"),

            class = "btn-success",

            style = "width: 100%;"

                    )

        )

            )),

            shiny::column(6, shiny::div(

        class = "white-box",

        shiny::h4("4. Full Expression Distribution"),

        shiny::div(

                    style = "margin-bottom: 10px;",

                    shiny::checkboxInput(ns("zero_baseline"), label = "Use 0 as baseline", value = TRUE)

        ),

        plotly::plotlyOutput(ns("gene_expr_box"), height = "400px"),

        shiny::uiOutput(ns("boxplot_order_status")),

        shiny::div(

                    style = "margin-top: 10px;",

                    shiny::actionButton(

            ns("export_boxplot_data_btn"),

            label = "Export Boxplot Data",

            icon = shiny::icon("table"),

            class = "btn-info btn-sm",

            style = "width: 100%;"

                    ),

                    shiny::helpText(

            "Opens a table of the per-sample expression values (long and wide",

            "formats) for the currently selected gene \u2014 copy to Excel /",

            "GraphPad Prism or download as CSV."

                    ),

                    shiny::hr(),

                    shiny::actionButton(

            ns("open_boxplot_image_export"),

            label = "Export Publication Plot",

            icon = shiny::icon("file-image"),

            class = "btn-success btn-sm",

            style = "width: 100%;"

                    ),

                    shiny::helpText(

            "Opens a publication-grade ggplot2 export modal (PDF/PNG/SVG +",

            "Live Preview + Copy R Code) for the currently selected gene's",

            "boxplot. Matches the interactive plotly styling."

                    )

        )

            ))

    ),

    shiny::hr(),

    shiny::fluidRow(

            shiny::column(12, shiny::div(

        class = "white-box",

        shiny::h4("Gene Expression Table (Click View to display boxplot | Click X to remove)"),

        shiny::div(

                    style = "margin-bottom: 10px;",

                    shiny::actionButton(ns("clear_all_genes_btn_quadrant"), label = "Clear Table", class = "btn-warning btn-sm"),

                    shiny::helpText("Click 'Clear Table' to remove all genes from the table")

        ),

        shiny::div(

                    style = "margin-bottom: 10px; font-size: 12px; color: #666;",

                    shiny::HTML("<b>Gene Sources:</b> Interest genes from sidebar | DE volcano clicks | Pathway genes (orange)")

        ),

        DT::dataTableOutput(ns("gene_expr_table"))

            ))

    ),

    )

}





#' @title Quadrant Linkage Server

#' @description Server logic with unified gene pool and bidirectional table-volcano sync.

#'   Now properly responds to table selection changes.

#' @param id Module ID

#' @param data_prep_list List from data prep module

#' @param gsea_res GseaRes object

#' @param table_controller Table controller with update_selection method

#' @keywords internal



# Subsection ----



mod_quadrant_server <- function(id, data_prep_list, gsea_res, table_controller) {

    shiny::moduleServer(id, function(input, output, session) {

    ns <- session$ns



    # Color Definitions

    COLOR_LEFT <- "#E41A1C"

    COLOR_RIGHT <- "#377EB8"

    COLOR_NS <- "#C0C0C0"

    COLOR_USER <- "#4DAF4A"

    COLOR_PATHWAY <- "#FF9800"

    COLOR_BOTH <- "#9C27B0"



    # Reactive Values

    # 监听 table_controller$selected_pathways 的变化

    selected_pathway_ids <- shiny::reactiveVal(character(0))

    selected_pathway_genes <- shiny::reactiveVal(character(0))

    current_boxplot_gene <- shiny::reactiveVal(NULL)



    # Get Reactive References

    data_prep_data <- data_prep_list$data

    highlight_genes_reactive <- data_prep_list$highlight_genes

    boxplot_order_ref <- data_prep_list$boxplot_order



    # =========================================

    # 关键：响应主Table勾选状态的变化

    # =========================================

    shiny::observe({

            # 监听 table_controller$selected_pathways 的变化

            table_selected <- table_controller$selected_pathways()



            if (!is.null(table_selected) && length(table_selected) > 0) {

        current <- selected_pathway_ids()

        # 只有当与当前状态不同时才更新

        if (!identical(sort(current), sort(table_selected))) {

                    selected_pathway_ids(table_selected)

                    message(sprintf("[Quad] Table selection changed: %s", paste(table_selected, collapse = ", ")))



                    # 更新pathway基因标记（使用最后一个选中的）

                    data_list <- data_prep_data()

                    if (!is.null(data_list)) {

            last_id <- tail(table_selected, 1)

            if (!is.null(data_list$gsea_res@geneSets[[last_id]])) {

                            pathway_genes <- data_list$gsea_res@geneSets[[last_id]]

                            selected_pathway_genes(toupper(pathway_genes))

                            message(sprintf("[Quad] Updated pathway genes from: %s (%d genes)", last_id, length(pathway_genes)))

            }

                    }

        }

            } else {

        # Table清空时同步清空

        if (length(selected_pathway_ids()) > 0) {

                    selected_pathway_ids(character(0))

                    selected_pathway_genes(character(0))

                    message("[Quad] Table selection cleared")

        }

            }

    })



    # Reset on Contrast Switch

    shiny::observeEvent(data_prep_data(), {

            selected_pathway_ids(character(0))

            selected_pathway_genes(character(0))

            current_boxplot_gene(NULL)

            boxplot_order_ref("default")

    })



    # Selected Pathways Display

    output$selected_pathways_display <- shiny::renderUI({

            sel_ids <- selected_pathway_ids()



            if (length(sel_ids) == 0) {

        shiny::div(

                    style = "margin-top: 10px; padding: 10px; background: #f8f9fa; border-radius: 5px;",

                    shiny::div(style = "color: #999; font-style: italic;", "No pathways selected")

        )

            } else {

        tag_list <- lapply(sel_ids, function(pid) {

                    shiny::tags$span(

            style = "display: inline-block; margin-right: 5px; margin-bottom: 5px; padding: 5px 10px; background: #FF9800; color: white; border-radius: 3px; font-size: 12px;",

            pid,

            shiny::tags$button(

                            class = "btn btn-xs",

                            style = "margin-left: 5px; padding: 0 5px; background: rgba(255,255,255,0.3); border: none; color: white;",

                            onclick = sprintf("Shiny.setInputValue('%s', '%s', {priority: 'event'})", ns("remove_pathway"), pid),

                            "x"

            )

                    )

        })



        tag_list <- c(tag_list, list(

                    shiny::tags$button(

            class = "btn btn-sm btn-warning",

            style = "margin-left: 10px;",

            onclick = sprintf("Shiny.setInputValue('%s', 'CLEAR_ALL', {priority: 'event'})", ns("remove_pathway")),

            "Clear All"

                    )

        ))



        shiny::div(

                    style = "margin-top: 10px; padding: 10px; background: #f8f9fa; border-radius: 5px;",

                    shiny::tags$strong("Selected Pathways: "),

                    tag_list

        )

            }

    })



    # Remove Pathway Event

    shiny::observeEvent(input$remove_pathway, {

            pathway_to_remove <- input$remove_pathway

            data_list <- data_prep_data()



            if (pathway_to_remove == "CLEAR_ALL") {

        selected_pathway_ids(character(0))

        selected_pathway_genes(character(0))

        # 同步清空主table勾选

        if (!is.null(table_controller$update_selection)) {

                    table_controller$update_selection("clear")

        }

        message("[Pathway] Cleared all selected pathways")

            } else {

        current <- selected_pathway_ids()

        new_selection <- setdiff(current, pathway_to_remove)

        selected_pathway_ids(new_selection)



        # 同步更新主table勾选

        if (!is.null(table_controller$update_selection)) {

                    table_controller$update_selection("remove", pathway_to_remove)

        }



        if (length(new_selection) > 0) {

                    last_id <- tail(new_selection, 1)

                    if (!is.null(data_list) && !is.null(data_list$gsea_res@geneSets[[last_id]])) {

            pathway_genes <- data_list$gsea_res@geneSets[[last_id]]

            selected_pathway_genes(toupper(pathway_genes))

                    }

        } else {

                    selected_pathway_genes(character(0))

        }

        message(sprintf("[Pathway] Removed: %s", pathway_to_remove))

            }

    })



    # Clear All Genes Button

    shiny::observeEvent(input$clear_all_genes_btn_quadrant, {

            highlight_genes_reactive(character(0))

            message("[Gene] Cleared all gene markers from quadrant")

    })



    # 1. Pathway Volcano Plot

    output$volcano_pathway <- plotly::renderPlotly({

            data_list <- data_prep_data()

            shiny::req(data_list)

            df <- data_list$df

            current_selections <- selected_pathway_ids()



            df$color <- ifelse(df$ID %in% current_selections, COLOR_PATHWAY,

        ifelse(df$NES > 0, COLOR_LEFT, COLOR_RIGHT)

            )

            df$size <- ifelse(df$ID %in% current_selections, 18, 10)

            df$opacity <- ifelse(length(current_selections) == 0, 0.8,

        ifelse(df$ID %in% current_selections, 1.0, 0.35)

            )

            df$linewidth <- ifelse(df$ID %in% current_selections, 3, 1)



            annotations_list <- list()

            if (length(current_selections) > 0) {

        selected_df <- df[df$ID %in% current_selections, ]

        for (i in seq_len(nrow(selected_df))) {

                    row <- selected_df[i, ]

                    ax_offset <- ifelse(i %% 2 == 1, 0, 50)

                    annotations_list[[i]] <- list(

            x = row$NES,

            y = -log10(row$pvalue),

            text = row$ID,

            showarrow = TRUE,

            arrowhead = 2,

            arrowsize = 1,

            arrowwidth = 2,

            arrowcolor = COLOR_PATHWAY,

            ax = ax_offset,

            ay = -30,

            font = list(size = 10, color = COLOR_PATHWAY),

            bgcolor = "rgba(255,255,255,0.95)",

            bordercolor = COLOR_PATHWAY,

            borderwidth = 2,

            borderpad = 4

                    )

        }

            }



            plotly::plot_ly(

        data = df,

        x = ~NES,

        y = ~ -log10(pvalue),

        type = "scatter",

        mode = "markers",

        marker = list(

                    color = ~color,

                    size = ~size,

                    opacity = ~opacity,

                    line = list(color = "white", width = ~linewidth)

        ),

        text = ~ sprintf("%s<br>NES: %.2f<br>P-value: %.2e<br>FDR: %.2e", ID, NES, pvalue, p.adjust),

        hoverinfo = "text",

        key = ~ID,

        source = ns("pathway_volcano")

            ) |>

        plotly::layout(

                    title = list(

            text = sprintf(

                            "Pathway Volcano: %s vs %s<br><sub>%d pathways | %d selected | %d significant (P<0.05)</sub>",

                            data_list$left_group, data_list$right_group,

                            nrow(df), length(current_selections), sum(df$p.adjust < 0.25, na.rm = TRUE)

            ),

            font = list(size = 14),

            x = 0.5,

            xanchor = "center"

                    ),

                    xaxis = list(title = "NES", zeroline = FALSE),

                    yaxis = list(title = "-log10 (P-value)", zeroline = FALSE),

                    showlegend = FALSE,

                    dragmode = "pan",

                    annotations = annotations_list

        )

    })



    # Pathway Volcano Click Event - 双向同步

    shiny::observeEvent(plotly::event_data("plotly_click", source = ns("pathway_volcano")), {

            click <- plotly::event_data("plotly_click", source = ns("pathway_volcano"))

            if (is.null(click) || is.null(click$key)) {

        return()

            }



            clicked_id <- click$key

            current <- selected_pathway_ids()

            data_list <- data_prep_data()



            if (clicked_id %in% current) {

        # 点击已选中的 → 取消选中

        new_selection <- setdiff(current, clicked_id)

        selected_pathway_ids(new_selection)



        # 同步更新主table勾选

        if (!is.null(table_controller$update_selection)) {

                    table_controller$update_selection("remove", clicked_id)

        }



        if (length(new_selection) > 0) {

                    last_id <- tail(new_selection, 1)

                    if (!is.null(data_list) && !is.null(data_list$gsea_res@geneSets[[last_id]])) {

            pathway_genes <- data_list$gsea_res@geneSets[[last_id]]

            selected_pathway_genes(toupper(pathway_genes))

                    }

        } else {

                    selected_pathway_genes(character(0))

        }

        message(sprintf("[Pathway] Deselected: %s", clicked_id))

            } else {

        # 点击未选中的 → 添加选中

        new_selection <- c(current, clicked_id)

        selected_pathway_ids(new_selection)



        # 同步更新主table勾选

        if (!is.null(table_controller$update_selection)) {

                    table_controller$update_selection("add", clicked_id)

        }



        if (!is.null(data_list) && !is.null(data_list$gsea_res@geneSets[[clicked_id]])) {

                    pathway_genes <- data_list$gsea_res@geneSets[[clicked_id]]

                    selected_pathway_genes(toupper(pathway_genes))

        }



        message(sprintf("[Pathway] Selected: %s", clicked_id))

            }

    })



    # 2. Gene Rank Distribution

    output$volcano_gene <- plotly::renderPlotly({

            data_list <- data_prep_data()

            shiny::req(data_list)



            genelist <- data_list$gsea_res@geneList

            rank_df <- data.frame(

        Rank = seq_along(genelist),

        Metric = as.numeric(genelist),

        Gene = names(genelist),

        stringsAsFactors = FALSE

            )



            pathway_genes <- selected_pathway_genes()

            rank_df$Color <- COLOR_NS

            rank_df$Size <- 4



            if (length(pathway_genes) > 0) {

        match_idx <- which(toupper(rank_df$Gene) %in% pathway_genes)

        if (length(match_idx) > 0) {

                    rank_df$Color[match_idx] <- COLOR_PATHWAY

                    rank_df$Size[match_idx] <- 12

        }

            }



            plotly::plot_ly(

        data = rank_df,

        x = ~Rank,

        y = ~Metric,

        type = "scattergl",

        mode = "markers",

        marker = list(color = ~Color, size = ~Size, opacity = 0.8, line = list(width = 0)),

        text = ~Gene,

        hoverinfo = "text"

            ) |>

        plotly::layout(

                    xaxis = list(title = "Gene Rank"),

                    yaxis = list(title = "Ranking Metric (Stat)"),

                    showlegend = FALSE,

                    title = list(

            text = ifelse(length(pathway_genes) > 0,

                            sprintf("Selected: %d pathway genes highlighted", length(pathway_genes)),

                            "Click pathway in volcano above to mark"

            ),

            font = list(size = 12)

                    )

        )

    })



    # 3. DE Volcano Plot

    output$de_volcano <- plotly::renderPlotly({

            data_list <- data_prep_data()

            shiny::req(data_list)



            contrast_id <- data_list$contrast_id

            left_group <- data_list$left_group

            right_group <- data_list$right_group



            de_df <- tryCatch(

        {

                    get_de_table(gsea_res, contrast_id)

        },

        error = function(e) NULL

            )



            shiny::req(de_df)



            if (!"logFC" %in% colnames(de_df)) de_df$logFC <- de_df$log2FoldChange

            if (!"pvalue" %in% colnames(de_df)) de_df$pvalue <- de_df$p.value

            if (!"padj" %in% colnames(de_df)) de_df$padj <- de_df$p.adjust



            de_df <- de_df[!is.na(de_df$logFC) & !is.na(de_df$pvalue), ]

            if (nrow(de_df) == 0) {

        return(NULL)

            }



            logfc_thresh <- if (!is.null(input$volcano_logfc_thresh)) input$volcano_logfc_thresh else 1

            pval_thresh <- if (!is.null(input$volcano_pval_thresh)) input$volcano_pval_thresh else 0.05



            de_df$x_axis <- de_df$logFC

            de_df$y_axis <- -log10(de_df$pvalue)



            inf_y <- is.infinite(de_df$y_axis)

            if (any(inf_y)) {

        max_y <- max(de_df$y_axis[!inf_y], na.rm = TRUE)

        de_df$y_axis[inf_y] <- max_y * 1.1

            }



            user_genes <- toupper(highlight_genes_reactive())

            pathway_genes <- selected_pathway_genes()

            de_df$gene_upper <- toupper(de_df$gene_symbol)



            de_df$is_user <- de_df$gene_upper %in% user_genes

            de_df$is_pathway <- de_df$gene_upper %in% pathway_genes

            de_df$is_significant <- abs(de_df$logFC) > logfc_thresh & de_df$pvalue < pval_thresh



            n_up <- sum(de_df$is_significant & de_df$logFC > 0, na.rm = TRUE)

            n_down <- sum(de_df$is_significant & de_df$logFC < 0, na.rm = TRUE)

            n_not_sig <- sum(!de_df$is_significant, na.rm = TRUE)

            n_user <- sum(de_df$is_user, na.rm = TRUE)

            n_pathway <- sum(de_df$is_pathway, na.rm = TRUE)

            n_both <- sum(de_df$is_user & de_df$is_pathway, na.rm = TRUE)



            de_df$color <- dplyr::case_when(

        de_df$is_user & de_df$is_pathway ~ COLOR_BOTH,

        de_df$is_user ~ COLOR_USER,

        de_df$is_pathway ~ COLOR_PATHWAY,

        de_df$is_significant & de_df$logFC > 0 ~ COLOR_LEFT,

        de_df$is_significant & de_df$logFC < 0 ~ COLOR_RIGHT,

        TRUE ~ COLOR_NS

            )



            de_df$size <- dplyr::case_when(

        de_df$is_user | de_df$is_pathway ~ 15,

        de_df$is_significant ~ 9,

        TRUE ~ 4

            )



            de_df$opacity <- dplyr::case_when(

        de_df$is_user | de_df$is_pathway ~ 1.0,

        de_df$is_significant ~ 0.7,

        TRUE ~ 0.5

            )



            de_df$linewidth <- dplyr::case_when(

        de_df$is_user | de_df$is_pathway ~ 1.0,

        TRUE ~ 0

            )



            de_df$plot_order <- dplyr::case_when(

        de_df$is_user | de_df$is_pathway ~ 3,

        de_df$is_significant ~ 2,

        TRUE ~ 1

            )

            de_df <- de_df[order(de_df$plot_order), ]



            current_selected <- selected_pathway_ids()

            pathway_label <- if (length(current_selected) > 0) {

        sprintf(" | From: %s", tail(current_selected, 1))

            } else {

        ""

            }



            title_text <- sprintf(

        "%s vs %s<br><sup>Up %d | Down %d | NS %d | Selected %d | Pway %d | Both %d%s</sup>",

        left_group, right_group, n_up, n_down, n_not_sig, n_user, n_pathway, n_both, pathway_label

            )



            annotations_list <- list()

            max_y_val <- max(de_df$y_axis, na.rm = TRUE)



            annotations_list[[1]] <- list(

        x = 0.99, y = 0.99, xref = "paper", yref = "paper",

        text = paste0("<b style='color:", COLOR_LEFT, ";'>High in ", left_group, "</b>"),

        showarrow = FALSE, font = list(size = 14),

        xanchor = "right", yanchor = "top",

        bgcolor = "rgba(255,255,255,0.9)",

        bordercolor = COLOR_LEFT, borderwidth = 2, borderpad = 6

            )

            annotations_list[[2]] <- list(

        x = 0.01, y = 0.99, xref = "paper", yref = "paper",

        text = paste0("<b style='color:", COLOR_RIGHT, ";'>High in ", right_group, "</b>"),

        showarrow = FALSE, font = list(size = 14),

        xanchor = "left", yanchor = "top",

        bgcolor = "rgba(255,255,255,0.9)",

        bordercolor = COLOR_RIGHT, borderwidth = 2, borderpad = 6

            )



            user_genes_df <- de_df[de_df$is_user, ]

            if (nrow(user_genes_df) > 0) {

        for (i in seq_len(min(nrow(user_genes_df), 20))) {

                    gene <- user_genes_df[i, ]

                    gene_color <- if (gene$is_user && gene$is_pathway) COLOR_BOTH else COLOR_USER

                    annotations_list[[length(annotations_list) + 1]] <- list(

            x = gene$x_axis,

            y = gene$y_axis,

            text = gene$gene_symbol,

            showarrow = TRUE,

            arrowhead = 0,

            arrowsize = 1,

            arrowwidth = 2,

            arrowcolor = gene_color,

            ax = ifelse(gene$logFC > 0, 50, -50),

            ay = -35,

            bgcolor = "rgba(255,255,255,0.85)",

            bordercolor = gene_color,

            borderwidth = 0.5,

            font = list(size = 12, color = gene_color)

                    )

        }

            }



            if (logfc_thresh > 0) {

        annotations_list[[length(annotations_list) + 1]] <- list(

                    x = logfc_thresh, y = max_y_val * 0.95,

                    text = sprintf("|logFC|=%.1f", logfc_thresh),

                    showarrow = FALSE, font = list(size = 10, color = "gray")

        )

        annotations_list[[length(annotations_list) + 1]] <- list(

                    x = -logfc_thresh, y = max_y_val * 0.95,

                    text = sprintf("|logFC|=%.1f", logfc_thresh),

                    showarrow = FALSE, font = list(size = 10, color = "gray")

        )

            }





            # ============================================================

            # 分层渲染：拆分数据为多个子集（从底到顶）

            # ============================================================



            df_ns <- de_df[!de_df$is_significant, ]

            df_up <- de_df[de_df$is_significant & de_df$logFC > 0, ]

            df_down <- de_df[de_df$is_significant & de_df$logFC < 0, ]

            df_pathway <- de_df[de_df$is_pathway & !de_df$is_user, ]

            df_user <- de_df[de_df$is_user & !de_df$is_pathway, ]

            df_both <- de_df[de_df$is_user & de_df$is_pathway, ]



            # 初始化 plot_ly 时设置 source（点击事件需要）

            p <- plotly::plot_ly(

        data = de_df,

        source = ns("deg_volcano")

            )



            # Layer 1: NS (灰色，最底层)

            if (nrow(df_ns) > 0) {

        p <- p |> plotly::add_trace(

                    data = df_ns,

                    x = df_ns$x_axis,

                    y = df_ns$y_axis,

                    type = "scatter",

                    mode = "markers",

                    marker = list(color = COLOR_NS, size = 4, opacity = 0.5, line = list(color = "white", width = 0)),

                    text = sprintf("%s<br>logFC: %.2f<br>-log10(p): %.2f<br>FDR: %.2e", df_ns$gene_symbol, df_ns$logFC, df_ns$y_axis, df_ns$padj),

                    hoverinfo = "text",

                    key = df_ns$gene_upper,

                    showlegend = FALSE,

                    inherit = FALSE

        )

            }



            # Layer 2: UP (红色)

            if (nrow(df_up) > 0) {

        p <- p |> plotly::add_trace(

                    data = df_up,

                    x = df_up$x_axis,

                    y = df_up$y_axis,

                    type = "scatter",

                    mode = "markers",

                    marker = list(color = COLOR_LEFT, size = 9, opacity = 0.7, line = list(color = "white", width = 0)),

                    text = sprintf("%s<br>logFC: %.2f<br>-log10(p): %.2f<br>FDR: %.2e", df_up$gene_symbol, df_up$logFC, df_up$y_axis, df_up$padj),

                    hoverinfo = "text",

                    key = df_up$gene_upper,

                    showlegend = FALSE,

                    inherit = FALSE

        )

            }



            # Layer 3: DOWN (蓝色)

            if (nrow(df_down) > 0) {

        p <- p |> plotly::add_trace(

                    data = df_down,

                    x = df_down$x_axis,

                    y = df_down$y_axis,

                    type = "scatter",

                    mode = "markers",

                    marker = list(color = COLOR_RIGHT, size = 9, opacity = 0.7, line = list(color = "white", width = 0)),

                    text = sprintf("%s<br>logFC: %.2f<br>-log10(p): %.2f<br>FDR: %.2e", df_down$gene_symbol, df_down$logFC, df_down$y_axis, df_down$padj),

                    hoverinfo = "text",

                    key = df_down$gene_upper,

                    showlegend = FALSE,

                    inherit = FALSE

        )

            }



            # Layer 4: 仅Pathway (橙色)

            if (nrow(df_pathway) > 0) {

        p <- p |> plotly::add_trace(

                    data = df_pathway,

                    x = df_pathway$x_axis,

                    y = df_pathway$y_axis,

                    type = "scatter",

                    mode = "markers",

                    marker = list(color = COLOR_PATHWAY, size = 15, opacity = 1.0, line = list(color = "white", width = 1.5)),

                    text = sprintf("%s<br>logFC: %.2f<br>-log10(p): %.2f<br>FDR: %.2e<br>[Pathway]", df_pathway$gene_symbol, df_pathway$logFC, df_pathway$y_axis, df_pathway$padj),

                    hoverinfo = "text",

                    key = df_pathway$gene_upper,

                    showlegend = FALSE,

                    inherit = FALSE

        )

            }



            # Layer 5: 仅User (绿色)

            if (nrow(df_user) > 0) {

        p <- p |> plotly::add_trace(

                    data = df_user,

                    x = df_user$x_axis,

                    y = df_user$y_axis,

                    type = "scatter",

                    mode = "markers",

                    marker = list(color = COLOR_USER, size = 15, opacity = 1.0, line = list(color = "white", width = 1.5)),

                    text = sprintf("%s<br>logFC: %.2f<br>-log10(p): %.2f<br>FDR: %.2e<br>[Selected]", df_user$gene_symbol, df_user$logFC, df_user$y_axis, df_user$padj),

                    hoverinfo = "text",

                    key = df_user$gene_upper,

                    showlegend = FALSE,

                    inherit = FALSE

        )

            }



            # Layer 6: Both (紫色，最顶层)

            if (nrow(df_both) > 0) {

        p <- p |> plotly::add_trace(

                    data = df_both,

                    x = df_both$x_axis,

                    y = df_both$y_axis,

                    type = "scatter",

                    mode = "markers",

                    marker = list(color = COLOR_BOTH, size = 18, opacity = 1.0, line = list(color = "white", width = 2)),

                    text = sprintf("%s<br>logFC: %.2f<br>-log10(p): %.2f<br>FDR: %.2e<br>[Both]", df_both$gene_symbol, df_both$logFC, df_both$y_axis, df_both$padj),

                    hoverinfo = "text",

                    key = df_both$gene_upper,

                    showlegend = FALSE,

                    inherit = FALSE

        )

            }



            # 布局设置

            p <- p |>

        plotly::layout(

                    title = list(text = title_text, font = list(size = 14), x = 0.5, xanchor = "center"),

                    xaxis = list(title = "logFC", zeroline = FALSE, showgrid = TRUE, gridcolor = "lightgray"),

                    yaxis = list(title = "-log10 (P-value)", zeroline = FALSE, showgrid = TRUE, gridcolor = "lightgray"),

                    showlegend = FALSE,

                    dragmode = "pan",

                    annotations = annotations_list,

                    shapes = list(

            list(

                            type = "line", x0 = logfc_thresh, x1 = logfc_thresh,

                            y0 = 0, y1 = max_y_val * 1.05,

                            line = list(color = "gray", dash = "dash", width = 1)

            ),

            list(

                            type = "line", x0 = -logfc_thresh, x1 = -logfc_thresh,

                            y0 = 0, y1 = max_y_val * 1.05,

                            line = list(color = "gray", dash = "dash", width = 1)

            ),

            list(

                            type = "line", x0 = min(de_df$x_axis) * 1.1, x1 = max(de_df$x_axis) * 1.1,

                            y0 = -log10(pval_thresh), y1 = -log10(pval_thresh),

                            line = list(color = "gray", dash = "dash", width = 1)

            )

                    )

        )



            p

    })



    # DE Volcano Click Event

    shiny::observeEvent(plotly::event_data("plotly_click", source = ns("deg_volcano")), {

            click <- plotly::event_data("plotly_click", source = ns("deg_volcano"))

            if (is.null(click) || is.null(click$key)) {

        return()

            }



            clicked_gene <- click$key

            current_boxplot_gene(clicked_gene)



            current_applied <- highlight_genes_reactive()

            highlight_genes_reactive(union(toupper(current_applied), toupper(clicked_gene)))

    })



    # 4. Gene Expression Table

    output$gene_expr_table <- DT::renderDataTable({

            genes <- highlight_genes_reactive()

            shiny::req(length(genes) > 0)



            data_list <- data_prep_data()

            shiny::req(data_list)

            symbol_map <- .rebuild_symbol_map(gsea_res, data_list$contrast_id)



            left_grp <- data_list$left_group

            right_grp <- data_list$right_group



            # 获取 DE volcano 的阈值

            pval_thresh <- input$volcano_pval_thresh %||% 0.05

            logfc_thresh <- input$volcano_logfc_thresh %||% 1



            user_genes <- toupper(highlight_genes_reactive())

            pathway_genes <- selected_pathway_genes()



            de_df <- tryCatch(

        {

                    get_de_table(gsea_res, data_list$contrast_id)

        },

        error = function(e) NULL

            )



            table_data <- lapply(genes, function(g) {

        gene_upper <- toupper(g)

        is_user <- gene_upper %in% user_genes

        is_pathway <- gene_upper %in% pathway_genes



        logfc <- NA_real_

        pval <- NA_real_

        padj_val <- NA_real_

        if (!is.null(de_df)) {

                    if ("gene_symbol" %in% colnames(de_df)) {

            idx <- which(toupper(de_df$gene_symbol) == gene_upper)

                    } else {

            idx <- which(rownames(de_df) == g)

                    }

                    if (length(idx) > 0) {

            row <- de_df[idx[1], ]

            logfc <- if ("logFC" %in% colnames(row)) row$logFC else row$log2FoldChange

            pval <- row$pvalue

            padj_val <- row$padj

                    }

        }



        updown <- ifelse(is.na(logfc), "-", ifelse(logfc > 0, "UP", "DOWN"))

        high_in <- ifelse(is.na(logfc), "-", ifelse(logfc > 0, left_grp, right_grp))



        # 保持数值为数值类型，不转字符串

        pval_display <- ifelse(is.na(pval), NA_real_, pval)

        padj_display <- ifelse(is.na(padj_val), NA_real_, padj_val)



        # 使用 DE volcano 阈值的 Significant 判断

        sig_label <- {

                    if (is.na(pval) || is.na(logfc)) {

            "-"

                    } else if (pval < pval_thresh && logfc > logfc_thresh) {

            "upreg"

                    } else if (pval < pval_thresh && logfc < -logfc_thresh) {

            "downreg"

                    } else {

            "ns"

                    }

        }



        # ✅ 修复后的Delete和View按钮生成

        delete_btn <- sprintf(

                    '<button class="btn btn-xs btn-danger" onclick="Shiny.setInputValue(\'%s\', {gene_to_remove: \'%s\', time: Date.now()}, {priority: \'event\'});" style="margin-right: 5px;">X</button>',

                    ns("delete_gene_action"), g

        )



        view_btn <- sprintf(

                    '<button class="btn btn-xs btn-primary" onclick="Shiny.setInputValue(\'%s\', {gene_to_view: \'%s\', time: Date.now()}, {priority: \'event\'});">View</button>',

                    ns("view_boxplot_action"), g

        )



        data.frame(

                    Gene = .get_display_symbol(g, symbol_map),

                    Log2FC = ifelse(is.na(logfc), "-", sprintf("%.3f", logfc)),

                    Pvalue = pval_display,

                    Padjust = padj_display,

                    UpDown = updown,

                    Significant = sig_label,

                    HighIn = high_in,

                    IsPathway = ifelse(is_pathway, "Yes", "No"),

                    Delete = delete_btn,

                    ViewBox = view_btn,

                    stringsAsFactors = FALSE,

                    check.names = FALSE

        )

            })



            table_df <- do.call(rbind, table_data)



            # 按 P-value 升序排序

            table_df <- table_df[order(table_df[["Pvalue"]], na.last = TRUE), ]



            # JS 渲染函数：科学计数法

            js_pval_render <- "

        function(data, type) {

                    if (type === 'sort' || type === 'type') return data;

                    if (data === null || isNaN(data)) return '-';

                    if (Math.abs(data) < 0.001) return data.toExponential(2);

                    return data.toFixed(4);

        }

            "



            dt <- DT::datatable(

        table_df,

        escape = FALSE,

        rownames = FALSE,

        selection = "none",

        extensions = c("Scroller"),

        options = list(

                    pageLength = 10,

                    scrollY = "50vh",

                    scroller = TRUE,

                    dom = "frtip",

                    search = list(caseInsensitive = TRUE),

                    columnDefs = list(

            list(orderable = FALSE, targets = c(7, 8)),

            list(className = "dt-center", targets = c(0, 2, 3, 4, 5, 6)),

            list(

                            targets = 2,

                            render = htmlwidgets::JS(js_pval_render)

            ),

            list(

                            targets = 3,

                            render = htmlwidgets::JS(js_pval_render)

            )

                    ),

                    order = list(list(2, "asc"))

        )

            )



            # Up/Down 列样式

            dt <- dt |> DT::formatStyle(

        columns = "UpDown",

        backgroundColor = DT::styleEqual(c("UP", "DOWN", "-"), c("#FFCDD2", "#BBDEFB", "transparent")),

        color = DT::styleEqual(c("UP", "DOWN", "-"), c(COLOR_LEFT, COLOR_RIGHT, "#666")),

        fontWeight = DT::styleEqual(c("UP", "DOWN"), c("bold", "bold"))

            )



            # Significant 列样式

            dt <- dt |> DT::formatStyle(

        columns = "Significant",

        color = DT::styleEqual(c("upreg", "downreg", "ns"), c("#E41A1C", "#377EB8", "#999999")),

        fontWeight = "bold"

            )



            # High In 列样式

            dt <- dt |> DT::formatStyle(

        columns = "HighIn",

        color = DT::styleEqual(c(left_grp, right_grp, "-"), c("#E41A1C", "#377EB8", "#999999"))

            )



            # In Pathway 列样式

            dt <- dt |> DT::formatStyle(

        columns = "IsPathway",

        backgroundColor = DT::styleEqual(c("Yes", "No"), c("#9C27B0", "transparent")),

        color = DT::styleEqual(c("Yes", "No"), c("#FFFFFF", "#999999")),

        fontWeight = DT::styleEqual(c("Yes", "No"), c("bold", "normal"))

            )



            # P-value 列样式

            dt <- dt |> DT::formatStyle(

        columns = "Pvalue",

        color = DT::styleInterval(0.001, c("red", "black")),

        fontWeight = DT::styleInterval(0.05, c("bold", "normal"))

            )



            # Adj P-value 列样式

            dt <- dt |> DT::formatStyle(

        columns = "Padjust",

        color = DT::styleInterval(0.001, c("red", "black")),

        fontWeight = DT::styleInterval(0.05, c("bold", "normal"))

            )



            dt

    })



    # ✅ 修复Delete基因事件监听

    shiny::observeEvent(input$delete_gene_action,

            {

        action_data <- input$delete_gene_action

        if (is.null(action_data) || is.null(action_data$gene_to_remove)) {

                    return()

        }



        gene_to_remove <- action_data$gene_to_remove

        current_applied <- highlight_genes_reactive()

        new_applied <- setdiff(toupper(current_applied), toupper(gene_to_remove))

        highlight_genes_reactive(new_applied)



        # 同时清除对应的箱线图

        if (!is.null(current_boxplot_gene()) && toupper(current_boxplot_gene()) == toupper(gene_to_remove)) {

                    current_boxplot_gene(NULL)

        }



        message(sprintf("[Gene] Removed from table: %s", gene_to_remove))

        shiny::showNotification(sprintf("Gene removed: %s", gene_to_remove), type = "message", duration = 2)

            },

            ignoreNULL = TRUE

    )



    # ✅ 修复View箱线图事件监听

    shiny::observeEvent(input$view_boxplot_action,

            {

        action_data <- input$view_boxplot_action

        if (is.null(action_data) || is.null(action_data$gene_to_view)) {

                    return()

        }



        gene_name <- action_data$gene_to_view

        current_boxplot_gene(gene_name)



        message(sprintf("[Boxplot] Displaying: %s", gene_name))

        shiny::showNotification(sprintf("Showing boxplot for: %s", gene_name), type = "message", duration = 2)

            },

            ignoreNULL = TRUE

    )



    # 5. Expression Boxplot

    output$gene_expr_box <- plotly::renderPlotly({

            current_gene <- current_boxplot_gene()



            if (is.null(current_gene)) {

        return(plotly::plot_ly() |> plotly::layout(

                    title = list(text = "Click gene in DE volcano or table to view expression", font = list(size = 14))

        ))

            }



            data_list <- data_prep_data()

            shiny::req(data_list)



            tryCatch(

        {

                    expr_mat <- get_expr_matrix(gsea_res, type = data_list$expression_type)

                    sample_meta <- get_sample_meta(gsea_res)



                    if (is.null(expr_mat) || is.null(sample_meta)) {

            return(plotly::plot_ly() |> plotly::layout(title = "Expression matrix not available"))

                    }



                    target_gene_upper <- toupper(current_gene)

                    gene_names_upper <- toupper(rownames(expr_mat))

                    match_idx <- which(gene_names_upper == target_gene_upper)



                    if (length(match_idx) == 0) {

            gene_meta <- gsea_res$expr_bundle$gene_meta

            if (!is.null(gene_meta) && nrow(gene_meta) > 0) {

                            rownames(gene_meta) <- if (is.null(rownames(gene_meta))) rownames(expr_mat) else rownames(gene_meta)

                            symbol_col <- intersect(c("SYMBOL", "symbol", "Gene", "gene_name", "gene_symbol"), colnames(gene_meta))[1]

                            if (!is.na(symbol_col)) {

                meta_symbols_upper <- toupper(as.character(gene_meta[[symbol_col]]))

                meta_matches <- which(meta_symbols_upper == target_gene_upper)

                if (length(meta_matches) > 0) {

                                    ensembl_id <- rownames(gene_meta)[meta_matches[1]]

                                    match_idx <- which(rownames(expr_mat) == ensembl_id)

                }

                            }

            }

                    }



                    if (length(match_idx) == 0 || is.na(match_idx)) {

            return(plotly::plot_ly() |> plotly::layout(

                            title = sprintf("Gene '%s' not found", current_gene)

            ))

                    }



                    symbol_map <- .rebuild_symbol_map(gsea_res, data_list$contrast_id)

                    actual_gene <- rownames(expr_mat)[match_idx[1]]

                    display_gene_name <- .get_display_symbol(current_gene, symbol_map)



                    expr_values <- expr_mat[actual_gene, ]

                    sample_names <- names(expr_values)

                    group_info <- sample_meta$group[match(sample_names, rownames(sample_meta))]



                    plot_data <- data.frame(

            Sample = sample_names,

            Expression = as.numeric(expr_values),

            Group = group_info,

            stringsAsFactors = FALSE

                    )



                    plot_data <- plot_data[!is.na(plot_data$Group), ]

                    if (nrow(plot_data) == 0) {

            return(plotly::plot_ly() |> plotly::layout(title = "No valid group data"))

                    }



                    current_confirmed_order <- boxplot_order_ref()

                    final_order_to_use <- if (!is.null(current_confirmed_order) &&

            current_confirmed_order != "default" &&

            current_confirmed_order != "") {

            current_confirmed_order

                    } else {

            "default"

                    }



                    actual_groups <- unique(as.character(plot_data$Group))

                    x_categories <- NULL



                    if (final_order_to_use != "default" && final_order_to_use != "" && !is.na(final_order_to_use)) {

            sep <- if (grepl("->", final_order_to_use, fixed = TRUE)) "->" else ","

            order_parts <- strsplit(final_order_to_use, sep)[[1]]

            order_parts <- trimws(order_parts)

            valid_parts <- order_parts[order_parts %in% actual_groups]

            x_categories <- c(valid_parts, setdiff(actual_groups, valid_parts))

                    } else {

            x_categories <- actual_groups

                    }



                    plot_data <- plot_data[plot_data$Group %in% x_categories, ]

                    plot_data$Group <- factor(plot_data$Group, levels = x_categories, ordered = TRUE)



                    unique_groups <- levels(plot_data$Group)

                    if (length(unique_groups) == 2) {

            group_colors <- c(COLOR_LEFT, COLOR_RIGHT)

            names(group_colors) <- unique_groups

                    } else {

            group_colors <- c(

                            "#E41A1C", "#377EB8", "#4DAF4A", "#984EA3",

                            "#FF7F00", "#A65628", "#F781BF", "#999999"

            )[seq_along(unique_groups)]

            names(group_colors) <- unique_groups

                    }



                    use_zero_baseline <- input$zero_baseline %||% FALSE



                    y_min <- min(plot_data$Expression, na.rm = TRUE)

                    y_max <- max(plot_data$Expression, na.rm = TRUE)



                    if (use_zero_baseline) {

            y_min <- min(y_min, 0)

            y_max <- max(y_max, 0)

                    }



                    y_range <- y_max - y_min

                    y_min <- y_min - y_range * 0.1

                    y_max <- y_max + y_range * 0.1



                    p <- ggplot2::ggplot(plot_data, ggplot2::aes(x = Group, y = Expression, fill = Group)) +

            ggplot2::geom_boxplot(alpha = 0.7, outlier.shape = NA) +

            ggplot2::geom_jitter(

                            width = 0.2, size = 3, alpha = 0.6,

                            ggplot2::aes(text = sprintf(

                "<b>Sample:</b> %s<br><b>Group:</b> %s<br><b>Expression:</b> %.3f",

                Sample, Group, Expression

                            ))

            ) +

            ggplot2::scale_fill_manual(values = group_colors) +

            ggplot2::scale_x_discrete(limits = x_categories, drop = FALSE) +

            ggplot2::coord_cartesian(ylim = c(y_min, y_max)) +

            ggplot2::theme_bw(base_size = 12) +

            ggplot2::labs(

                            title = sprintf("%s", display_gene_name),

                            y = data_list$expression_type, x = NULL

            ) +

            ggplot2::theme(

                            legend.position = "none",

                            axis.text.x = ggplot2::element_text(angle = 45, hjust = 1)

            )



                    if (use_zero_baseline) {

            p <- p + ggplot2::geom_hline(

                            yintercept = 0, linetype = "dashed",

                            color = "red", alpha = 0.7, size = 0.8

            )

                    }



                    ply <- plotly::ggplotly(p, tooltip = "text")

                    ply |> plotly::layout(

            xaxis = list(categoryorder = "array", categoryarray = x_categories, title = ""),

            dragmode = FALSE

                    )

        },

        error = function(e) {

                    return(plotly::plot_ly() |> plotly::layout(

            title = sprintf("Error: %s", e$message)

                    ))

        }

            )

    })



    # Boxplot Order Status

    output$boxplot_order_status <- shiny::renderUI({

            order_info <- boxplot_order_ref()

            if (is.null(order_info) || order_info == "default") {

        shiny::div(style = "font-size: 12px; color: #666;", "Order: Default")

            } else {

        shiny::div(

                    style = "font-size: 12px; color: #666;",

                    "Order:", gsub(",", " -> ", order_info)

        )

            }

    })



    # ==========================================================================

    # 7. Export Boxplot Data Button (panel 4, "Full Expression Distribution")

    # --------------------------------------------------------------------------

    # Opens a modal with two TSV tables (long + wide) of the per-sample

    # expression values for the currently selected gene. Each table has its

    # own "Copy" / "Download CSV" controls. The user can paste the wide

    # table directly into GraphPad Prism (Column format) or Excel.

    # ==========================================================================

    current_boxplot_data <- shiny::reactiveVal(NULL)

    current_boxplot_filenames <- shiny::reactiveVal(list(long = "long.csv", wide = "wide.csv"))



    # Helper: turn an arbitrary string into a filename-safe token

    .sanitize_filename <- function(x) {

            x <- gsub("[^A-Za-z0-9_.-]+", "_", x %||% "gene")

            x <- gsub("^_+|_+$", "", x)

            if (!nzchar(x)) x <- "gene"

            x

    }



    # Helper: write a string to the system clipboard (uses clipr when available)

    .write_to_clipboard <- function(text) {

            if (requireNamespace("clipr", quietly = TRUE)) {

        clipr::write_clip(text)

        return(invisible(TRUE))

            }

            con <- file("clipboard", "w")

            on.exit(close(con), add = TRUE)

            writeLines(text, con)

            invisible(TRUE)

    }



    shiny::observeEvent(input$export_boxplot_data_btn, {

            data_list <- data_prep_data()

            shiny::req(data_list)



            current_gene <- current_boxplot_gene()

            if (is.null(current_gene) || !nzchar(current_gene)) {

        shiny::showNotification(

                    "No gene selected. Click a gene in the DE volcano or in the gene table first.",

                    type = "warning",

                    duration = 3

        )

        return()

            }



            # Use the same display symbol that the boxplot title shows

            symbol_map <- .rebuild_symbol_map(gsea_res, data_list$contrast_id)

            display_gene <- .get_display_symbol(current_gene, symbol_map)



            result <- tryCatch(

        extract_boxplot_data(

                    GSEAlens_res = gsea_res,

                    contrast_id  = data_list$contrast_id,

                    gene_symbol  = display_gene,

                    expr_type    = data_list$expression_type

        ),

        error = function(e) {

                    shiny::showNotification(

            sprintf("Failed to extract boxplot data: %s", e$message),

            type = "error",

            duration = 5

                    )

                    NULL

        }

            )

            if (is.null(result)) return()



            current_boxplot_data(result)



            # Pre-build the two TSV strings and the safe filenames

            tsv_long <- .serialize_df_tsv(result$long)

            tsv_wide <- .serialize_df_tsv(result$wide)

            safe_gene     <- .sanitize_filename(result$gene)

            safe_contrast <- .sanitize_filename(result$contrast_id)

            current_boxplot_filenames(list(

        long = sprintf("GSEAlens_%s_%s_box_long.csv", safe_gene, safe_contrast),

        wide = sprintf("GSEAlens_%s_%s_box_wide.csv", safe_gene, safe_contrast)

            ))



            # Format the sample-count line for the header

            n_samples <- nrow(result$long)

            group_counts <- table(result$long$Group)

            group_summary <- paste(

        sprintf("%s: %d", names(group_counts), as.integer(group_counts)),

        collapse = ", "

            )



            shiny::showModal(shiny::modalDialog(

        title = sprintf(

                    "Expression Data: %s (n = %d samples; %s)",

                    result$gene, n_samples, group_summary

        ),

        size = "l",

        easyClose = TRUE,

        footer = shiny::modalButton("Close"),

        shiny::tagList(

                    # ---- Section 1: Wide format ------------------------------------

                    shiny::div(

            style = "margin-top: 4px;",

            shiny::tags$b("Wide format"),

            shiny::helpText(

                            style = "margin: 0 0 6px 0;",

                            "First row = sample IDs, second row = expression values. ",

                            "Left-most cell = gene name. Paste into GraphPad Prism (Column data) ",

                            "or Excel."

            )

                    ),

                    shiny::div(

            style = "display: flex; gap: 8px; align-items: center; margin-bottom: 6px;",

            shiny::actionButton(

                            ns("copy_wide_tsv_btn"),

                            label = "Copy Wide",

                            icon = shiny::icon("copy"),

                            class = "btn-success btn-sm"

            ),

            shiny::downloadButton(

                            ns("download_wide_csv_btn"),

                            label = "Download Wide CSV",

                            class = "btn-info btn-sm"

            )

                    ),

                    shiny::div(

            style = "background: #f5f5f5; padding: 12px; border-radius: 5px; max-height: 160px; overflow: auto;",

            shiny::tags$pre(

                            shiny::code(tsv_wide),

                            style = "font-size: 11px; white-space: pre; word-break: keep-all; margin: 0;"

            )

                    ),



                    shiny::hr(),



                    # ---- Section 2: Long format ------------------------------------

                    shiny::div(

            shiny::tags$b("Long format"),

            shiny::helpText(

                            style = "margin: 0 0 6px 0;",

                            "One row per sample. Suitable for GraphPad Prism (Grouped data) ",

                            "and most R / Python plotting tools."

            )

                    ),

                    shiny::div(

            style = "display: flex; gap: 8px; align-items: center; margin-bottom: 6px;",

            shiny::actionButton(

                            ns("copy_long_tsv_btn"),

                            label = "Copy Long",

                            icon = shiny::icon("copy"),

                            class = "btn-success btn-sm"

            ),

            shiny::downloadButton(

                            ns("download_long_csv_btn"),

                            label = "Download Long CSV",

                            class = "btn-info btn-sm"

            )

                    ),

                    shiny::div(

            style = "background: #f5f5f5; padding: 12px; border-radius: 5px; max-height: 220px; overflow: auto;",

            shiny::tags$pre(

                            shiny::code(tsv_long),

                            style = "font-size: 11px; white-space: pre; word-break: keep-all; margin: 0;"

            )

                    )

        )

            ))

    })



    # Copy handlers ---------------------------------------------------------

    shiny::observeEvent(input$copy_wide_tsv_btn, {

            result <- current_boxplot_data()

            shiny::req(result)

            tsv <- .serialize_df_tsv(result$wide)

            .write_to_clipboard(tsv)

            shiny::showNotification("Wide table copied to clipboard.", type = "message", duration = 2)

    })



    shiny::observeEvent(input$copy_long_tsv_btn, {

            result <- current_boxplot_data()

            shiny::req(result)

            tsv <- .serialize_df_tsv(result$long)

            .write_to_clipboard(tsv)

            shiny::showNotification("Long table copied to clipboard.", type = "message", duration = 2)

    })



    # Download handlers -----------------------------------------------------

    output$download_wide_csv_btn <- shiny::downloadHandler(

            filename = function() current_boxplot_filenames()$wide,

            content  = function(file) {

        result <- current_boxplot_data()

        tsv <- if (is.null(result)) "" else .serialize_df_tsv(result$wide)

        # writeLines adds a final "\n", matching the modal preview

        writeLines(if (nzchar(tsv)) tsv else "Gene\t(empty)\n", file, useBytes = TRUE)

            }

    )



    output$download_long_csv_btn <- shiny::downloadHandler(

            filename = function() current_boxplot_filenames()$long,

            content  = function(file) {

        result <- current_boxplot_data()

        tsv <- if (is.null(result)) "" else .serialize_df_tsv(result$long)

        writeLines(if (nzchar(tsv)) tsv else "Sample\tGroup\tExpression\n", file, useBytes = TRUE)

            }

    )



    # ============================================================

    # Boxplot Image Export Center (panel 4, "Full Expression Distribution")

    # ------------------------------------------------------------

    # Publication-grade ggplot2 export for the selected gene's boxplot.

    # Mirrors the volcano export modal pattern + internal/external preview.

    # ============================================================

    .build_boxplot_export_modal <- function() {

            shiny::modalDialog(

        title = "Export Publication Plot - Gene Boxplot",

        size = "l", footer = NULL, easyClose = TRUE,

        shiny::fluidRow(

                    shiny::column(5,

            shiny::h5("Dimensions"),

            shiny::fluidRow(

                            shiny::column(6,

                shiny::numericInput(ns("box_exp_width"),  "Width (inch)",  value = 5, min = 2, max = 20)

                            ),

                            shiny::column(6,

                shiny::numericInput(ns("box_exp_height"), "Height (inch)", value = 5, min = 2, max = 20)

                            )

            ),

            shiny::numericInput(ns("box_exp_dpi"), "DPI", value = 300, min = 72, max = 600),



            shiny::h6("Canvas Margin (pt)"),

            shiny::fluidRow(

                            shiny::column(3, shiny::numericInput(ns("box_exp_margin_top"),    "Top",    value = 15, min = 0, max = 80)),

                            shiny::column(3, shiny::numericInput(ns("box_exp_margin_bottom"), "Bottom", value = 15, min = 0, max = 80)),

                            shiny::column(3, shiny::numericInput(ns("box_exp_margin_left"),   "Left",   value = 18, min = 0, max = 80)),

                            shiny::column(3, shiny::numericInput(ns("box_exp_margin_right"),  "Right",  value = 18, min = 0, max = 80))

            ),

            shiny::hr(),

            shiny::h5("Download"),

            shiny::fluidRow(

                            shiny::column(6,

                shiny::downloadButton(ns("box_exp_download_pdf"),

                                    "Download PDF", class = "btn-danger btn-block",

                                    icon = shiny::icon("file-pdf"))

                            ),

                            shiny::column(6,

                shiny::downloadButton(ns("box_exp_download_png"),

                                    "Download PNG", class = "btn-primary btn-block",

                                    icon = shiny::icon("file-image"))

                            )

            ),

            shiny::hr(),

            shiny::fluidRow(

                            shiny::column(6,

                shiny::selectInput(ns("box_exp_format"), "Other format",

                                    choices = c("SVG" = "svg", "TIFF" = "tiff"), selected = "svg")

                            ),

                            shiny::column(6,

                shiny::div(style = "margin-top: 22px;",

                                    shiny::downloadButton(ns("box_exp_download_other"),

                    "Download", class = "btn-default btn-block")

                )

                            )

            ),

            shiny::hr(),

            shiny::fluidRow(

                            shiny::column(6,

                shiny::actionButton(ns("box_exp_copy_code"),

                                    "Copy R Code", class = "btn-info btn-block",

                                    icon = shiny::icon("clipboard"))

                            ),

                            shiny::column(6,

                shiny::actionButton(ns("box_exp_dismiss"),

                                    "Close", class = "btn-default btn-block")

                            )

            )

                    ),

                    shiny::column(7,

            shiny::h5("Live Preview"),

            shiny::div(style = "background:#f5f5f5; border:1px solid #ddd; border-radius:4px; padding:8px; width:100%; height:520px; display:flex; align-items:center; justify-content:center; overflow:auto;",

                            shiny::plotOutput(ns("box_exp_preview")) |>

                shinycssloaders::withSpinner(type = 6, color = "#28a745")

            ),

            shiny::tags$small(style = "color: #666; display:block; margin-top:6px;",

                            "Preview is auto-scaled to fit the window while keeping the export aspect ratio (tall figures appear as vertical thumbnails, wide figures as horizontal ones).")

                    )

        )

            )

    }



    shiny::observeEvent(input$open_boxplot_image_export, {

            # Auto-extract data if not already cached (so the user doesn't need

            # to click "Export Boxplot Data" first)

            result <- current_boxplot_data()

            if (is.null(result) || is.null(result$long) || nrow(result$long) == 0) {

        data_list <- data_prep_data()

        current_gene <- current_boxplot_gene()

        if (is.null(data_list) || is.null(current_gene) || !nzchar(current_gene)) {

                    shiny::showNotification(

            "No gene selected. Click a gene in the DE volcano or gene table first.",

            type = "warning", duration = 4); return()

        }

        symbol_map  <- .rebuild_symbol_map(gsea_res, data_list$contrast_id)

        display_gene <- .get_display_symbol(current_gene, symbol_map)

        result <- tryCatch(

                    extract_boxplot_data(

            GSEAlens_res = gsea_res, contrast_id  = data_list$contrast_id,

            gene_symbol  = display_gene, expr_type = data_list$expression_type),

                    error = function(e) {

            shiny::showNotification(

                            sprintf("Failed to extract boxplot data: %s", e$message),

                            type = "error", duration = 5); NULL })

        if (is.null(result)) return()

        current_boxplot_data(result)

            }

            shiny::showModal(.build_boxplot_export_modal())

    })

    shiny::observeEvent(input$box_exp_dismiss, shiny::removeModal())



    .boxplot_export_code <- function() {

            result <- current_boxplot_data()

            if (is.null(result)) return("")

            data_list <- data_prep_data()

            if (is.null(data_list)) return("")

            lg <- if (is.null(data_list$left_group))  "Left"  else data_list$left_group

            rg <- if (is.null(data_list$right_group)) "Right" else data_list$right_group

            generate_boxplot_image_code(

        box_data_long     = result$long,

        gene_symbol       = result$gene,

        expr_type         = data_list$expression_type,

        left_group        = lg,

        right_group       = rg,

        use_zero_baseline = isTRUE(input$zero_baseline),

        margin_top    = if (is.null(input$box_exp_margin_top)    || is.na(input$box_exp_margin_top))    15 else input$box_exp_margin_top,

        margin_bottom = if (is.null(input$box_exp_margin_bottom) || is.na(input$box_exp_margin_bottom)) 15 else input$box_exp_margin_bottom,

        margin_left   = if (is.null(input$box_exp_margin_left)   || is.na(input$box_exp_margin_left))   18 else input$box_exp_margin_left,

        margin_right  = if (is.null(input$box_exp_margin_right)  || is.na(input$box_exp_margin_right))  18 else input$box_exp_margin_right,

        fig_width     = if (is.null(input$box_exp_width)         || is.na(input$box_exp_width))          5 else input$box_exp_width,

        fig_height    = if (is.null(input$box_exp_height)        || is.na(input$box_exp_height))         5 else input$box_exp_height,

        fig_dpi    = if (is.null(input$box_exp_dpi)   || is.na(input$box_exp_dpi))   300 else input$box_exp_dpi

            )

    }



    .boxplot_preview_plot <- shiny::reactive({

            code <- tryCatch(.boxplot_export_code(),

        error = function(e) {

                    shiny::showNotification(

            sprintf("[boxplot preview] code generation failed: %s", e$message),

            type = "error", duration = 8)

                    ""

        })

            if (!nzchar(code)) return(NULL)

            eval_env <- new.env(parent = globalenv())

            eval_env$p <- NULL

            eval_env$print <- base::invisible

            tryCatch(eval(parse(text = code), envir = eval_env),

                                error = function(e) {

                                    shiny::showNotification(

                                        sprintf("[boxplot preview] %s", e$message),

                                        type = "error", duration = 8)

                                    NULL

                                })

            eval_env$p

    })



    output$box_exp_preview <- shiny::renderPlot(

            {

        p <- .boxplot_preview_plot()

        shiny::req(p)

        p

            },

            res  = 72,

            width  = function() {

        d <- .vol_preview_dims(input$box_exp_width, input$box_exp_height); d$width

            },

            height = function() {

        d <- .vol_preview_dims(input$box_exp_width, input$box_exp_height); d$height

            }

    )



    .boxplot_render_to_file <- function(file, fmt) {

            code <- .boxplot_export_code()

            if (!nzchar(code)) return()

            eval_env <- new.env(parent = globalenv())

            eval_env$p <- NULL

            eval_env$print <- base::invisible

            tryCatch(eval(parse(text = code), envir = eval_env),

                                error = function(e) shiny::showNotification(

                                    sprintf("Render failed: %s", e$message), type = "error", duration = 8))

            if (is.null(eval_env$p)) return()

            ggplot2::ggsave(file, eval_env$p,

                                            width  = if (is.null(input$box_exp_width))  5 else input$box_exp_width,

                                            height = if (is.null(input$box_exp_height)) 5 else input$box_exp_height,

                                            dpi    = if (is.null(input$box_exp_dpi))   300 else input$box_exp_dpi,

                                            device = fmt)

    }



    output$box_exp_download_pdf <- shiny::downloadHandler(

            filename = function() sprintf("GSEAlens_boxplot_%s.pdf", Sys.Date()),

            content  = function(file) .boxplot_render_to_file(file, "pdf")

    )

    output$box_exp_download_png <- shiny::downloadHandler(

            filename = function() sprintf("GSEAlens_boxplot_%s.png", Sys.Date()),

            content  = function(file) .boxplot_render_to_file(file, "png")

    )

    output$box_exp_download_other <- shiny::downloadHandler(

            filename = function() sprintf("GSEAlens_boxplot_%s.%s", Sys.Date(),

                                    if (is.null(input$box_exp_format)) "svg" else input$box_exp_format),

            content  = function(file) .boxplot_render_to_file(file, input$box_exp_format)

    )



    shiny::observeEvent(input$box_exp_copy_code, {

            code <- .boxplot_export_code()

            if (!nzchar(code)) {

        shiny::showNotification("Nothing to copy.", type = "warning"); return()

            }

            ok <- tryCatch({ clipr::write_clip(code); TRUE }, error = function(e) FALSE)

            if (isTRUE(ok)) {

        shiny::showNotification("R code copied to clipboard.", type = "message")

            } else {

        shiny::showModal(shiny::modalDialog(

                    title = "Reproducible R Code (copy manually)",

                    size = "l", easyClose = TRUE,

                    shiny::pre(style = "max-height: 60vh; overflow-y: auto; font-size: 11px;", code),

                    footer = shiny::modalButton("Close")

        ))

            }

    })



    # ============================================================

    # Pathway Volcano Export Center

    # ------------------------------------------------------------

    # ggplot2 + ggsave (no kaleido/orca dependency), with a reproducible

    # R code generator (generate_volcano_code in 15_code_generator.R).

    # ============================================================



    .build_volcano_export_modal <- function() {

            shiny::modalDialog(

        title = "Export Publication Plot - Pathway Volcano",

        size = "l", footer = NULL, easyClose = TRUE,

        shiny::fluidRow(

                    shiny::column(5,

            shiny::h5("Dimensions"),

            shiny::fluidRow(

                            shiny::column(6,

                shiny::numericInput(ns("vol_exp_width"),  "Width (inch)",  value = 8, min = 2, max = 24)

                            ),

                            shiny::column(6,

                shiny::numericInput(ns("vol_exp_height"), "Height (inch)", value = 6, min = 2, max = 24)

                            )

            ),

            shiny::numericInput(ns("vol_exp_dpi"), "DPI", value = 300, min = 72, max = 600),



            shiny::h6("Canvas Margin (pt)"),

            shiny::fluidRow(

                            shiny::column(3, shiny::numericInput(ns("vol_exp_margin_top"),    "Top",    value = 15, min = 0, max = 80)),

                            shiny::column(3, shiny::numericInput(ns("vol_exp_margin_bottom"), "Bottom", value = 15, min = 0, max = 80)),

                            shiny::column(3, shiny::numericInput(ns("vol_exp_margin_left"),   "Left",   value = 18, min = 0, max = 80)),

                            shiny::column(3, shiny::numericInput(ns("vol_exp_margin_right"),  "Right",  value = 18, min = 0, max = 80))

            ),

            shiny::hr(),

            shiny::checkboxInput(ns("vol_exp_show_stats"),

                            label = "Show stats subtitle (pathways / selected / significant)",

                            value = FALSE),

            shiny::hr(),

            shiny::h5("Download"),

            shiny::fluidRow(

                            shiny::column(6,

                shiny::downloadButton(ns("vol_exp_download_pdf"),

                                    "Download PDF",

                                    class = "btn-danger btn-block",

                                    icon  = shiny::icon("file-pdf"))

                            ),

                            shiny::column(6,

                shiny::downloadButton(ns("vol_exp_download_png"),

                                    "Download PNG",

                                    class = "btn-primary btn-block",

                                    icon  = shiny::icon("file-image"))

                            )

            ),

            shiny::hr(),

            shiny::fluidRow(

                            shiny::column(6,

                shiny::selectInput(ns("vol_exp_format"), "Other format",

                                    choices = c("SVG" = "svg", "TIFF" = "tiff"),

                                    selected = "svg")

                            ),

                            shiny::column(6,

                shiny::div(style = "margin-top: 22px;",

                                    shiny::downloadButton(ns("vol_exp_download_other"),

                    "Download", class = "btn-default btn-block")

                )

                            )

            ),

            shiny::hr(),

            shiny::fluidRow(

                            shiny::column(6,

                shiny::actionButton(ns("vol_exp_copy_code"),

                                    "Copy R Code",

                                    class = "btn-info btn-block",

                                    icon = shiny::icon("clipboard"))

                            ),

                            shiny::column(6,

                shiny::actionButton(ns("vol_exp_dismiss"),

                                    "Close",

                                    class = "btn-default btn-block")

                            )

            )

                    ),

                    shiny::column(7,

            shiny::h5("Live Preview"),

            shiny::div(style = "background:#f5f5f5; border:1px solid #ddd; border-radius:4px; padding:8px; width:100%; height:520px; display:flex; align-items:center; justify-content:center; overflow:auto;",

                            shiny::plotOutput(ns("vol_exp_preview")) |>

                shinycssloaders::withSpinner(type = 6, color = "#28a745")

            ),

            shiny::tags$small(style = "color: #666; display:block; margin-top:6px;",

                            "X axis: NES | Y axis: -log10(pvalue). Preview auto-scaled to fit while keeping the export aspect ratio.")

                    )

        )

            )

    }



    volcano_export_df <- shiny::reactive({

            data_list <- data_prep_data()

            shiny::req(data_list)

            data_list$df[, c("ID", "NES", "pvalue", "p.adjust")]

    })



    shiny::observeEvent(input$open_volcano_export, {

            shiny::showModal(.build_volcano_export_modal())

    })

    shiny::observeEvent(input$vol_exp_dismiss, shiny::removeModal())



    # Live preview reactive

    # IRON FIX (2026-06-30): suppress `print(p)` inside the eval'd code so

    # it does not open a separate windows()/pdf() device and conflict with

    # renderPlot's own device. Also surface errors as notifications and add

    # NA guards to width/height/res (numericInput returns NA on cleared).

    #

    # Preview device matching export physical size (2026-07-06 v3):
    # Shiny's renderPlot does NOT accept res as a function (only width/height
    # can be functions). So we fix res=72 and compute pixel dimensions as
    # inch * 72, which makes the device's physical size = width_px/72 = w_in
    # inches, matching the export. A pixel cap protects memory for large
    # figures (the cap downscales pixels but physical inches are preserved
    # via the cap / res relationship).
    .vol_preview_dims <- function(w_in, h_in, res = 72, max_px = 1600) {

            w_in <- suppressWarnings(as.numeric(w_in[1]))

            h_in <- suppressWarnings(as.numeric(h_in[1]))

            if (length(w_in) == 0 || is.na(w_in) || w_in <= 0) w_in <- 8

            if (length(h_in) == 0 || is.na(h_in) || h_in <= 0) h_in <- 6

            scale <- min(1, max_px / (max(w_in, h_in) * res))

            list(width  = round(w_in * res * scale),

                        height = round(h_in * res * scale))

    }

    .volcano_preview_plot <- shiny::reactive({

            code <- tryCatch(.volcano_export_code(),

        error = function(e) {

                    shiny::showNotification(

            sprintf("[vol preview] code generation failed: %s", e$message),

            type = "error", duration = 8)

                    ""

        })

            if (!nzchar(code)) return(NULL)

            eval_env <- new.env(parent = globalenv())

            eval_env$p <- NULL

            eval_env$print <- base::invisible
            eval_env$gsea_res <- gsea_res

            tryCatch(eval(parse(text = code), envir = eval_env),

                                error = function(e) {

                                    shiny::showNotification(

                                        sprintf("[vol preview] %s", e$message),

                                        type = "error", duration = 8)

                                    NULL

                                })

            eval_env$p

    })

    output$vol_exp_preview <- shiny::renderPlot(

            {

        p <- .volcano_preview_plot()

        shiny::req(p)

        p

            },

            res  = 72,

            width  = function() {

        d <- .vol_preview_dims(input$vol_exp_width, input$vol_exp_height); d$width

            },

            height = function() {

        d <- .vol_preview_dims(input$vol_exp_width, input$vol_exp_height); d$height

            }

    )



    .volcano_export_code <- function() {

            data_list <- data_prep_data()

            lg <- if (is.null(data_list$left_group))  "Left"  else data_list$left_group

            rg <- if (is.null(data_list$right_group)) "Right" else data_list$right_group

            sel_ids <- selected_pathway_ids()

            n_sel <- length(sel_ids)

            show_stats <- isTRUE(input$vol_exp_show_stats)

            generate_volcano_code(

                            gsea_res_var = "gsea_res",

                            contrast_id  = data_list$contrast_id,

                            left_group = lg, right_group = rg,

                            n_selected = n_sel,

                            selected_ids = sel_ids,

                            show_annotations = show_stats,

                            target_collection = if (is.null(data_list$collections)) "ALL" else data_list$collections,

                            margin_top    = if (is.null(input$vol_exp_margin_top)    || is.na(input$vol_exp_margin_top))    15 else input$vol_exp_margin_top,

                            margin_bottom = if (is.null(input$vol_exp_margin_bottom) || is.na(input$vol_exp_margin_bottom)) 15 else input$vol_exp_margin_bottom,

                            margin_left   = if (is.null(input$vol_exp_margin_left)   || is.na(input$vol_exp_margin_left))   18 else input$vol_exp_margin_left,

                            margin_right  = if (is.null(input$vol_exp_margin_right)  || is.na(input$vol_exp_margin_right))  18 else input$vol_exp_margin_right,

                            fig_width     = if (is.null(input$vol_exp_width)         || is.na(input$vol_exp_width))          8 else input$vol_exp_width,

                            fig_height    = if (is.null(input$vol_exp_height)        || is.na(input$vol_exp_height))         6 else input$vol_exp_height,
                            fig_dpi    = if (is.null(input$vol_exp_dpi)   || is.na(input$vol_exp_dpi))   300 else input$vol_exp_dpi)

    }



    .volcano_render_to_file <- function(file, fmt) {

            code <- .volcano_export_code()

            if (!nzchar(code)) return()

            # parent=globalenv() so lexical lookup resolves attached packages.

            eval_env <- new.env(parent = globalenv())

            eval_env$p <- NULL

            eval_env$print <- base::invisible   # suppress popup device; ggsave draws itself
            eval_env$gsea_res <- gsea_res

            tryCatch(eval(parse(text = code), envir = eval_env),

                                error = function(e) shiny::showNotification(

                                    sprintf("Render failed: %s", e$message), type = "error", duration = 8))

            if (is.null(eval_env$p)) return()

            ggplot2::ggsave(file, eval_env$p,

                                            width  = if (is.null(input$vol_exp_width))  8 else input$vol_exp_width,

                                            height = if (is.null(input$vol_exp_height)) 6 else input$vol_exp_height,

                                            dpi    = if (is.null(input$vol_exp_dpi))   300 else input$vol_exp_dpi,

                                            device = fmt)

    }



    output$vol_exp_download_pdf <- shiny::downloadHandler(

            filename = function() sprintf("GSEAlens_volcano_%s.pdf", Sys.Date()),

            content  = function(file) .volcano_render_to_file(file, "pdf")

    )



    output$vol_exp_download_png <- shiny::downloadHandler(

            filename = function() sprintf("GSEAlens_volcano_%s.png", Sys.Date()),

            content  = function(file) .volcano_render_to_file(file, "png")

    )



    output$vol_exp_download_other <- shiny::downloadHandler(

            filename = function() sprintf("GSEAlens_volcano_%s.%s", Sys.Date(),

                                    if (is.null(input$vol_exp_format)) "svg" else input$vol_exp_format),

            content  = function(file) .volcano_render_to_file(file, input$vol_exp_format)

    )



    shiny::observeEvent(input$vol_exp_copy_code, {

            code <- .volcano_export_code()

            ok <- tryCatch({ clipr::write_clip(code); TRUE }, error = function(e) FALSE)

            if (isTRUE(ok)) {

        shiny::showNotification("R code copied to clipboard.", type = "message")

            } else {

        shiny::showModal(shiny::modalDialog(

                    title = "Reproducible R Code (copy manually)",

                    size = "l", easyClose = TRUE,

                    shiny::pre(style = "max-height: 60vh; overflow-y: auto; font-size: 11px;", code),

                    footer = shiny::modalButton("Close")

        ))

            }

    })



    # ============================================================

    # DE Volcano Export Center (gene-level)

    # ------------------------------------------------------------

    # Mirrors the Pathway Volcano export but uses generate_de_volcano_code

    # (six-category classification: both/user/pathway/up/down/ns).

    # ============================================================

    .build_de_volcano_export_modal <- function() {

            shiny::modalDialog(

        title = "Export Publication Plot - DE Volcano",

        size = "l", footer = NULL, easyClose = TRUE,

        shiny::fluidRow(

                    shiny::column(5,

            shiny::h5("Dimensions"),

            shiny::fluidRow(

                            shiny::column(6,

                shiny::numericInput(ns("de_exp_width"),  "Width (inch)",  value = 8, min = 2, max = 24)

                            ),

                            shiny::column(6,

                shiny::numericInput(ns("de_exp_height"), "Height (inch)", value = 6, min = 2, max = 24)

                            )

            ),

            shiny::numericInput(ns("de_exp_dpi"), "DPI", value = 300, min = 72, max = 600),



            shiny::h6("Canvas Margin (pt)"),

            shiny::fluidRow(

                            shiny::column(3, shiny::numericInput(ns("de_exp_margin_top"),    "Top",    value = 15, min = 0, max = 80)),

                            shiny::column(3, shiny::numericInput(ns("de_exp_margin_bottom"), "Bottom", value = 15, min = 0, max = 80)),

                            shiny::column(3, shiny::numericInput(ns("de_exp_margin_left"),   "Left",   value = 18, min = 0, max = 80)),

                            shiny::column(3, shiny::numericInput(ns("de_exp_margin_right"),  "Right",  value = 18, min = 0, max = 80))

            ),

            shiny::hr(),

            shiny::h5("Classification thresholds"),

            shiny::numericInput(ns("de_exp_logfc"), "logFC threshold",

                                value = if (is.null(input$volcano_logfc_thresh)) 1

                                        else input$volcano_logfc_thresh,

                                min = 0, max = 22, step = 0.5),

            shiny::numericInput(ns("de_exp_pval"), "P-value threshold",

                                value = if (is.null(input$volcano_pval_thresh)) 0.05

                                        else input$volcano_pval_thresh,

                                min = 0.001, max = 1, step = 0.01),

            shiny::hr(),

            shiny::checkboxInput(ns("de_exp_show_stats"),

                            label = "Show stats + direction banners (Up/Down | High-in)",

                            value = FALSE),

            shiny::hr(),

            shiny::h5("Download"),

            shiny::fluidRow(

                            shiny::column(6,

                shiny::downloadButton(ns("de_exp_download_pdf"),

                                    "Download PDF", class = "btn-danger btn-block",

                                    icon = shiny::icon("file-pdf"))

                            ),

                            shiny::column(6,

                shiny::downloadButton(ns("de_exp_download_png"),

                                    "Download PNG", class = "btn-primary btn-block",

                                    icon = shiny::icon("file-image"))

                            )

            ),

            shiny::hr(),

            shiny::fluidRow(

                            shiny::column(6,

                shiny::selectInput(ns("de_exp_format"), "Other format",

                                    choices = c("SVG" = "svg", "TIFF" = "tiff"), selected = "svg")

                            ),

                            shiny::column(6,

                shiny::div(style = "margin-top: 22px;",

                                    shiny::downloadButton(ns("de_exp_download_other"),

                    "Download", class = "btn-default btn-block")

                )

                            )

            ),

            shiny::hr(),

            shiny::fluidRow(

                            shiny::column(6,

                shiny::actionButton(ns("de_exp_copy_code"),

                                    "Copy R Code", class = "btn-info btn-block",

                                    icon = shiny::icon("clipboard"))

                            ),

                            shiny::column(6,

                shiny::actionButton(ns("de_exp_dismiss"),

                                    "Close", class = "btn-default btn-block")

                            )

            )

                    ),

                    shiny::column(7,

            shiny::h5("Live Preview"),

            shiny::div(style = "background:#f5f5f5; border:1px solid #ddd; border-radius:4px; padding:8px; width:100%; height:520px; display:flex; align-items:center; justify-content:center; overflow:auto;",

                            shiny::plotOutput(ns("de_exp_preview")) |>

                shinycssloaders::withSpinner(type = 6, color = "#28a745")

            ),

            shiny::tags$small(style = "color: #666; display:block; margin-top:6px;",

                            "Six categories: Both (purple) / Selected (green) / Pathway (orange) / Up (red) / Down (blue) / NS (gray). Preview auto-scaled to fit while keeping the export aspect ratio.")

                    )

        )

            )

    }



    shiny::observeEvent(input$open_de_volcano_export, {

            data_list <- data_prep_data()

            de_df <- tryCatch(get_de_table(gsea_res, data_list$contrast_id),

                        error = function(e) NULL)

            if (is.null(de_df) || nrow(de_df) == 0) {

        shiny::showNotification("No DE data to export.", type = "warning"); return()

            }

            shiny::showModal(.build_de_volcano_export_modal())

    })

    shiny::observeEvent(input$de_exp_dismiss, shiny::removeModal())



    .de_volcano_export_df <- shiny::reactive({

            data_list <- data_prep_data()

            shiny::req(data_list)

            de_df <- tryCatch(get_de_table(gsea_res, data_list$contrast_id),

                        error = function(e) NULL)

            shiny::req(de_df)

            if (!"logFC" %in% colnames(de_df)) de_df$logFC <- de_df$log2FoldChange

            if (!"pvalue" %in% colnames(de_df)) de_df$pvalue <- de_df$p.value

            if (!"padj" %in% colnames(de_df)) de_df$padj <- de_df$p.adjust

            de_df <- de_df[!is.na(de_df$logFC) & !is.na(de_df$pvalue), ]

            de_df

    })



    .de_volcano_export_code <- function() {

            de_df <- .de_volcano_export_df()

            if (is.null(de_df) || nrow(de_df) == 0) return("")

            data_list <- data_prep_data()

            if (is.null(data_list)) return("")

            lg <- if (is.null(data_list$left_group))  "Left"  else data_list$left_group

            rg <- if (is.null(data_list$right_group)) "Right" else data_list$right_group

            user_genes <- toupper(highlight_genes_reactive())

            pathway_genes <- toupper(selected_pathway_genes())

            logfc_t <- if (is.null(input$de_exp_logfc)) 1 else input$de_exp_logfc

            pval_t  <- if (is.null(input$de_exp_pval))  0.05 else input$de_exp_pval

            show_stats <- isTRUE(input$de_exp_show_stats)

            generate_de_volcano_code(

        gsea_res_var  = "gsea_res",

        contrast_id   = data_list$contrast_id,

        user_genes    = user_genes,

        pathway_genes = pathway_genes,

        logfc_thresh  = logfc_t,

        pval_thresh   = pval_t,

        left_group    = lg, right_group = rg,

        show_annotations = show_stats,

        margin_top    = if (is.null(input$de_exp_margin_top)    || is.na(input$de_exp_margin_top))    15 else input$de_exp_margin_top,

        margin_bottom = if (is.null(input$de_exp_margin_bottom) || is.na(input$de_exp_margin_bottom)) 15 else input$de_exp_margin_bottom,

        margin_left   = if (is.null(input$de_exp_margin_left)   || is.na(input$de_exp_margin_left))   18 else input$de_exp_margin_left,

        margin_right  = if (is.null(input$de_exp_margin_right)  || is.na(input$de_exp_margin_right))  18 else input$de_exp_margin_right,

        fig_width     = if (is.null(input$de_exp_width)         || is.na(input$de_exp_width))          8 else input$de_exp_width,

        fig_height    = if (is.null(input$de_exp_height)        || is.na(input$de_exp_height))         6 else input$de_exp_height,

        fig_dpi    = if (is.null(input$de_exp_dpi)   || is.na(input$de_exp_dpi))   300 else input$de_exp_dpi

            )

    }



    .de_volcano_preview_plot <- shiny::reactive({

            code <- tryCatch(.de_volcano_export_code(),

        error = function(e) {

                    shiny::showNotification(

            sprintf("[de vol preview] code generation failed: %s", e$message),

            type = "error", duration = 8)

                    ""

        })

            if (!nzchar(code)) return(NULL)

            eval_env <- new.env(parent = globalenv())

            eval_env$p <- NULL

            eval_env$print <- base::invisible
            eval_env$gsea_res <- gsea_res

            tryCatch(eval(parse(text = code), envir = eval_env),

                                error = function(e) {

                                    shiny::showNotification(

                                        sprintf("[de vol preview] %s", e$message),

                                        type = "error", duration = 8)

                                    NULL

                                })

            eval_env$p

    })



    output$de_exp_preview <- shiny::renderPlot(

            {

        p <- .de_volcano_preview_plot()

        shiny::req(p)

        p

            },

            res  = 72,

            width  = function() {

        d <- .vol_preview_dims(input$de_exp_width, input$de_exp_height); d$width

            },

            height = function() {

        d <- .vol_preview_dims(input$de_exp_width, input$de_exp_height); d$height

            }

    )



    .de_volcano_render_to_file <- function(file, fmt) {

            code <- .de_volcano_export_code()

            if (!nzchar(code)) return()

            eval_env <- new.env(parent = globalenv())

            eval_env$p <- NULL

            eval_env$print <- base::invisible
            eval_env$gsea_res <- gsea_res

            tryCatch(eval(parse(text = code), envir = eval_env),

                                error = function(e) shiny::showNotification(

                                    sprintf("Render failed: %s", e$message), type = "error", duration = 8))

            if (is.null(eval_env$p)) return()

            ggplot2::ggsave(file, eval_env$p,

                                            width  = if (is.null(input$de_exp_width))  8 else input$de_exp_width,

                                            height = if (is.null(input$de_exp_height)) 6 else input$de_exp_height,

                                            dpi    = if (is.null(input$de_exp_dpi))   300 else input$de_exp_dpi,

                                            device = fmt)

    }



    output$de_exp_download_pdf <- shiny::downloadHandler(

            filename = function() sprintf("GSEAlens_de_volcano_%s.pdf", Sys.Date()),

            content  = function(file) .de_volcano_render_to_file(file, "pdf")

    )

    output$de_exp_download_png <- shiny::downloadHandler(

            filename = function() sprintf("GSEAlens_de_volcano_%s.png", Sys.Date()),

            content  = function(file) .de_volcano_render_to_file(file, "png")

    )

    output$de_exp_download_other <- shiny::downloadHandler(

            filename = function() sprintf("GSEAlens_de_volcano_%s.%s", Sys.Date(),

                                    if (is.null(input$de_exp_format)) "svg" else input$de_exp_format),

            content  = function(file) .de_volcano_render_to_file(file, input$de_exp_format)

    )



    shiny::observeEvent(input$de_exp_copy_code, {

            code <- .de_volcano_export_code()

            if (!nzchar(code)) {

        shiny::showNotification("Nothing to copy.", type = "warning"); return()

            }

            ok <- tryCatch({ clipr::write_clip(code); TRUE }, error = function(e) FALSE)

            if (isTRUE(ok)) {

        shiny::showNotification("R code copied to clipboard.", type = "message")

            } else {

        shiny::showModal(shiny::modalDialog(

                    title = "Reproducible R Code (copy manually)",

                    size = "l", easyClose = TRUE,

                    shiny::pre(style = "max-height: 60vh; overflow-y: auto; font-size: 11px;", code),

                    footer = shiny::modalButton("Close")

        ))

            }

    })

    })

}

#' @title GSEAlens Global Imports

#' @description Centralized management of all external dependency imports to avoid redundant definitions

#' @keywords internal

#' @name utils-imports

NULL





# 基础 R 包导入（不需要 @importFrom，因为已在 Depends/Imports 中）





#' @importFrom grDevices dev.off png colorRampPalette

#' @importFrom graphics plot text par lines points abline legend

#' @importFrom methods is new slot slotNames

NULL



# stats 包函数（显式导入常用函数）

#' @importFrom stats p.adjust terms var na.omit setNames

#' @importFrom stats sd median quantile reorder aggregate

NULL



# utils 包函数

#' @importFrom utils head tail packageVersion data combn read.csv

NULL





# Tidyverse 核心





#' @importFrom dplyr filter arrange mutate select left_join right_join

#' @importFrom dplyr distinct desc group_by summarise ungroup bind_rows

#' @importFrom dplyr rename pull slice slice_head slice_tail if_else coalesce

#' @importFrom tidyr pivot_longer pivot_wider separate unite drop_na

#' @importFrom tibble tibble as_tibble enframe deframe rownames_to_column

#' @importFrom stringr str_split str_trim str_detect str_replace str_wrap

#' @importFrom stringr str_replace_all str_to_title str_sub str_c

NULL





# 可视化





#' @importFrom ggplot2 ggplot aes geom_point geom_line geom_col geom_boxplot

#' @importFrom ggplot2 geom_jitter geom_hline geom_vline geom_text annotate

#' @importFrom ggplot2 geom_tile geom_segment geom_rect

#' @importFrom ggplot2 scale_color_manual scale_fill_manual scale_x_continuous

#' @importFrom ggplot2 scale_y_continuous scale_size_manual scale_alpha_manual

#' @importFrom ggplot2 labs theme element_text element_blank element_rect

#' @importFrom ggplot2 theme_bw theme_minimal coord_cartesian ggsave

#' @importFrom ggplot2 facet_wrap facet_grid

NULL





# Shiny 生态系统





#' @importFrom shiny shinyApp fluidPage titlePanel sidebarLayout sidebarPanel

#' @importFrom shiny mainPanel tabsetPanel tabPanel br tags HTML icon

#' @importFrom shiny selectInput textInput actionButton observeEvent reactiveVal

#' @importFrom shiny reactive renderPlot renderText showModal modalDialog

#' @importFrom shiny updateSelectInput updateSelectizeInput invalidateLater

#' @importFrom shiny isolate moduleServer NS tagList div span hr h4

#' @importFrom shiny req validate need showNotification

#' @importFrom shinycssloaders withSpinner

NULL



#' @importFrom DT datatable renderDataTable dataTableOutput formatStyle

#' @importFrom DT styleInterval styleEqual

NULL



#' @importFrom plotly plot_ly renderPlotly plotlyOutput layout add_annotations

#' @importFrom plotly event_data ggplotly style

NULL





# 生物信息学核心





#' @importFrom limma topTable eBayes contrasts.fit makeContrasts

NULL



#' @importFrom edgeR cpm rpkm DGEList calcNormFactors

NULL



#' @importFrom DESeq2 DESeq results resultsNames counts

#' @importFrom DESeq2 vst estimateSizeFactors

NULL



#' @importFrom SummarizedExperiment assay colData rowData

NULL



#' @importFrom S4Vectors metadata metadata<-

NULL





# GSEA 相关





#' @importFrom clusterProfiler GSEA

NULL



#' @importFrom enrichplot dotplot cnetplot heatplot ridgeplot gseaplot upsetplot

NULL



#' @importFrom graphics barplot

NULL



# GseaVis 不导入具体函数，使用 :: 调用避免冲突





# 热图与可视化





#' @importFrom ComplexHeatmap Heatmap HeatmapAnnotation draw rowAnnotation

NULL



#' @importFrom circlize colorRamp2

NULL



#' @importFrom grid gpar unit grid.text

NULL





# 并行计算与工具





#' @importFrom future plan multisession sequential

#' @importFrom future.apply future_lapply

NULL



#' @importFrom htmltools tags HTML

NULL



#' @importFrom htmlwidgets saveWidget JS

NULL



#' @importFrom jsonlite fromJSON toJSON

NULL



#' @importFrom clipr write_clip

NULL



#' @importFrom msigdbr msigdbr msigdbr_collections

NULL



#' @importFrom rlang .data

#' @importFrom rlang `%||%`

NULL





# ---- Native R utilities (base packages) -------------------------------------

# `tools` ships with base R but Bioconductor requires explicit importFrom

# for any `::` access in package code. See utils-accessors.R ::file_ext use.

#' @importFrom tools file_ext

NULL





# ---- RNG encapsulation ------------------------------------------------------

# withr::with_seed / local_seed replace set.seed() (reviewer comment #4).

# Declared in Imports (not Suggests) because R/ code calls them directly.

#' @importFrom withr with_seed local_seed

NULL





# ---- GSEA scoring helper ----------------------------------------------------

# enrichit::gseaScores is the GSEA running-score core used by vis-gsea-core.R.

#' @importFrom enrichit gseaScores

NULL





# ---- Network / graph --------------------------------------------------------

# igraph powers the Pathway Relation module and Hubgene Network layouts.

#' @importFrom igraph graph_from_data_frame layout_with_fr layout_with_kk

#' @importFrom igraph layout_in_circle V vcount ecount

NULL





# ---- Interactive network rendering ------------------------------------------

# visNetwork powers the Hubgene Network shiny module.

#' @importFrom visNetwork visNetwork visNetworkOutput renderVisNetwork

#' @importFrom visNetwork visPhysics visLayout visInteraction visOptions visEvents

NULL





# ---- Shiny UI helpers -------------------------------------------------------

# shinyjs powers enable/disable of the edge-detail control in Pathway Relation.

#' @importFrom shinyjs enable disable

NULL





# ---- Progress reporting -----------------------------------------------------

# progressr powers the progressor inside batch_calc_gsea.

#' @importFrom progressr handlers progressor

NULL





utils::globalVariables(c(

    ".", "gene_symbol", "stat", "abs_stat", "ID", "NES", "Collection",

    "Combo_Name", "Display_Collection", "Enriched_In", "Expression",

    "Group", "Sample", "Subcollection", "URL", "Description", "Description.y",

    "Detail_Page", "Pathway_Link", "Rank", "setSize", "pvalue", "gs_collection",

    "gs_description", "gs_name", "gs_subcollection", "gs_url", "phase",

    "read.csv", "relative_sec", "rss_mb", "x", "y"

))

# Section: Data Accessor Utilities ----

#' @title Data Accessor Utilities
#' @description Provides unified data extraction interface, isolating underlying
#'   object structure differences. Fixes DESeq2 sample name matching issues.
#' @keywords internal
#' @name utils-accessors
NULL



# Section: Expression Matrix Accessors ----


#' @title Get Expression Matrix
#' @description Extract expression matrix from GseaEnv or GseaRes object, supporting
#'   multiple normalization methods including raw counts, CPM, logCPM, VST, FPKM,
#'   TPM, and log-normalized counts.
#' @param obj A GseaEnv or GseaRes object.
#' @param type Character string specifying the type of expression values to return.
#'   Options include: \code{"default"} (default display matrix), \code{"raw"}
#'   (raw counts), \code{"cpm"} (counts per million), \code{"logcpm"}
#'   (log2-transformed CPM), \code{"vst"} (variance stabilizing transformation,
#'   DESeq2 only), \code{"fpkm"} (fragments per kilobase per million),
#'   \code{"tpm"} (transcripts per million), and \code{"lognorm"} (log2 of
#'   normalized counts).
#' @param ... Additional arguments passed to backend-specific methods.
#' @return A numeric matrix with genes as rows and samples as columns.
#' @export
#'
#' @examples
#' # Load pre-computed inputs shipped with the package
#' data(preprocessed_limma, package = "GSEAlens")
#'
#' precomp <- preprocessed_limma
#' data(gsea_pathwaysets_toy, package = "GSEAlens")
#'
#' pathways <- gsea_pathwaysets_toy
#' gsea_env <- setup_gsea_env(
#'   fit = precomp$fit,
#'   pathway_obj = pathways,
#'   expr_data = precomp$gsea_limma_voom_data
#' )
#' expr <- get_expr_matrix(gsea_env)
#' dim(expr)
get_expr_matrix <- function(obj, type = "default", ...) {
    UseMethod("get_expr_matrix")
}

#' @export
get_expr_matrix.GseaEnv <- function(obj, type = "default", ...) {
    .get_expr_internal(obj$expr_bundle, obj$backend_info, type, ...)
}

#' @export
get_expr_matrix.GseaRes <- function(obj, type = "default", ...) {
    .get_expr_internal(obj$expr_bundle, obj$backend_info, type, ...)
}

#' @title Internal Expression Matrix Extraction Logic
#' @description Core logic for extracting and computing expression matrices.
#'   Handles backend-specific computations (DESeq2, limma-voom) and normalizes
#'   sample ordering between expression matrices and metadata.
#' @keywords internal
#'
.get_expr_internal <- function(expr_bundle, backend_info, type = "default", ...) {
    # Normalize type names
    type_normalized <- tolower(type)
    type_aliases <- c(
    "log2cpm" = "logcpm",
    "cpm" = "cpm",
    "log2tpm" = "logtpm",
    "tpm" = "tpm",
    "vst" = "vst",
    "log2fpkm" = "logfpkm",
    "fpkm" = "fpkm",
    "lognorm" = "lognorm",
    "default" = "default",
    "raw" = "raw"
    )

    if (type_normalized %in% names(type_aliases)) {
    type_normalized <- type_aliases[[type_normalized]]
    }

    backend <- backend_info$backend

    # Return default display matrix if requested
    if (type_normalized == "default" && !is.null(expr_bundle$display_expr)) {
    expr_mat <- expr_bundle$display_expr
    if (!is.null(expr_bundle$sample_meta)) {
            if (!is.null(rownames(expr_bundle$sample_meta))) {
        common_samples <- intersect(colnames(expr_mat), rownames(expr_bundle$sample_meta))
        if (length(common_samples) > 0) {
                    expr_mat <- expr_mat[, common_samples, drop = FALSE]
        }
            }
    }
    return(expr_mat)
    }

    # Retrieve raw data
    raw_counts <- expr_bundle$raw_counts
    sample_meta <- expr_bundle$sample_meta
    gene_lengths <- expr_bundle$gene_meta$length

    if (is.null(raw_counts)) {
    rlang::warn("Raw count matrix not available; cannot compute expression values dynamically.", .class = "GSEAlens_no_raw_counts")
    return(NULL)
    }

    # Compute expression values based on type and backend
    res <- switch(type_normalized,
    "raw" = raw_counts,

    # CPM (generic)
    "cpm" = {
            if (backend == "limma_voom" && !is.null(expr_bundle$dge_list)) {
        edgeR::cpm(expr_bundle$dge_list, log = FALSE)
            } else if (backend == "deseq2" && !is.null(expr_bundle$dds_obj)) {
        counts <- DESeq2::counts(expr_bundle$dds_obj, normalized = FALSE)
        t(t(counts) / colSums(counts)) * 1e6
            } else {
        t(t(raw_counts) / colSums(raw_counts)) * 1e6
            }
    },
    "logcpm" = {
            if (backend == "limma_voom" && !is.null(expr_bundle$dge_list)) {
        edgeR::cpm(expr_bundle$dge_list, log = TRUE)
            } else if (backend == "deseq2" && !is.null(expr_bundle$dds_obj)) {
        counts <- DESeq2::counts(expr_bundle$dds_obj, normalized = FALSE)
        log2(t(t(counts) / colSums(counts)) * 1e6 + 1)
            } else {
        log2(t(t(raw_counts) / colSums(raw_counts)) * 1e6 + 1)
            }
    },
    "fpkm" = {
            if (is.null(gene_lengths)) {
        rlang::warn("FPKM calculation requires gene length information (gene_meta$length); falling back to CPM.", .class = "GSEAlens_fpkm_no_length")
        t(t(raw_counts) / colSums(raw_counts)) * 1e6
            } else {
        gene_lengths_kb <- gene_lengths / 1000
        rpm <- t(t(raw_counts) / colSums(raw_counts)) * 1e6
        rpm / gene_lengths_kb
            }
    },
    "logfpkm" = {
            if (is.null(gene_lengths)) {
        rlang::warn("logFPKM calculation requires gene length information; falling back to logCPM.", .class = "GSEAlens_logfpkm_no_length")
        log2(t(t(raw_counts) / colSums(raw_counts)) * 1e6 + 1)
            } else {
        gene_lengths_kb <- gene_lengths / 1000
        rpm <- t(t(raw_counts) / colSums(raw_counts)) * 1e6
        fpkm <- rpm / gene_lengths_kb
        log2(fpkm + 1)
            }
    },

    # VST (DESeq2-exclusive)
    "vst" = {
            if (backend != "deseq2") {
        rlang::warn(sprintf("VST is a DESeq2-exclusive method; current backend is '%s', falling back to logCPM.", backend), .class = "GSEAlens_vst_wrong_backend")
        if (backend == "limma_voom" && !is.null(expr_bundle$dge_list)) {
                    edgeR::cpm(expr_bundle$dge_list, log = TRUE)
        } else {
                    log2(t(t(raw_counts) / colSums(raw_counts)) * 1e6 + 1)
        }
            } else if (!is.null(expr_bundle$vst_matrix)) {
        expr_bundle$vst_matrix
            } else if (!is.null(expr_bundle$dds_obj)) {
        tryCatch(
                    {
            SummarizedExperiment::assay(DESeq2::vst(expr_bundle$dds_obj, blind = FALSE))
                    },
                    error = function(e) {
            rlang::warn(sprintf("VST calculation failed, falling back to logCPM: %s", e$message), .class = "GSEAlens_vst_failed")
            counts <- DESeq2::counts(expr_bundle$dds_obj, normalized = FALSE)
            log2(t(t(counts) / colSums(counts)) * 1e6 + 1)
                    }
        )
            } else {
        rlang::abort("VST matrix was not pre-computed and cannot be computed from dds_obj.", .class = "GSEAlens_vst_unavailable")
            }
    },

    # Log2 normalized counts (DESeq2)
    "lognorm" = {
            if (backend == "deseq2" && !is.null(expr_bundle$dds_obj)) {
        norm_counts <- DESeq2::counts(expr_bundle$dds_obj, normalized = TRUE)
        log2(norm_counts + 1)
            } else {
        if (backend == "limma_voom" && !is.null(expr_bundle$dge_list)) {
                    edgeR::cpm(expr_bundle$dge_list, log = TRUE)
        } else {
                    log2(t(t(raw_counts) / colSums(raw_counts)) * 1e6 + 1)
        }
            }
    },

    # Unknown type
    rlang::abort(sprintf("Unsupported expression value type: %s", type), .class = "GSEAlens_bad_expr_type")
    )

    # Align expression matrix column order with sample metadata
    if (!is.null(sample_meta) && !is.null(rownames(sample_meta))) {
    common_samples <- intersect(colnames(res), rownames(sample_meta))
    if (length(common_samples) == 0) {
            rlang::warn("Expression matrix column names do not match sample metadata row names; please check sample identifiers.", .class = "GSEAlens_sample_mismatch")
    } else {
            res <- res[, common_samples, drop = FALSE]
    }
    }

    return(res)
}


# Section: Differential Expression Table Accessors ----


#' @title Get Differential Expression Table
#' @description Extract differential expression results for a specific contrast
#'   from the stored results. Automatically handles reverse contrasts (e.g.,
#'   B_vs_A) by retrieving the forward contrast and flipping the sign of logFC.
#' @param obj A GseaRes object containing differential expression results.
#' @param contrast_id Character string specifying the contrast ID (e.g., "A_vs_B").
#'   If the reverse contrast exists in storage, it will be retrieved and
#'   logFC/stat signs will be inverted.
#' @return A data frame containing at minimum: gene_symbol, logFC, and p-value
#'   columns. Additional columns (stat, pvalue, padj, t) may be present depending
#'   on the analysis backend.
#' @export
#'
#' @examples
#' # Load a pre-computed GseaRes shipped with the package
#' data(precomputed_gseares, package = "GSEAlens")
#'
#' gsea_res <- precomputed_gseares
#' de_result <- get_de_table(gsea_res, contrast_id = "untrt_vs_trt")
#' head(de_result)
get_de_table <- function(obj, contrast_id) {
    UseMethod("get_de_table")
}

#' @export
get_de_table.GseaRes <- function(obj, contrast_id) {
    if (contrast_id %in% names(obj$de_store)) {
    return(obj$de_store[[contrast_id]])
    }

    parts <- strsplit(contrast_id, "_vs_")[[1]]
    if (length(parts) == 2) {
    reverse_id <- paste(parts[2], parts[1], sep = "_vs_")

    if (reverse_id %in% names(obj$de_store)) {
            de_df <- obj$de_store[[reverse_id]]

            if ("logFC" %in% colnames(de_df)) {
        de_df$logFC <- -de_df$logFC
            }
            if ("stat" %in% colnames(de_df)) {
        de_df$stat <- -de_df$stat
            }
            if ("t" %in% colnames(de_df)) {
        de_df$t <- -de_df$t
            }

            message(sprintf(
        "Auto-mapped reverse contrast: %s -> %s (logFC sign flipped)",
        contrast_id, reverse_id
            ))
            return(de_df)
    }
    }

    rlang::abort(
    c(
            sprintf("Contrast '%s' and its reverse not found in de_store.", contrast_id),
            i = sprintf("Available: %s", paste(names(obj$de_store), collapse = ", "))
    ),
    contrast_id = contrast_id,
    .class = "GSEAlens_contrast_not_found"
    )
}


# Section: Metadata Accessors ----


#' @title Get Sample Metadata
#' @description Extract and process sample metadata, handling DFrame conversion
#'   from DESeq2 and unifying group column naming conventions across backends.
#' @param obj GseaEnv or GseaRes object
#' @return A data frame with sample metadata. Includes a standardized 'group'
#'   column derived from the target factor (inferred from dds_obj@design for
#'   DESeq2 workflows). Sample row names are preserved for downstream matching.
#' @export
#'
#' @examples
#' # Load a pre-computed GseaRes shipped with the package
#' data(precomputed_gseares, package = "GSEAlens")
#'
#' gsea_res <- precomputed_gseares
#' meta <- get_sample_meta(gsea_res)
#' head(meta)
get_sample_meta <- function(obj) {
    UseMethod("get_sample_meta")
}

#' @export
get_sample_meta.GseaEnv <- function(obj) {
    .process_sample_meta(obj$expr_bundle$sample_meta, obj$expr_bundle)
}

#' @export
get_sample_meta.GseaRes <- function(obj) {
    .process_sample_meta(obj$expr_bundle$sample_meta, obj$expr_bundle)
}

#' @title Internal Sample Metadata Processing
#' @description Automatically infers target_factor from dds_obj@design for
#'   DESeq2 workflows, handles DFrame conversion, and creates a unified
#'   'group' column for consistent downstream processing.
#' @param sample_meta Raw sample metadata (possibly DFrame).
#' @param expr_bundle Expression bundle containing raw_counts, dds_obj, and
#'   other components for row name recovery.
#' @keywords internal
#'
.process_sample_meta <- function(sample_meta, expr_bundle) {
    if (is.null(sample_meta)) {
    message("[SampleMeta] Input is NULL")
    return(NULL)
    }

    # Convert DFrame/DataFrame to data.frame
    if (inherits(sample_meta, "DFrame") || inherits(sample_meta, "DataFrame")) {
    message("[SampleMeta] Converting DFrame to data.frame")
    original_rownames <- rownames(sample_meta)
    original_ncol <- ncol(sample_meta)
    sample_meta <- as.data.frame(sample_meta, stringsAsFactors = FALSE)

    if ((is.null(rownames(sample_meta)) || all(rownames(sample_meta) == "")) &&
            !is.null(original_rownames)) {
            rownames(sample_meta) <- original_rownames
            message(sprintf(
        "[SampleMeta] Restored %d rownames after DFrame conversion",
        length(original_rownames)
            ))
    }

    if (ncol(sample_meta) != original_ncol) {
            rlang::warn(
        sprintf("Column count mismatch after conversion: %d vs %d", ncol(sample_meta), original_ncol),
        .class = "GSEAlens_sample_meta_col_mismatch"
            )
    }
    } else if (!is.data.frame(sample_meta)) {
    sample_meta <- as.data.frame(sample_meta, stringsAsFactors = FALSE)
    }

    # Recover row names if missing
    if (is.null(rownames(sample_meta)) || all(rownames(sample_meta) == "")) {
    message("[SampleMeta] Row names missing, attempting recovery...")

    if (!is.null(expr_bundle$raw_counts)) {
            if (ncol(expr_bundle$raw_counts) == nrow(sample_meta)) {
        rownames(sample_meta) <- colnames(expr_bundle$raw_counts)
        message("[SampleMeta] Recovered rownames from raw_counts column names")
            }
    }

    if ((is.null(rownames(sample_meta)) || all(rownames(sample_meta) == "")) &&
            !is.null(expr_bundle$dds_obj)) {
            dds_colnames <- colnames(expr_bundle$dds_obj)
            if (length(dds_colnames) == nrow(sample_meta)) {
        rownames(sample_meta) <- dds_colnames
        message("[SampleMeta] Recovered rownames from dds_obj")
            }
    }
    }

    target_factor <- NULL

    # Strategy A: Infer from dds_obj@design (DESeq2 workflow, highest priority)
    if (!is.null(expr_bundle$dds_obj)) {
    tryCatch(
            {
        design_formula <- DESeq2::design(expr_bundle$dds_obj)
        if (inherits(design_formula, "formula")) {
                    design_terms <- attr(terms(design_formula), "term.labels")
                    if (length(design_terms) > 0) {
            inferred_factor <- tail(design_terms, 1)
            if (inferred_factor %in% colnames(sample_meta)) {
                            target_factor <- inferred_factor
                            message(sprintf(
                "[SampleMeta] Inferred target_factor from dds_obj@design: '%s'",
                target_factor
                            ))
            }
                    }
        }
            },
            error = function(e) {
        message(sprintf("[SampleMeta] Failed to infer from dds_obj@design: %s", e$message))
            }
    )
    }

    # Strategy B: Use stored target_factor
    if (is.null(target_factor) && !is.null(expr_bundle$target_factor)) {
    if (expr_bundle$target_factor %in% colnames(sample_meta)) {
            target_factor <- expr_bundle$target_factor
            message(sprintf("[SampleMeta] Using stored target_factor: '%s'", target_factor))
    }
    }

    # Strategy C: Fallback to first valid factor
    if (is.null(target_factor)) {
    is_limma_backend <- !is.null(expr_bundle$dge_list) && is.null(expr_bundle$dds_obj)

    if (!is_limma_backend) {
            rlang::warn("Could not determine target_factor from dds_obj, using first valid factor.", .class = "GSEAlens_sample_meta_factor_fallback")
    }

    factor_cols <- names(sample_meta)[vapply(sample_meta, is.factor, logical(1))]
    valid_factors <- factor_cols[vapply(sample_meta[factor_cols], function(x) length(levels(x)) > 1, logical(1))]
    if (length(valid_factors) > 0) {
            target_factor <- valid_factors[1]
    }
    }

    # Create unified 'group' column
    if (!"group" %in% colnames(sample_meta)) {
    alt_names <- c(
            "group", "Group", "condition", "Condition",
            "treatment", "Treatment", "grp"
    )
    found_name <- NULL

    if (!is.null(target_factor) && target_factor %in% colnames(sample_meta)) {
            found_name <- target_factor
    } else {
            for (name in alt_names) {
        if (name %in% colnames(sample_meta)) {
                    found_name <- name
                    break
        }
            }
    }

    if (!is.null(found_name)) {
            sample_meta$group <- sample_meta[[found_name]]
            message(sprintf("[SampleMeta] Mapped column '%s' to 'group'", found_name))
    } else {
            if (!is.null(target_factor) && target_factor %in% colnames(sample_meta)) {
        sample_meta$group <- sample_meta[[target_factor]]
        message(sprintf("[SampleMeta] Using inferred factor '%s' as 'group'", target_factor))
            } else {
        factor_cols <- names(sample_meta)[vapply(sample_meta, is.factor, logical(1))]
        if (length(factor_cols) > 0) {
                    valid_factors <- factor_cols[vapply(sample_meta[factor_cols], function(x) length(levels(x)) > 1, logical(1))]
                    if (length(valid_factors) > 0) {
            sample_meta$group <- as.character(sample_meta[[valid_factors[1]]])
            rlang::warn(sprintf("Using first valid factor '%s' as group.", valid_factors[1]), .class = "GSEAlens_sample_meta_factor_fallback")
                    }
        }
            }
    }
    }

    # Convert numeric group codes to character labels
    if ("group" %in% colnames(sample_meta)) {
    if (is.numeric(sample_meta$group) || is.integer(sample_meta$group)) {
            message("[SampleMeta] Converting numeric group codes to character labels...")

            converted <- FALSE
            if (!is.null(expr_bundle$dds_obj)) {
        tryCatch(
                    {
            original_colData <- as.data.frame(SummarizedExperiment::colData(expr_bundle$dds_obj))
            factor_cols <- names(original_colData)[vapply(original_colData, is.factor, logical(1))]

            for (fc in factor_cols) {
                            if (length(levels(original_colData[[fc]])) > 1) {
                group_levels <- levels(original_colData[[fc]])
                sample_match <- match(rownames(sample_meta), rownames(original_colData))
                if (any(!is.na(sample_match))) {
                                    group_codes <- as.numeric(original_colData[[fc]])[sample_match]
                                    mapped_groups <- group_levels[group_codes]
                                    if (any(!is.na(mapped_groups))) {
                    sample_meta$group <- mapped_groups
                    message(sprintf("[SampleMeta] Converted using dds_obj$colData[['%s']]", fc))
                    converted <- TRUE
                    break
                                    }
                }
                            }
            }
                    },
                    error = function(e) {
            message(sprintf("[SampleMeta] Error accessing dds_obj: %s", e$message))
                    }
        )
            }

            if (!converted && !is.null(expr_bundle$dge_list)) {
        original_samples <- expr_bundle$dge_list$samples
        if ("group" %in% colnames(original_samples) && is.factor(original_samples$group)) {
                    group_levels <- levels(original_samples$group)
                    group_codes <- as.numeric(original_samples$group)
                    mapped_groups <- group_levels[group_codes]
                    names(mapped_groups) <- rownames(original_samples)
                    sample_meta$group <- mapped_groups[rownames(sample_meta)]
                    message("[SampleMeta] Converted using dge_list$group")
        }
            }

            if (is.numeric(sample_meta$group)) {
        rlang::warn("Could not convert numeric group codes to character labels.", .class = "GSEAlens_sample_meta_numeric_group")
            }
    }

    if (!is.factor(sample_meta$group)) {
            sample_meta$group <- as.factor(sample_meta$group)
    }
    }

    # Final validation and reporting
    if (is.null(rownames(sample_meta)) || all(rownames(sample_meta) == "")) {
    rlang::warn("Failed to recover rownames; sample matching will fail.", .class = "GSEAlens_sample_meta_no_rownames")
    } else {
    unique_groups <- if (!is.null(sample_meta$group)) {
            paste(unique(sample_meta$group), collapse = ", ")
    } else {
            "N/A"
    }
    message(sprintf(
            "[SampleMeta] Final: %d samples, groups: %s",
            nrow(sample_meta), unique_groups
    ))
    }

    return(sample_meta)
}


#' @title Get Contrast Registry
#' @description Retrieve the registered contrasts used for differential expression
#'   analysis, including contrast definitions and metadata.
#' @param obj A GseaEnv or GseaRes object.
#' @return The contrast registry object stored in the input object.
#' @export
#'
#' @examples
#' # Load a pre-computed GseaRes shipped with the package
#' data(precomputed_gseares, package = "GSEAlens")
#'
#' gsea_res <- precomputed_gseares
#' registry <- get_contrast_registry(gsea_res)
#' print(registry)
get_contrast_registry <- function(obj) {
    UseMethod("get_contrast_registry")
}

#' @export
get_contrast_registry.GseaEnv <- function(obj) {
    obj$contrast_registry
}

#' @export
get_contrast_registry.GseaRes <- function(obj) {
    obj$contrast_registry
}

#' @title Get Gene Set Information
#' @description Retrieve gene set collection metadata including pathway definitions
#'   and annotation dictionary.
#' @param obj A GseaEnv or GseaRes object.
#' @return An object containing geneset_info with pathway definitions and
#'   metadata dictionary.
#' @export
#'
#' @examples
#' # Load a pre-computed GseaRes shipped with the package
#' data(precomputed_gseares, package = "GSEAlens")
#'
#' gsea_res <- precomputed_gseares
#' gs_info <- get_geneset_info(gsea_res)
#' names(gs_info)
get_geneset_info <- function(obj) {
    UseMethod("get_geneset_info")
}

#' @export
get_geneset_info.GseaRes <- function(obj) {
    obj$geneset_info
}

#' @export
get_geneset_info.GseaEnv <- function(obj) {
    obj$geneset
}


# Section: Additional Data Helper Functions ----


#' @title Read Additional Data
#' @description Read and validate additional data from CSV or RDS file. Supports
#'   gzip-compressed formats (.gz). The data frame must contain an 'ID' column
#'   as the primary key for joining with pathway results.
#' @param file_path Character string specifying the file path. Supported formats:
#'   \itemize{
#'     \item \code{.csv} / \code{.csv.gz} - Comma-separated values (optionally
#'       gzip compressed)
#'     \item \code{.rds} / \code{.rds.gz} - R serialized data (optionally gzip
#'       compressed)
#'   }
#' @return A data frame with validated ID column. Rows are deduplicated by ID,
#'   keeping the first occurrence of each unique ID.
#' @export
#'
#' @examples
#' # Build a small CSV in a temp directory and read it back.
#' tmp_csv <- file.path(tempdir(), "pathway_annotations_demo.csv")
#' utils::write.csv(
#'   data.frame(
#'     ID = c("HALLMARK_HYPOXIA", "HALLMARK_INFLAMMATORY_RESPONSE"),
#'     Brief_Description = c("Hypoxia", "Inflammatory response"),
#'     stringsAsFactors = FALSE
#'   ),
#'   tmp_csv, row.names = FALSE
#' )
#'
#' add_data <- read_addition_data(tmp_csv)
#' add_data
read_addition_data <- function(file_path) {
    if (!file.exists(file_path)) {
    rlang::abort(sprintf("File not found: %s", file_path), .class = "GSEAlens_file_not_found")
    }

    ext <- tools::file_ext(file_path)

    if (grepl("rds", ext, ignore.case = TRUE)) {
    data <- readRDS(file_path)
    } else if (grepl("csv", ext, ignore.case = TRUE) || ext == "gz") {
    if (grepl("\\.gz$", file_path)) {
            con <- gzfile(file_path, "rt", encoding = "UTF-8")
            data <- tryCatch({
        read.csv(con, stringsAsFactors = FALSE, check.names = FALSE)
            }, finally = {
        close(con)
            })
    } else {
            data <- read.csv(file_path,
        stringsAsFactors = FALSE,
        check.names = FALSE, encoding = "UTF-8"
            )
    }
    } else {
    rlang::abort(
            sprintf("Unsupported file format: %s. Supported formats: .csv, .csv.gz, .rds, .rds.gz", ext),
            .class = "GSEAlens_unsupported_format"
    )
    }

    if (!is.data.frame(data)) {
    rlang::abort(sprintf("Input file must contain a data.frame, got: %s", class(data)[1]), .class = "GSEAlens_bad_file_format")
    }

    if (!"ID" %in% colnames(data)) {
    rlang::abort(
            sprintf("Data frame must contain 'ID' column as the primary key. Available columns: %s", paste(colnames(data), collapse = ", ")),
            .class = "GSEAlens_missing_id_col"
    )
    }

    data$ID <- as.character(data$ID)

    if (any(duplicated(data$ID))) {
    n_dup <- sum(duplicated(data$ID))
    message(
            "[read_addition_data] Found ", n_dup,
            " duplicate ID(s), keeping first occurrence"
    )
    data <- data[!duplicated(data$ID), ]
    }

    message(
    "[read_addition_data] Loaded: ", nrow(data), " rows x ",
    ncol(data), " columns from ", basename(file_path)
    )

    return(data)
}

#' @title Create Additional Data RDS File
#' @description Convert a CSV file to standardized gzip-compressed RDS format
#'   with validation. The output file will be saved to the specified directory
#'   with a standardized naming convention.
#' @param csv_path Character string specifying the input CSV file path.
#' @param output_dir Character string specifying the output directory path.
#'   Default: current working directory.
#' @param output_name Character string specifying the output filename.
#'   Default: \code{"addition_data_gsealens.rds"}
#' @param overwrite Logical scalar indicating whether to overwrite an existing
#'   file. Default: \code{FALSE}.
#' @return Invisible \code{TRUE} on success. The RDS file path is printed to
#'   the console.
#' @export
#'
#' @examples
#' # Build a small CSV in a temp directory, then convert to RDS.
#' tmp_csv <- file.path(tempdir(), "annotations_demo.csv")
#' utils::write.csv(
#'   data.frame(
#'     ID = c("HALLMARK_HYPOXIA", "HALLMARK_INFLAMMATORY_RESPONSE"),
#'     Brief_Description = c("Hypoxia", "Inflammatory response"),
#'     stringsAsFactors = FALSE
#'   ),
#'   tmp_csv, row.names = FALSE
#' )
#'
#' out_rds <- file.path(tempdir(), "annotations_demo.rds")
#' creat_addition_data_rdsfile(
#'   tmp_csv,
#'   output_dir = tempdir(),
#'   output_name = "annotations_demo.rds",
#'   overwrite = TRUE
#' )
#' file.exists(out_rds)
creat_addition_data_rdsfile <- function(csv_path,
                                        output_dir = getwd(),
                                        output_name = "addition_data_gsealens.rds",
                                        overwrite = FALSE) {
    if (!file.exists(csv_path)) {
    rlang::abort(sprintf("CSV file not found: %s", csv_path), .class = "GSEAlens_csv_not_found")
    }

    message("[creat_addition_data_rdsfile] Reading: ", csv_path)
    data <- read.csv(csv_path,
    stringsAsFactors = FALSE,
    check.names = FALSE, encoding = "UTF-8"
    )

    if (!is.data.frame(data)) {
    rlang::abort(sprintf("CSV content must be a data.frame, got: %s", class(data)[1]), .class = "GSEAlens_bad_csv_format")
    }

    if (!"ID" %in% colnames(data)) {
    rlang::abort(
            sprintf("CSV must contain 'ID' column as the primary key. Available columns: %s", paste(colnames(data), collapse = ", ")),
            .class = "GSEAlens_missing_id_col"
    )
    }

    data$ID <- as.character(data$ID)

    if (any(duplicated(data$ID))) {
    n_dup <- sum(duplicated(data$ID))
    message("[creat_addition_data_rdsfile] Removed ", n_dup, " duplicate ID(s)")
    data <- data[!duplicated(data$ID), ]
    }

    if (!dir.exists(output_dir)) {
    dir.create(output_dir, recursive = TRUE, showWarnings = FALSE)
    }

    output_path <- file.path(normalizePath(output_dir), output_name)

    if (file.exists(output_path)) {
    if (!overwrite) {
            rlang::abort(
        sprintf("Output file already exists: %s. Use overwrite = TRUE to replace.", output_path),
        .class = "GSEAlens_file_exists"
            )
    }
    message("[creat_addition_data_rdsfile] Overwriting existing file...")
    }

    saveRDS(data, output_path, compress = "gzip")

    message("[creat_addition_data_rdsfile] Saved: ", output_path)
    message("   Dimensions: ", nrow(data), " rows x ", ncol(data), " columns")
    message("   Compression: gzip")
    message("   File size: ", format(file.info(output_path)$size / 1024, digits = 2), " KB")

    invisible(TRUE)
}

#' @title Create Additional Data Template
#' @description Generate a template CSV file containing all pathway IDs from
#'   the GseaRes object. Users can edit this template to add custom annotations
#'   such as Brief_Description, Custom_Category, Custom_Tag, and User_Notes.
#' @param gsea_res A GseaRes object (generated by \code{batch_calc_gsea}).
#' @param output_path Character string specifying the output template file path.
#'   Default: \code{"addition_template_gsealens.csv"}.
#' @param include_cols Character vector specifying additional columns to include
#'   besides ID. Default includes standard annotation columns.
#' @return Invisible \code{TRUE} on success.
#' @export
#'
#' @examples
#' # Load pre-computed GseaRes shipped with the package and write a template
#' # CSV to a temporary file (no side effects on the user's filesystem).
#' data(precomputed_gseares, package = "GSEAlens")
#'
#' gsea_res <- precomputed_gseares
#' out_csv <- file.path(tempdir(), "addition_template_demo.csv")
#' create_addition_template(gsea_res, output_path = out_csv)
#'
#' # The template contains one row per pathway ID, ready to edit.
#' template <- utils::read.csv(out_csv)
#' utils::head(template)
create_addition_template <- function(gsea_res,
                                                                            output_path = "addition_template_gsealens.csv",
                                                                            include_cols = c(
                                                                                "Brief_Description",
                                                                                "Custom_Category",
                                                                                "Custom_Tag",
                                                                                "User_Notes"
                                                                            )) {
    if (!inherits(gsea_res, "GseaRes")) {
    rlang::abort("Input must be a GseaRes object.", .class = "GSEAlens_bad_class")
    }

    if (is.null(gsea_res$geneset_info$meta_dict)) {
    rlang::abort("GseaRes object missing geneset_info$meta_dict.", .class = "GSEAlens_missing_meta_dict")
    }

    meta_dict <- gsea_res$geneset_info$meta_dict
    all_ids <- unique(meta_dict$ID)

    template <- data.frame(
    ID = sort(all_ids),
    stringsAsFactors = FALSE
    )

    for (col in include_cols) {
    template[[col]] <- NA_character_
    }

    output_dir <- dirname(output_path)
    if (output_dir != "" && !dir.exists(output_dir)) {
    dir.create(output_dir, recursive = TRUE, showWarnings = FALSE)
    }

    utils::write.csv(template, output_path,
    row.names = FALSE, quote = TRUE,
    fileEncoding = "UTF-8"
    )

    message("[create_addition_template] Template created: ", output_path)
    message("   Contains ", nrow(template), " pathway IDs")
    message("   Columns: ", paste(colnames(template), collapse = ", "))
    message("")
    message("Instructions:")
    message("  1. Edit the CSV file to fill in annotations")
    message("  2. Convert to RDS: creat_addition_data_rdsfile('", basename(output_path), "')")
    message(
    "  3. Load in GSEAlens: launch_gsea_app(gsea_res, '",
    gsub("\\.csv$", ".rds", basename(output_path)), "')"
    )

    invisible(TRUE)
}


# Section: Gene Symbol Processing Utilities ----
# Cross-species, cross-backend gene name handling utilities
# Compatible with both limma-voom and DESeq2 pipelines

# Housekeeping gene probes for detection (uppercase for matching)
DETECTION_PROBES <- c(
    # Animal housekeeping genes
    "GAPDH", "ACTB", "ACTIN", "TUBA", "TUBB", "UBC", "UBB", "RPLP0", "RPS18",
    "RPL13A", "PPIA", "YWHAZ", "HPRT1", "B2M",
    # Plant housekeeping genes
    "ACTIN2", "ACTIN7", "TUB8", "EF1A", "UBQ10", "UBQ11",
    # Eukaryotic common
    "18S", "28S", "RPS5", "RPSA"
)


#' @title Detect Gene Symbol Column
#' @description Automatically detect the gene symbol column in gene metadata.
#'   Uses housekeeping gene probe matching for robust cross-species detection.
#' @param gene_meta A data frame containing gene metadata.
#' @return The detected column name if successful, or \code{NULL} if detection
#'   fails.
#' @keywords internal
#'
.detect_gene_symbol_column <- function(gene_meta) {
    if (is.null(gene_meta) || !is.data.frame(gene_meta) || nrow(gene_meta) == 0) {
    return(NULL)
    }

    # Strategy 1: Probe matching (most robust)
    for (col_name in colnames(gene_meta)) {
    col_data <- gene_meta[[col_name]]
    if (!is.character(col_data) && !is.factor(col_data)) next
    if (length(col_data) == 0) next

    col_upper <- toupper(as.character(col_data))
    matched <- sum(col_upper %in% DETECTION_PROBES, na.rm = TRUE)

    if (matched >= 2) {
            message(sprintf(
        "[GeneDetector] Detected gene column '%s' by probe matching (matched %d housekeeping genes)",
        col_name, matched
            ))
            return(col_name)
    }
    }

    # Strategy 2: Fallback to common column names
    fallback_names <- c(
    "SYMBOL", "symbol", "Gene", "gene_name", "gene",
    "GeneSymbol", "gene_symbol", "gene_id", "GeneID",
    "Gene_Name", "geneID"
    )
    for (name in fallback_names) {
    if (name %in% colnames(gene_meta)) {
            message(sprintf("[GeneDetector] Using fallback column '%s'", name))
            return(name)
    }
    }

    rlang::warn("Failed to detect gene symbol column in gene_meta.", .class = "GSEAlens_no_symbol_col")
    return(NULL)
}


#' @title Rebuild Gene Symbol Case Mapping
#' @description Reconstruct gene symbol case mapping from differential expression
#'   results for display purposes.
#' @param gsea_res A GseaRes object.
#' @param contrast_id Character string specifying the contrast ID.
#' @return A named character vector where names are uppercase gene symbols and
#'   values are the original case. Returns \code{NULL} if mapping cannot be
#'   constructed.
#' @keywords internal
#'
.rebuild_symbol_map <- function(gsea_res, contrast_id) {
    de_df <- gsea_res$de_store[[contrast_id]]
    if (is.null(de_df) || !"gene_symbol" %in% colnames(de_df)) {
    rlang::warn(sprintf("de_store not available for contrast '%s'.", contrast_id), .class = "GSEAlens_no_de_store")
    return(NULL)
    }

    gene_syms <- de_df$gene_symbol
    gene_syms <- gene_syms[!is.na(gene_syms) & gene_syms != ""]

    if (length(gene_syms) == 0) {
    rlang::warn("No valid gene symbols found in de_store.", .class = "GSEAlens_no_symbols")
    return(NULL)
    }

    symbol_map <- setNames(gene_syms, toupper(gene_syms))
    message(sprintf(
    "[SymbolMap] Built mapping for %d genes (contrast: %s)",
    length(symbol_map), contrast_id
    ))
    return(symbol_map)
}


#' @title Get Display Gene Symbol
#' @description Retrieve the original case gene symbol based on the case mapping.
#' @param gene A gene identifier (case-insensitive).
#' @param symbol_map A case mapping vector generated by \code{.rebuild_symbol_map}.
#' @return The original case gene symbol, or the input if no mapping exists.
#' @keywords internal
#'
.get_display_symbol <- function(gene, symbol_map) {
    if (is.null(symbol_map) || is.null(gene) || is.na(gene) || gene == "") {
    return(gene)
    }
    upper_gene <- toupper(as.character(gene))
    if (upper_gene %in% names(symbol_map)) {
    return(symbol_map[[upper_gene]])
    }
    return(gene)
}


#' @title Batch Get Display Gene Symbols
#' @param genes A character vector of gene identifiers.
#' @param symbol_map A case mapping vector generated by \code{.rebuild_symbol_map}.
#' @return A character vector of gene symbols in their original case.
#' @keywords internal
#'
.get_display_symbols <- function(genes, symbol_map) {
    vapply(genes, function(g) .get_display_symbol(g, symbol_map), character(1), USE.NAMES = FALSE)
}


#' @title Convert Expression Matrix Row Names to Gene Symbols
#' @description Automatically detect and convert expression matrix row names,
#'   supporting Ensembl ID to gene symbol conversion.
#' @param expr_mat An expression matrix.
#' @param gene_meta A gene metadata data frame containing both ID and symbol columns.
#' @param gsea_res A GseaRes object (used to access backend information).
#' @return The expression matrix with updated row names.
#' @keywords internal
#'
.convert_expr_rownames <- function(expr_mat, gene_meta, gsea_res) {
    if (is.null(expr_mat) || is.null(rownames(expr_mat))) {
    return(expr_mat)
    }

    current_rownames <- rownames(expr_mat)

    # Check if row names are Ensembl IDs
    is_ensembl <- any(grepl("^ENS[A-Z]{0,3}G[0-9]+", current_rownames, ignore.case = TRUE))

    if (!is_ensembl) {
    detected_col <- .detect_gene_symbol_column(gene_meta)
    if (!is.null(detected_col)) {
            message(sprintf("[ExprConvert] Row names already appear to be gene symbols"))
    }
    return(expr_mat)
    }

    message("[ExprConvert] Detected Ensembl IDs as row names, attempting conversion...")

    gene_symbol_col <- .detect_gene_symbol_column(gene_meta)

    if (is.null(gene_symbol_col)) {
    rlang::abort(
            "Ensembl IDs detected but gene symbol column not found in gene_meta. Cannot convert row names for heatmap plotting.",
            .class = "GSEAlens_no_symbol_col_for_ensembl"
    )
    }

    gm <- gene_meta

    geneid_col <- NULL
    for (name in c("GeneID", "gene_id", "geneID", "ENSEMBL", "ensembl_id")) {
    if (name %in% colnames(gm)) {
            geneid_col <- name
            break
    }
    }

    if (is.null(geneid_col)) {
    rlang::abort(
            c(
        "Ensembl IDs detected but GeneID column not found in gene_meta.",
        i = sprintf("Available columns: %s", paste(colnames(gm), collapse = ", "))
            ),
            .class = "GSEAlens_no_geneid_col"
    )
    }

    ensembl_ids <- gm[[geneid_col]]
    gene_symbols <- gm[[gene_symbol_col]]
    names(gene_symbols) <- ensembl_ids

    gene_symbols <- gene_symbols[!duplicated(names(gene_symbols))]

    converted <- gene_symbols[current_rownames]
    valid_mask <- !is.na(converted) & converted != ""

    n_converted <- sum(valid_mask)
    n_total <- length(current_rownames)

    if (n_converted == 0) {
    rlang::abort(
            sprintf("Conversion failed: no genes matched between expr_mat rownames and gene_meta$%s", geneid_col),
            .class = "GSEAlens_ensembl_conversion_failed"
    )
    }

    rownames(expr_mat)[valid_mask] <- converted[valid_mask]

    message(sprintf(
    "[ExprConvert] Successfully converted %d/%d genes (%.1f%%)",
    n_converted, n_total, 100 * n_converted / n_total
    ))

    return(expr_mat)
}

# Section: Shiny Module - Master Table ----



#' @title Master Workspace Table UI

#' @description Interactive data table module for displaying GSEA pathway results

#'   with checkbox selection and modal integration.

#' @param id Module ID

#' @return Shiny UI tagList

#' @keywords internal

mod_master_table_ui <- function(id) {

    ns <- shiny::NS(id)

    shiny::tagList(

    shiny::tags$head(

            shiny::tags$script(shiny::HTML(sprintf("

        Shiny.addCustomMessageHandler('%s', function(message) {

                    var ids = message.ids;

                    var ns = message.ns;

                    var checkboxes = document.querySelectorAll('.joint-plot-checkbox');

                    var now = Date.now();

                    checkboxes.forEach(function(cb) {

            // Skip checkboxes the user clicked within the last 800ms to avoid

            // server write-back overwriting native browser state.

            // (fixes the issue where fast R->JS write-back resets user-checked

            //  boxes to unchecked)

            if (cb._lastUserClick && (now - cb._lastUserClick) < 800) return;

            var id = cb.getAttribute('data-id');

            var shouldBeChecked = ids.indexOf(id) !== -1;

            if (cb.checked !== shouldBeChecked) {

                            cb.checked = shouldBeChecked;

            }

                    });

        });

            ", ns("updateCheckbox")))),

            # External selection update message handler

            shiny::tags$script(shiny::HTML(sprintf("

        Shiny.addCustomMessageHandler('%s', function(message) {

                    var action = message.action;

                    var ids = message.ids || [];

                    var checkboxes = document.querySelectorAll('.joint-plot-checkbox');



                    if (action === 'set') {

            checkboxes.forEach(function(cb) {

                            var id = cb.getAttribute('data-id');

                            cb.checked = ids.includes(id);

            });

                    } else if (action === 'add') {

            checkboxes.forEach(function(cb) {

                            var id = cb.getAttribute('data-id');

                            if (ids.includes(id)) {

                cb.checked = true;

                            }

            });

                    } else if (action === 'remove') {

            checkboxes.forEach(function(cb) {

                            var id = cb.getAttribute('data-id');

                            if (ids.includes(id)) {

                cb.checked = false;

                            }

            });

                    } else if (action === 'clear') {

            checkboxes.forEach(function(cb) {

                            cb.checked = false;

            });

                    }

        });

            ", ns("externalUpdate"))))

    ),

    shiny::div(

            id = ns("table-wrapper"),

            class = "master-table-container",

            style = "width: 100%; overflow-x: auto;",

            shiny::tags$div(

        style = "margin-bottom: 10px; color: #666; font-size: 12px;",

        shiny::HTML("Drag column headers to reorder | Check 'Joint Plot' column to select pathways | Click 'Dashboard' for details")

            ),

            DT::dataTableOutput(ns("table"))

    ),

    shiny::tags$div(

            style = "display: none;",

            shiny::textInput(ns("joint_selection_store"), label = NULL, value = "")

    )

    )

}





#' @title Master Workspace Table Server

#' @description Provides data display, column display control with decoupled checkbox state.

#'   Supports merging addition_data columns into the main table.

#'   Includes external selection update capabilities for bidirectional sync.

#' @param id Module ID

#' @param data_prep Reactive data from the data preprocessing module

#' @param addition_data Optional. A data frame with pathway annotations to merge.

#' @return List containing selected_pathways, show_modal, and selection control functions

#' @keywords internal

#' @importFrom dplyr left_join

mod_master_table_server <- function(id, data_prep, addition_data = NULL) {

    shiny::moduleServer(id, function(input, output, session) {

    ns <- session$ns

    joint_selected <- shiny::reactiveVal(character(0))

    has_interaction <- shiny::reactiveVal(FALSE)





    to_safe <- function(x) gsub("'", "\\'", x, fixed = TRUE)

    to_original <- function(x) gsub("\\'", "'", x, fixed = TRUE)



    # Validate and prepare addition_data

    .validate_addition_data_internal <- function(add_data) {

            if (is.null(add_data)) {

        return(NULL)

            }

            if (!is.data.frame(add_data)) {

        warning("[MasterTable] addition_data must be a data.frame, ignoring")

        return(NULL)

            }

            if (!"ID" %in% colnames(add_data)) {

        warning("[MasterTable] addition_data must contain 'ID' column, ignoring")

        return(NULL)

            }

            add_data$ID <- as.character(add_data$ID)

            return(add_data)

    }



    addition_data_validated <- .validate_addition_data_internal(addition_data)





    # =========================================

    # Key fix: Joint plot toggle event - use identical() for boolean comparison

    # =========================================

    shiny::observeEvent(input$joint_plot_toggle, {

            toggle <- input$joint_plot_toggle

            if (is.null(toggle) || !is.list(toggle)) {

        return()

            }



            # Key fix: JavaScript passes strings "true"/"false", need to convert

            is_checked <- identical(toggle$checked, TRUE) || identical(toggle$checked, "true")



            has_interaction(TRUE) # Mark as interacted when event fires



            current <- joint_selected()



            if (is_checked) {

        # Add to selected list

        if (!(toggle$id %in% current)) {

                    joint_selected(c(current, toggle$id))

                    message(sprintf("[MasterTable] Added pathway: %s", toggle$id))

        }

            } else {

        # Remove from selected list

        new_sel <- setdiff(current, toggle$id)

        joint_selected(new_sel)

        message(sprintf("[MasterTable] Removed pathway: %s", toggle$id))

            }

    })



    # External selection update interface (for other modules to call)

    update_selection <- function(action = c("set", "add", "remove", "clear"), ids = character(0)) {

            action <- match.arg(action)

            current <- joint_selected()



            switch(action,

        "set" = {

                    has_interaction(TRUE)

                    joint_selected(unique(ids))

                    session$sendCustomMessage(type = ns("externalUpdate"), message = list(

            action = "set",

            ids = to_safe(unique(ids))

                    ))

                    message("[MasterTable] Selection set to: ", paste(ids, collapse = ", "))

        },

        "add" = {

                    has_interaction(TRUE)

                    new_sel <- union(current, ids)

                    joint_selected(unique(new_sel))

                    session$sendCustomMessage(type = ns("externalUpdate"), message = list(

            action = "add",

            ids = to_safe(unique(ids))

                    ))

                    message("[MasterTable] Added to selection: ", paste(ids, collapse = ", "))

        },

        "remove" = {

                    has_interaction(TRUE)

                    new_sel <- setdiff(current, ids)

                    joint_selected(new_sel)

                    session$sendCustomMessage(type = ns("externalUpdate"), message = list(

            action = "remove",

            ids = to_safe(unique(ids))

                    ))

                    message("[MasterTable] Removed from selection: ", paste(ids, collapse = ", "))

        },

        "clear" = {

                    has_interaction(TRUE)

                    joint_selected(character(0))

                    session$sendCustomMessage(type = ns("externalUpdate"), message = list(

            action = "clear",

            ids = character(0)

                    ))

                    message("[MasterTable] Selection cleared")

        }

            )

    }



    # =========================================

    # Table rendering

    # =========================================

    output$table <- DT::renderDataTable(

            {

        data_list <- data_prep()

        shiny::validate(shiny::need(data_list, "Waiting for data to load..."))

        shiny::validate(shiny::need(nrow(data_list$df) > 0, "No data under current filter criteria"))



        df <- data_list$df

        left_grp <- data_list$left_group

        right_grp <- data_list$right_group



        # Calculate Enriched_In

        df$Enriched_In <- ifelse(df$NES > 0, left_grp, right_grp)

        df$Enriched_In <- factor(df$Enriched_In, levels = c(left_grp, right_grp))



        # Merge Description

        if (!is.null(data_list$task_obj$meta$meta_dict)) {

                    meta_dict <- data_list$task_obj$meta$meta_dict

                    if ("Brief_Description" %in% colnames(meta_dict)) {

            desc_map <- setNames(meta_dict$Brief_Description, meta_dict$ID)

            df$Description <- desc_map[df$ID]

                    } else if ("Description" %in% colnames(meta_dict)) {

            desc_map <- setNames(meta_dict$Description, meta_dict$ID)

            df$Description <- desc_map[df$ID]

                    } else {

            df$Description <- df$ID

                    }

        } else {

                    df$Description <- df$ID

        }

        df$Description[is.na(df$Description)] <- df$ID[is.na(df$Description)]



        # Merge addition_data columns

        if (!is.null(addition_data_validated)) {

                    add_cols <- setdiff(colnames(addition_data_validated), "ID")

                    df <- dplyr::left_join(df, addition_data_validated, by = "ID", suffix = c("", "_add"))

        }



        df$NES_display <- round(df$NES, 2)



        # Get current selection state

        current_selection <- isolate(joint_selected())



        # Interactive columns

        # onclick first stamps cb._lastUserClick; the message carries a ts field

        # to prevent Shiny internal input deduplication

        df$Select_for_Plot <- sprintf(

                    '<input type="checkbox" class="joint-plot-checkbox" data-id="%s" %s onclick="this._lastUserClick=Date.now();Shiny.setInputValue(\'%s\', {id: \'%s\', checked: this.checked, ts: Date.now()}, {priority: \'event\'});"/>',

                    df$Safe_ID,

                    ifelse(df$Safe_ID %in% current_selection, 'checked="checked"', ""),

                    ns("joint_plot_toggle"),

                    df$Safe_ID

        )



        df$Detail_Btn <- sprintf(

                    '<button class="btn btn-sm btn-success" onclick="Shiny.setInputValue(\'%s\', \'%s\', {priority: \'event\'})">Dashboard</button>',

                    ns("show_modal"),

                    df$Safe_ID

        )



        # Display columns

        base_cols <- c(

                    "Rank", "Select_for_Plot", "Detail_Btn", "ID", "Enriched_In",

                    "NES_display", "pvalue", "p.adjust", "setSize", "Description"

        )



        if (!is.null(addition_data_validated)) {

                    add_cols_display <- setdiff(colnames(addition_data_validated), "ID")

                    add_cols_valid <- add_cols_display[vapply(add_cols_display, function(col) {

            !all(is.na(df[[col]]) | df[[col]] == "" | is.null(df[[col]]))

                    }, logical(1))]

                    base_cols <- c(base_cols, add_cols_valid)

        }



        display_cols <- intersect(base_cols, colnames(df))

        dt_data <- df[, display_cols, drop = FALSE]



        col_name_map <- c(

                    "Rank" = "Rank",

                    "Select_for_Plot" = "Joint Plot",

                    "Detail_Btn" = "Dashboard",

                    "ID" = "Pathway",

                    "Enriched_In" = "Enriched In",

                    "NES_display" = "NES",

                    "pvalue" = "P-value",

                    "p.adjust" = "FDR",

                    "setSize" = "Size",

                    "Description" = "Description"

        )



        for (col in display_cols) {

                    if (!col %in% names(col_name_map)) {

            col_name_map[col] <- col

                    }

        }



        names(dt_data) <- col_name_map[display_cols]



        # Column definitions

        col_defs <- list()



        if ("Select_for_Plot" %in% display_cols) {

                    idx <- which(display_cols == "Select_for_Plot") - 1

                    col_defs[[length(col_defs) + 1]] <- list(

            width = "40px", targets = idx, className = "dt-center", orderable = FALSE

                    )

        }



        if ("Detail_Btn" %in% display_cols) {

                    idx <- which(display_cols == "Detail_Btn") - 1

                    col_defs[[length(col_defs) + 1]] <- list(

            width = "60px", targets = idx, className = "dt-center", orderable = FALSE

                    )

        }



        if ("Rank" %in% display_cols) {

                    idx <- which(display_cols == "Rank") - 1

                    col_defs[[length(col_defs) + 1]] <- list(

            width = "20px", targets = idx, className = "dt-center"

                    )

        }



        if ("ID" %in% display_cols) {

                    idx <- which(display_cols == "ID") - 1

                    col_defs[[length(col_defs) + 1]] <- list(

            width = "200px", targets = idx

                    )

        }



        if ("Enriched_In" %in% display_cols) {

                    idx <- which(display_cols == "Enriched_In") - 1

                    col_defs[[length(col_defs) + 1]] <- list(

            width = "80px", targets = idx, className = "dt-center"

                    )

        }



        numeric_cols <- c("NES_display", "pvalue", "p.adjust", "setSize")

        for (numeric_col in numeric_cols) {

                    if (numeric_col %in% display_cols) {

            idx <- which(display_cols == numeric_col) - 1

            col_defs[[length(col_defs) + 1]] <- list(

                            width = "60px", targets = idx, className = "dt-center"

            )

                    }

        }



        if ("Description" %in% display_cols) {

                    idx <- which(display_cols == "Description") - 1

                    col_defs[[length(col_defs) + 1]] <- list(

            width = "800px", targets = idx, className = "description-cell"

                    )

        }



        # JS render function: scientific notation for small values

        js_pval_render <- "

        function(data, type) {

                    if (type === 'sort' || type === 'type') return data;

                    if (data === null || isNaN(data)) return '-';

                    if (Math.abs(data) < 0.001) return data.toExponential(2);

                    return data.toFixed(4);

        }

            "



        # Find P-value and FDR column indices

        pval_idx <- which(display_cols == "pvalue") - 1

        padj_idx <- which(display_cols == "p.adjust") - 1



        # Add P-value and FDR render definitions

        col_defs[[length(col_defs) + 1]] <- list(

                    targets = pval_idx,

                    render = htmlwidgets::JS(js_pval_render)

        )

        col_defs[[length(col_defs) + 1]] <- list(

                    targets = padj_idx,

                    render = htmlwidgets::JS(js_pval_render)

        )





        # Render table

        dt <- DT::datatable(

                    dt_data,

                    escape = FALSE,

                    selection = "none",

                    rownames = FALSE,

                    extensions = c("Buttons", "Scroller", "ColReorder", "FixedColumns"),

                    options = list(

            scrollX = TRUE,

            scrollY = "70vh",

            scroller = TRUE,

            deferRender = TRUE,

            pageLength = -1,

            dom = "Bfrtip",

            buttons = c("copy", "csv", "excel", "colvis"),

            colReorder = TRUE,

            columnDefs = col_defs,

            order = list(list(0, "asc")),

            autoWidth = FALSE,

            fixedColumns = list(leftColumns = 2),

            scrollCollapse = TRUE,

            language = list(emptyTable = "No data available", zeroRecords = "No matching records found")

                    )

        )



        if ("Enriched In" %in% names(dt_data)) {

                    dt <- dt |> DT::formatStyle(

            columns = "Enriched In",

            backgroundColor = DT::styleEqual(c(left_grp, right_grp), c("#fee0d2", "#deebf7")),

            color = DT::styleEqual(c(left_grp, right_grp), c("#cc212f", "#1052bd")),

            fontWeight = "bold"

                    )

        }



        if ("NES" %in% names(dt_data)) {

                    dt <- dt |> DT::formatStyle(

            columns = "NES",

            color = DT::styleInterval(0, c("#1052bd", "#cc212f")),

            fontWeight = "bold"

                    )

        }



        if ("FDR" %in% names(dt_data)) {

                    dt <- dt |> DT::formatStyle(

            columns = "FDR",

            backgroundColor = DT::styleInterval(c(0.01, 0.05, 0.25), c("#fc9272", "#fdb9a2", "#fee0d2", "transparent")),

            fontWeight = DT::styleInterval(0.05, c("bold", "normal"))

                    )

        }



        if ("P-value" %in% names(dt_data)) {

                    dt <- dt |> DT::formatStyle(

            columns = "P-value",

            backgroundColor = DT::styleInterval(c(0.01, 0.05), c("#fc9272", "#fee0d2", "transparent")),

            fontWeight = DT::styleInterval(0.05, c("bold", "normal"))

                    )

        }



        dt

            },

            server = TRUE

    )



    # Sync checkbox state

    # Use debounce(250ms) to coalesce multiple joint_selected changes from rapid

    # clicks, preventing N clicks from triggering N full DOM write-backs and

    # reducing accumulated latency.

    # Note: UI feedback from external APIs (update_selection / remove_pathways /

    # clear_selection) is still synchronized immediately by their respective

    # externalUpdate messages and is NOT affected by debounce.

    joint_selected_d <- shiny::debounce(joint_selected, 250)

    shiny::observe({

            sel <- joint_selected_d()

            session$sendCustomMessage(type = ns("updateCheckbox"), message = list(

        ids = sel, ns = ns("")

            ))

    })



    # =========================================

    # Key fix: selected_pathways reactive - ensure checkbox state is returned

    # =========================================

    selected_pathways <- shiny::reactive({

            # Return checkbox interaction state first

            if (isTRUE(has_interaction())) {

        return(to_original(joint_selected()))

            }

            # Fall back to row click

            data_list <- data_prep()

            if (is.null(data_list) || is.null(input$table_rows_selected)) {

        return(character(0))

            }

            return(data_list$df$ID[input$table_rows_selected])

    })



    show_modal_trigger <- shiny::reactive({

            input$show_modal

    })



    remove_pathways <- function(ids) {

            ids <- ids[!is.na(ids)]

            if (length(ids) == 0) {

        return()

            }

            has_interaction(TRUE)

            safe_ids <- to_safe(ids)

            new_sel <- setdiff(joint_selected(), safe_ids)

            joint_selected(new_sel)

    }



    clear_selection <- function() {

            has_interaction(TRUE)

            joint_selected(character(0))

    }



    # Return all interfaces

    return(list(

            selected_pathways = selected_pathways,

            show_modal = show_modal_trigger,

            remove_pathways = remove_pathways,

            clear_selection = clear_selection,

            update_selection = update_selection

    ))

    })

}

# Section: Multi-Plot Module ----



## Subsection: UI Component ----



#' @title Combined Pathway Plotting UI

#' @keywords internal



mod_multi_plot_ui <- function(id) {

    ns <- shiny::NS(id)

    shiny::tagList(

    shiny::div(

            style = "margin-top: 20px; padding: 15px; background-color: #f8f9fa; border-radius: 8px;",

            shiny::h4("Combined Pathway Plotting"),

            shiny::uiOutput(ns("selection_info")),

            shiny::selectizeInput(

        ns("pathway_selector"),

        label = "Selected Pathways (click x to remove):",

        choices = character(0),

        multiple = TRUE,

        options = list(plugins = list("remove_button"), placeholder = "Select pathways..."),

        width = "100%"

            ),

            shiny::actionButton(

        ns("clear_btn"),

        "Clear Selection",

        class = "btn-warning",

        style = "margin-bottom: 15px;"

            ),

            shiny::plotOutput(ns("multi_plot"), height = "800px", width = "100%"),

            shiny::hr(),

            shiny::actionButton(

        ns("open_combined_export"),

        label = "Export Publication Plot",

        class = "btn-success",

        icon  = shiny::icon("file-image"),

        style = "width: 100%;"

            )

    )

    )

}



## Subsection: Server Component ----



#' @title Combined Pathway Plotting Server

#' @keywords internal



mod_multi_plot_server <- function(id, data_prep, table_controller, gsea_res = NULL) {

    shiny::moduleServer(id, function(input, output, session) {

    ns <- session$ns



    selected_ids <- table_controller$selected_pathways

    updating <- shiny::reactiveVal(FALSE)



    # Sync selection to selectize

    shiny::observe({

            sel <- selected_ids()

            updating(TRUE)

            shiny::updateSelectizeInput(session, "pathway_selector", choices = sel, selected = sel)

            updating(FALSE)

    })



    # Listen for selectize removal

    shiny::observeEvent(input$pathway_selector,

            {

        if (updating()) {

                    return()

        }

        current <- selected_ids()

        new_sel <- input$pathway_selector

        if (is.null(new_sel)) new_sel <- character(0)



        removed <- setdiff(current, new_sel)

        if (length(removed) > 0) {

                    table_controller$remove_pathways(removed)

        }

            },

            ignoreInit = TRUE

    )



    # Clear button

    shiny::observeEvent(input$clear_btn, {

            table_controller$clear_selection()

    })



    # Render selection info

    output$selection_info <- shiny::renderUI({

            n <- length(selected_ids())

            if (n == 0) {

        shiny::tags$div(

                    style = "color: #856404; background-color: #fff3cd; padding: 10px;",

                    "Check the 'Combined Display' column in the pathway table to select pathways"

        )

            } else {

        shiny::tags$div(

                    style = "color: #155724; background-color: #d4edda; padding: 10px;",

                    sprintf("%d pathway(s) selected", n)

        )

            }

    })



    # Combined plot reactive (extracted so the export modal can reuse it).
    # Returns a patchwork object from plot_directional_gsea, or NULL on error.
    .combined_plot_error <- shiny::reactiveVal(NULL)

    .combined_plot <- shiny::reactive({
            sel <- selected_ids()
            shiny::req(sel, length(sel) > 0)

            data_list <- data_prep()
            shiny::req(data_list)

            colors <- trimws(strsplit(data_list$custom_colors, ",")[[1]])
            if (length(colors) < length(sel)) {
        colors <- rep(colors, length.out = length(sel))
            }

            .combined_plot_error(NULL)
            tryCatch(
        {
                    if (!is.null(gsea_res)) {
            contrast_id <- data_list$contrast_id
            full_task <- extract_gsea_task(gsea_res, contrast_id, "ALL")
            plot_directional_gsea(
                            directional_gsea_obj = full_task,
                            target_pathways = sel,
                            subPlot = data_list$plot_subtype,
                            curveCol = colors
            )
                    } else {
            plot_directional_gsea(
                            directional_gsea_obj = list(
                gsea_res = data_list$gsea_res,
                meta = list(
                                    left_group = data_list$left_group,
                                    right_group = data_list$right_group
                )
                            ),
                            target_pathways = sel,
                            subPlot = data_list$plot_subtype,
                            curveCol = colors
            )
                    }
        },
        error = function(e) {
                    .combined_plot_error(e$message)
                    NULL
        }
            )
    })

    # Combined plot rendering
    output$multi_plot <- shiny::renderPlot({
            p <- .combined_plot()
            if (is.null(p)) {
        err <- .combined_plot_error()
        graphics::plot(1, type = "n", axes = FALSE)
        graphics::text(1, 1,
                    sprintf("Plotting Error: %s", if (is.null(err)) "unknown" else err),
                    col = "red")
        return(invisible(NULL))
            }
            p
    })

    # ============================================================
    # Export Image Modal (mirrors Joint Canvas / module 14)
    # ============================================================

    .build_combined_export_modal <- function() {
            shiny::modalDialog(
        title = "Export Publication Plot - Combined Pathway Plotting",
        size = "l", footer = NULL, easyClose = TRUE,
        shiny::fluidRow(
                    shiny::column(5,
            shiny::h5("Dimensions"),
            shiny::fluidRow(
                            shiny::column(6,
                shiny::numericInput(ns("cp_width"),  "Width (inch)",  value = 12, min = 4, max = 40)
                            ),
                            shiny::column(6,
                shiny::numericInput(ns("cp_height"), "Height (inch)", value = 10, min = 4, max = 40)
                            )
            ),
            shiny::numericInput(ns("cp_dpi"), "DPI", value = 300, min = 72, max = 600),
            shiny::hr(),
            shiny::h5("Download"),
            shiny::fluidRow(
                            shiny::column(6,
                shiny::downloadButton(ns("cp_download_pdf"),
                                    "Download PDF",
                                    class = "btn-danger btn-block",
                                    icon  = shiny::icon("file-pdf"))
                            ),
                            shiny::column(6,
                shiny::downloadButton(ns("cp_download_png"),
                                    "Download PNG",
                                    class = "btn-primary btn-block",
                                    icon  = shiny::icon("file-image"))
                            )
            ),
            shiny::hr(),
            shiny::fluidRow(
                            shiny::column(6,
                shiny::selectInput(ns("cp_format"), "Other format",
                                    choices = c("SVG" = "svg", "TIFF" = "tiff"),
                                    selected = "svg")
                            ),
                            shiny::column(6,
                shiny::div(style = "margin-top: 22px;",
                                    shiny::downloadButton(ns("cp_download_other"),
                    "Download", class = "btn-default btn-block")
                )
                            )
            ),
            shiny::hr(),
            shiny::fluidRow(
                            shiny::column(6,
                shiny::actionButton(ns("cp_copy_code"),
                                    "Copy R Code",
                                    class = "btn-info btn-block",
                                    icon = shiny::icon("clipboard"))
                            ),
                            shiny::column(6,
                shiny::actionButton(ns("cp_dismiss"), "Close",
                                    class = "btn-default btn-block")
                            )
            )
                    ),
                    shiny::column(7,
            shiny::h5("Live Preview"),
            shiny::div(style = "background:#f5f5f5; border:1px solid #ddd; border-radius:4px; padding:8px; width:100%; height:520px; display:flex; align-items:center; justify-content:center; overflow:auto;",
                            shiny::plotOutput(ns("cp_preview")) |>
                shinycssloaders::withSpinner(type = 6, color = "#28a745")
            ),
            shiny::tags$small(style = "color: #666; display:block; margin-top:6px;",
                            "Combined Pathway Plotting uses plot_directional_gsea (patchwork). Preview auto-scaled to fit while keeping the export aspect ratio.")
                    )
        )
            )
    }

    shiny::observeEvent(input$open_combined_export, {
            if (is.null(.combined_plot())) {
        shiny::showNotification("Select pathways first.", type = "warning"); return()
            }
            shiny::showModal(.build_combined_export_modal())
    })
    shiny::observeEvent(input$cp_dismiss, shiny::removeModal())

    # Preview device matching export physical size (res fixed at 72).
    .cp_preview_dims <- function(w_in, h_in, res = 72, max_px = 1600) {
            w_in <- suppressWarnings(as.numeric(w_in[1]))
            h_in <- suppressWarnings(as.numeric(h_in[1]))
            if (length(w_in) == 0 || is.na(w_in) || w_in <= 0) w_in <- 12
            if (length(h_in) == 0 || is.na(h_in) || h_in <= 0) h_in <- 10
            scale <- min(1, max_px / (max(w_in, h_in) * res))
            list(width  = round(w_in * res * scale),
                        height = round(h_in * res * scale))
    }

    output$cp_preview <- shiny::renderPlot(
            {
        .combined_plot()
            },
            res  = 72,
            width  = function() {
        d <- .cp_preview_dims(input$cp_width, input$cp_height); d$width
            },
            height = function() {
        d <- .cp_preview_dims(input$cp_width, input$cp_height); d$height
            }
    )

    .cp_render_to_file <- function(file, fmt) {
            p <- .combined_plot()
            if (is.null(p)) {
        shiny::showNotification("No plot to export.", type = "warning"); return()
            }
            ggplot2::ggsave(
        file, p,
        width  = if (is.null(input$cp_width))  12 else input$cp_width,
        height = if (is.null(input$cp_height)) 10 else input$cp_height,
        dpi    = if (is.null(input$cp_dpi))   300 else input$cp_dpi,
        device = fmt,
        limitsize = FALSE
            )
    }

    output$cp_download_pdf <- shiny::downloadHandler(
            filename = function() sprintf("GSEAlens_combined_plot_%s.pdf", Sys.Date()),
            content  = function(file) .cp_render_to_file(file, "pdf")
    )
    output$cp_download_png <- shiny::downloadHandler(
            filename = function() sprintf("GSEAlens_combined_plot_%s.png", Sys.Date()),
            content  = function(file) .cp_render_to_file(file, "png")
    )
    output$cp_download_other <- shiny::downloadHandler(
            filename = function() sprintf("GSEAlens_combined_plot_%s.%s", Sys.Date(),
                                    if (is.null(input$cp_format)) "svg" else input$cp_format),
            content  = function(file) .cp_render_to_file(file, input$cp_format)
    )

    # ----- Code export (mirrors module 13/14 pattern) -----
    # Generates self-contained R code reproducing the current
    # plot_directional_gsea() call so the user can reproduce the figure.
    .cp_export_code <- function() {
            sel <- selected_ids()
            if (is.null(sel) || length(sel) == 0) return("")
            data_list <- data_prep()
            if (is.null(data_list)) return("")

            colors <- trimws(strsplit(data_list$custom_colors, ",")[[1]])
            if (length(colors) == 0) colors <- NULL

            lg <- if (is.null(data_list$left_group))  "Left"  else data_list$left_group
            rg <- if (is.null(data_list$right_group)) "Right" else data_list$right_group
            sub <- if (is.null(data_list$plot_subtype)) 3 else data_list$plot_subtype

            generate_combined_plot_code(
        gsea_res_var   = "gsea_res",
        contrast_id    = data_list$contrast_id,
        target_pathways = sel,
        subPlot        = sub,
        curve_colors   = colors,
        left_group     = lg,
        right_group    = rg
            )
    }

    shiny::observeEvent(input$cp_copy_code, {
            code <- .cp_export_code()
            if (!nzchar(code)) {
        shiny::showNotification("Nothing to copy.", type = "warning"); return()
            }
            ok <- tryCatch({ clipr::write_clip(code); TRUE }, error = function(e) FALSE)
            if (isTRUE(ok)) {
        shiny::showNotification("R code copied to clipboard.", type = "message")
            } else {
        # Fallback: show code in a new modal so the user can manually copy
        shiny::showModal(shiny::modalDialog(
                    title = "Reproducible R Code (copy manually)",
                    size = "l", easyClose = TRUE,
                    shiny::div(
            style = "background:#f5f5f5; padding:15px; border-radius:5px; max-height:500px; overflow:auto;",
            shiny::tags$pre(shiny::code(code),
                            style = "font-size:11px; white-space:pre-wrap; word-break:break-all;")
                    ),
                    footer = shiny::modalButton("Close")
        ))
            }
    })
    })
}

#' @title Extract Pathway Core Genes (Leading Edge)

#' @description Parse the core_enrichment field from GSEA results to extract the gene list.

#'   This is the fundamental data interface for all visualization modules in 03/04.

#' @param gsea_res_obj GseaRes object or single GSEA result (list(status=, data=, ...))

#' @param pathway_id Character, pathway ID

#' @return Character vector, core genes (uppercase normalized)

#' @export

#'

#' @examples

#' # Load a pre-computed GseaRes and extract a task

#' data(precomputed_gseares, package = "GSEAlens")
#'
#' gsea_res <- precomputed_gseares

#' task <- extract_gsea_task(gsea_res, contrast_id = "untrt_vs_trt")

#' res_df <- as.data.frame(task$gsea_res)

#' pw_id <- res_df$ID[1]

#' core_genes <- get_core_genes_for_pathway(task, pw_id)

#' head(core_genes)

get_core_genes_for_pathway <- function(gsea_res_obj, pathway_id) {

    # 统一处理输入：可能是GseaRes对象或已提取的task结果

    if (inherits(gsea_res_obj, "GseaRes")) {

    # 需要指定 contrast_id，这里简化处理，假设传入的是单个对比结果

    stop("For GseaRes object, please use extract_gsea_task to extract specific contrast first")

    }



    # 处理提取后的GseaTask对象或直接的结果列表

    if (inherits(gsea_res_obj, "GseaTask")) {

    gsea_result <- gsea_res_obj$gsea_res

    } else if (is.list(gsea_res_obj) && !is.null(gsea_res_obj$data)) {

    # 来自 results 列表的直接元素

    if (gsea_res_obj$status != "Success") {

            return(character(0))

    }

    gsea_result <- gsea_res_obj$data

    } else {

    gsea_result <- gsea_res_obj

    }



    # 从S4对象中提取结果表

    if (methods::is(gsea_result, "gseaResult")) {

    res_df <- as.data.frame(gsea_result@result)

    } else {

    return(character(0))

    }



    # 匹配通路ID

    row_idx <- which(res_df$ID == pathway_id)

    if (length(row_idx) == 0) {

    return(character(0))

    }



    # 解析 core_enrichment 字段（以 / 分隔的基因字符串）

    core_str <- as.character(res_df$core_enrichment[row_idx[1]])

    if (is.na(core_str) || core_str == "") {

    return(character(0))

    }



    # 分割并清洗

    genes <- unlist(strsplit(core_str, "/"))

    genes <- toupper(trimws(genes))

    genes <- genes[genes != ""]



    return(unique(genes))

}



# ==============================================================================

# Section ----

# ==============================================================================



## Subsection ----



### Sub-subsection ----



#' @title Batch Extract Core Genes for Multiple Pathways

#' @description Batch data preparation for 03 module Network/UpSet/Chord visualizations.

#' @param gsea_task_obj GseaTask object (single contrast)

#' @param pathway_ids Character vector, pathway ID list

#' @return Named list, names = pathway_id, values = core_genes vector

#' @export

#'



#' @examples

#' # Load a pre-computed GseaRes and extract a task

#' data(precomputed_gseares, package = "GSEAlens")
#'
#' gsea_res <- precomputed_gseares

#' task <- extract_gsea_task(gsea_res, contrast_id = "untrt_vs_trt")

#' res_df <- as.data.frame(task$gsea_res)

#' pw_ids <- head(res_df$ID, 3)

#' genes_list <- get_core_genes_list(task, pw_ids)

#' length(genes_list)

get_core_genes_list <- function(gsea_task_obj, pathway_ids) {

    if (!inherits(gsea_task_obj, "GseaTask")) {

    stop("Must pass a GseaTask object")

    }



    result <- lapply(pathway_ids, function(pid) {

    get_core_genes_for_pathway(gsea_task_obj, pid)

    })

    names(result) <- pathway_ids

    return(result)

}



# ==============================================================================

# Section ----

# ==============================================================================



## Subsection ----



### Sub-subsection ----



#' @title Calculate ORA Ratio (for DotPlot)

#' @description Calculate overlap ratio between pathway genes and differentially

#'   expressed genes.

#' @param pathway_genes Gene set of pathway (defined by TERM2GENE)

#' @param de_genes Differentially expressed genes (filtered by p-value threshold)

#' @param ratio_mode Character, "ora" (intersection/pathway size) or "leading"

#'   (intersection/DE size)

#' @return Numeric, ratio value

#' @export

#'



#' @examples

#' ratio <- calculate_overlap_ratio(c("GAPDH", "TP53", "MYC"), c("TP53", "MYC", "EGFR"))

calculate_overlap_ratio <- function(pathway_genes, de_genes, ratio_mode = c("ora", "leading")) {

    ratio_mode <- match.arg(ratio_mode)



    pathway_genes <- toupper(pathway_genes)

    de_genes <- toupper(de_genes)



    overlap <- intersect(pathway_genes, de_genes)



    if (ratio_mode == "ora") {

    # 经典 ORA：交集 / 通路总基因数

    return(length(overlap) / length(pathway_genes))

    } else {

    # Leading Edge 模式：交集 / DE 基因总数

    return(length(overlap) / length(de_genes))

    }

}



# ==============================================================================

# Section ----

# ==============================================================================



## Subsection ----



### Sub-subsection ----



#' @title Get Full Pathway Gene Set (TERM2GENE)

#' @description Extract complete gene list for a pathway from geneset_info.

#' @param gsea_res GseaRes object

#' @param pathway_id Pathway ID

#' @return Character vector

#' @export

#'



#' @examples

#' # Load a pre-computed GseaRes shipped with the package

#' data(precomputed_gseares, package = "GSEAlens")
#'
#' gsea_res <- precomputed_gseares

#' pw_id <- gsea_res$geneset_info$term2gene$gs_name[1]

#' genes <- get_term_genes(gsea_res, pw_id)

#' head(genes)

get_term_genes <- function(gsea_res, pathway_id) {

    term2gene <- gsea_res$geneset_info$term2gene

    genes <- term2gene$gene_symbol[term2gene$gs_name == pathway_id]

    return(toupper(unique(genes)))

}



# ==============================================================================

# Section ----

# ==============================================================================



## Subsection ----



### Sub-subsection ----



#' @title Safe Parameter Validation with Fallback (Global Strategy)

#' @description Unified handling of invalid parameters (e.g., ncol <= 0) to ensure

#'   03/04 module stability.

#' @param value Input value

#' @param default Default value

#' @param min_val Minimum value (inclusive)

#' @param max_val Maximum value (inclusive, optional)

#' @param param_name Parameter name (for warning messages)

#' @return Validated value

#' @export

#'



#' @examples

#' workers <- validate_param(4, default = 2, min_val = 1, max_val = 8, param_name = "workers")

validate_param <- function(value, default, min_val = 1, max_val = NULL, param_name = "parameter") {

    # 处理NULL或NA

    if (is.null(value) || (length(value) == 1 && is.na(value))) {

    message(sprintf("[Param Check] %s is NULL/NA, fallback to default: %s", param_name, default))

    return(default)

    }



    # 优先 is.numeric 判断，避免触发 as.numeric 的强制转换警告

    # 仅对非 numeric 输入（如字符串）才走 suppressWarnings 兜底分支

    val <- if (is.numeric(value)) {

    as.numeric(value[1])

    } else {

    parsed <- suppressWarnings(as.numeric(value[1]))

    if (is.na(parsed)) {

            message(sprintf("[Param Check] %s conversion failed, fallback to: %s", param_name, default))

            return(default)

    }

    parsed

    }



    # 检查范围

    if (val < min_val) {

    message(sprintf("[Param Check] %s=%s < min(%s), fallback to: %s", param_name, val, min_val, default))

    return(default)

    }



    if (!is.null(max_val) && val > max_val) {

    message(sprintf("[Param Check] %s=%s > max(%s), capped to: %s", param_name, val, max_val, max_val))

    return(max_val)

    }



    return(as.integer(val))

}





# ==============================================================================

# Section ----

# ==============================================================================



## Subsection ----



### Sub-subsection ----



#' @title Build Pathway Similarity Network Edges (Safe Version)

#' @description Safely construct edge list for pathway network based on Jaccard

#'   similarity of core genes. Includes defensive checks for edge cases.

#'   Now includes Overlap Coefficient and Dice Coefficient for comprehensive

#'   similarity assessment.

#' @param core_genes_list Named list: pathway_id -> core_gene_vector

#' @param min_shared_genes Minimum number of shared genes between pathways to form an edge

#' @return Data frame with columns: from, to, shared, weight (Jaccard index),

#'   overlap_coef, dice_coef, shared_genes

#' @export

#'



#' @examples

#' # Build edges from two pathways sharing 2 core genes

#' core_genes <- list(

#'   PATHWAY_A = c("GENE_A", "GENE_B", "GENE_C"),

#'   PATHWAY_B = c("GENE_B", "GENE_C", "GENE_D")

#' )

#' edges <- build_edge_list_safely(core_genes, min_shared_genes = 2)

#' print(edges)

build_edge_list_safely <- function(core_genes_list, min_shared_genes = 2) {

    # ==== 防御性检查 ====



    # 防御1：检查输入是否为 NULL 或空列表

    if (is.null(core_genes_list) || length(core_genes_list) == 0) {

    message("[build_edge_list_safely] Input core_genes_list is NULL or empty")

    return(NULL)

    }



    # 防御2：移除空的核心基因向量

    valid_idx <- which(vapply(core_genes_list, function(x) {

    length(x) > 0 && !all(is.na(x))

    }, logical(1)))



    if (length(valid_idx) == 0) {

    message("[build_edge_list_safely] All pathways have empty core genes")

    return(NULL)

    }



    core_genes_list <- core_genes_list[valid_idx]



    pathway_ids <- names(core_genes_list)

    n <- length(pathway_ids)



    # 防御3：如果只有 0 或 1 个 pathway，无法形成边

    if (n <= 1) {

    message(sprintf("[build_edge_list_safely] Only %d pathway(s), cannot form edges", n))

    return(NULL)

    }



    # 防御4：如果 min_shared_genes <= 0，设置默认值

    if (is.null(min_shared_genes) || min_shared_genes < 1) {

    message("[build_edge_list_safely] min_shared_genes must be >= 1, using default 2")

    min_shared_genes <- 2

    }



    # ==== 边构建 ====



    # 预分配边存储（使用列表避免动态扩展）

    edges_list <- vector("list", length = n * (n - 1) / 2)

    edge_count <- 0



    # 双层循环计算 Jaccard 相似度

    for (i in seq_len(n - 1)) {

    for (j in (i + 1):n) {

            p1 <- pathway_ids[i]

            p2 <- pathway_ids[j]



            genes1 <- core_genes_list[[p1]]

            genes2 <- core_genes_list[[p2]]



            # 跳过任何包含 NA 的基因向量

            if (any(is.na(genes1)) || any(is.na(genes2))) next



            # 大小写不敏感的交集/并集计算

            genes1_upper <- toupper(as.character(genes1))

            genes2_upper <- toupper(as.character(genes2))



            # 计算共享基因数

            shared_count <- length(intersect(genes1_upper, genes2_upper))



            # 检查是否满足最小共享阈值

            if (shared_count < min_shared_genes) next



            # 计算 Jaccard 相似度

            union_count <- length(union(genes1_upper, genes2_upper))

            jaccard <- if (union_count > 0) shared_count / union_count else 0



            # 计算 Overlap Coefficient (Simpson Index)

            # 公式：|A∩B| / min(|A|, |B|)

            # 适合检测子集关系

            min_size <- min(length(genes1_upper), length(genes2_upper))

            overlap_coef <- if (min_size > 0) shared_count / min_size else 0



            # 计算 Dice Coefficient

            # 公式： 2|A∩B| / (|A| + |B|)

            # 对交集更敏感，范围 [0, 1]

            sum_size <- length(genes1_upper) + length(genes2_upper)

            dice_coef <- if (sum_size > 0) (2 * shared_count) / sum_size else 0



            # 获取共享基因列表

            shared_genes_list <- intersect(genes1_upper, genes2_upper)



            edge_count <- edge_count + 1

            edges_list[[edge_count]] <- data.frame(

        from = p1,

        to = p2,

        shared = shared_count,

        weight = jaccard, # Jaccard index (primary weight)

        overlap_coef = overlap_coef, # Overlap coefficient

        dice_coef = dice_coef, # Dice coefficient

        shared_genes = I(list(shared_genes_list)),

        stringsAsFactors = FALSE

            )

    }

    }



    # ==== 返回结果 ====



    if (edge_count == 0) {

    message(

            "[build_edge_list_safely] No edges formed with current threshold (min_shared=",

            min_shared_genes, ")"

    )

    return(NULL)

    }



    # 合并边列表

    edge_df <- do.call(rbind, edges_list[seq_len(edge_count)])

    rownames(edge_df) <- NULL



    message(sprintf(

    "[build_edge_list_safely] Built %d edges from %d pathways",

    nrow(edge_df), n

    ))



    return(edge_df)

}

# Tests for validate_param()

# Type: pure function (no dependencies)

# Coverage: NULL/NA/non-numeric fallback; numeric string coercion;

#           min/max boundary; integer return type; vector input; NULL max_val



test_that("validate_param returns default for NULL", {

  expect_equal(validate_param(NULL, default = 5), 5)

})



test_that("validate_param returns default for NA", {

  expect_equal(validate_param(NA, default = 5), 5)

})



test_that("validate_param returns default for non-numeric string", {

  expect_equal(validate_param("abc", default = 5), 5)

})



test_that("validate_param coerces numeric string to integer", {

  expect_equal(validate_param("4", default = 1, min_val = 1, max_val = 8), 4L)

})



test_that("validate_param returns default when value below min_val", {

  expect_equal(validate_param(0, default = 2, min_val = 1), 2)

})



test_that("validate_param caps to max_val when value above", {

  expect_equal(validate_param(100, default = 2, min_val = 1, max_val = 8), 8)

})



test_that("validate_param returns integer type for valid numeric input", {

  result <- validate_param(4, default = 2, min_val = 1, max_val = 8)

  expect_type(result, "integer")

})



test_that("validate_param accepts value exactly at min_val boundary", {

  expect_equal(validate_param(1, default = 2, min_val = 1), 1L)

})



test_that("validate_param accepts value exactly at max_val boundary", {

  expect_equal(validate_param(8, default = 2, min_val = 1, max_val = 8), 8L)

})



test_that("validate_param honors NULL max_val (no upper bound)", {

  expect_equal(validate_param(1000, default = 2, min_val = 1, max_val = NULL), 1000L)

})



test_that("validate_param uses first element of vector input", {

  expect_equal(validate_param(c(3, 99, 99), default = 2, min_val = 1, max_val = 8), 3L)

})

# Tests for build_edge_list_safely()

# Type: pure function (no dependencies)

# Coverage: NULL/empty/single inputs; threshold gating; three similarity

#           coefficients (Jaccard / Overlap / Dice); case-insensitivity;

#           NA gene handling (BUG-1 FIX: skipped, not silently dropped);

#           invalid min_shared_genes fallback; multi-pathway topology

# Also serves as regression guard for ISSUE-R2 (plyr -> dplyr migration).

#

# Naming convention for fixtures:

#   PATHWAY_A, PATHWAY_B, PATHWAY_C : dummy pathway identifiers

#   GENE_A, GENE_B, GENE_C, ...      : dummy gene symbols belonging to pathways

#   The exact letters carry no biology; they only encode "which gene is shared

#   with which other pathway" so that expected Jaccard / Overlap / Dice values

#   are easy to compute by hand from the test comments.



test_that("build_edge_list_safely returns NULL for NULL input", {

  expect_null(build_edge_list_safely(NULL))

})



test_that("build_edge_list_safely returns NULL for empty list", {

  expect_null(build_edge_list_safely(list()))

})



test_that("build_edge_list_safely returns NULL for single pathway", {

  cgl <- list(PATHWAY_A = c("GENE_A", "GENE_B", "GENE_C"))

  expect_null(build_edge_list_safely(cgl))

})



test_that("build_edge_list_safely returns NULL when no pair meets threshold", {

  cgl <- list(

    PATHWAY_A = c("GENE_A", "GENE_B"),

    PATHWAY_B = c("GENE_X", "GENE_Y")  # no overlap with PATHWAY_A

  )

  edges <- build_edge_list_safely(cgl, min_shared_genes = 2)

  expect_null(edges)

})



test_that("build_edge_list_safely forms edge when shared >= threshold", {

  cgl <- list(

    PATHWAY_A = c("GENE_A", "GENE_B", "GENE_C"),

    PATHWAY_B = c("GENE_B", "GENE_C", "GENE_D")

  )

  edges <- build_edge_list_safely(cgl, min_shared_genes = 2)

  expect_s3_class(edges, "data.frame")

  expect_equal(nrow(edges), 1L)

  # shared = {GENE_B, GENE_C} = 2, union = {GENE_A, GENE_B, GENE_C, GENE_D} = 4

  # Jaccard = 2/4 = 0.5

  expect_equal(edges$shared[1], 2L)

  expect_equal(edges$weight[1], 0.5)

  # Overlap Coefficient = shared / min(|A|, |B|) = 2 / 3

  expect_equal(edges$overlap_coef[1], 2 / 3)

  # Dice = 2*shared / (|A| + |B|) = 4 / 6

  expect_equal(edges$dice_coef[1], 4 / 6)

})



test_that("build_edge_list_safely is case-insensitive", {

  cgl <- list(

    PATHWAY_A = c("GENE_A", "GENE_B"),

    PATHWAY_B = c("gene_a", "gene_b")  # lowercase input

  )

  edges <- build_edge_list_safely(cgl, min_shared_genes = 2)

  expect_equal(nrow(edges), 1L)

  expect_equal(edges$shared[1], 2L)

})



# ---- BUG-1 FIX: NA genes cause the entire pair to be SKIPPED, not silently dropped ----

# Implementation: `if (any(is.na(genes1)) || any(is.na(genes2))) next`

# Therefore a 2-pathway input where one contains NA -> no edges -> NULL.

test_that("build_edge_list_safely skips pairs containing any NA gene", {

  cgl <- list(

    PATHWAY_A = c("GENE_A", NA, "GENE_B"),   # contains NA -> any(is.na(.)) is TRUE

    PATHWAY_B = c("GENE_A", "GENE_B", "GENE_C")

  )

  expect_null(build_edge_list_safely(cgl, min_shared_genes = 2))

})



test_that("build_edge_list_safely still builds edges from clean pairs when others have NA", {

  cgl <- list(

    PATHWAY_A = c("GENE_A", "GENE_B", "GENE_C"),     # clean

    PATHWAY_B = c("GENE_A", "GENE_B", "GENE_D"),     # clean, shares {GENE_A, GENE_B} with A

    PATHWAY_C = c("GENE_A", NA, "GENE_B")            # contains NA -> skipped in all pairs involving C

  )

  edges <- build_edge_list_safely(cgl, min_shared_genes = 2)

  expect_s3_class(edges, "data.frame")

  expect_equal(nrow(edges), 1L)   # only PATHWAY_A - PATHWAY_B edge survives

})



test_that("build_edge_list_safely falls back to default when min_shared_genes invalid", {

  cgl <- list(

    PATHWAY_A = c("GENE_A", "GENE_B", "GENE_C"),

    PATHWAY_B = c("GENE_A", "GENE_B", "GENE_D")

  )

  # min_shared_genes = 0 should fallback to default 2

  edges <- build_edge_list_safely(cgl, min_shared_genes = 0)

  expect_s3_class(edges, "data.frame")

  expect_equal(nrow(edges), 1L)

})



test_that("build_edge_list_safely builds 3 edges for 3 fully-connected pathways", {

  cgl <- list(

    PATHWAY_A = c("GENE_A", "GENE_B", "GENE_C", "GENE_D"),

    PATHWAY_B = c("GENE_A", "GENE_B", "GENE_C", "GENE_E"),

    PATHWAY_C = c("GENE_A", "GENE_B", "GENE_C", "GENE_F")

  )

  edges <- build_edge_list_safely(cgl, min_shared_genes = 2)

  expect_equal(nrow(edges), 3L)  # C(3,2) = 3 pairs

})

# Tests for calculate_overlap_ratio()

# Type: pure function (no dependencies)

# Coverage: ORA / leading modes; identical / disjoint / partial overlap;

#           case-insensitivity; dedupe behavior

#

# Naming convention for fixtures:

#   GENE_A, GENE_B, ... are dummy gene symbols (purely syntactic placeholders)

#   They stand in for real symbols like TP53, MYC, GAPDH but carry no biology.



test_that("calculate_overlap_ratio returns 1 for identical sets in ora mode", {

  pathway <- c("GENE_A", "GENE_B", "GENE_C")

  de      <- c("GENE_A", "GENE_B", "GENE_C")

  expect_equal(calculate_overlap_ratio(pathway, de, ratio_mode = "ora"), 1)

})



test_that("calculate_overlap_ratio returns 0 for disjoint sets in ora mode", {

  pathway <- c("GENE_A", "GENE_B")

  de      <- c("GENE_C", "GENE_D")

  expect_equal(calculate_overlap_ratio(pathway, de, ratio_mode = "ora"), 0)

})



test_that("calculate_overlap_ratio handles partial overlap in ora mode", {

  pathway <- c("GENE_A", "GENE_B", "GENE_C", "GENE_D")

  de      <- c("GENE_C", "GENE_D", "GENE_E", "GENE_F")

  # overlap = {GENE_C, GENE_D} = 2; pathway = 4 -> 0.5

  expect_equal(calculate_overlap_ratio(pathway, de, ratio_mode = "ora"), 0.5)

})



test_that("calculate_overlap_ratio uses DE size as denominator in leading mode", {

  pathway <- c("GENE_A", "GENE_B", "GENE_C", "GENE_D")  # 4 genes

  de      <- c("GENE_C", "GENE_D")                      # 2 genes

  # overlap = {GENE_C, GENE_D} = 2

  # ora mode:     2/4 = 0.5 (pathway size)

  # leading mode: 2/2 = 1.0 (DE size)

  expect_equal(calculate_overlap_ratio(pathway, de, ratio_mode = "ora"), 0.5)

  expect_equal(calculate_overlap_ratio(pathway, de, ratio_mode = "leading"), 1)

})



test_that("calculate_overlap_ratio is case-insensitive", {

  pathway <- c("GENE_A", "GENE_B", "GENE_C")

  de      <- c("gene_a", "gene_b", "gene_c")  # lowercase input

  expect_equal(calculate_overlap_ratio(pathway, de), 1)

})



test_that("calculate_overlap_ratio does NOT deduplicate pathway genes", {

  # Implementation: intersect(toupper(.)) without unique()

  # pathway = c("GENE_A", "GENE_A", "GENE_B", "GENE_C") -> toupper keeps duplicates

  # intersect returns {GENE_A, GENE_B, GENE_C} (set semantics)

  # length(overlap) = 3; length(pathway_genes) = 4 -> 3/4

  pathway <- c("GENE_A", "GENE_A", "GENE_B", "GENE_C")

  de      <- c("GENE_A", "GENE_B", "GENE_C")

  expect_equal(calculate_overlap_ratio(pathway, de), 3 / 4)

})



test_that("calculate_overlap_ratio rejects invalid ratio_mode via match.arg", {

  expect_error(

    calculate_overlap_ratio(c("GENE_A"), c("GENE_A"), ratio_mode = "invalid"),

    class = "simpleError"

  )

})

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE, warning = FALSE, message = FALSE)

## ----install-package, eval=FALSE----------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# BiocManager::install("GSEAlens")

## ----install-github, eval=FALSE-----------------------------------------------
# if (!requireNamespace("pak", quietly = TRUE))
#     install.packages("pak")
# pak::pkg_install("DDL095/GSEAlens")

## ----setup-environment, results='hide'----------------------------------------
library(airway)
data(airway)
expression_data <- airway

## ----limma-voom-data-preparation----------------------------------------------
library(edgeR)
library(limma)
group_level <- expression_data$dex
design <- model.matrix(~0+group_level)
colnames(design) <- levels(group_level)
compare_end <- combn(levels(group_level), 2, simplify = FALSE)
contrast_strings <- sapply(compare_end, function(x) paste(x[2], x[1], sep = " - "))
contrast_matrix <- makeContrasts(contrasts = contrast_strings, levels = design)
genes_df <- data.frame(
  gene_id = SummarizedExperiment::rowData(expression_data)$gene_id,
  symbol = SummarizedExperiment::rowData(expression_data)$symbol,
  gene_biotype = SummarizedExperiment::rowData(expression_data)$gene_biotype
)
genes_df$Length <- SummarizedExperiment::rowData(expression_data)$gene_seq_end -
                   SummarizedExperiment::rowData(expression_data)$gene_seq_start + 1
gsea_limma_voom_data <- edgeR::DGEList(
  counts = SummarizedExperiment::assay(expression_data, "counts"),
  genes = genes_df,
  norm.factors = NULL,
  group = group_level,
  remove.zeros = TRUE
)
# Filter low-expression genes, keep protein-coding RNAs, deduplicate symbols
total_counts <- edgeR::cpm(gsea_limma_voom_data) |> rowSums()
dup_symbols <- gsea_limma_voom_data$genes$symbol[duplicated(gsea_limma_voom_data$genes$symbol)]
keep <- rep(TRUE, nrow(gsea_limma_voom_data))
for (gene in dup_symbols) {
  idx <- which(gsea_limma_voom_data$genes$symbol == gene)
  best_idx <- idx[which.max(total_counts[idx])]
  remove_idx <- idx[idx != best_idx]
  keep[remove_idx] <- FALSE
}
gsea_limma_voom_data <- gsea_limma_voom_data[keep, ]
rownames(gsea_limma_voom_data) <- gsea_limma_voom_data$genes$symbol
keep_biotype <- gsea_limma_voom_data$genes$gene_biotype == "protein_coding"
gsea_limma_voom_data <- gsea_limma_voom_data[keep_biotype, ]
gsea_limma_voom_data <- edgeR::normLibSizes(gsea_limma_voom_data, method = "TMM")
isexpr <- rowSums(edgeR::cpm(gsea_limma_voom_data) > 1) >= 3
gsea_limma_voom_data <- gsea_limma_voom_data[isexpr, ]

## ----limma-voom-processing----------------------------------------------------
VoomOutPut <- voom(gsea_limma_voom_data, design)
fit <- lmFit(object = VoomOutPut, design = design) |>
  contrasts.fit(contrasts = contrast_matrix) |>
  eBayes()

## ----deseq2-se-workflow-------------------------------------------------------
library("DESeq2")
dds_se <- DESeqDataSet(expression_data, design = ~ cell + dex)
# Keep protein-coding genes only
gene_biotypes <- SummarizedExperiment::rowData(dds_se)$gene_biotype
keep_protein_coding <- gene_biotypes == "protein_coding"
dds_se <- dds_se[keep_protein_coding, ]
# Remove low-expression genes (require >=10 reads in >=3 samples)
smallestGroupSize <- 3
keep <- rowSums(DESeq2::counts(dds_se) >= 10) >= smallestGroupSize
dds_se <- dds_se[keep, ]
# Deduplicate gene symbols, keeping the highest-count row
rownames(dds_se) <- SummarizedExperiment::rowData(dds_se)$gene_name
total_counts <- rowSums(SummarizedExperiment::assay(dds_se))
dup_genes <- rownames(dds_se)[duplicated(rownames(dds_se))]
keep <- rep(TRUE, nrow(dds_se))
for (gene in dup_genes) {
  idx <- which(rownames(dds_se) == gene)
  best_idx <- idx[which.max(total_counts[idx])]
  remove_idx <- idx[idx != best_idx]
  keep[remove_idx] <- FALSE
}
dds_se <- dds_se[keep, ]
# Run DESeq2 fitting
dds_se <- DESeq(dds_se)

## ----deseq2-matrix-workflow---------------------------------------------------
DDS_rawdata <- expression_data
# Keep protein-coding genes only
gene_biotypes <- SummarizedExperiment::rowData(DDS_rawdata)$gene_biotype
keep_protein_coding <- gene_biotypes == "protein_coding"
DDS_rawdata <- DDS_rawdata[keep_protein_coding, ]
# Remove low-expression genes
smallestGroupSize <- 3
keep_epd <- rowSums(SummarizedExperiment::assay(DDS_rawdata, "counts") >= 10) >= smallestGroupSize
DDS_rawdata <- DDS_rawdata[keep_epd, ]
# Deduplicate gene symbols
rownames(DDS_rawdata) <- SummarizedExperiment::rowData(DDS_rawdata)$gene_name
total_counts <- rowSums(SummarizedExperiment::assay(DDS_rawdata))
keep_name <- rep(TRUE, nrow(DDS_rawdata))
dup_genes <- rownames(DDS_rawdata)[duplicated(rownames(DDS_rawdata))]
for (gene in dup_genes) {
  idx <- which(rownames(DDS_rawdata) == gene)
  best_idx <- idx[which.max(total_counts[idx])]
  remove_idx <- idx[idx != best_idx]
  keep_name[remove_idx] <- FALSE
}
DDS_rawdata <- DDS_rawdata[keep_name, ]
# Extract counts and sample metadata into plain matrices
cts <- SummarizedExperiment::assay(DDS_rawdata, "counts")
coldata <- as.data.frame(SummarizedExperiment::colData(DDS_rawdata))
coldata <- coldata[, c("cell", "dex")]
coldata$cell <- factor(coldata$cell)
coldata$dex <- factor(coldata$dex)
# Build DESeqDataSet from matrices and run DESeq2
dds <- DESeqDataSetFromMatrix(countData = cts, colData = coldata, design = ~ dex)
dds <- DESeq(dds)

## ----sessionInfo--------------------------------------------------------------
sessionInfo()

#' @title Initialize GSEA Environment Object

#' @description Unified entry point supporting limma-voom (no intercept design) and DESeq2 objects.

#' Automatically extracts contrasts, differential analysis results, and expression matrices to generate

#' a standardized GseaEnv object.

#' @param fit Differential analysis object. Supports \code{MArrayLM} (limma) or \code{DESeqDataSet} (DESeq2).

#' @param pathway_obj Gene set object. Typically generated by \code{build_gsea_pathways()}.

#' @param expr_data Optional. Expression matrix or DGEList. If NULL, extraction from fit object will be attempted (partial support only).

#' @param target_factor Only for DESeq2. Specifies the target factor of interest. If NULL, automatically inferred as the last term in the design formula.


#' @param verbose Logical. If TRUE (default), print progress messages. Set to FALSE for silent batch-mode use.
#' @return Returns a standardized object of class \code{GseaEnv}.

#' @export

#' @examples

#' # Limma-voom workflow using pre-computed inputs shipped with the package

#' data(preprocessed_limma, package = "GSEAlens")
#'
#' precomp <- preprocessed_limma

#' data(gsea_pathwaysets_toy, package = "GSEAlens")
#'
#' pathways <- gsea_pathwaysets_toy

#' env <- setup_gsea_env(

#'   fit       = precomp$fit,

#'   pathway_obj = pathways,

#'   expr_data = precomp$gsea_limma_voom_data

#' )

#' class(env)

setup_gsea_env <- function(fit, pathway_obj, expr_data = NULL, target_factor = NULL,
                                                    verbose = TRUE) {

    # IRON FIX (2026-07-01, D-2): add `verbose` to allow silent batch-mode use.
    if (isTRUE(verbose)) message("\nStarting GSEAlens engine...")



    # 1. 校验基因集对象

    if (is.null(pathway_obj) || is.null(pathway_obj$TERM2GENE)) {

    stop("Invalid pathway_obj! Please use build_gsea_pathways() to generate a valid gene set object.")

    }



    # 2. 识别后端类型并分发提取

    backend_data <- NULL

    backend_info <- NULL



    if (inherits(fit, "MArrayLM")) {

    message("Detected input type: [Limma-Voom]")



    # 调用 Limma 提取器

    backend_data <- .extract_limma_data(fit, expr_data)



    backend_info <- list(

            backend = "limma_voom",

            input_class = "MArrayLM",

            design_formula = NULL, # Limma fit 对象通常不直接保留公式

            target_factor = NA

    )

    } else if (inherits(fit, "DESeqDataSet")) {

    message("Detected input type: [DESeq2]")



    # 调用 DESeq2 提取器

    backend_data <- .extract_deseq2_data(fit, target_factor)



    # 记录实际使用的 target_factor

    actual_factor <- if (is.null(target_factor)) {

            utils::tail(attr(terms(DESeq2::design(fit)), "term.labels"), 1)

    } else {

            target_factor

    }



    backend_info <- list(

            backend = "deseq2",

            input_class = "DESeqDataSet",

            design_formula = DESeq2::design(fit),

            target_factor = actual_factor

    )

    } else {

    stop("Unsupported input type! Please provide MArrayLM (limma) or DESeqDataSet (DESeq2) object.")

    }



    # 3. 组装基因集信息

    #

    geneset_info <- list(

    name = pathway_obj$SuperTag %||% pathway_obj$name,

    term2gene = pathway_obj$TERM2GENE %||% pathway_obj$term2gene,

    meta_dict = {

            md <- pathway_obj$meta_dict %||% pathway_obj$TERM2NAME

            if (!"Subcollection" %in% colnames(md)) md$Subcollection <- ""

            if (!"Combo_Name" %in% colnames(md)) {

        md$Combo_Name <- ifelse(

                    is.na(md$Subcollection) | md$Subcollection == "",

                    md$Collection,

                    paste0(md$Collection, ":", md$Subcollection)

        )

            }

            if (!"Species" %in% colnames(md)) {

        md$Species <- pathway_obj$species %||% "HS"

            }

            md

    },

    used_collections = pathway_obj$collections_used %||% pathway_obj$used_collections,

    species = pathway_obj$species %||% "HS" # 新增

    )



    # 4. 构建最终对象

    env_obj <- create_gsea_env(

    backend_info = backend_info,

    contrast_registry = backend_data$contrast_registry,

    de_store = backend_data$de_store,

    expr_bundle = backend_data$expr_bundle,

    geneset = geneset_info,

    raw_obj = list(fit = fit) # 保留原始对象引用

    )



    # 5. 最终校验

    .check_gsea_env(env_obj)



    if (isTRUE(verbose)) {

    message("GseaEnv object built successfully!")

    message(sprintf("   Contains %d contrast groups", nrow(backend_data$contrast_registry)))

    message(sprintf("   Contains %d pathways", nrow(pathway_obj$meta_dict)))

    }



    return(env_obj)

}





# 查看函数





#' @title View GSEA Environment Object Overview

#' @description Print detailed information about the GseaEnv object to the console, including backend type,

#' contrast list, and gene set status.

#' @param env_obj GseaEnv object

#' @return Invisibly returns the input object (for chaining).

#' @export



#' @examples

#' # Build a GseaEnv from pre-computed inputs shipped with the package

#' data(preprocessed_limma, package = "GSEAlens")
#'
#' precomp <- preprocessed_limma

#' data(gsea_pathwaysets_toy, package = "GSEAlens")
#'
#' pathways <- gsea_pathwaysets_toy

#' env <- setup_gsea_env(

#'   fit       = precomp$fit,

#'   pathway_obj = pathways,

#'   expr_data = precomp$gsea_limma_voom_data

#' )

#' inspect_gsea_env(env)

inspect_gsea_env <- function(env_obj) {

    if (!inherits(env_obj, "GseaEnv")) stop("Input object is not of class GseaEnv.")



    # 解析信息

    bi <- env_obj$backend_info

    reg <- env_obj$contrast_registry

    gs <- env_obj$geneset

    expr <- env_obj$expr_bundle



    # 控制台输出

    message("")

    message(paste(rep("=", 60), collapse = ""))

    message("       GSEAlens Environment Summary")

    message(paste(rep("=", 60), collapse = ""))



    # 1. 后端信息

    message("[1] Backend Information")

    message(sprintf("   Type        : %s", bi$backend))

    message(sprintf("   Input Class : %s", bi$input_class))

    if (bi$backend == "deseq2") {

    message(sprintf("   Target Factor: %s", bi$target_factor))

    }

    message("")



    # 2. 对比组注册表

    message(sprintf("[2] Contrast Registry (%d comparisons)", nrow(reg)))

    if (nrow(reg) > 0) {

    # 打印前几个，防止刷屏

    print_head <- utils::head(reg[, c("contrast_id", "left_group", "right_group")], 5)

    for (i in seq_len(nrow(print_head))) {

            message(sprintf(

        "   [%d] %s  ( %s vs %s )",

        i, print_head$contrast_id[i],

        print_head$left_group[i], print_head$right_group[i]

            ))

    }

    if (nrow(reg) > 5) message("   ... (remaining omitted)")

    } else {

    message("   No contrasts detected!")

    }

    message("")



    # 3. 基因集信息

    message("[3] Gene Set Database")

    message(sprintf("   Name        : %s", gs$name))

    message(sprintf("   Pathways    : %d", nrow(gs$meta_dict)))

    message("")



    # 4. 表达数据状态

    message("[4] Expression Data Status")

    if (!is.null(expr$raw_counts)) {

    message(sprintf(

            "   Raw Counts  : %d genes x %d samples",

            nrow(expr$raw_counts), ncol(expr$raw_counts)

    ))

    message(sprintf("   Display Mat : %s", ifelse(!is.null(expr$display_expr), "Ready", "Missing")))

    } else {

    message("   Expression matrix not included (heatmap functionality will be unavailable)")

    }

    message("")



    # 5. 下一步指引

    message("[5] Next Step")

    message("   Please run the computation engine:")

    message("   > res <- batch_calc_gsea(env_obj)")

    message(paste(rep("=", 60), collapse = ""))



    invisible(env_obj)

}



#' @export

print.GseaEnv <- function(x, ...) {

    inspect_gsea_env(x)

}

# Tests for setup_gsea_env() input validation contract

# Type: validation gateway tests (no full backend construction)

# Coverage: pathway_obj NULL/missing-TERM2GENE rejection; unsupported fit class

#           rejection; error class and message assertions.

#

# BUG-4 FIX: original sample expected intercept-design validation to fire, but

# the real implementation checks pathway_obj FIRST and would always reject

# pathway_obj = list() before reaching fit-class dispatch. These tests align

# with the actual control flow.



# Helper: a pathway_obj that passes the gateway check (has TERM2GENE)

build_valid_pathway_obj <- function() {

  list(

    TERM2GENE = data.frame(

      gs_name = c("PATHWAY_DUMMY"),

      gene_symbol = c("GENE_A"),

      stringsAsFactors = FALSE

    ),

    meta_dict = data.frame(

      Collection = c("H"),

      Subcollection = c(""),

      stringsAsFactors = FALSE

    ),

    SuperTag = "test",

    collections_used = data.frame(),

    species = "HS"

  )

}



test_that("setup_gsea_env rejects NULL pathway_obj", {

  expect_error(

    setup_gsea_env(fit = list(), pathway_obj = NULL),

    regexp = "pathway_obj"

  )

})



test_that("setup_gsea_env rejects pathway_obj missing TERM2GENE field", {

  bad_pathway <- list(not_term2gene = "wrong")

  expect_error(

    setup_gsea_env(fit = list(), pathway_obj = bad_pathway),

    regexp = "pathway_obj"

  )

})



test_that("setup_gsea_env rejects empty list pathway_obj", {

  expect_error(

    setup_gsea_env(fit = list(), pathway_obj = list()),

    regexp = "pathway_obj"

  )

})



test_that("setup_gsea_env rejects fit that is neither MArrayLM nor DESeqDataSet", {

  # Must use a *valid* pathway_obj to get past the first gateway

  # and reach the fit-class dispatch.

  expect_error(

    setup_gsea_env(

      fit = list(not_a_fit = TRUE),  # no MArrayLM / DESeqDataSet class

      pathway_obj = build_valid_pathway_obj()

    ),

    regexp = "Unsupported input type"

  )

})



test_that("setup_gsea_env rejects plain numeric fit with valid pathway_obj", {

  expect_error(

    setup_gsea_env(

      fit = 42,

      pathway_obj = build_valid_pathway_obj()

    ),

    regexp = "Unsupported input type"

  )

})

# GSEAlens Interactive Analysis Workstation ---------------------------------------------------

#' @title GSEAlens Interactive Analysis Workstation

#' @description Modular Shiny application based on GseaRes object, supporting

#'   Limma and DESeq2 dual backends. Provides interactive visualization for

#'   GSEA pathway enrichment analysis including volcano plots, heatmaps,

#'   expression boxplots, and pathway relationship networks.

#' @param gsea_res GseaRes object (generated by batch_calc_gsea())

#' @param addition_data Optional. A data frame or file path containing

#'   pathway annotations to merge into the main table. Can also be:

#'   \itemize{

#'     \item A data.frame with 'ID' column as primary key

#'     \item A character vector: file path to .csv, .csv.gz, .rds, or .rds.gz

#'     \item NULL: automatically searches for "addition_data_gsealens.rds"

#'       or "addition_data_gsealens.csv" in the working directory

#'   }

#'   Additional columns will be merged to the main pathway table.

#' @return Shiny application object

#' @export

#' @examples

#' if(interactive()){

#' # Basic usage

#' gsea_res <- batch_calc_gsea(gsea_env)

#' launch_gsea_app(gsea_res)

#'

#' # With addition data from file path

#' launch_gsea_app(gsea_res, "pathway_annotations.csv")

#'

#' # With pre-loaded data frame

#' add_data <- read.csv("annotations.csv")

#' launch_gsea_app(gsea_res, add_data)

#' }

launch_gsea_app <- function(gsea_res, addition_data = NULL) {

    # Step 1: Validate GseaRes object ------------------------------------------------



    if (!inherits(gsea_res, "GseaRes")) {

    stop("Please provide a valid GseaRes object (generated by batch_calc_gsea())")

    }



    # Step 2: Handle addition_data ----------------------------------------------------



    if (!is.null(addition_data)) {

    addition_data <- .process_addition_data(addition_data)

    } else {

    # Auto-detect from working directory

    addition_data <- .auto_detect_addition_data()

    }



    # Step 3: Start Shiny application -------------------------------------------------



    .launch_gsea_shiny(gsea_res, addition_data)

}



# Process Addition Data -----------------------------------------------------------------------





#' @title Process Addition Data

#' @description Internal function to handle various addition_data input types.

#'   Supports data.frame objects or file paths (.csv, .rds formats).

#' @param addition_data A data.frame or character vector (file path)

#' @return A validated data.frame with addition data

#' @keywords internal

.process_addition_data <- function(addition_data) {

    # If already a data.frame, validate and return

    if (is.data.frame(addition_data)) {

    .validate_addition_data(addition_data)

    return(addition_data)

    }



    # If character, treat as file path

    if (is.character(addition_data)) {

    if (length(addition_data) != 1) {

            stop("When addition_data is a character, it must be a single file path")

    }

    if (!file.exists(addition_data)) {

            stop("Addition data file not found: ", addition_data)

    }



    # Detect format and read

    if (grepl("\\.rds", addition_data, ignore.case = TRUE)) {

            data <- readRDS(addition_data)

    } else if (grepl("\\.csv", addition_data, ignore.case = TRUE)) {

            data <- read.csv(addition_data,

        stringsAsFactors = FALSE,

        check.names = FALSE, encoding = "UTF-8"

            )

    } else {

            stop("Unsupported addition_data format. Use .csv or .rds")

    }



    .validate_addition_data(data)

    return(data)

    }



    stop("addition_data must be a data.frame or file path (character)")

}



# Validate Addition Data ----------------------------------------------------------------------





#' @title Validate Addition Data

#' @description Internal function to validate addition_data structure.

#'   Checks that input is a data.frame with required 'ID' column and

#'   removes duplicate entries if present.

#' @param data A data.frame to validate

#' @return Validated data.frame with character ID column

#' @keywords internal

.validate_addition_data <- function(data) {

    if (!is.data.frame(data)) {

    stop("addition_data must be a data.frame")

    }

    if (!"ID" %in% colnames(data)) {

    stop("addition_data must contain 'ID' column as primary key")

    }

    # Ensure ID is character

    data$ID <- as.character(data$ID)

    # Deduplicate

    if (any(duplicated(data$ID))) {

    n_dup <- sum(duplicated(data$ID))

    message("[GSEAlens] Removed ", n_dup, " duplicate ID(s) from addition_data")

    data <- data[!duplicated(data$ID), ]

    }

    message(

    "[GSEAlens] addition_data validated: ", nrow(data), " rows x ",

    ncol(data), " columns"

    )

    return(data)

}



# Auto-detect Addition Data -------------------------------------------------------------------





#' @title Auto-detect Addition Data

#' @description Search working directory for default addition_data files.

#'   Checks for 'addition_data_gsealens.rds' or 'addition_data_gsealens.csv'

#'   in the current working directory.

#' @return A validated data.frame or NULL if no file found

#' @keywords internal

.auto_detect_addition_data <- function() {

    wd <- getwd()



    # Priority: RDS > CSV > not found

    rds_path <- file.path(wd, "addition_data_gsealens.rds")

    csv_path <- file.path(wd, "addition_data_gsealens.csv")



    if (file.exists(rds_path)) {

    message("[GSEAlens] Auto-detected addition_data: ", basename(rds_path))

    data <- readRDS(rds_path)

    return(.validate_addition_data(data))

    }



    if (file.exists(csv_path)) {

    message("[GSEAlens] Auto-detected addition_data: ", basename(csv_path))

    message("[GSEAlens] Converting CSV to RDS for faster loading...")

    data <- read.csv(csv_path,

            stringsAsFactors = FALSE,

            check.names = FALSE, encoding = "UTF-8"

    )

    # Validate and save as RDS for future use

    data <- .validate_addition_data(data)

    saveRDS(data, rds_path, compress = "gzip")

    message("[GSEAlens] Saved RDS cache: ", basename(rds_path))

    return(data)

    }



    message("[GSEAlens] No addition_data found, using default table display")

    return(NULL)

}



# Launch GSEAlens Shiny App (Internal) --------------------------------------------------------





#' @title Launch GSEAlens Shiny App (Internal)

#' @description Core Shiny application launcher. Builds the UI layout and

#'   initializes all server modules for interactive GSEA analysis.

#' @param gsea_res A valid GseaRes object

#' @param addition_data Optional data.frame with pathway annotations

#' @return A Shiny app object

#' @keywords internal

.launch_gsea_shiny <- function(gsea_res, addition_data) {

    ui <- shiny::fluidPage(

    shiny::tags$head(

            shiny::tags$style(shiny::HTML("

        /* IRON FIX (2026-06-30): modal scroll-to-top bug.

                    * Previously only .modal-body had max-height: 85vh, but the dialog

                    * itself was unconstrained and vertically centered by Bootstrap, so

                    * when the Live Preview image grew taller than the viewport the

                    * browser pushed the modal header above the viewport and the user

                    * could not scroll back to it. Switching to a flex layout with a

                    * bounded .modal-dialog / .modal-content keeps header & footer

                    * pinned and only the .modal-body scrolls. */

        .modal {

                    display: flex !important;

                    align-items: flex-start !important;

                    overflow-y: auto !important;

        }

        .modal-dialog {

                    max-width: 90vw !important;

                    width: 90vw !important;

                    margin: 4vh auto !important;

        }

        .modal-content {

                    max-height: 92vh !important;

                    display: flex !important;

                    flex-direction: column !important;

        }

        .modal-header { flex: 0 0 auto !important; }

        .modal-body {

                    flex: 1 1 auto !important;

                    min-height: 0 !important;

                    overflow-y: auto !important;

                    background-color: #fafafa;

        }

        .modal-footer { flex: 0 0 auto !important; }

        .white-box {

                    background-color: white; padding: 15px; border-radius: 8px;

                    box-shadow: 0 2px 8px rgba(0,0,0,0.1); margin-bottom: 15px;

        }

        .description-cell {

                    white-space: normal !important; word-wrap: break-word !important;

                    max-width: 300px; font-size: 12px; line-height: 1.4;

        }

        .dt-center { text-align: center !important; }

        .well { background-color: #f8f9fa; border: 1px solid #e9ecef; border-radius: 8px; padding: 15px; }

            "))

    ),

    shinyjs::useShinyjs(),

    shiny::titlePanel(shiny::HTML("GSEAlens: Seek Inside")),

    shiny::sidebarLayout(

            shiny::sidebarPanel(

        width = 3,

        mod_data_prep_ui("data_prep")

            ),

            shiny::mainPanel(

        width = 9,

        shiny::tabsetPanel(

                    id = "main_tabs",

                    shiny::tabPanel(

            title = "Main Workspace",

            shiny::br(),

            mod_master_table_ui("master_table"),

            shiny::hr(),

            mod_multi_plot_ui("multi_plot")

                    ),

                    shiny::tabPanel(

            title = "Holographic Quadruple Linkage",

            shiny::br(),

            mod_quadrant_ui("quadrant")

                    ),

                    shiny::tabPanel(

            title = "Pathway Relationship Exploration",

            shiny::br(),

            mod_pathway_relation_ui("pathway_relation")

                    ),

                    shiny::tabPanel(

            title = "HubGene Network",

            value = "hubgene_vis",

            shiny::br(),

            mod_hubgene_vis_ui("hubgene_vis")

                    ),

                    shiny::tabPanel(

            title = "AI Interpretation",

            value = "ai_interpretation",

            mod_ai_abs_page_ui("ai_abs")

                    ),

                    shiny::tabPanel(

            title = "Joint GSEA Canvas",

            shiny::br(),

            mod_joint_canvas_ui("joint_canvas")

                    )

        )

            )

    ),

    mod_pathway_modal_ui("pathway_modal")

    )



    # Server part -------------------------------------------------------------------------

    server <- function(input, output, session) {

    # Module 1: Data preprocessing module

    data_prep_list <- mod_data_prep_server("data_prep", gsea_res)



    # Module 2: Master table module

    table_result <- mod_master_table_server("master_table", data_prep_list$data, addition_data)



    # Module 3: Combined plotting module (gsea_res for cross-collection support)

    mod_multi_plot_server("multi_plot", data_prep_list$data, table_result, gsea_res)



    # Module 4: Quadrant linkage module (pass table_controller for bidirectional sync)

    mod_quadrant_server("quadrant", data_prep_list, gsea_res, table_controller = table_result)



    # Module 5: Detail modal module

    mod_pathway_modal_server(

            "pathway_modal",

            data_prep_list$data,

            table_result$show_modal,

            gsea_res,

            pending_genes_reactive = data_prep_list$get_pending_genes,

            update_pending_genes = data_prep_list$update_pending_genes

    )



    # Module 6: Pathway relation exploration module

    mod_pathway_relation_server("pathway_relation", data_prep_list, gsea_res, table_result)



    # Shared gsea_res reference (for downstream modules)

    shiny::shinyOptions(gsea_res_shared = gsea_res)



    # Module 7: HubGene Network module

    mod_hubgene_vis_server("hubgene_vis", data_prep_list, table_result, gsea_res)



    # Module 8: AI analysis module

    mod_ai_abs_page_server("ai_abs", gsea_res, data_prep_list, table_result)



    # Module 9: Joint GSEA canvas module

    mod_joint_canvas_server("joint_canvas", gsea_res, data_prep_list, table_result)

    }



    message("GSEAlens started")

    message(sprintf("   Backend type: %s", gsea_res$backend_info$backend))

    message(sprintf("   Number of comparison groups: %d", nrow(gsea_res$contrast_registry)))



    shiny::shinyApp(ui, server)

}

#' @title Align GSEA Performance Data with System Monitoring Data

#' @description Aligns timestamps generated by batch_calc_gsea with CSV data from

#'   an independent monitoring script, producing a time series data frame suitable

#'   for publication.

#' @param gsea_res A GseaRes object (must contain metadata$gsea_benchmark)

#' @param monitor_csv Path to the CSV file generated by the monitoring script

#' @return A aligned data frame (containing unified time axis fields such as relative_sec)

#' @noRd

#' @examples

#' \dontrun{

#' res <- readRDS("GSEA_Capsule_[...].rds")

#' aligned <- align_benchmark_data(res, "system_monitor.csv")

#' plot(aligned$relative_sec, aligned$rss_mb, type = "l")

#' }

align_benchmark_data <- function(gsea_res, monitor_csv) {

    if (!inherits(gsea_res, "GseaRes")) {

    stop("Input must be a GseaRes object")

    }



    if (!file.exists(monitor_csv)) {

    stop("Monitoring CSV does not exist: ", monitor_csv)

    }



    # 提取GSEA时间窗口

    bench <- gsea_res$metadata$gsea_benchmark

    start_ms <- bench$start_ms

    end_ms <- bench$end_ms



    # 读取Python生成的CSV（格式更简洁）

    monitor <- read.csv(monitor_csv, stringsAsFactors = FALSE)



    # Python和R的时间戳都是Unix毫秒，直接对齐

    aligned <- monitor[monitor$timestamp_ms >= (start_ms - 5000) &

    monitor$timestamp_ms <= (end_ms + 5000), ]



    if (nrow(aligned) == 0) {

    warning("No overlapping time data found")

    return(NULL)

    }



    # 转换相对时间（秒）

    aligned$relative_sec <- (aligned$timestamp_ms - start_ms) / 1000



    # 阶段标记

    total_duration <- (end_ms - start_ms) / 1000

    aligned$phase <- ifelse(

    aligned$relative_sec < 0, "pre_start",

    ifelse(aligned$relative_sec > total_duration, "post_end",

            ifelse(aligned$relative_sec < total_duration * 0.2, "startup",

        ifelse(aligned$relative_sec < total_duration * 0.8, "core_compute", "cleanup")

            )

    )

    )



    attr(aligned, "gsea_info") <- bench

    class(aligned) <- c("GseaBenchmarkAligned", "data.frame")



    message(sprintf(

    "Alignment complete: %d sampling points, spanning %.1f seconds",

    nrow(aligned), max(aligned$relative_sec) - min(aligned$relative_sec)

    ))



    invisible(aligned)

}





# Section: Visualization ----



#' @importFrom ggplot2 ggplot aes geom_line geom_vline labs theme_minimal scale_color_manual

#' @title Plot GSEA Benchmark Memory Curve

#' @description Generates a standard memory usage curve plot for publication.

#' @param aligned_data Aligned data frame returned by align_benchmark_data

#' @param highlight_phases Whether to color by phase, default TRUE

#' @return A ggplot object

#' @export

#' @examples

#' # Build a small synthetic aligned-data frame (mimics the output of

#' # align_benchmark_data() without needing an external CSV or GseaRes object).

#' syn <- data.frame(

#'   timestamp_ms = seq(0, 20000, by = 1000),

#'   rss_mb = c(120, 150, 220, 280, 310, 330, 340, 335, 320, 305,

#'              290, 270, 255, 245, 240, 235, 230, 225, 220, 215, 210),

#'   relative_sec = seq(0, 20, by = 1),

#'   phase = c(rep("startup", 4), rep("core_compute", 12), rep("cleanup", 5))

#' )

#' attr(syn, "gsea_info") <- list(duration_sec = 20, workers = 2)

#' class(syn) <- c("GseaBenchmarkAligned", "data.frame")

#' plot_gsea_memory(syn)

#' plot_gsea_memory(syn, highlight_phases = FALSE)

plot_gsea_memory <- function(aligned_data, highlight_phases = TRUE) {

    if (!requireNamespace("ggplot2", quietly = TRUE)) {

    stop("ggplot2 package required: install.packages('ggplot2')")

    }



    info <- attr(aligned_data, "gsea_info")



    p <- ggplot2::ggplot(aligned_data, ggplot2::aes(x = relative_sec, y = rss_mb)) +

    ggplot2::geom_line(linewidth = 0.8, color = "steelblue") +

    ggplot2::geom_vline(xintercept = 0, linetype = "dashed", color = "darkgreen") +

    ggplot2::geom_vline(xintercept = info$duration_sec, linetype = "dashed", color = "darkred") +

    ggplot2::labs(

            title = sprintf("GSEA Memory Usage Profile (%d cores)", info$workers),

            subtitle = sprintf("Total duration: %.1f seconds", info$duration_sec),

            x = "Time (seconds from GSEA start)",

            y = "Memory (MB)",

            caption = sprintf(

        "Peak: %.0f MB | Average: %.0f MB",

        max(aligned_data$rss_mb, na.rm = TRUE),

        mean(aligned_data$rss_mb[aligned_data$phase %in% c("startup", "core_compute", "cleanup")], na.rm = TRUE)

            )

    ) +

    ggplot2::theme_minimal()



    if (highlight_phases) {

    # 添加阶段背景色（高级用法，可选）

    p <- p + ggplot2::aes(color = phase) +

            ggplot2::scale_color_manual(values = c(

        pre_start = "gray70",

        startup = "green3",

        core_compute = "steelblue",

        cleanup = "orange",

        post_end = "gray70"

            ), name = "Phase")

    }



    p

}





# Section: Print Method ----



#' @title Print Method for GSEA Benchmark Alignment Results

#' @description Formats and displays a summary of aligned GSEA benchmark data,

#'   including computation parameters, monitoring statistics, and phase-wise

#'   memory usage.

#' @param x An object of class 'GseaBenchmarkAligned'

#' @param ... Additional arguments passed to print

#' @return The input object x (invisibly)

#' @noRd

#' @exportS3Method

#' @examples

#' \dontrun{

#' data <- align_benchmark_data(gsea_res, "system_monitor.csv")

#' print(data)

#' }

print.GseaBenchmarkAligned <- function(x, ...) {

    info <- attr(x, "gsea_info")



    # NOTE: cat() is the idiomatic way to write to stdout inside an S3 print()

    # method (cf. print.data.frame, print.lm). Do NOT convert to message(),

    # which would redirect output to stderr and break interactive use.

    cat("========================================\n")

    cat("   GSEA Benchmark Alignment Results\n")

    cat("========================================\n")

    cat(sprintf("GSEA Computation Time: %s\n", info$start_time))

    cat(sprintf("Parallel Cores:        %d\n", info$workers))

    cat(sprintf("GSEA Total Duration:   %.2f seconds\n", info$duration_sec))

    cat("----------------------------------------\n")

    cat(sprintf("Monitoring Samples:    %d\n", nrow(x)))

    cat(sprintf(

    "Monitoring Span:       %.1f seconds (%.1f to %.1f)\n",

    max(x$relative_sec) - min(x$relative_sec),

    min(x$relative_sec), max(x$relative_sec)

    ))

    cat(sprintf("Peak Memory:           %.0f MB\n", max(x$rss_mb, na.rm = TRUE)))

    cat(sprintf("Average Memory:        %.0f MB\n", mean(x$rss_mb[x$phase %in% c("startup", "core_compute", "cleanup")], na.rm = TRUE)))

    cat("========================================\n")



    # 各阶段统计

    cat("\nMemory Statistics by Phase:\n")

    stage_stats <- tapply(x$rss_mb, x$phase, function(v) {

    sprintf("Mean %.0f MB, Peak %.0f MB", mean(v, na.rm = TRUE), max(v, na.rm = TRUE))

    })

    for (phase in names(stage_stats)) {

    if (!is.na(stage_stats[phase])) {

            cat(sprintf("  [%s]: %s\n", phase, stage_stats[phase]))

    }

    }



    invisible(x)

}





# Section: Scalability Analysis ----



#' @title Generate Scalability Analysis Report (Multi-core Comparison)

#' @description Compares GSEA performance across different core counts (for publication).

#' @param gsea_res_list A list of multiple GseaRes objects (named e.g., list("1_core" = res1, "4_core" = res4))

#' @param monitor_csv_list Corresponding list of monitoring CSV paths (with same names as gsea_res_list)

#' @return A comparison data frame with columns: run_name, workers, duration_sec, mem_peak_mb (if available), mem_avg_mb (if available)

#' @noRd

#' @examples

#' \dontrun{

#' results <- analyze_scalability(

#'   gsea_res_list = list("1_core" = res1, "4_core" = res4),

#'   monitor_csv_list = c("1_core" = "monitor_1.csv", "4_core" = "monitor_4.csv")

#' )

#' print(results)

#' }

analyze_scalability <- function(gsea_res_list, monitor_csv_list = NULL) {

    if (is.null(names(gsea_res_list))) {

    names(gsea_res_list) <- paste0("run_", seq_along(gsea_res_list))

    }



    results <- lapply(names(gsea_res_list), function(name) {

    res <- gsea_res_list[[name]]



    if (is.null(res$metadata$gsea_benchmark)) {

            warning(name, " missing benchmark data, skipping")

            return(NULL)

    }



    bench <- res$metadata$gsea_benchmark



    # 基础信息

    out <- data.frame(

            run_name = name,

            workers = bench$workers,

            duration_sec = bench$duration_sec,

            stringsAsFactors = FALSE

    )



    # 如果有监控数据，添加内存统计

    if (!is.null(monitor_csv_list) && name %in% names(monitor_csv_list)) {

            csv_path <- monitor_csv_list[[name]]

            if (file.exists(csv_path)) {

        monitor <- read.csv(csv_path, stringsAsFactors = FALSE)

        # 筛选GSEA时间段

        aligned <- monitor[monitor$timestamp_ms >= bench$start_ms &

                    monitor$timestamp_ms <= bench$end_ms, ]

        if (nrow(aligned) > 0) {

                    out$mem_peak_mb <- max(aligned$rss_mb, na.rm = TRUE)

                    out$mem_avg_mb <- mean(aligned$rss_mb, na.rm = TRUE)

        }

            }

    }



    out

    })



    do.call(rbind, results)

}

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE, warning = FALSE, message = FALSE)

## ----install-package, eval=FALSE----------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# BiocManager::install("GSEAlens")

## ----install-github, eval=FALSE-----------------------------------------------
# if (!requireNamespace("pak", quietly = TRUE))
#     install.packages("pak")
# pak::pkg_install("DDL095/GSEAlens")

## ----setup-environment, results='hide'----------------------------------------
library(GSEAlens)
library(airway)

## ----load-prepared-objects----------------------------------------------------
data(preprocessed_limma, package = "GSEAlens")
preproc_limma         <- preprocessed_limma
fit                   <- preproc_limma$fit
gsea_limma_voom_data  <- preproc_limma$gsea_limma_voom_data
data(preprocessed_dds_se, package = "GSEAlens")
dds_se <- preprocessed_dds_se
data(preprocessed_dds, package = "GSEAlens")
dds <- preprocessed_dds

## ----build-gsea-pathways------------------------------------------------------
# Real call (slow on the Bioconductor build machine):
# gsea_pathwaysets <- build_gsea_pathways(
#   species = "HS", auto_select = c("H", "C2:CP:REACTOME", "C5:GO:BP")
# )
# For the vignette we load a pre-computed lightweight pathway object instead:
data(gsea_pathwaysets_toy, package = "GSEAlens")
gsea_pathwaysets <- gsea_pathwaysets_toy

## ----setup-gsea-env-----------------------------------------------------------
# limma-voom workflow (needs the DGEList because fit alone lacks raw counts)
gseadata_limmavoom <- setup_gsea_env(fit = fit, pathway_obj = gsea_pathwaysets, expr_data = gsea_limma_voom_data)
# DESeq2 SummarizedExperiment workflow
gseadata_se <- setup_gsea_env(fit = dds_se, pathway_obj = gsea_pathwaysets)
# DESeq2 Count matrix workflow
gseadata_dds <- setup_gsea_env(fit = dds, pathway_obj = gsea_pathwaysets)

## ----batch-calc-limma, eval=FALSE---------------------------------------------
# # Write vignette outputs to a temporary directory to avoid polluting the
# # Bioconductor build machine's working directory.
# out_dir <- tempdir()
# # limma-voom workflow
# gsea_res_limmavoom <- batch_calc_gsea(gseadata_limmavoom,
#                                                  custom_series_name = "limmavoom_data",
#                                                  output_dir = out_dir,
#                                                  workers = 2,
#                                                  force = TRUE)
# # DESeq2 SummarizedExperiment workflow
# gsea_res_se <- batch_calc_gsea(gseadata_se,
#                                           custom_series_name = "dds_se_data",
#                                           output_dir = out_dir,
#                                           workers = 2,
#                                           force = TRUE)
# # DESeq2 Count matrix workflow
# gsea_res_dds <- batch_calc_gsea(gseadata_dds,
#                                            custom_series_name = "dds_data",
#                                            output_dir = out_dir,
#                                            workers = 2,
#                                            force = TRUE)

## ----import-capsule, eval=FALSE-----------------------------------------------
# gsea_res <- import_gsea_capsule("/path/to/your/files/")
# # Or read the RDS directly:
# # gsea_res <- readRDS("/path/of/your/file/")

## ----launch-app, eval=FALSE---------------------------------------------------
# # Basic launch
# launch_gsea_app(gsea_res)
# # With explicit pathway annotations:
# # launch_gsea_app(gsea_res, addition_data = "pathway_annotations.csv")

## ----sessionInfo--------------------------------------------------------------
sessionInfo()

#!/usr/bin/env Rscript
# Purpose: Convert .rds files in data/ to .rda format so that data() can load them.
#
# Background:
#   - Bioconductor reviewer (mireia-bioinfo) requested migrating rds from
#     inst/extdata/ to data/ so users can load via data(object_name).
#   - R's data() does NOT support .rds directly; it requires .rda (or .RData).
#   - This script converts each data/<name>.rds to data/<name>.rda, ensuring
#     the saved object name matches the file basename (so that
#     data(<name>) loads an object called <name>).
#
# Run this BEFORE R CMD build, e.g.:
#   Rscript inst/scripts/convert_data_to_rda.R
#
# After running, data/ will contain only .rda files (the .rds versions are
# removed to keep the directory clean).

data_dir <- file.path("data")
if (!dir.exists(data_dir)) {
  stop("data/ directory not found. Run from package root.")
}

rds_files <- list.files(data_dir, pattern = "\\.rds$", full.names = TRUE)
if (length(rds_files) == 0) {
  message("No .rds files in data/. Nothing to convert.")
  quit(status = 0)
}

message(sprintf("Found %d .rds files to convert:", length(rds_files)))

for (f in rds_files) {
  obj_name <- tools::file_path_sans_ext(basename(f))
  obj <- readRDS(f)
  # Ensure the object in the .rda is named after the file basename
  assign(obj_name, obj)
  rda_path <- file.path(data_dir, paste0(obj_name, ".rda"))
  save(list = obj_name, file = rda_path, compress = "xz", version = 2)
  message(sprintf("  %s -> %s", basename(f), basename(rda_path)))
  # Remove the original .rds to avoid confusion
  unlink(f)
}

# Final listing
message("\nFinal data/ contents:")
for (f in list.files(data_dir, full.names = TRUE)) {
  message(sprintf("  %s (%.1f KB)", basename(f), file.info(f)$size / 1024))
}

# ==============================================================================

# File: R/utils_hubgene.R

# Purpose: HubGene network construction and visualization utilities

# ==============================================================================



# ------------------------------------------------------------------------------

# Modified: build_hubgene_network() function - Added de_df parameter passing

# ------------------------------------------------------------------------------



#' Build HubGene Network Data Structure (Enhanced with Leading Edge)

#'

#' Build complete network data with leading edge information for edges.

#' Now properly passes de_df to extract_hub_genes for stat extraction.

#'

#' @param gsea_task GseaTask object

#' @param pathway_ids Character vector of selected pathway IDs

#' @param min_hub_degree Minimum degree threshold for hub genes (default: 2)

#' @param de_df Optional DE data frame containing stat column

#' @param res_df Optional pathway result data frame

#' @param seed Random seed for reproducibility (default: 123)

#'

#' @return A list with three components:

#'   \item{nodes}{A list containing pathway and gene data frames}

#'   \item{edges}{Edge data frame with source, target, and leading edge info}

#'   \item{hub_df}{Hub gene data frame from extraction}

#'

#' @export



#' @examples

#' # Load pre-computed GseaRes shipped with the package and pick a task

#' data(precomputed_gseares, package = "GSEAlens")
#'
#' gsea_res <- precomputed_gseares

#' task <- extract_gsea_task(gsea_res, contrast_id = "untrt_vs_trt")

#'

#' # Pick top 3 enriched pathway IDs from the task result

#' res_df <- as.data.frame(task$gsea_res)

#' top_ids <- utils::head(res_df$ID, 3)

#'

#' # Build the hub-gene network from these pathways

#' network <- build_hubgene_network(task, top_ids)

#' names(network)

build_hubgene_network <- function(gsea_task, pathway_ids,

                                                                    min_hub_degree = 2,

                                                                    de_df = NULL,

                                                                    res_df = NULL,

                                                                    seed = 123) {

    # Extract hub genes (now includes leading edge info)

    # Fixed: Properly pass de_df parameter

    hub_df <- extract_hub_genes(gsea_task, pathway_ids, min_hub_degree, de_df = de_df)



    if (is.null(hub_df) || nrow(hub_df) == 0) {

    return(list(nodes = NULL, edges = NULL, hub_df = NULL))

    }



    # Extract gene sets from GSEA result

    gene_sets <- gsea_task$gsea_res@geneSets[pathway_ids]



    # Get pathway result info if not provided

    if (is.null(res_df)) {

    res_df <- as.data.frame(gsea_task$gsea_res@result)

    }



    # Build pathway nodes

    pathway_nodes <- data.frame(

    id = pathway_ids,

    type = "pathway",

    stringsAsFactors = FALSE

    )



    # Add pathway statistics

    pw_stats <- res_df[match(pathway_ids, res_df$ID), ]

    pathway_nodes$NES <- pw_stats$NES

    pathway_nodes$FDR <- pw_stats$p.adjust

    pathway_nodes$pvalue <- pw_stats$pvalue

    pathway_nodes$n_core <- pw_stats$setSize

    pathway_nodes$direction <- ifelse(pathway_nodes$NES > 0, "up", "down")



    # Count total genes per pathway

    pathway_nodes$n_total <- vapply(pathway_ids, function(pw) {

    length(gene_sets[[pw]])

    }, FUN.VALUE = integer(1L))



    # Calculate core ratio

    pathway_nodes$core_ratio <- pathway_nodes$n_core / pathway_nodes$n_total



    # Build gene nodes (only hub genes)

    hub_genes <- hub_df[hub_df$is_hub, ]



    gene_nodes <- data.frame(

    id = hub_genes$gene,

    type = "gene",

    degree = hub_genes$degree,

    stat = hub_genes$stat,

    pathways = hub_genes$pathways,

    stringsAsFactors = FALSE

    )



    # Gene direction based on stat

    gene_nodes$direction <- ifelse(gene_nodes$stat > 0, "up",

    ifelse(gene_nodes$stat < 0, "down", "neutral")

    )



    # Build edges with leading edge info

    edges_list <- lapply(seq_len(nrow(hub_genes)), function(i) {

    g <- hub_genes$gene[i]

    g_upper <- toupper(g)

    leading_edges <- hub_genes$pathway_leading_edges[[i]]



    connected_pws <- names(gene_sets)[vapply(gene_sets, function(x) g_upper %in% toupper(x), FUN.VALUE = logical(1L))]



    if (length(connected_pws) > 0) {

            edge_info <- lapply(connected_pws, function(pw) {

        is_leading <- if (pw %in% names(leading_edges)) {

                    leading_edges[[pw]]

        } else {

                    FALSE

        }



        data.frame(

                    source = g,

                    target = pw,

                    is_leading_edge = is_leading,

                    edge_width = ifelse(is_leading, 3, 1),

                    stringsAsFactors = FALSE

        )

            })



            return(do.call(rbind, edge_info))

    } else {

            return(NULL)

    }

    })



    if (length(edges_list) > 0 && !all(vapply(edges_list, is.null, FUN.VALUE = logical(1L)))) {

    # plyr was removed from DESCRIPTION; dplyr::bind_rows produces an

    # equivalent result here because every data.frame in edges_list is built

    # by the same code path above (same columns), so column-set alignment is

    # guaranteed. For heterogeneous column sets, bind_rows fills missing

    # columns with NA exactly like plyr::rbind.fill did.

    edges <- dplyr::bind_rows(edges_list)

    } else {

    edges <- NULL

    }



    return(list(

    nodes = list(pathway = pathway_nodes, gene = gene_nodes),

    edges = edges,

    hub_df = hub_df

    ))

}





# ------------------------------------------------------------------------------

# Modified: extract_hub_genes() function - Added debug output for diagnosis

# ------------------------------------------------------------------------------



#' Extract Hub Genes from Selected Pathways (Enhanced with Leading Edge)

#'

#' Extract hub genes with their leading edge status for each pathway.

#' A gene is considered a hub if it appears in at least min_degree pathways.

#'

#' @param gsea_task GseaTask object

#' @param pathway_ids Character vector of pathway IDs to analyze

#' @param min_degree Minimum number of pathways a gene must appear in (default: 2)

#' @param de_df Optional differential expression data frame with stat column

#'

#' @return A data frame with columns:

#'   \item{gene}{Gene symbol}

#'   \item{degree}{Number of pathways the gene appears in}

#'   \item{pathways}{Comma-separated list of pathway IDs}

#'   \item{is_hub}{Logical indicating if gene is a hub (degree >= min_degree)}

#'   \item{stat}{Statistic value from de_df or geneList}

#'   \item{pathway_leading_edges}{List column with per-pathway leading edge status}

#'

#' @export



#' @examples

#' # Load pre-computed GseaRes shipped with the package and pick a task

#' data(precomputed_gseares, package = "GSEAlens")
#'
#' gsea_res <- precomputed_gseares

#' task <- extract_gsea_task(gsea_res, contrast_id = "untrt_vs_trt")

#'

#' # Pick top 3 enriched pathway IDs from the task result

#' res_df <- as.data.frame(task$gsea_res)

#' top_ids <- utils::head(res_df$ID, 3)

#'

#' # Extract hub genes appearing in >= 2 of these pathways

#' hubs <- extract_hub_genes(task, top_ids, min_degree = 2)

#' utils::head(hubs)

extract_hub_genes <- function(gsea_task, pathway_ids, min_degree = 2, de_df = NULL) {

    # Validate inputs

    if (is.null(pathway_ids) || length(pathway_ids) == 0) {

    return(NULL)

    }



    # Extract gene sets from GSEA result

    gene_sets <- gsea_task$gsea_res@geneSets[pathway_ids]



    if (length(gene_sets) == 0) {

    return(NULL)

    }



    # ===========================================

    # Extract Leading Edge gene lists

    # ===========================================

    leading_edge_genes <- list()

    res_df <- as.data.frame(gsea_task$gsea_res@result)



    for (pw_id in pathway_ids) {

    row_idx <- which(res_df$ID == pw_id)

    if (length(row_idx) > 0) {

            core_str <- as.character(res_df$core_enrichment[row_idx[1]])

            if (!is.na(core_str) && core_str != "") {

        core_genes <- unlist(strsplit(core_str, "/"))

        core_genes <- toupper(trimws(core_genes))

        core_genes <- core_genes[core_genes != ""]

        leading_edge_genes[[pw_id]] <- unique(core_genes)

            } else {

        leading_edge_genes[[pw_id]] <- character(0)

            }

    } else {

            leading_edge_genes[[pw_id]] <- character(0)

    }

    }



    # ===========================================

    # Source filtering: keep only expressed genes

    # ===========================================



    valid_genes <- NULL



    if (!is.null(de_df)) {

    # Preferentially use genes with stat values from DE table

    if ("gene_symbol" %in% colnames(de_df) && "stat" %in% colnames(de_df)) {

            valid_genes <- de_df$gene_symbol[!is.na(de_df$stat)]

            valid_genes <- toupper(trimws(as.character(valid_genes)))

            valid_genes <- valid_genes[!is.na(valid_genes) & valid_genes != ""]

            valid_genes <- unique(valid_genes)

            message("[extract_hub_genes] Using stat from de_df, ", length(valid_genes), " genes with valid stat")

    } else if ("gene_symbol" %in% colnames(de_df) && "logFC" %in% colnames(de_df)) {

            valid_genes <- de_df$gene_symbol[!is.na(de_df$logFC)]

            valid_genes <- toupper(trimws(as.character(valid_genes)))

            valid_genes <- valid_genes[!is.na(valid_genes) & valid_genes != ""]

            valid_genes <- unique(valid_genes)

            message("[extract_hub_genes] Using logFC from de_df, ", length(valid_genes), " genes with valid logFC")

    }

    }



    # Fallback to geneList if DE table is unavailable

    if (is.null(valid_genes) || length(valid_genes) == 0) {

    if (!is.null(gsea_task$gsea_res@geneList)) {

            valid_genes <- toupper(names(gsea_task$gsea_res@geneList))

            valid_genes <- unique(valid_genes)

            message("[extract_hub_genes] Falling back to geneList, ", length(valid_genes), " genes")

    }

    }



    if (is.null(valid_genes) || length(valid_genes) == 0) {

    warning("[extract_hub_genes] Cannot determine valid genes, returning NULL")

    return(NULL)

    }



    # Filter gene_sets

    gene_sets <- lapply(gene_sets, function(genes) {

    genes_upper <- toupper(trimws(as.character(genes)))

    genes_upper <- genes_upper[!is.na(genes_upper) & genes_upper != ""]

    intersect(genes_upper, valid_genes)

    })



    gene_sets <- gene_sets[vapply(gene_sets, length, FUN.VALUE = integer(1L)) > 0]



    if (length(gene_sets) == 0) {

    return(NULL)

    }



    # ===========================================

    # Count gene occurrences across pathways

    # ===========================================



    all_genes <- unlist(gene_sets)

    gene_counts <- table(toupper(all_genes))



    gene_info <- data.frame(

    gene = names(gene_counts),

    degree = as.integer(gene_counts),

    stringsAsFactors = FALSE

    )



    gene_info$pathways <- vapply(gene_info$gene, function(g) {

    g_upper <- toupper(g)

    pws <- names(gene_sets)[vapply(gene_sets, function(x) g_upper %in% toupper(x), FUN.VALUE = logical(1L))]

    paste(pws, collapse = ", ")

    }, FUN.VALUE = character(1L))



    # Determine leading edge status for each pathway

    gene_info$pathway_leading_edges <- lapply(gene_info$gene, function(g) {

    g_upper <- toupper(g)

    leading_status <- vapply(pathway_ids, function(pw_id) {

            if (pw_id %in% names(leading_edge_genes)) {

        g_upper %in% toupper(leading_edge_genes[[pw_id]])

            } else {

        FALSE

            }

    }, FUN.VALUE = logical(1L))

    names(leading_status) <- pathway_ids

    return(leading_status)

    })



    gene_info$is_hub <- gene_info$degree >= min_degree



    # ===========================================

    # Critical fix: Properly read stat information

    # ===========================================



    if (!is.null(de_df) && is.data.frame(de_df)) {

    # Ensure gene_symbol column exists

    if ("gene_symbol" %in% colnames(de_df)) {

            # Create gene name to stat mapping

            de_df$gene_upper <- toupper(as.character(de_df$gene_symbol))



            # Prefer stat column

            if ("stat" %in% colnames(de_df)) {

        stat_map <- setNames(de_df$stat, de_df$gene_upper)

        gene_info$stat <- as.numeric(stat_map[gene_info$gene])

        message("[extract_hub_genes] Mapped stat from de_df$stat")

            }

            # Use logFC if no stat column

            else if ("logFC" %in% colnames(de_df)) {

        stat_map <- setNames(de_df$logFC, de_df$gene_upper)

        gene_info$stat <- as.numeric(stat_map[gene_info$gene])

        message("[extract_hub_genes] Mapped stat from de_df$logFC")

            }

            # Use log2FoldChange if no logFC column (DESeq2)

            else if ("log2FoldChange" %in% colnames(de_df)) {

        stat_map <- setNames(de_df$log2FoldChange, de_df$gene_upper)

        gene_info$stat <- as.numeric(stat_map[gene_info$gene])

        message("[extract_hub_genes] Mapped stat from de_df$log2FoldChange")

            } else {

        gene_info$stat <- 0

        warning("[extract_hub_genes] de_df has no stat/logFC/log2FoldChange column")

            }



            # Handle NA values

            gene_info$stat[is.na(gene_info$stat)] <- 0

    } else {

            gene_info$stat <- 0

            warning("[extract_hub_genes] de_df has no gene_symbol column")

    }

    } else {

    # No de_df available, use geneList values as fallback

    gene_list <- gsea_task$gsea_res@geneList

    if (!is.null(gene_list)) {

            stat_map <- setNames(as.numeric(gene_list), toupper(names(gene_list)))

            gene_info$stat <- as.numeric(stat_map[gene_info$gene])

            gene_info$stat[is.na(gene_info$stat)] <- 0

            message("[extract_hub_genes] Mapped stat from geneList (fallback)")

    } else {

            gene_info$stat <- 0

    }

    }



    gene_info$log2FC <- gene_info$stat



    # Sort by degree descending

    gene_info <- gene_info[order(gene_info$degree, decreasing = TRUE), ]



    return(gene_info)

}





#' Prepare HubGene Nodes for Plotting

#'

#' Prepare node data with layout positions for plotly visualization.

#'

#' @param network_data Output from \code{\link{build_hubgene_network}}

#' @param layout Layout algorithm: "fr" (Fruchterman-Reingold),

#'   "kk" (Kamada-Kawai), or "circle"

#' @param seed Random seed for reproducibility (default: 42)

#'

#' @return A named list with three components:

#'   \item{pathway}{Pathway node data frame with coordinates}

#'   \item{gene}{Gene node data frame with coordinates}

#'   \item{all}{Combined node data frame}

#'

#' @export



#' @examples

#' # Build a minimal network_data object (2 pathways + 2 shared genes)

#' network_data <- list(

#'   nodes = list(

#'     pathway = data.frame(

#'       id = c("PATHWAY_A", "PATHWAY_B"), type = "pathway",

#'       NES = c(2.0, -1.5), direction = c("up", "down"),

#'       stringsAsFactors = FALSE

#'     ),

#'     gene = data.frame(

#'       id = c("GENE_X", "GENE_Y"), type = "gene",

#'       stringsAsFactors = FALSE

#'     )

#'   ),

#'   edges = data.frame(

#'     from = c("PATHWAY_A", "PATHWAY_A", "PATHWAY_B", "PATHWAY_B"),

#'     to   = c("GENE_X", "GENE_Y", "GENE_X", "GENE_Y"),

#'     stringsAsFactors = FALSE

#'   ),

#'   hub_df = data.frame(gene = c("GENE_X", "GENE_Y"), degree = c(2, 2),

#'                       stringsAsFactors = FALSE)

#' )

#' nodes <- prepare_hubgene_nodes(network_data, layout = "fr", seed = 42)

#' if (!is.null(nodes)) head(nodes$all[, c("id", "type")])

prepare_hubgene_nodes <- function(network_data, layout = "fr", seed = 42) {

    if (is.null(network_data) || is.null(network_data$nodes)) {

    return(NULL)

    }



    nodes <- network_data$nodes

    edges <- network_data$edges



    if (nrow(edges) == 0) {

    return(NULL)

    }



    # Combine all nodes for igraph

    all_nodes <- dplyr::bind_rows(nodes$pathway, nodes$gene)

    all_nodes$id <- as.character(all_nodes$id)



    # Build igraph graph

    g <- igraph::graph_from_data_frame(

    edges,

    vertices = all_nodes,

    directed = FALSE

    )



    # Calculate layout

    # Only stochastic layouts (Fruchterman-Reingold) require a seed for reproducibility.

    # kk and circle are deterministic. withr::with_seed sets the seed locally and

    # restores the caller's RNG state on exit, so the user's global RNG is not polluted.

    # The seed parameter (default 42) is exposed to callers for reproducibility control.

    compute_layout <- function() {

    if (layout == "fr") {

            igraph::layout_with_fr(g)

    } else if (layout == "kk") {

            igraph::layout_with_kk(g)

    } else if (layout == "circle") {

            igraph::layout_in_circle(g)

    } else {

            igraph::layout_with_fr(g)

    }

    }



    # Default empty/null layout falls back to FR (stochastic); otherwise respect the

    # deterministic algorithm choice.

    needs_seed <- is.null(layout) || layout == "" || layout == "fr"

    coords <- if (needs_seed) {

    withr::with_seed(seed, compute_layout())

    } else {

    compute_layout()

    }



    # Add coordinates to nodes

    all_nodes$x <- coords[, 1]

    all_nodes$y <- coords[, 2]



    # Calculate node sizes

    # Pathway nodes: based on -log10(FDR) * scale

    if ("FDR" %in% colnames(all_nodes)) {

    all_nodes$size_pathway <- -log10(all_nodes$FDR + 1e-10) * 8 + 15

    all_nodes$size_pathway <- pmin(all_nodes$size_pathway, 40)

    } else {

    all_nodes$size_pathway <- 25

    }



    # Gene nodes: based on degree * scale

    if ("degree" %in% colnames(all_nodes)) {

    all_nodes$size_gene <- all_nodes$degree * 5 + 10

    all_nodes$size_gene <- pmin(all_nodes$size_gene, 30)

    } else {

    all_nodes$size_gene <- 15

    }



    # Assign final size based on type

    all_nodes$size <- ifelse(all_nodes$type == "pathway",

    all_nodes$size_pathway,

    all_nodes$size_gene

    )



    # Split back into pathway and gene nodes

    pathway_nodes_out <- all_nodes[all_nodes$type == "pathway", ]

    gene_nodes_out <- all_nodes[all_nodes$type == "gene", ]



    return(list(

    pathway = pathway_nodes_out,

    gene = gene_nodes_out,

    all = all_nodes

    ))

}





#' Color Nodes by Direction

#'

#' Assign colors based on NES (pathways) or log2FC (genes).

#'

#' @param node_data Node data frame with type and direction columns

#' @param color_mode Color mode: "logFC", "pathway", or "uniform"

#' @param left_group Left group name for legend label

#' @param right_group Right group name for legend label

#'

#' @return Node data with added color and color_label columns

#'

#' @export



#' @examples

#' # Build a minimal node data frame (pathway + gene nodes)

#' nodes <- data.frame(

#'   name   = c("P_POS", "P_NEG", "G_UP", "G_DOWN"),

#'   type   = c("pathway", "pathway", "gene", "gene"),

#'   NES    = c( 2.5, -2.0, NA, NA),

#'   log2FC = c( NA,  NA, 1.5, -1.2),

#'   stringsAsFactors = FALSE

#' )

#' colored <- color_by_direction(nodes, color_mode = "logFC")

#' head(colored$color)

color_by_direction <- function(node_data, color_mode = "logFC",

                                                                left_group = "A", right_group = "B") {

    # Define color palette (consistent with volcano/NES plots)

    color_up <- "#E41A1C" # Red for up-regulated

    color_down <- "#377EB8" # Blue for down-regulated

    color_neutral <- "#999999" # Gray for neutral

    color_hub <- "#FFD700" # Gold for hub genes



    if (color_mode == "logFC") {

    # For gene nodes: color by log2FC

    # For pathway nodes: color by NES



    node_data$color <- vapply(seq_len(nrow(node_data)), function(i) {

            row <- node_data[i, ]



            if (row$type == "pathway") {

        # Pathway: color by NES direction

        if (!is.na(row$NES)) {

                    if (row$NES > 0) {

            return(color_up)

                    } else if (row$NES < 0) {

            return(color_down)

                    }

        }

        return(color_neutral)

            } else if (row$type == "gene") {

        # Gene: color by log2FC

        if (!is.na(row$log2FC)) {

                    if (row$log2FC > 0) {

            return(color_up)

                    } else if (row$log2FC < 0) {

            return(color_down)

                    }

        }

        return(color_neutral)

            }



            return(color_neutral)

    }, FUN.VALUE = character(1L))

    } else if (color_mode == "pathway") {

    # Color genes by which pathway they connect to most

    node_data$color <- color_hub # Default hub color

    } else {

    # Uniform color

    node_data$color <- color_neutral

    }



    # Add color legend labels

    node_data$color_label <- vapply(seq_len(nrow(node_data)), function(i) {

    row <- node_data[i, ]



    if (row$type == "pathway") {

            if (!is.na(row$NES)) {

        if (row$NES > 0) {

                    return(paste0("Up in ", left_group))

        } else {

                    return(paste0("Up in ", right_group))

        }

            }

            return("Neutral")

    } else {

            if (!is.na(row$log2FC)) {

        if (row$log2FC > 0) {

                    return(paste0("Up in ", left_group))

        } else if (row$log2FC < 0) {

                    return(paste0("Up in ", right_group))

        }

            }

            return("Neutral")

    }

    }, FUN.VALUE = character(1L))



    return(node_data)

}





#' Generate HubGene Hover Text

#'

#' Generate HTML text for plotly hover tooltips.

#'

#' @param node Node data row (as list with named elements)

#' @param node_type Node type: "pathway" or "gene"

#' @param left_group Left group name

#' @param right_group Right group name

#'

#' @return Character string with HTML content for hover tooltip

#'

#' @export



#' @examples

#' # Example with a pathway node

#' pathway_node <- list(

#'   id = "HALLMARK_INFLAMMATORY_RESPONSE",

#'   NES = 2.3, FDR = 0.001, pvalue = 0.0005,

#'   n_core = 15, n_total = 50

#' )

#' hover_pathway <- generate_hubgene_hover_text(pathway_node, "pathway")

#' cat(hover_pathway)

#'

#' # Example with a gene node

#' gene_node <- list(id = "GENE_A", log2FC = 1.8)

#' hover_gene <- generate_hubgene_hover_text(gene_node, "gene")

#' cat(hover_gene)

generate_hubgene_hover_text <- function(node, node_type,

                                        left_group = "A",

                                        right_group = "B") {

    # Safe type conversion - ensure all numeric fields are properly converted

    safe_num <- function(x, default = 0) {

    if (is.null(x) || is.na(x)) {

            return(default)

    }

    x <- as.numeric(x)

    if (is.na(x)) {

            return(default)

    }

    return(x)

    }



    safe_char <- function(x, default = "") {

    if (is.null(x) || is.na(x) || is.factor(x)) {

            return(default)

    }

    return(as.character(x))

    }



    if (node_type == "pathway") {

    # Pathway hover text

    nes <- safe_num(node$NES)

    fdr <- safe_num(node$FDR)

    pval <- safe_num(node$pvalue)

    n_core <- safe_num(node$n_core, 0)

    n_total <- safe_num(node$n_total, 1)

    core_ratio <- if (n_total > 0) n_core / n_total else 0



    direction_text <- if (nes > 0) {

            paste0("Up in ", left_group)

    } else {

            paste0("Up in ", right_group)

    }



    id <- safe_char(node$id, "Unknown")



    hover <- sprintf(

            "<b style='font-size:14px;'>%s</b><br><br>

            <table style='font-size:12px;'>

        <tr><td><b>NES:</b></td><td>%.3f</td></tr>

        <tr><td><b>FDR:</b></td><td>%.2e</td></tr>

        <tr><td><b>p-value:</b></td><td>%.2e</td></tr>

        <tr><td><b>Direction:</b></td><td>%s</td></tr>

        <tr><td><b>Core Genes:</b></td><td>%d / %d (%.1f%%)</td></tr>

            </table>",

            id,

            nes,

            fdr,

            pval,

            direction_text,

            as.integer(n_core),

            as.integer(n_total),

            core_ratio * 100

    )

    } else if (node_type == "gene") {

    # Gene hover text

    id <- safe_char(node$id, "Unknown")

    log2fc <- safe_num(node$log2FC)

    degree <- safe_num(node$degree, 0)

    pathways <- safe_char(node$pathways, "None")

    color <- safe_char(node$color, "#999999")



    if (log2fc > 0) {

            direction_text <- paste0("Up in ", left_group)

    } else if (log2fc < 0) {

            direction_text <- paste0("Up in ", right_group)

    } else {

            direction_text <- "Neutral"

    }



    n_pathways <- length(unlist(strsplit(pathways, ", ")))



    hover <- sprintf(

            "<b style='font-size:14px; color:%s;'>%s</b><br><br>

            <table style='font-size:12px;'>

        <tr><td><b>log2FC:</b></td><td>%.3f</td></tr>

        <tr><td><b>Direction:</b></td><td>%s</td></tr>

        <tr><td><b>Hub Degree:</b></td><td>%d</td></tr>

        <tr><td><b>Pathways:</b></td><td>%d</td></tr>

            </table><br>

            <b>Appears in:</b><br>

            <small>%s</small>",

            color,

            id,

            log2fc,

            direction_text,

            as.integer(degree),

            n_pathways,

            pathways

    )

    }



    return(hover)

}





#' Get Color Legend Configuration

#'

#' Generate color legend data for HubGene network plot.

#'

#' @param left_group Left group name (e.g., treatment or case)

#' @param right_group Right group name (e.g., control or reference)

#'

#' @return A named list containing:

#'   \item{pathway_up}{Color for up-regulated pathways}

#'   \item{pathway_down}{Color for down-regulated pathways}

#'   \item{gene_up}{Color for up-regulated genes}

#'   \item{gene_down}{Color for down-regulated genes}

#'   \item{labels}{Named list of legend labels}

#'

#' @export



#' @examples

#' legend <- get_hubgene_legend("Control", "Treatment")

get_hubgene_legend <- function(left_group = "A", right_group = "B") {

    list(

    pathway_up = "#E41A1C",

    pathway_down = "#377EB8",

    gene_up = "#E41A1C",

    gene_down = "#377EB8",

    labels = list(

            pathway_up = paste0("Pathway: Up in ", left_group),

            pathway_down = paste0("Pathway: Up in ", right_group),

            gene_up = paste0("Gene: Up in ", left_group),

            gene_down = paste0("Gene: Up in ", right_group),

            hub_gene = "Hub Gene (degree >= 2)"

    )

    )

}

# Tests for color_by_direction()

# Type: pure function (no dependencies)

# Coverage: NES-based pathway coloring; log2FC-based gene coloring;

#           NA handling; row/column preservation; color_label generation

#

# Naming convention for fixtures:

#   PATHWAY_POS / PATHWAY_NEG : pathway nodes with positive / negative NES

#   PATHWAY_ZERO / PATHWAY_NA : pathway nodes with NES == 0 / NA

#   GENE_UP / GENE_DOWN        : gene nodes with positive / negative log2FC

#   GENE_ZERO / GENE_NA        : gene nodes with log2FC == 0 / NA

# These names describe the test condition (not biology).



# Helper: build a minimal node_data frame consistent with the function contract

build_test_nodes <- function() {

  data.frame(

    name = c("PATHWAY_POS", "PATHWAY_NEG", "PATHWAY_ZERO", "PATHWAY_NA",

             "GENE_UP", "GENE_DOWN", "GENE_ZERO", "GENE_NA"),

    type = c("pathway", "pathway", "pathway", "pathway",

             "gene", "gene", "gene", "gene"),

    NES    = c( 2.5,  -2.0,   0,  NA,    NA,   NA,   NA,   NA),

    log2FC = c( NA,   NA,  NA,  NA,    1.5, -1.2,   0,   NA),

    stringsAsFactors = FALSE

  )

}



test_that("color_by_direction colors pathways by NES sign in logFC mode", {

  nodes <- build_test_nodes()

  result <- color_by_direction(nodes, color_mode = "logFC")

  expect_equal(result$color[result$name == "PATHWAY_POS"],  "#E41A1C")  # NES > 0 -> red

  expect_equal(result$color[result$name == "PATHWAY_NEG"],  "#377EB8")  # NES < 0 -> blue

  expect_equal(result$color[result$name == "PATHWAY_ZERO"], "#999999")  # NES == 0 -> gray

})



test_that("color_by_direction colors genes by log2FC sign in logFC mode", {

  nodes <- build_test_nodes()

  result <- color_by_direction(nodes, color_mode = "logFC")

  expect_equal(result$color[result$name == "GENE_UP"],   "#E41A1C")  # logFC > 0 -> red

  expect_equal(result$color[result$name == "GENE_DOWN"], "#377EB8")  # logFC < 0 -> blue

  expect_equal(result$color[result$name == "GENE_ZERO"], "#999999")  # logFC == 0 -> gray

})



test_that("color_by_direction handles NA NES/log2FC as neutral gray", {

  nodes <- build_test_nodes()

  result <- color_by_direction(nodes, color_mode = "logFC")

  expect_equal(result$color[result$name == "PATHWAY_NA"], "#999999")  # pathway NA -> gray

  expect_equal(result$color[result$name == "GENE_NA"], "#999999")     # gene NA -> gray

})



test_that("color_by_direction preserves row order and adds color + color_label", {

  nodes <- build_test_nodes()

  result <- color_by_direction(nodes, color_mode = "logFC")

  expect_equal(nrow(result), nrow(nodes))

  expect_equal(nrow(nodes), 8L)

  expect_true(all(c("color", "color_label") %in% names(result)))

  # Row order preserved

  expect_equal(result$name, nodes$name)

})



test_that("color_by_direction generates color_label with group names", {

  nodes <- build_test_nodes()

  result <- color_by_direction(nodes, color_mode = "logFC",

                               left_group = "Treatment", right_group = "Control")

  expect_equal(result$color_label[result$name == "PATHWAY_POS"],

               "Up in Treatment")

  expect_equal(result$color_label[result$name == "PATHWAY_NEG"],

               "Up in Control")

})



test_that("color_by_direction uses uniform color in fallback mode", {

  nodes <- build_test_nodes()

  result <- color_by_direction(nodes, color_mode = "other")

  expect_true(all(result$color == "#999999"))

})

# ==============================================================================

# Section: Data Preprocessing Module UI

# ==============================================================================



#' @title Data Preprocessing Module UI

#' @description

#' User interface for the data preprocessing module, providing controls for:

#' \itemize{

#'   \item Contrast selection and gene set subgroup filtering

#'   \item Joint GSEA canvas for multi-pathway visualization

#'   \item Differential expression gene marker selection

#'   \item Group display order customization

#'   \item Expression metrics configuration

#' }

#' @param id Module namespace ID

#' @keywords internal



mod_data_prep_ui <- function(id) {

    ns <- shiny::NS(id)

    shiny::tagList(

    shiny::selectInput(

            ns("selected_contrast"),

            label = "Select Contrast",

            choices = NULL

    ),

    shiny::h4("Data Filtering"),

    shiny::selectizeInput(

            ns("selected_collections"),

            label = "Select Gene Set Subgroup:",

            choices = NULL,

            multiple = TRUE,

            options = list(plugins = list("remove_button"), placeholder = "Select subgroup or keep ALL")

    ),

    shiny::selectInput(

            ns("sort_by"),

            label = "Global Sorting Strategy:",

            choices = c(

        "By NES Absolute Value (Desc)" = "abs_nes_desc",

        "By NES (Desc)" = "nes_desc",

        "By P-value (Asc)" = "pval_asc"

            ),

            selected = "abs_nes_desc"

    ),

    shiny::actionButton(

            ns("run_btn"),

            label = "Confirm Contrast and Gene Sets",

            class = "btn-success",

            style = "width: 100%; font-weight: bold; margin-top: 15px; font-size: 16px;"

    ),

    shiny::helpText(

            style = "margin-top: 10px; text-align: center; color: #28a745;",

            "Click this button after modifying settings"

    ),

    shiny::hr(),

    shiny::h4("Joint GSEA Canvas"),

    shiny::selectizeInput(

            ns("joint_contrasts"),

            label = "Select Contrasts (Multi-select, supports permutations):",

            choices = NULL,

            multiple = TRUE,

            options = list(

        plugins = list("remove_button"),

        placeholder = "Select contrasts...",

        maxItems = 999

            )

    ),

    shiny::numericInput(

            ns("joint_ncol"),

            "Number per row (ncol):",

            value = 2,

            min = 1,

            max = 10,

            step = 1

    ),

    shiny::actionButton(

            ns("joint_generate"),

            "Generate/Update Multi-Pathway Canvas",

            class = "btn-success",

            style = "width: 100%; font-weight: bold; margin-top: 10px;"

    ),

    shiny::hr(),

    shiny::h4("Differential Expression Gene Markers"),

    shiny::div(

            style = "background-color: #e8f4f8; padding: 10px; border-radius: 5px; border-left: 4px solid #17a2b8;",

            shiny::selectizeInput(

        ns("pending_genes"),

        label = "Select Genes of Interest (Multi-select supported):",

        choices = NULL,

        multiple = TRUE,

        options = list(

                    plugins = list("remove_button"),

                    placeholder = "Enter gene names (e.g., TP53)",

                    maxItems = 999,

                    closeAfterSelect = FALSE,

                    selectOnTab = TRUE

        )

            ),

            shiny::actionButton(

        ns("apply_genes_btn"),

        label = "Confirm and Apply Gene Markers",

        class = "btn-info",

        style = "width: 100%; margin-top: 10px; font-weight: bold;"

            ),

            shiny::actionButton(

        ns("clear_all_genes_btn"),

        label = "Clear All",

        class = "btn-warning",

        style = "width: 100%; margin-top: 10px; font-weight: bold;"

            ),

            shiny::div(

        style = "background-color: #d4edda; padding: 10px; border-radius: 5px; margin-top: 10px;",

        shiny::HTML(

                    "<strong style='color: #155724;'>Confirmed Gene Markers:</strong><br>

    <small style='color: #666;'>Retained after switching contrasts</small>"

        ),

        shiny::uiOutput(ns("confirmed_genes_display"))

            ),

            shiny::uiOutput(ns("applied_genes_display")),

            shiny::hr(),

            shiny::h4("Group Display Order"),

            shiny::div(

        style = "background-color: #fff3cd; padding: 10px; border-radius: 5px; border-left: 4px solid #ffc107;",

        shiny::selectInput(

                    ns("boxplot_order_pending"),

                    label = "Group Ordering Options:",

                    choices = c("Default Order" = "default"),

                    selected = "default"

        ),

        shiny::actionButton(

                    ns("apply_order_btn"),

                    label = "Confirm Order (Refresh Boxplot)",

                    class = "btn-warning",

                    style = "width: 100%; margin-top: 10px; font-weight: bold;"

        ),

        shiny::uiOutput(ns("order_status_display")),

        shiny::helpText(

                    style = "margin-top: 8px; color: #666;",

                    "Click confirm after selecting order; boxplot will auto-sort"

        )

            ),

            shiny::hr(),

            shiny::h4("Joint Plotting Control"),

            shiny::selectInput(

        ns("plot_subtype"),

        label = "GSEAvis Style:",

        choices = c("1: Classic Enrichment Only" = "1", "2: Enrichment + Heatmap Strip" = "2", "3: Full Strip with Rank" = "3"),

        selected = "3"

            ),

            shiny::textInput(

        ns("custom_colors"),

        label = "Multi-Pathway Custom Colors:",

        value = "#E41A1C, #377EB8, #4DAF4A, #984EA3",

        placeholder = "e.g., #FF0000, #00FF00"

            ),

            shiny::hr(),

            shiny::h4("Expression Metrics"),

            shiny::selectInput(

        ns("expression_type"),

        label = "Select Expression Data Type:",

        choices = NULL,

        selected = NULL

            )

    )

    )

}



# ==============================================================================

# Section: Data Preprocessing Module Server

# ==============================================================================



#' @title Data Preprocessing Module Server

#' @description

#' Server logic for the data preprocessing module. Handles:

#' \itemize{

#'   \item Contrast and gene set selection

#'   \item Data processing and sorting

#'   \item Gene marker management (add, confirm, clear)

#'   \item Boxplot ordering configuration

#'   \item Joint canvas parameter passing

#'   \item Expression data type selection based on backend

#' }

#' @param id Module namespace ID

#' @param gsea_res GSEA results object containing contrast registry and backend info

#' @return A list of reactive functions and values:

#' \itemize{

#'   \item \code{data}: Reactive data object with processed GSEA results

#'   \item \code{highlight_genes}: Reactive character vector of confirmed gene markers

#'   \item \code{boxplot_order}: Reactive value for boxplot group ordering

#'   \item \code{joint_contrasts}: Reactive vector of contrasts for joint canvas

#'   \item \code{joint_ncol}: Reactive number of columns for joint canvas

#'   \item \code{joint_generate}: Event reactive for canvas regeneration trigger

#'   \item \code{update_pending_genes}: Function to update pending genes from modal

#'   \item \code{get_pending_genes}: Function to retrieve current pending genes

#' }

#' @keywords internal



mod_data_prep_server <- function(id, gsea_res) {

    shiny::moduleServer(id, function(input, output, session) {

    ns <- session$ns



    # === 新增：获取物种信息并更新 label ===

    species <- gsea_res$geneset_info$species %||% "HS"



    shiny::observe({

            shiny::updateSelectizeInput(

        session,

        "selected_collections",

        label = sprintf("Select Gene Set Subgroup [%s]:", species)

            )

    })

    # === 新增结束 ===



    result_data <- shiny::reactiveVal(NULL)

    has_initialized <- shiny::reactiveVal(FALSE)



    applied_genes <- shiny::reactiveVal(character(0))

    applied_boxplot_order <- shiny::reactiveVal("default")

    pending_genes_internal <- shiny::reactiveVal(character(0))



    canvas_contrasts_val <- shiny::reactiveVal(character(0))

    canvas_ncol_val <- shiny::reactiveVal(3)



    current_contrast_cache <- shiny::reactiveVal(NULL)

    pending_genes_internal <- shiny::reactiveVal(character(0))



    # 根据后端类型动态设置表达数据类型选项

    shiny::observe({

            backend <- gsea_res$backend_info$backend



            if (backend == "limma_voom") {

        choices <- c(

                    "log2(CPM)" = "logcpm",

                    "CPM (raw)" = "cpm",

                    "log2(FPKM)" = "logfpkm",

                    "FPKM (raw)" = "fpkm"

        )

        selected <- "cpm"

        message("Detected Limma-voom backend, loading corresponding expression types")

            } else if (backend == "deseq2") {

        choices <- c(

                    "log2(CPM)" = "logcpm",

                    "CPM (raw)" = "cpm",

                    "VST (Variance Stabilizing Transform)" = "vst",

                    "log2(Normalized Counts + 1)" = "lognorm"

        )

        selected <- "cpm"

        message("Detected DESeq2 backend, loading corresponding expression types")

            } else {

        choices <- c("log2(CPM)" = "logcpm", "CPM (raw)" = "cpm")

        selected <- "cpm"

            }



            shiny::updateSelectInput(

        session,

        "expression_type",

        choices = choices,

        selected = selected

            )

    })



    # 初始化对比组（包含排列：正向+反向）

    shiny::observe({

            registry <- gsea_res$contrast_registry

            all_contrasts <- list()



            # 构建排列：包含正向和反向

            for (i in seq_len(nrow(registry))) {

        row <- registry[i, ]

        # 正向

        all_contrasts[[row$contrast_id]] <- paste(row$left_group, "vs", row$right_group)

        # 反向（排列）

        rev_id <- paste(row$right_group, row$left_group, sep = "_vs_")

        rev_display <- paste(row$right_group, "vs", row$left_group, "(reversed)")

        all_contrasts[[rev_id]] <- rev_display

            }



            choices <- setNames(names(all_contrasts), unlist(all_contrasts))



            # 更新单选（主工作台）

            shiny::updateSelectInput(

        session,

        "selected_contrast",

        choices = choices,

        selected = registry$contrast_id[1]

            )



            # 更新多选（联合画布）- 默认全选所有排列

            shiny::updateSelectizeInput(

        session,

        "joint_contrasts",

        choices = choices,

        selected = names(all_contrasts)[seq_len(min(4, length(all_contrasts)))]

            )

    })





    # 基因列表更新（根治版：已确认基因独立维护）

    shiny::observeEvent(input$selected_contrast,

            {

        contrast_id <- input$selected_contrast

        shiny::req(contrast_id)



        if (!is.null(current_contrast_cache()) && current_contrast_cache() == contrast_id) {

                    return()

        }



        message(sprintf("Switching contrast: %s", contrast_id))

        current_contrast_cache(contrast_id)



        tryCatch(

                    {

            de_df <- get_de_table(gsea_res, contrast_id)

            if (!is.null(de_df) && "gene_symbol" %in% colnames(de_df)) {

                            gene_choices <- de_df$gene_symbol

                            gene_choices <- gene_choices[!is.na(gene_choices)]

                            gene_choices <- sort(unique(gene_choices))



                            current_applied <- applied_genes()



                            applied_upper <- toupper(current_applied)

                            choices_upper <- toupper(gene_choices)



                            selected_upper <- applied_upper[applied_upper %in% choices_upper]

                            pending_selected <- gene_choices[choices_upper %in% selected_upper]



                            pending_selected <- unique(pending_selected)



                            shiny::updateSelectizeInput(

                session,

                "pending_genes",

                choices = gene_choices,

                selected = pending_selected,

                server = length(gene_choices) > 1000

                            )



                            pending_genes_internal(pending_selected)



                            message(sprintf(

                "Pending refilled from applied (%d in current choices; %d total in applied)",

                length(pending_selected), length(current_applied)

                            ))



                            message(sprintf("Confirmed gene markers remain unchanged (%d genes)", length(current_applied)))

            }

                    },

                    error = function(e) {

            message("Failed to update gene list: ", e$message)

                    }

        )

            },

            ignoreInit = FALSE

    )



    # 生成排列组合

    generate_all_permutations <- function(groups) {

            n <- length(groups)

            if (n <= 1) {

        return(list(default = "Default Order"))

            }



            # Use an environment as a mutable accumulator to avoid the `<<-` super-assignment.

            # Semantics are equivalent to the previous recursive <<- pattern, but the

            # scope of the mutation is explicit and does not leak into parent frames.

            state <- new.env(parent = emptyenv())

            state$perms <- list()



            permute <- function(arr, l, r) {

        if (l == r) {

                    perm_str <- paste(arr, collapse = ",")

                    label_str <- paste(arr, collapse = "->")

                    state$perms[[perm_str]] <- label_str

                    return()

        }

        for (i in l:r) {

                    tmp <- arr[l]

                    arr[l] <- arr[i]

                    arr[i] <- tmp

                    permute(arr, l + 1, r)

                    tmp <- arr[l]

                    arr[l] <- arr[i]

                    arr[i] <- tmp

        }

            }

            permute(groups, 1, n)

            return(as.list(state$perms))

    }



    generate_limited_perms <- function(groups, max_perms = 100, seed = 123) {

            perms <- list()

            n <- length(groups)

            count <- 0



            perms[[paste(groups, collapse = ",")]] <- paste(groups, collapse = "->")

            count <- count + 1



            perms[[paste(rev(groups), collapse = ",")]] <- paste(rev(groups), collapse = "->")

            count <- count + 1



            for (i in 2:min(n, max_perms / 2)) {

        shifted <- groups[c(i:n, seq_len(i - 1))]

        perms[[paste(shifted, collapse = ",")]] <- paste(shifted, collapse = "->")

        count <- count + 1

        if (count >= max_perms) break

            }



            if (count < max_perms) {

        # Local RNG scope: seed is required for reproducible random permutation

        # sampling when max_perms exceeds deterministic (shift-based) permutations.

        # The seed is exposed to users via the `seed` parameter (default 123).

        # withr::local_seed restores the caller's RNG state on function exit.

        withr::local_seed(seed)

        for (i in seq_len(max_perms - count)) {

                    shuffled <- sample(groups)

                    perm_str <- paste(shuffled, collapse = ",")

                    if (!(perm_str %in% names(perms))) {

            perms[[perm_str]] <- paste(shuffled, collapse = "->")

                    }

        }

            }



            return(perms)

    }



    # 动态更新Boxplot排序选项

    shiny::observe({

            sample_meta <- tryCatch(get_sample_meta(gsea_res), error = function(e) NULL)

            if (is.null(sample_meta) || !"group" %in% colnames(sample_meta)) {

        return()

            }



            all_groups <- levels(sample_meta$group)

            if (length(all_groups) <= 1) {

        return()

            }



            total_perms <- factorial(length(all_groups))



            if (total_perms > 100) {

        perms <- generate_limited_perms(all_groups, max_perms = 100)

        perms[["limited"]] <- "--- Showing first 100 only ---"

            } else {

        perms <- generate_all_permutations(all_groups)

            }



            perms <- c(list(default = "Default Order"), perms)



            shiny::updateSelectInput(

        session,

        "boxplot_order_pending",

        choices = perms,

        selected = "default"

            )

    })



    # 基因确认按钮

    # 关键修复：input$pending_genes observer应该追加而非覆盖

    # 当用户在输入框手动输入时，只更新pending_genes_internal

    # 不再无条件覆盖Modal添加的基因

    shiny::observeEvent(input$pending_genes,

            {

        genes <- input$pending_genes

        if (is.null(genes)) genes <- character(0)



        # 清理基因名

        genes_clean <- toupper(trimws(genes))

        genes_clean <- genes_clean[genes_clean != ""]



        # 获取当前pending

        current_pending <- pending_genes_internal()



        # 判断：是追加还是覆盖？

        # 如果用户输入的基因不在当前pending中，认为是手动追加

        # 如果用户输入的基因与当前pending完全不同（通过手动清空后输入），认为是覆盖

        if (length(genes_clean) == 0) {

                    # 用户清空了输入框，保持pending不变

                    pending_genes_internal(current_pending)

        } else if (length(current_pending) == 0) {

                    # 当前pending为空，直接设置

                    pending_genes_internal(genes_clean)

        } else {

                    # 当前pending有值，比较差异

                    # 如果输入的内容比pending多很多，认为是手动输入（覆盖）

                    # 否则认为是追加

                    genes_not_in_pending <- setdiff(genes_clean, current_pending)

                    pending_not_in_genes <- setdiff(current_pending, genes_clean)



                    # 如果大部分pending都在输入中，且输入中有新基因，认为是追加

                    if (length(pending_not_in_genes) < length(current_pending) * 0.5 && length(genes_not_in_pending) > 0) {

            # 追加模式：保留原有 + 添加新的

            pending_genes_internal(unique(c(current_pending, genes_clean)))

                    } else {

            # 覆盖模式：直接替换

            pending_genes_internal(genes_clean)

                    }

        }



        message(sprintf("[PendingInput] User input updated: %d genes", length(genes_clean)))

            },

            ignoreNULL = FALSE,

            ignoreInit = TRUE

    )



    shiny::observeEvent(input$apply_genes_btn, {

            genes_to_apply <- pending_genes_internal()



            # 合并：已有的 + 新输入的

            current_applied <- applied_genes()

            new_applied <- union(toupper(current_applied), toupper(genes_to_apply))

            applied_genes(new_applied)



            message(sprintf(

        "Applied genes: %d (merged: %d existing + %d new)",

        length(new_applied), length(current_applied), length(genes_to_apply)

            ))

            shiny::showNotification(sprintf("Applied genes: %d (merged)", length(new_applied)),

        type = "message", duration = 3

            )

    })



    # 【新增】Clear All 按钮

    shiny::observeEvent(input$clear_all_genes_btn, {

            applied_genes(character(0))

            pending_genes_internal(character(0))

            shiny::updateSelectizeInput(

        session,

        "pending_genes",

        selected = character(0)

            )

            message("Cleared all applied genes")

            shiny::showNotification("Cleared all gene markers", type = "message", duration = 2)

    })



    # 排序确认按钮

    shiny::observeEvent(input$apply_order_btn,

            {

        order_to_apply <- input$boxplot_order_pending

        applied_boxplot_order(order_to_apply)



        message(sprintf(

                    "Order Confirm: User selected: %s | Quad-link module triggered update",

                    if (order_to_apply == "default") "default" else order_to_apply

        ))



        shiny::showNotification(

                    sprintf(

            "Order applied: %s | Boxplot refreshed",

            if (order_to_apply == "default") "Default order" else gsub(",", " -> ", order_to_apply)

                    ),

                    type = "message",

                    duration = 2

        )

            },

            ignoreInit = TRUE

    )



    # 显示实时排序状态

    output$order_status_display <- shiny::renderUI({

            pending <- input$boxplot_order_pending

            confirmed <- applied_boxplot_order()



            status_text <- if (pending == "default") {

        "Default Order"

            } else {

        sprintf("Pending: %s", gsub(",", " -> ", pending))

            }



            confirmed_text <- if (confirmed != "default" && !is.na(confirmed) && confirmed != "") {

        sprintf("<br>Applied: %s", gsub(",", " -> ", confirmed))

            } else {

        ""

            }



            shiny::div(

        style = "background: #e7f3ff; padding: 8px; border-radius: 4px; font-size: 12px; color: #004085; margin-top: 10px;",

        shiny::HTML(paste0(status_text, confirmed_text))

            )

    })



    # ═══════════════════════════════════════════════════════════════════════════

    # 修复：显示原始大小写的基因名

    # ═══════════════════════════════════════════════════════════════════════════

    output$confirmed_genes_display <- shiny::renderUI({

            genes <- applied_genes()



            if (length(genes) == 0) {

        shiny::span("(None)", style = "color: #999;")

            } else {

        # 按需重建 symbol_map（获取原始大小写）

        current_contrast <- input$selected_contrast

        if (!is.null(current_contrast) && current_contrast != "") {

                    symbol_map <- .rebuild_symbol_map(gsea_res, current_contrast)

                    display_genes <- .get_display_symbols(genes, symbol_map)

        } else {

                    display_genes <- genes

        }



        shiny::div(

                    style = "margin-top: 5px; padding: 8px; background: #f1f8f4; border-radius: 3px;",

                    shiny::strong(length(genes), " genes:", style = "color: #28a745;"),

                    shiny::br(),

                    shiny::span(paste(display_genes, collapse = ", "), style = "font-size: 12px; color: #555;")

        )

            }

    })



    # 监听画布参数变化

    shiny::observeEvent(input$canvas_contrasts,

            {

        canvas_contrasts_val(input$canvas_contrasts)

            },

            ignoreNULL = FALSE

    )



    shiny::observeEvent(input$canvas_ncol, {

            canvas_ncol_val(input$canvas_ncol)

    })





    # 核心数据处理

    process_data_core <- function(contrast_id, collections, sort_by, expr_type, plot_subtype, colors, is_auto = FALSE) {

            if (is.null(contrast_id)) {

        return(NULL)

            }



            task_obj <- tryCatch(

        {

                    extract_gsea_task(gsea_res, contrast_id, target_collection = collections)

        },

        error = function(e) {

                    message(sprintf("Task extraction failed: %s", e$message))

                    return(NULL)

        }

            )



            if (is.null(task_obj)) {

        return(NULL)

            }



            meta <- task_obj$meta

            gsea_res_obj <- task_obj$gsea_res

            df <- as.data.frame(gsea_res_obj@result)



            if (nrow(df) == 0) {

        return(NULL)

            }



            df$abs_NES <- abs(df$NES)

            sort_config <- switch(sort_by,

        "nes_desc" = list(col = "NES", desc = TRUE),

        "abs_nes_desc" = list(col = "abs_NES", desc = TRUE),

        "pval_asc" = list(col = "pvalue", desc = FALSE),

        list(col = "abs_NES", desc = TRUE)

            )



            sort_order <- order(df[[sort_config$col]], decreasing = sort_config$desc)

            df <- df[sort_order, ]

            df$Rank <- seq_len(nrow(df))

            df$Safe_ID <- gsub("'", "\\\\'", df$ID)



            reg_row <- gsea_res$contrast_registry[gsea_res$contrast_registry$contrast_id == contrast_id, ]



            if (nrow(reg_row) == 0) {

        parts <- strsplit(contrast_id, "_vs_")[[1]]

        if (length(parts) == 2) {

                    left <- parts[1]

                    right <- parts[2]

        } else {

                    left <- contrast_id

                    right <- "Background"

        }

            } else {

        left <- reg_row$left_group[1]

        right <- reg_row$right_group[1]

            }



            list(

        df = df,

        gsea_res = gsea_res_obj,

        task_obj = task_obj,

        left_group = left,

        right_group = right,

        contrast_id = contrast_id,

        contrast_name = sprintf("%s vs %s", left, right),

        expression_type = expr_type,

        plot_subtype = as.numeric(plot_subtype),

        custom_colors = colors,

        is_preview = is_auto,

        backend = gsea_res$backend_info$backend,

        joint_contrasts = canvas_contrasts_val(),

        joint_ncol = canvas_ncol_val(),

        collections = collections

            )

    }



    # 自动初始化

    shiny::observeEvent(input$selected_contrast,

            {

        if (has_initialized()) {

                    return()

        }



        shiny::invalidateLater(200, session)



        shiny::isolate({

                    if (is.null(input$selected_contrast)) {

            return()

                    }



                    task_temp <- tryCatch(

            extract_gsea_task(gsea_res, input$selected_contrast, "ALL"),

            error = function(e) NULL

                    )

                    if (is.null(task_temp)) {

            return()

                    }



                    df_temp <- as.data.frame(task_temp$gsea_res@result)

                    available <- unique(c(df_temp$Collection, df_temp$Combo_Name))

                    available <- setdiff(available, c(NA, "Unknown"))



                    if (length(available) == 0) {

            return()

                    }



                    # Sort collections: Hallmark (H/MH) first, then by main code

                    # (C1/M1, C2/M2, ...) in natural numeric order; within the same

                    # main code the main collection precedes its sub-collections

                    # (e.g. C2 before C2:CGP, C2 before C2:CP:REACTOME).

                    avail_chr <- as.character(available)

                    main_code <- sub(":.*$", "", avail_chr)

                    coll_num <- ifelse(

            main_code %in% c("H", "MH"), 0L,

            suppressWarnings(as.integer(gsub("[^0-9]", "", main_code)))

                    )

                    coll_num[is.na(coll_num)] <- 99L

                    has_sub <- grepl(":", avail_chr)

                    sub_part <- ifelse(has_sub, sub("^[^:]+:", "", avail_chr), "")

                    ord <- order(coll_num, has_sub, sub_part)

                    sorted_available <- avail_chr[ord]



                    # Default to Hallmark when present; otherwise fall back to the

                    # collection with the fewest entries (original heuristic).

                    hallmark <- intersect(c("H", "MH"), sorted_available)

                    if (length(hallmark) > 0) {

            min_col <- hallmark[1]

                    } else {

            counts <- vapply(sorted_available, function(x) {

                            sum(startsWith(as.character(df_temp$Combo_Name), x))

            }, numeric(1))

            min_col <- sorted_available[which.min(counts)[1]]

                    }



                    shiny::updateSelectizeInput(

            session,

            "selected_collections",

            choices = c("ALL", sorted_available),

            selected = min_col

                    )



                    auto_data <- process_data_core(

            input$selected_contrast,

            min_col,

            "abs_nes_desc",

            input$expression_type %||% "logcpm",

            "3",

            "#E41A1C, #377EB8, #4DAF4A, #984EA3",

            TRUE

                    )



                    if (!is.null(auto_data)) {

            result_data(auto_data)

            has_initialized(TRUE)

            shiny::showNotification(sprintf("Auto-loaded: %s", min_col), type = "default", duration = 3)

                    }

        })

            },

            ignoreNULL = TRUE,

            ignoreInit = TRUE

    )



    # 手动确认按钮

    shiny::observeEvent(input$run_btn, {

            shiny::req(input$selected_contrast)



            cols <- input$selected_collections

            if (is.null(cols) || length(cols) == 0) cols <- "ALL"



            manual_data <- process_data_core(

        input$selected_contrast,

        cols,

        input$sort_by,

        input$expression_type,

        input$plot_subtype,

        input$custom_colors,

        FALSE

            )



            if (is.null(manual_data)) {

        shiny::showNotification("No data for selected configuration", type = "error", duration = 5)

        return()

            }



            result_data(manual_data)

            shiny::showNotification("Workspace updated", type = "message", duration = 3)

    })



    # 画布生成触发器

    joint_generate_event <- shiny::eventReactive(input$joint_generate,

            {

        list(

                    contrasts = input$joint_contrasts,

                    ncol = input$joint_ncol,

                    timestamp = Sys.time()

        )

            },

            ignoreNULL = TRUE

    )



    # Modal传递基因接口（追加模式）- 完全重写

    update_pending_genes <- function(new_genes_from_modal) {

            # 防御性检查

            if (is.null(new_genes_from_modal) || length(new_genes_from_modal) == 0) {

        message("[UpdatePending] No new genes from modal, skipping")

        return(invisible(NULL))

            }



            # 清理基因名：去除空白 + 转大写

            new_genes_clean <- toupper(trimws(as.character(new_genes_from_modal)))

            new_genes_clean <- new_genes_clean[!is.na(new_genes_clean)]

            new_genes_clean <- new_genes_clean[new_genes_clean != ""]

            new_genes_clean <- unique(new_genes_clean)



            # 直接获取当前pending_genes_internal的值

            current_pending <- tryCatch(

        {

                    shiny::isolate(pending_genes_internal())

        },

        error = function(e) {

                    message(sprintf("[UpdatePending] Error: %s", e$message))

                    character(0)

        }

            )



            # 追加合并（不是替换）

            updated_pending <- unique(c(current_pending, new_genes_clean))



            # 更新reactive值

            pending_genes_internal(updated_pending)



            message(sprintf(

        "[UpdatePending] Merged: %d existing + %d new = %d total (pending)",

        length(current_pending), length(new_genes_clean), length(updated_pending)

            ))



            # 获取当前contrast的基因choices

            gene_choices_upper <- character(0)



            tryCatch(

        {

                    contrast_id <- shiny::isolate(input$selected_contrast)

                    if (!is.null(contrast_id) && contrast_id != "") {

            de_df <- get_de_table(gsea_res, contrast_id)

            if (!is.null(de_df) && "gene_symbol" %in% colnames(de_df)) {

                            gene_choices <- de_df$gene_symbol

                            gene_choices <- gene_choices[!is.na(gene_choices)]

                            gene_choices <- trimws(gene_choices)

                            gene_choices <- gene_choices[gene_choices != ""]

                            gene_choices_upper <- toupper(sort(unique(gene_choices)))

            }

                    }

        },

        error = function(e) {

                    message(sprintf("[UpdatePending] Error getting gene choices: %s", e$message))

        }

            )



            # 如果没有choices，使用pending作为choices

            if (length(gene_choices_upper) == 0) {

        gene_choices_upper <- updated_pending

            }



            # 刷新selectizeInput

            tryCatch(

        {

                    shiny::updateSelectizeInput(

            session,

            "pending_genes",

            choices = gene_choices_upper,

            selected = updated_pending,

            options = list(

                            plugins = list("remove_button"),

                            placeholder = "Enter gene names (e.g., TP53)",

                            maxItems = 999,

                            closeAfterSelect = FALSE,

                            selectOnTab = TRUE

            ),

            server = FALSE

                    )

                    message("[UpdatePending] UI refreshed")

        },

        error = function(e) {

                    message(sprintf("[UpdatePending] UI update error: %s", e$message))

        }

            )



            invisible(NULL)

    }





    return(list(

            data = shiny::reactive({

        result_data()

            }),

            highlight_genes = applied_genes,

            boxplot_order = applied_boxplot_order,

            joint_contrasts = shiny::reactive({

        input$joint_contrasts

            }),

            joint_ncol = shiny::reactive({

        input$joint_ncol

            }),

            joint_generate = joint_generate_event,

            update_pending_genes = update_pending_genes,

            get_pending_genes = function() {

        shiny::isolate(pending_genes_internal())

            }

    ))

    })

}

# Section: Visualization and Report Generation ----



#' GSEA Visualization and Report Generation

#'

#' Provides static plotting functions and interactive HTML report generation

#' capabilities. This is a documentation landing page for the visualization

#' family of functions in this file.

#'

#' @return NULL (this is a documentation landing page; individual functions

#'   return their own objects, documented separately).

#' @name visualization

NULL



#' @title Plot Directional GSEA

#' @description Professional-grade GSEA plotting engine with seamless support for single or multiple pathway visualization.

#'   Automatically generates elegant legends and native red-blue gene distribution track.

#' @param directional_gsea_obj A wrapped GseaTask object (returned by extract_gsea_task)

#' @param target_pathways Vector of pathway IDs to plot (supports single or multiple)

#' @param main_title Main title name

#' @param subPlot Controls the number of subplots generated (1: enrichment plot only; 2: enrichment + heatmap track; 3: full rank track)

#' @param curveCol Custom curve color vector

#' @param add_pval Whether to add statistical annotations on the plot. Default FALSE.

#' @param show_contrast_in_axis Logical. Whether to display contrast group information on the x-axis of the Rank plot.

#'   Default FALSE; recommended to set TRUE only in combined canvas mode.

#' @param ... Additional parameters passed to internal engine

#' @return ggplot2 composite object

#' @importFrom ggplot2 ggplot aes geom_col scale_fill_gradient2 geom_hline scale_x_continuous theme_bw theme element_blank element_text margin coord_cartesian labs geom_vline annotate scale_color_manual

#' @importFrom grDevices colorRampPalette

#' @importFrom patchwork plot_annotation wrap_plots

#' @importFrom stringr str_to_title str_wrap

#' @export



#' @examples

#' # Load pre-computed GseaRes and pick a task + an enriched pathway ID

#' data(precomputed_gseares, package = "GSEAlens")
#'
#' gsea_res <- precomputed_gseares

#' task <- extract_gsea_task(gsea_res, contrast_id = "untrt_vs_trt")

#' res_df <- as.data.frame(task$gsea_res)

#' top_pathway <- res_df$ID[1]

#'

#' # Render the directional GSEA plot (enrichment + rank + heatmap tracks)

#' p <- plot_directional_gsea(task, target_pathways = top_pathway)

#' class(p)

plot_directional_gsea <- function(directional_gsea_obj, target_pathways,

                                                                    main_title = NULL, subPlot = 3,

                                                                    curveCol = NULL, add_pval = FALSE,

                                                                    show_contrast_in_axis = FALSE,

                                                                    ...) {

    # 1. 解析对象与提取基础数据

    res <- directional_gsea_obj$gsea_res

    res@result$Description <- res@result$ID

    meta <- directional_gsea_obj$meta

    df <- as.data.frame(res)



    # Guard: filter out pathways not present in the current GSEA result

    # (e.g. stale IDs from a different gene set collection)

    valid_mask <- target_pathways %in% df$ID

    if (!all(valid_mask)) {

    missing_pws <- target_pathways[!valid_mask]

    warning(sprintf(

            "Skipping %d pathway(s) not found in GSEA results: %s",

            length(missing_pws), paste(missing_pws, collapse = ", ")

    ))

    target_pathways <- target_pathways[valid_mask]

    if (length(target_pathways) == 0) {

            stop("None of the selected pathways exist in the current GSEA result. Please re-select pathways from the active gene set collection.")

    }

    }



    n_lines <- length(target_pathways)



    # 1b. Unified label formatter: strip first "_" prefix, title-case the rest

    .format_pathway_label <- function(pw_id) {

    tit <- unlist(strsplit(pw_id, split = "_"))

    if (length(tit) > 1) {

            ft <- paste(stringr::str_to_title(tit[seq(2, length(tit))]), collapse = " ")

    } else {

            ft <- paste(stringr::str_to_title(tit), collapse = " ")

    }

    stringr::str_wrap(ft, width = 45)

    }



    # 1c. Auto-generate main_title when caller does not provide one

    if (is.null(main_title)) {

    main_title <- if (n_lines == 1) {

            .format_pathway_label(target_pathways[1])

    } else {

            sprintf("Combined Display: %d Pathway(s)", n_lines)

    }

    }



    # 2. 高级颜色池分配逻辑

    if (is.null(curveCol) || length(curveCol) < n_lines) {

    base_colors <- c(

            "#E41A1C", "#377EB8", "#4DAF4A", "#984EA3", "#FF7F00",

            "#A65628", "#F781BF", "#1B9E77", "#D95F02", "#7570B3"

    )

    curveCol <- if (n_lines <= length(base_colors)) {

            base_colors[seq_len(n_lines)]

    } else {

            grDevices::colorRampPalette(base_colors)(n_lines)

    }

    }



    curveCol_use <- curveCol[seq_len(n_lines)]

    curveCol_gsea <- curveCol_use

    if (n_lines == 1) {

    curveCol_gsea <- c(curveCol_use[1], curveCol_use[1])

    }



    # 3. 核心绘图（原生引擎）

    p_list <- .gsea_nb_core(

    object = res,

    geneSetID = target_pathways,

    subPlot = subPlot,

    curveCol = curveCol_gsea,

    addPval = add_pval,

    show_contrast_in_axis = show_contrast_in_axis,

    left_group = meta$left_group,

    right_group = meta$right_group,

    ...

    )


    # 4. 终极图例拦截与覆写逻辑

    if (n_lines > 1) {

    df_sub <- df[match(target_pathways, df$ID), ]

    raw_desc <- df_sub$Description

    name_id <- df_sub$ID



    nice_labels <- vapply(name_id, .format_pathway_label, character(1))



    names(curveCol_use) <- raw_desc

    override_scale <- ggplot2::scale_color_manual(

            name = "Term Name",

            values = curveCol_use,

            breaks = raw_desc,

            labels = nice_labels

    )

    p_list$p1 <- p_list$p1 + override_scale

    if (subPlot >= 2 && !is.null(p_list$p2)) {

            p_list$p2 <- p_list$p2 + override_scale

    }

    }



    # 5. 组合子图
    plots <- list(p_list$p1)
    if (subPlot >= 2 && !is.null(p_list$p2)) {
            plots[[2]] <- p_list$p2
    }
    if (subPlot == 3 && !is.null(p_list$p3)) {
            plots[[3]] <- p_list$p3
    }

    # Vertical height ratios (Panel1 : Panel2 : Panel3).
    # Single pathway: classic GSEA 0.5/0.2/0.3.
    # Multi-pathway: Panel 2 is faceted into N rows, so give it more vertical
    # room (0.30) so each facet gets usable height instead of being squished
    # and surrounded by inflated whitespace.
    if (length(plots) == 3) {
            heights <- if (n_lines >= 2) c(0.45, 0.30, 0.25) else c(0.5, 0.2, 0.3)
    } else if (length(plots) == 2) {
            heights <- if (n_lines >= 2) c(0.60, 0.40) else c(0.7, 0.3)
    } else {
            heights <- 1
    }

    p_base <- patchwork::wrap_plots(plots, ncol = 1, heights = heights)



    # 6. 添加结构化大标题

    p_final <- p_base + patchwork::plot_annotation(

    title = main_title,

    subtitle = sprintf("<- %s | %s ->", meta$left_group, meta$right_group),

    theme = ggplot2::theme(

            plot.title = ggplot2::element_text(size = 14, face = "bold", hjust = 0.5),

            plot.subtitle = ggplot2::element_text(size = 12, hjust = 0.5, color = "grey40")

    )

    )



    return(p_final)

}



# Section: HTML Report Generation ----



#' @title Generate GSEA HTML Report

#' @description Automatically avoids column name contamination, intelligently extracts expression matrices from objects,

#' and outputs comfortable web reports with dual p-value display and heatmaps.

#' Uses the ComplexHeatmap engine to draw high-quality heatmaps, perfectly replicating pheatmap aesthetics.

#' @param res_obj GseaTask object

#' @param output_base_dir Output folder path. If left empty (NULL), it will automatically follow the original project location of the calculation capsule.

#' @param p_adjust_cutoff FDR filtering threshold, default 1

#' @param top_plots Number of pathways to plot detailed sub-pages, format c(positive_count, negative_count)

#' @param dpi Resolution for GSEA enrichment plots, default 200

#' @return Path to the generated HTML report (invisible)

#' @importFrom ComplexHeatmap Heatmap HeatmapAnnotation draw

#' @importFrom circlize colorRamp2

#' @importFrom grid gpar

#' @export



#' @examples
#' \donttest{
#' data(precomputed_gseares, package = "GSEAlens")
#' task <- extract_gsea_task(precomputed_gseares, contrast_id = "untrt_vs_trt")
#' report_path <- generate_gsea_html_report(task,
#'                                          output_base_dir = tempdir())
#' }
generate_gsea_html_report <- function(res_obj, output_base_dir = NULL,

                                                                            p_adjust_cutoff = 1, top_plots = c(15, 15),

                                                                            dpi = 200) {

    if (!requireNamespace("DT", quietly = TRUE)) stop("Missing 'DT' package.")

    if (!requireNamespace("ComplexHeatmap", quietly = TRUE)) stop("Missing 'ComplexHeatmap' package.")

    if (!requireNamespace("circlize", quietly = TRUE)) stop("Missing 'circlize' package.")



    meta <- res_obj$meta

    res <- res_obj$gsea_res

    df <- as.data.frame(res)



    # 1. 自动寻路逻辑

    if (is.null(output_base_dir)) {

    if (!is.null(meta$project_info) && !is.null(meta$project_info$series_dir)) {

            output_base_dir <- file.path(

        meta$project_info$series_dir,

        sprintf("HTML_Report_%s_vs_%s", meta$left_group, meta$right_group)

            )

    } else {

            output_base_dir <- "GSEA_Results"

    }

    }



    bundle_dir <- output_base_dir

    if (!dir.exists(bundle_dir)) dir.create(bundle_dir, recursive = TRUE)

    details_dir <- file.path(bundle_dir, "Details")

    if (!dir.exists(details_dir)) dir.create(details_dir, recursive = TRUE)



    main_html_path <- file.path(bundle_dir, paste0(basename(bundle_dir), ".html"))



    # 2. 核心清洗区

    if (!is.null(meta$meta_dict)) {

    dict_cols <- colnames(meta$meta_dict)

    df_cols <- colnames(df)

    cols_to_add <- setdiff(dict_cols, df_cols)

    safe_dict <- meta$meta_dict |> dplyr::select(ID, dplyr::any_of(cols_to_add))

    df_clean <- df |> dplyr::left_join(safe_dict, by = "ID")

    } else {

    df_clean <- df

    }



    df_clean <- df_clean |>

    dplyr::filter(p.adjust <= p_adjust_cutoff)



    # Probe column availability up-front so the mutate() below can use base R
    # `if` (lazy evaluation) instead of the magrittr `.` placeholder, which
    # is undefined under native pipe `|>`.
    has_combo      <- "Combo_Name"     %in% names(df_clean)
    has_collection <- "Collection"     %in% names(df_clean)
    has_url        <- "URL"            %in% names(df_clean)
    has_desc_y     <- "Description.y"  %in% names(df_clean)



    df_clean <- df_clean |>

    dplyr::mutate(

            Enriched_In = factor(ifelse(NES > 0, meta$left_group, meta$right_group),

        levels = c(meta$left_group, meta$right_group)

            ),

            Display_Collection = if (has_combo) Combo_Name else if (has_collection) Collection else "Unknown",

            Display_Collection = as.factor(ifelse(is.na(Display_Collection), "Unknown", Display_Collection)),

            Pathway_Link = if (has_url) {

        ifelse(is.na(URL) | URL == "", sprintf("<b>%s</b>", ID),

                    sprintf('<a href="%s" target="_blank" style="color: #0056b3; text-decoration: none;">%s</a>', URL, ID)

        )

            } else {

        sprintf("<b>%s</b>", ID)

            },

            Description = if (has_desc_y) Description.y else Description

    ) |>

    dplyr::arrange(desc(abs(NES))) |>

    dplyr::mutate(Rank = dplyr::row_number())



    if (nrow(df_clean) == 0) {

    message("Warning: No significant pathways found under current cutoff. Skipping report generation.")

    return(invisible(NULL))

    }



    # 3. 表达矩阵智能雷达

    message("[Smart Radar] Detecting expression matrix for heatmap plotting...")

    expr_mat <- NULL

    cpm_mat <- NULL

    sample_meta_sub <- NULL

    n_left <- 0



    if (!is.null(meta$expr_bundle)) {

    expr_bundle <- meta$expr_bundle

    sample_meta <- expr_bundle$sample_meta



    if (!is.null(sample_meta) && "group" %in% colnames(sample_meta)) {

            left_samples <- rownames(sample_meta)[sample_meta$group == meta$left_group]

            right_samples <- rownames(sample_meta)[sample_meta$group == meta$right_group]

            target_samples <- c(left_samples, right_samples)



            if (length(target_samples) > 0) {

        message(sprintf(

                    "   [Heatmap Ready] Matched samples: Left group (%s) %d | Right group (%s) %d",

                    meta$left_group, length(left_samples), meta$right_group, length(right_samples)

        ))



        # 获取 display_expr (Z-score 底图)

        if (!is.null(expr_bundle$display_expr)) {

                    expr_mat <- expr_bundle$display_expr[, target_samples, drop = FALSE]

        } else if (!is.null(expr_bundle$raw_counts)) {

                    expr_mat <- log2(expr_bundle$raw_counts[, target_samples, drop = FALSE] + 1)

        }



        # 获取 raw_counts (用于显示数值)

        if (!is.null(expr_bundle$raw_counts)) {

                    cpm_mat <- edgeR::cpm(expr_bundle$raw_counts[, target_samples, drop = FALSE], log = FALSE)

        }



        sample_meta_sub <- sample_meta[target_samples, , drop = FALSE]

        n_left <- length(left_samples)

            }

    }

    }



    # 4. 遍历生成子图与详情页

    pos_plot_ids <- df_clean |>

    dplyr::filter(NES > 0) |>

    dplyr::arrange(desc(NES)) |>

    head(top_plots[1]) |>

    dplyr::pull(ID)

    neg_plot_ids <- df_clean |>

    dplyr::filter(NES < 0) |>

    dplyr::arrange(NES) |>

    head(top_plots[2]) |>

    dplyr::pull(ID)

    target_plot_ids <- c(pos_plot_ids, neg_plot_ids)



    detail_links <- character(nrow(df_clean))



    for (i in seq_len(nrow(df_clean))) {

    pw_id <- df_clean$ID[i]



    if (!(pw_id %in% target_plot_ids)) {

            detail_links[i] <- '<button class="btn btn-sm btn-outline-secondary" disabled style="padding: 2px 10px;">Skipped</button>'

            next

    }



    safe_pw_name <- gsub("[^A-Za-z0-9_.-]", "_", pw_id)

    detail_filename <- sprintf("Detail_Rank%03d_%s.html", i, safe_pw_name)

    gsea_png_name <- sprintf("GSEA_Rank%03d_%s.png", i, safe_pw_name)

    heat_png_name <- sprintf("Heatmap_Rank%03d_%s.png", i, safe_pw_name)



    # 绘制 GSEA 主图

    tryCatch(

            {

        p_gsea <- plot_directional_gsea(res_obj, target_pathways = pw_id, main_title = pw_id, subPlot = 3, add_pval = FALSE)

        ggplot2::ggsave(file.path(details_dir, gsea_png_name), plot = p_gsea, width = 8, height = 6, dpi = dpi, bg = "white")

            },

            error = function(e) {

        message(sprintf("Failed to plot %s: %s", pw_id, e$message))

            }

    )



    # 绘制热图 (ComplexHeatmap 升级版)

    heat_html_tag <- "<p class='text-muted' style='margin-top:20px;'>No expression data available for visualization.</p>"

    if (!is.null(expr_mat)) {

            all_genes <- res@geneSets[[pw_id]]

            match_list_idx <- which(toupper(rownames(expr_mat)) %in% toupper(all_genes))

            plot_genes <- rownames(expr_mat)[match_list_idx]



            if (length(plot_genes) >= 2) {

        plot_mat <- expr_mat[plot_genes, , drop = FALSE]

        plot_mat <- plot_mat[apply(plot_mat, 1, var) > 1e-6, , drop = FALSE]



        if (nrow(plot_mat) >= 2) {

                    # Z-score 标准化

                    z_mat <- t(scale(t(plot_mat)))

                    z_mat[is.na(z_mat)] <- 0

                    z_mat[z_mat > 1] <- 1

                    z_mat[z_mat < -1] <- -1



                    # 提取 CPM 数值 (用于显示)

                    display_numbers <- round(cpm_mat[rownames(z_mat), , drop = FALSE])



                    # ComplexHeatmap 配置

                    # 1. 颜色映射 (复刻 pheatmap 的淡蓝/白/淡红渐变)

                    col_fun <- circlize::colorRamp2(c(-1, 0, 1), c("#67a9cf", "#f7f7f7", "#ef8a62"))



                    # 2. 列注释 (分组条)

                    grp_col <- c("#E41A1C", "#377EB8")

                    names(grp_col) <- c(meta$left_group, meta$right_group)



                    top_ann <- ComplexHeatmap::HeatmapAnnotation(

            Group = sample_meta_sub$group,

            col = list(Group = grp_col),

            annotation_name_gp = grid::gpar(fontsize = 12, fontface = "bold"),

            simple_anno_size = grid::unit(0.6, "cm")

                    )



                    # 3. 行注释 (Leading Edge, 可选)

                    # leading_status <- ifelse(toupper(rownames(z_mat)) %in% toupper(core_genes), "YES", "NO")

                    # right_ann <- rowAnnotation(LeadingEdge = leading_status, ...)



                    # 4. 单元格渲染函数 (绘制数值)

                    cell_fun <- function(j, i, x, y, width, height, fill) {

            val <- display_numbers[i, j]

            # 保持原有设置：固定黑色字体，字号 13

            grid::grid.text(val, x, y, gp = grid::gpar(fontsize = 13, col = "black", fontface = "bold"))

                    }



                    # 5. 构建 Heatmap 对象

                    ht <- ComplexHeatmap::Heatmap(

            z_mat,

            name = "Z-Score",

            col = col_fun,

            cluster_rows = FALSE,

            cluster_columns = FALSE,

            column_split = factor(sample_meta_sub$group, levels = c(meta$left_group, meta$right_group)),

            cluster_column_slices = FALSE,

            top_annotation = top_ann,

            cell_fun = cell_fun,

            row_names_gp = grid::gpar(fontsize = 15),

            column_names_gp = grid::gpar(fontsize = 15, fontface = "bold"),

            rect_gp = grid::gpar(col = "white", lwd = 1),

            show_heatmap_legend = TRUE,

            width = NULL,

            height = NULL

                    )



                    # 6. 保存图片

                    png(file.path(details_dir, heat_png_name),

            width = 800, height = 600, res = 100

                    )

                    # IRON FIX (2026-07-01): ensure dev.off() runs even if draw() errors,
                    # otherwise the residual png device will swallow subsequent plots.
                    on.exit(dev.off(), add = TRUE)

                    ComplexHeatmap::draw(ht,

            merge_legend = TRUE,

            column_title = sprintf(

                            "Row-Scaled Z-Score [-1, 1]\nEnriched in: %s",

                            as.character(df_clean$Enriched_In[i])

            )

                    )



                    heat_html_tag <- sprintf("<img src='%s' class='img-fluid shadow-sm border'>", heat_png_name)

        }

            }

    }



    # 拼接 HTML

    html_content <- sprintf('

            <!DOCTYPE html><html><head><meta charset="utf-8"><title>Detail: %s</title>

            <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.1.3/dist/css/bootstrap.min.css" rel="stylesheet"></head>

            <body class="container mt-4 mb-5"><h2 class="text-primary">%s</h2>

            <h6 class="text-muted mb-4">Original ID: %s</h6>

            <p class="lead"><strong>Rank:</strong> %d | <strong>NES:</strong> %.3f | <strong>FDR:</strong> %.4e</p><hr>

            <div class="row mb-5"><div class="col-md-5"><h4 class="mb-3">Enrichment Plot</h4><img src="%s" class="img-fluid border shadow-sm"></div>

            <div class="col-md-7"><h4 class="mb-3">Expression Heatmap</h4><div style="overflow:auto; max-height:800px;">%s</div></div></div>

            </body></html>

    ', pw_id, pw_id, pw_id, i, df_clean$NES[i], df_clean$p.adjust[i], gsea_png_name, heat_html_tag)



    writeLines(html_content, con = file.path(details_dir, detail_filename))

    detail_links[i] <- sprintf('<a href="./Details/%s" target="_blank" class="btn btn-sm btn-success" style="padding: 2px 10px; text-decoration: none;">Dashboard</a>', detail_filename)

    }



    df_clean$Detail_Page <- detail_links



    # 5. 生成主交互式数据表

    display_df <- df_clean |>

    dplyr::select(Rank, Detail_Page, Pathway = Pathway_Link, Collection = Display_Collection, Enriched_In, Size = setSize, NES, pvalue, p.adjust, Description)



    dt_table <- DT::datatable(

    display_df,

    rownames = FALSE, escape = FALSE, filter = "top",

    caption = htmltools::tags$caption(

            style = "caption-side: top; text-align: center; font-size: 150%; font-weight: bold;",

            sprintf("GSEA Report: %s vs %s", meta$left_group, meta$right_group)

    ),

    extensions = c("Buttons", "Scroller"),

    options = list(dom = "Bfrtip", deferRender = TRUE, scrollY = 600, scroller = TRUE, pageLength = -1, buttons = c("copy", "csv", "excel"), autoWidth = TRUE)

    ) |>

    DT::formatRound(columns = c("NES"), digits = 3) |>

    DT::formatSignif(columns = c("pvalue", "p.adjust"), digits = 4) |>

    DT::formatStyle("Enriched_In", backgroundColor = DT::styleEqual(c(meta$left_group, meta$right_group), c("#fee0d2", "#deebf7"))) |>

    DT::formatStyle("NES", color = DT::styleInterval(0, c("blue", "red")), fontWeight = "bold")



    # IRON FIX (2026-07-01): use withr::local_dir so saveWidget errors do not
    # leak a half-changed working directory. withr is already in Imports.
    withr::local_dir(bundle_dir)
    htmlwidgets::saveWidget(dt_table,
    file = basename(main_html_path), selfcontained = FALSE, libdir = "lib",
    title = sprintf("GSEA Report: %s vs %s [%s]", meta$left_group, meta$right_group, meta$geneset_name)
    )



    message(sprintf("Done! HTML report generated at: %s", bundle_dir))

    return(invisible(bundle_dir))

}

# ==============================================================================

# File: R/vis-gsea-core.R

# Purpose: Internal GSEA visualization core (GseaVis-independent)

# ==============================================================================



#' @title Internal GSEA Data Extractor

#' @description Extract running score data from gseaResult object.

#'   Adapted from GseaVis::gsInfo, using enrichit::gseaScores internally.

#' @param object A gseaResult object.

#' @param geneSetID Character, gene set ID.

#' @return data.frame with runningScore, position, geneList, etc.

#' @keywords internal

.gs_info <- function(object, geneSetID) {

    gseaScores <- enrichit::gseaScores



    geneList <- object@geneList



    if (is.numeric(geneSetID)) {

    geneSetID <- object@result[geneSetID, "ID"]

    }



    geneSet <- object@geneSets[[geneSetID]]

    if (is.null(geneSet)) {

    stop(sprintf("Gene set '%s' not found in the GSEA result object. It may belong to a different gene set collection.", geneSetID))

    }

    exponent <- object@params[["exponent"]]



    df <- gseaScores(geneList, geneSet, exponent, fortify = TRUE)

    df$ymin <- 0

    df$ymax <- 0

    pos <- df$position == 1

    h <- diff(range(df$runningScore)) / 20

    df$ymin[pos] <- -h

    df$ymax[pos] <- h

    df$geneList <- geneList

    df$Description <- object@result[geneSetID, "Description"]



    return(df)

}





#' @title Internal GSEA Native Plotting Engine

#' @description Core plotting logic migrated from GseaVis::gseaNb,

#'   stripped of unused features (newGsea, filePath, KEGG, geneExpHt, etc.).

#'   Returns a list of ggplot objects for downstream assembly via patchwork.

#' @param object gseaResult object.

#' @param geneSetID Character vector of pathway IDs.

#' @param subPlot Integer, 1/2/3.

#' @param curveCol Curve color vector.

#' @param htCol Heatmap gradient colors.

#' @param lineSize Line width for enrichment curve.

#' @param addPval Logical, add NES/Pvalue text annotation (single pathway only).

#' @param pvalX X position for pval text (relative fraction).

#' @param pvalY Y position for pval text (relative fraction).

#' @param pvalSize Text size for pval annotation.

#' @param pCol Color for pval text.

#' @param nesDigit NES rounding digits.

#' @param pDigit P-value rounding digits.

#' @param show_contrast_in_axis Logical, if TRUE shows "Left vs Right" on X-axis.

#' @param left_group Left group label.

#' @param right_group Right group label.

#' @param base_size Theme base size.

#' @param subRatio Relative heights for 3 panels.

#' @param rankSeq X-axis break interval for rank plot.

#' @param htHeight Relative height of heatmap strip inside Panel 2.

#' @param htAlpha Alpha for heatmap rectangles.

#' @return Named list: p1 (curve), p2 (heatmap strip), p3 (rank distribution),

#'   data_ga, gsdata, glist.

#' @keywords internal

.gsea_nb_core <- function(object,

                                                    geneSetID,

                                                    subPlot = 3,

                                                    curveCol = c("#76BA99", "#EB4747", "#996699"),

                                                    htCol = c("#08519C", "#A50F15"),

                                                    lineSize = 0.8,

                                                    addPval = FALSE,

                                                    pvalX = 0.9,

                                                    pvalY = 0.9,

                                                    pvalSize = 4,

                                                    pCol = "grey0",

                                                    nesDigit = 2,

                                                    pDigit = 2,

                                                    show_contrast_in_axis = FALSE,

                                                    left_group = "Left",

                                                    right_group = "Right",

                                                    base_size = 14,

                                                    subRatio = c(0.5, 0.2, 0.3),

                                                    rankSeq = 5000,

                                                    htHeight = 0.3,

                                                    htAlpha = 0.8) {



    # ============================================================================

    # 1. Data preparation

    # ============================================================================

    glist <- object@geneList



    # Running score data for all pathways

    gsdata_list <- lapply(geneSetID, function(setid) {

    tmp <- .gs_info(object, geneSetID = setid)

    tmp$id <- setid

    tmp

    })

    gsdata <- dplyr::bind_rows(gsdata_list)



    # In-pathway gene positions (only used for tick marks in Panel 2)

    gsdata1_list <- lapply(unique(gsdata$Description), function(setid) {

    tmp <- gsdata[gsdata$Description == setid, ]

    tmp[tmp$position == 1, ]

    })

    gsdata1 <- dplyr::bind_rows(gsdata1_list)



    # Pathway statistics

    data_ga <- as.data.frame(object)

    data_ga <- data_ga[data_ga$ID %in% geneSetID, ]

    data_ga <- data_ga[match(unique(gsdata$id), data_ga$ID), ]



    n_pathways <- length(geneSetID)



    # ============================================================================

    # 2. Panel 1: Enrichment Curve

    # ============================================================================

    if (n_pathways == 1) {

    if (length(curveCol) == 1) {

            p1 <- ggplot2::ggplot(gsdata, ggplot2::aes(x = .data$x, y = .data$runningScore)) +

        ggplot2::geom_line(color = curveCol[1], linewidth = lineSize)

    } else {

            p1 <- ggplot2::ggplot(gsdata, ggplot2::aes(x = .data$x, y = .data$runningScore)) +

        ggplot2::geom_line(ggplot2::aes(color = .data$runningScore), linewidth = lineSize) +

        ggplot2::scale_color_gradient(low = curveCol[1], high = curveCol[2])

    }

    } else {

    mulcol <- curveCol[seq_len(min(n_pathways, length(curveCol)))]

    names(mulcol) <- geneSetID

    gsdata$id <- factor(gsdata$id, levels = geneSetID)



    p1 <- ggplot2::ggplot(gsdata, ggplot2::aes(x = .data$x, y = .data$runningScore)) +

            ggplot2::geom_line(ggplot2::aes(color = .data$id), linewidth = lineSize) +

            ggplot2::scale_color_manual(values = mulcol, name = "Term Name")

    }



    p1 <- p1 +

    ggplot2::geom_hline(yintercept = 0, color = "black", linetype = "dashed") +

    ggplot2::theme_bw(base_size = base_size) +

    ggplot2::scale_x_continuous(expand = c(0, 0)) +

    ggplot2::theme(

            legend.position = if (n_pathways == 1) "none" else "right",

            axis.text.x = ggplot2::element_blank(),

            axis.ticks.x = ggplot2::element_blank(),

            axis.title.x = ggplot2::element_blank(),

            axis.line.x = ggplot2::element_blank(),

            panel.grid = ggplot2::element_blank(),

            plot.margin = ggplot2::margin(t = 0, r = 0.2, b = 0, l = 0.2, unit = "cm")

    ) +

    ggplot2::ylab("Running Enrichment Score")



    # P-value annotation (single pathway only)

    if (isTRUE(addPval) && n_pathways == 1 && nrow(data_ga) > 0) {

    p_label <- paste0(

            "NES: ", round(data_ga$NES[1], digits = nesDigit), "\n",

            "Pvalue: ", ifelse(data_ga$pvalue[1] < 0.001, "< 0.001", round(data_ga$pvalue[1], digits = pDigit)), "\n",

            "Adjusted Pvalue: ", ifelse(data_ga$p.adjust[1] < 0.001, "< 0.001", round(data_ga$p.adjust[1], digits = pDigit))

    )



    px <- pvalX * nrow(gsdata[gsdata$id == geneSetID[1], ])

    py <- pvalY * sum(abs(range(gsdata$runningScore))) + min(gsdata$runningScore)



    p1 <- p1 +

            ggplot2::annotate(

        "text",

        x = px,

        y = py,

        label = p_label,

        size = pvalSize,

        color = pCol,

        fontface = "italic",

        hjust = 1

            )

    }



    # ============================================================================

    # 3. Panel 2: Heatmap Strip + Gene Ticks

    # ============================================================================

    p2 <- NULL



    if (subPlot >= 2) {

    # Bin geneList into 10 intervals for heatmap

    df_rank <- data.frame(pos = seq_along(glist), fc = glist)

    qt <- stats::quantile(df_rank$fc, probs = c(0.1, 0.9))



    df_rank <- df_rank |>

            dplyr::mutate(fc = dplyr::case_when(

        .data$fc >= qt[2] ~ qt[2],

        .data$fc <= qt[1] ~ qt[1],

        .default = .data$fc

            ))



    break_intervals <- cut(df_rank$fc, breaks = 10, include.lowest = TRUE)

    intervals <- levels(break_intervals)



    interval_bounds <- data.frame(do.call(rbind, strsplit(gsub("[()\\[\\]]", "", intervals), ",")))

    interval_bounds$X1 <- as.numeric(vapply(strsplit(interval_bounds$X1, split = "\\[|\\("), "[", character(1), 2))

    interval_bounds$X2 <- as.numeric(vapply(strsplit(interval_bounds$X2, split = "\\]|\\)"), "[", character(1), 1))



    start_positions <- vapply(interval_bounds[, 1], function(bound) which.min(abs(df_rank$fc - bound)), integer(1))

    start_positions[1] <- nrow(df_rank)

    end_positions <- vapply(interval_bounds[, 2], function(bound) which.min(abs(df_rank$fc - bound)), integer(1))

    interval_means <- tapply(df_rank$fc, break_intervals, mean)



    result_df <- data.frame(

            Interval = seq_len(10),

            Start = interval_bounds[, 1],

            End = interval_bounds[, 2],

            xmax = start_positions,

            xmin = end_positions,

            ymin = 0,

            ymax = htHeight,

            Mean_LogFC = as.numeric(interval_means)

    )



    # Replicate for each pathway

    result_df_all <- dplyr::bind_rows(lapply(unique(gsdata$id), function(setid) {

            result_df$id <- setid

            result_df

    }))



    # Factor ordering

    if (gsdata$id[1] == gsdata$Description[1]) {

            result_df_all$id <- factor(result_df_all$id, levels = geneSetID)

            gsdata1$id <- factor(gsdata1$id, levels = geneSetID)

    } else {

            result_df_all$id <- factor(result_df_all$id, levels = data_ga$ID)

            gsdata1$id <- factor(gsdata1$id, levels = data_ga$ID)

    }



    # Build plot

    p2 <- ggplot2::ggplot() +

            ggplot2::geom_rect(

        data = result_df_all,

        ggplot2::aes(

                    xmin = .data$xmin, xmax = .data$xmax,

                    ymin = .data$ymin, ymax = .data$ymax,

                    fill = .data$Mean_LogFC

        ),

        color = NA, inherit.aes = FALSE, alpha = htAlpha, show.legend = FALSE

            ) +

            ggplot2::scale_fill_gradient2(low = htCol[1], mid = "white", high = htCol[2], midpoint = 0)



    if (n_pathways > 1) {

            p2 <- p2 +

        ggplot2::geom_segment(

                    data = gsdata1,

                    ggplot2::aes(x = .data$x, xend = .data$x, y = 0, yend = 1, color = .data$id),

                    show.legend = FALSE

        ) +

        ggplot2::scale_color_manual(values = mulcol)

    } else {

            p2 <- p2 +

        ggplot2::geom_segment(

                    data = gsdata1,

                    ggplot2::aes(x = .data$x, xend = .data$x, y = 0, yend = 1),

                    color = "black",

                    show.legend = FALSE

        )

    }



    p2 <- p2 +

            ggplot2::scale_x_continuous(expand = c(0, 0)) +

            ggplot2::scale_y_continuous(expand = c(0, 0)) +

            ggplot2::theme_bw(base_size = base_size) +

            ggplot2::theme(

        axis.ticks = ggplot2::element_blank(),

        axis.text = ggplot2::element_blank(),

        axis.title.y = ggplot2::element_blank(),

        panel.grid = ggplot2::element_blank(),

        axis.line.x = ggplot2::element_blank(),

        strip.background = ggplot2::element_blank(),

        strip.text = ggplot2::element_blank(),

        panel.spacing = ggplot2::unit(0, "cm"),

        plot.margin = ggplot2::margin(t = 0, r = 0.2, b = 0, l = 0.2, unit = "cm")

            ) +

            ggplot2::xlab("Rank in Ordered Dataset")



    if (n_pathways > 1) {

            p2 <- p2 + ggplot2::facet_wrap(~id, ncol = 1)

    }



    if (subPlot > 2) {

            p2 <- p2 + ggplot2::theme(axis.title.x = ggplot2::element_blank())

    }

    }



    # ============================================================================

    # 4. Panel 3: Ranked List Distribution (Native Red-Blue)

    # ============================================================================

    p3 <- NULL



    if (subPlot == 3) {

    df_rank_plot <- data.frame(x = seq_along(glist), y = as.numeric(glist))



    x_axis_label <- if (isTRUE(show_contrast_in_axis)) {

            sprintf("%s vs %s", left_group, right_group)

    } else {

            "Rank in Ordered Dataset"

    }



    p3 <- ggplot2::ggplot(df_rank_plot, ggplot2::aes(x = .data$x, y = .data$y)) +

            ggplot2::geom_col(ggplot2::aes(fill = .data$y), width = 1, color = NA, show.legend = FALSE) +

            ggplot2::scale_fill_gradient2(low = "#08519C", mid = "white", high = "#A50F15", midpoint = 0) +

            ggplot2::geom_hline(yintercept = 0, linewidth = 0.5, color = "black", linetype = "dashed") +

            ggplot2::scale_x_continuous(breaks = seq(0, length(glist), rankSeq)) +

            ggplot2::theme_bw(base_size = base_size) +

            ggplot2::theme(

        panel.grid = ggplot2::element_blank(),

        axis.text = ggplot2::element_text(colour = "black"),

        plot.margin = ggplot2::margin(t = 0, r = 0.2, b = 0.2, l = 0.2, unit = "cm")

            ) +

            ggplot2::coord_cartesian(expand = 0) +

            ggplot2::labs(x = x_axis_label, y = "Ranked List")



    # Joint canvas mode: bold X-axis label

    if (isTRUE(show_contrast_in_axis)) {

            p3 <- p3 +

        ggplot2::theme(axis.title.x = ggplot2::element_text(face = "bold", size = 18))

    }



    # Zero cross and group annotations

    z_cross <- sum(glist > 0)

    m_rank <- length(glist)



    p3 <- p3 +

            ggplot2::geom_vline(xintercept = z_cross, linetype = "dashed", color = "grey50") +

            ggplot2::annotate(

        "text",

        x = z_cross, y = 0,

        label = paste0("Zero cross at ", z_cross),

        vjust = 1.5, hjust = -0.05, size = 3, color = "grey30"

            ) +

            ggplot2::annotate(

        "text",

        x = m_rank * 0.01, y = max(glist) * 0.85,

        label = sprintf("'%s' (pos)", left_group),

        color = "#A50F15", hjust = 0, size = 4, fontface = "italic"

            ) +

            ggplot2::annotate(

        "text",

        x = m_rank * 0.99, y = min(glist) * 0.85,

        label = sprintf("'%s' (neg)", right_group),

        color = "#08519C", hjust = 1, size = 4, fontface = "italic"

            )

    }



    # ============================================================================

    # 5. Return

    # ============================================================================

    return(list(

    p1 = p1,

    p2 = p2,

    p3 = p3,

    data_ga = data_ga,

    gsdata = gsdata,

    glist = glist

    ))

}

#' @title Core Class Definitions and Validation Functions

#' @description Define GseaEnv, GseaRes, GseaTask structures and provide internal validation logic.

#' @keywords internal

#' @name class_validations



NULL





# 1. Core Class Definitions





#' @title Define GseaEnv Class

#' @description Standardized GSEA input environment object.

#' @param backend_info Backend information object

#' @param contrast_registry Contrast registry

#' @param de_store Differential expression store

#' @param expr_bundle Expression data bundle

#' @param geneset Gene set collection

#' @param raw_obj Raw data object

#' @return A GseaEnv object

#' @examples

#' gsea_env <- create_gsea_env(

#'   backend_info = list(),

#'   contrast_registry = data.frame(),

#'   de_store = data.frame(),

#'   expr_bundle = list(),

#'   geneset = character(),

#'   raw_obj = NULL

#' )

#' @export



create_gsea_env <- function(backend_info, contrast_registry, de_store, expr_bundle, geneset, raw_obj) {

    structure(

    list(

            backend_info = backend_info,

            contrast_registry = contrast_registry,

            de_store = de_store,

            expr_bundle = expr_bundle,

            geneset = geneset,

            raw_backend_obj = raw_obj # 保留原始对象以备不时之需，但下游不应直接访问

    ),

    class = "GseaEnv"

    )

}



#' @title Define GseaRes Class

#' @description GSEA computation result capsule.

#' @param metadata Metadata information

#' @param backend_info Backend information object

#' @param contrast_registry Contrast registry

#' @param de_store Differential expression store

#' @param expr_bundle Expression data bundle

#' @param geneset_info Gene set information

#' @param results GSEA results

#' @return A GseaRes object

#' @examples

#' gsea_res <- create_gsea_res(

#'   metadata = list(),

#'   backend_info = list(),

#'   contrast_registry = data.frame(),

#'   de_store = data.frame(),

#'   expr_bundle = list(),

#'   geneset_info = list(),

#'   results = list()

#' )

#' @export



create_gsea_res <- function(metadata, backend_info, contrast_registry, de_store, expr_bundle, geneset_info, results) {

    structure(

    list(

            metadata = metadata,

            backend_info = backend_info,

            contrast_registry = contrast_registry,

            de_store = de_store,

            expr_bundle = expr_bundle,

            geneset_info = geneset_info,

            results = results

    ),

    class = "GseaRes"

    )

}



#' @title Define GseaTask Class

#' @description Single contrast extraction result object.

#' @param gsea_res GseaRes object

#' @param meta Metadata for the task

#' @return A GseaTask object

#' @examples

#' gsea_task <- create_gsea_task(

#'   gsea_res = NULL,

#'   meta = data.frame()

#' )

#' @export



create_gsea_task <- function(gsea_res, meta) {

    structure(

    list(

            gsea_res = gsea_res,

            meta = meta

    ),

    class = "GseaTask"

    )

}





# 2. Internal Validation Functions





#' @title Validate Limma Design Matrix

#' @description Enforce no-intercept design (~ 0 + group) to ensure accurate contrast parsing.

#' @param fit MArrayLM object from limma

#' @return TRUE if validation passes, otherwise stops with an error

#' @keywords internal



.validate_limma_design <- function(fit) {

    # 检查是否包含截距项

    # Intercept / (Intercept) 是 R 中默认的截距命名

    design_matrix <- fit$design

    has_intercept <- any(grepl("Intercept", colnames(design_matrix), ignore.case = TRUE))



    if (has_intercept) {

    rlang::abort(

            c(

        "Intercept term detected in limma design matrix.",

        i = "GSEAlens requires a no-intercept design (~ 0 + group).",

        i = "Example: design <- model.matrix(~ 0 + group, data = samples); fit <- lmFit(expr, design)",

        i = "Reason: No-intercept design makes colnames(fit) directly correspond to group names."

            ),

            .class = "GSEAlens_limma_intercept_detected"

    )

    }



    # 检查是否有足够的列进行对比

    if (ncol(design_matrix) < 2) {

    rlang::warn(

            "Design matrix contains only 1 column; no between-group comparisons possible.",

            .class = "GSEAlens_limma_single_col"

    )

    }



    return(TRUE)

}



#' @title Validate DESeq2 Target Factor

#' @description Check if target_factor exists in colData and is properly formatted.

#' @param dds DESeqDataSet object

#' @param target_factor String, the target factor to validate

#' @return TRUE if validation passes, otherwise stops with an error

#' @keywords internal



.validate_deseq2_design <- function(dds, target_factor) {

    col_data <- as.data.frame(SummarizedExperiment::colData(dds))



    if (!target_factor %in% colnames(col_data)) {

    rlang::abort(

            c(

        sprintf("target_factor '%s' not found in colData.", target_factor),

        i = sprintf("Available column names: %s", paste(colnames(col_data), collapse = ", "))

            ),

            target_factor = target_factor,

            .class = "GSEAlens_deseq2_missing_factor"

    )

    }



    # 检查是否为因子

    if (!is.factor(col_data[[target_factor]])) {

    rlang::warn(

            sprintf("target_factor '%s' is not a factor type; attempting automatic conversion...", target_factor),

            .class = "GSEAlens_deseq2_factor_coerce"

    )

    # 这里不实际转换，只是警告，因为 DESeq2 通常在构建时已处理

    }



    return(TRUE)

}



#' @title Validate GseaEnv Object Integrity

#' @description Internal function to ensure object structure conforms to specification.

#' @param env_obj GseaEnv object to validate

#' @return TRUE if validation passes, otherwise stops with an error

#' @keywords internal



.check_gsea_env <- function(env_obj) {

    if (!inherits(env_obj, "GseaEnv")) {

    rlang::abort(

            "Input object is not of class GseaEnv.",

            .class = "GSEAlens_bad_class"

    )

    }



    required_slots <- c("backend_info", "contrast_registry", "de_store", "expr_bundle", "geneset")

    missing_slots <- setdiff(required_slots, names(env_obj))



    if (length(missing_slots) > 0) {

    rlang::abort(

            c(

        "GseaEnv object structure is incomplete.",

        i = sprintf("Missing slots: %s", paste(missing_slots, collapse = ", "))

            ),

            missing_slots = missing_slots,

            .class = "GSEAlens_incomplete_env"

    )

    }



    # 检查 contrast_registry 必要字段

    reg <- env_obj$contrast_registry

    if (!is.data.frame(reg) || !all(c("contrast_id", "left_group", "right_group") %in% colnames(reg))) {

    rlang::abort(

            "contrast_registry must contain columns: contrast_id, left_group, right_group.",

            .class = "GSEAlens_bad_registry"

    )

    }



    return(TRUE)

}



#' @title Validate GseaRes Object Integrity

#' @description Internal function to ensure GseaRes object contains required components.

#' @param res_obj GseaRes object to validate

#' @return TRUE if validation passes, otherwise stops with an error

#' @keywords internal



.check_gsea_res <- function(res_obj) {

    if (!inherits(res_obj, "GseaRes")) {

    rlang::abort(

            "Input object is not of class GseaRes.",

            .class = "GSEAlens_bad_class"

    )

    }

    # 简单检查 results 列表是否存在

    if (is.null(res_obj$results)) {

    rlang::abort(

            "No computation results in GseaRes object.",

            .class = "GSEAlens_empty_res"

    )

    }

    return(TRUE)

}

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE, warning = FALSE, message = FALSE)

## ----install-package, eval=FALSE----------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# BiocManager::install("GSEAlens")

## ----install-github, eval=FALSE-----------------------------------------------
# if (!requireNamespace("pak", quietly = TRUE))
#     install.packages("pak")
# pak::pkg_install("DDL095/GSEAlens")

## ----setup-environment, results='hide'----------------------------------------
library(GSEAlens)
library(airway)

## ----load-prepared-objects----------------------------------------------------
data(preprocessed_limma, package = "GSEAlens")
preproc_limma         <- preprocessed_limma
fit                   <- preproc_limma$fit
gsea_limma_voom_data  <- preproc_limma$gsea_limma_voom_data
data(preprocessed_dds_se, package = "GSEAlens")
dds_se <- preprocessed_dds_se
data(preprocessed_dds, package = "GSEAlens")
dds <- preprocessed_dds

## ----build-gsea-pathways, eval=FALSE------------------------------------------
# # 实际调用（在 Bioconductor 构建机上较慢，因需加载多个 MSigDB 集合；编译时跳过）
# # gsea_pathwaysets <- build_gsea_pathways(
# #   species = "HS", auto_select = c("H", "C2:CP:REACTOME", "C5:GO:BP")
# # )

## ----load-precomputed-pathways------------------------------------------------
# 为加速 vignette 编译，此处加载预计算的轻量级基因集对象
# （Hallmark + KEGG_LEGACY，共 236 条通路）。
# 重新生成方法见 inst/scripts/make_gsea_pathwaysets_toy.R。
data(gsea_pathwaysets_toy, package = "GSEAlens")
gsea_pathwaysets <- gsea_pathwaysets_toy

## ----setup-gsea-limma---------------------------------------------------------
gseadata_limmavoom <- setup_gsea_env(fit = fit,pathway_obj = gsea_pathwaysets,expr_data = gsea_limma_voom_data)

## ----setup-gsea-se------------------------------------------------------------
gseadata_se <- setup_gsea_env(fit = dds_se,pathway_obj = gsea_pathwaysets)

## ----setup-gsea-dds-----------------------------------------------------------
gseadata_dds <- setup_gsea_env(fit = dds,pathway_obj = gsea_pathwaysets)

## ----batch-calc-gsea, eval=FALSE----------------------------------------------
# # 将 vignette 输出写入临时目录，避免污染 Bioconductor 构建机器的工作目录。
# out_dir <- tempdir()
# # limma-voom 流程
# gsea_res_limmavoom <- batch_calc_gsea(gseadata_limmavoom,
#                                                  custom_series_name = "limmavoom_data",
#                                                  output_dir = out_dir,
#                                                  workers = 2,  # 根据比对数量和电脑性能调整
#                                                  force = TRUE)
# # DESeq2 SummarizedExperiment 流程
# gsea_res_se <- batch_calc_gsea(gseadata_se,
#                                           custom_series_name = "dds_se_data",
#                                           output_dir = out_dir,
#                                           workers = 2,
#                                           force = TRUE)
# # DESeq2 Count matrix 流程
# gsea_res_dds <- batch_calc_gsea(gseadata_dds,
#                                            custom_series_name = "dds_data",
#                                            output_dir = out_dir,
#                                            workers = 2,
#                                            force = TRUE)

## ----import-capsule, eval=FALSE-----------------------------------------------
# gsea_res <- import_gsea_capsule("/path/to/your/files/")
# # 或者直接读取 RDS 文件：
# # gsea_res <- readRDS("/path/of/your/file/")

## ----launch-app, eval=FALSE---------------------------------------------------
# launch_gsea_app(gsea_res)

## ----launch-app-with-data, eval=FALSE-----------------------------------------
# launch_gsea_app(gsea_res, addition_data = "pathway_annotations.csv")

## ----sessionInfo--------------------------------------------------------------
sessionInfo()

# Use rlang::`%||%` (imported via utils-imports.R @importFrom rlang %||%).
# This avoids the package-local semantics divergence (length-0 / all-NA fallback)
# and ensures consistent NA handling across the whole codebase. (B-4 IRON FIX, 2026-07-01)

#' @title Backend Data Extractor

#' @description Extract standardized data structures from limma or DESeq2 objects.

#' @keywords internal

#' @name backends



NULL





# 1. Limma 后端提取器





#' @title Extract Limma-Voom Data

#' @description Extract contrast groups and differential analysis results from MArrayLM object.

#' @param fit MArrayLM object (must have been processed with eBayes)

#' @param expr_data Optional. DGEList or expression matrix.

#' @return A list containing contrast_registry, de_store, and expr_bundle

#' @keywords internal

.extract_limma_data <- function(fit, expr_data = NULL) {

    # 1. 强制校验：无截距设计

    .validate_limma_design(fit)



    # 2. 解析对比组

    coef_names <- colnames(fit)



    # 检查是否包含对比符号 " - " (标准 limma contrast 命名)

    is_contrast_obj <- !is.null(fit$contrasts) || any(grepl(" - ", coef_names))



    if (!is_contrast_obj) {

    stop(

            "\n[Limma Input Error] Design matrix column names detected as group names (e.g., 'GroupA', 'GroupB') instead of contrasts (e.g., 'GroupA - GroupB').\nGSEAlens requires a fit object with contrasts already defined.\nPlease use makeContrasts and contrasts.fit to define your comparison groups."

    )

    }



    # 3. 构建 contrast_registry

    parsed_contrasts <- lapply(coef_names, function(name) {

    parts <- strsplit(name, " - ")[[1]]

    if (length(parts) == 2) {

            return(list(left = trimws(parts[1]), right = trimws(parts[2])))

    } else {

            return(list(left = name, right = "Background"))

    }

    })



    contrast_registry <- tibble::tibble(

    contrast_id = vapply(parsed_contrasts, function(x) paste(x$left, x$right, sep = "_vs_"), character(1)),

    left_group = vapply(parsed_contrasts, `[[`, character(1), "left"),

    right_group = vapply(parsed_contrasts, `[[`, character(1), "right"),

    source_name = coef_names,

    backend = "limma_voom"

    )



    # 4. 提取 DE 结果

    de_store <- list()



    for (i in seq_len(nrow(contrast_registry))) {

    reg_row <- contrast_registry[i, ]

    tt <- limma::topTable(fit, coef = reg_row$source_name, number = Inf, sort.by = "none")

    de_store[[reg_row$contrast_id]] <- .standardize_de_columns(df = tt, backend = "limma_voom")

    }



    # 5. 构建 expr_bundle

    expr_bundle <- .build_expr_bundle(expr_data, backend = "limma_voom")



    return(list(

    contrast_registry = contrast_registry,

    de_store = de_store,

    expr_bundle = expr_bundle

    ))

}





# 2. DESeq2 后端提取器





#' @title Extract DESeq2 Data

#' @description Extract contrast groups and differential analysis results from DESeqDataSet object.

#' @param dds DESeqDataSet object (must have been processed with DESeq())

#' @param target_factor Character. Target factor. If NULL, automatically inferred as the last term in the design formula.

#' @return A list containing contrast_registry, de_store, and expr_bundle

#' @keywords internal



.extract_deseq2_data <- function(dds, target_factor = NULL) {

    # 1. 确定 target_factor

    design_formula <- DESeq2::design(dds)

    design_terms <- attr(terms(design_formula), "term.labels")



    if (is.null(target_factor)) {

    target_factor <- utils::tail(design_terms, 1)

    message(sprintf("[DESeq2] target_factor not specified, automatically inferred as: '%s'", target_factor))

    }



    # 校验 target_factor

    .validate_deseq2_design(dds, target_factor)



    # 2. 获取所有 levels

    col_data <- as.data.frame(SummarizedExperiment::colData(dds))

    factor_levels <- levels(col_data[[target_factor]])



    if (length(factor_levels) < 2) {

    stop(sprintf("Factor '%s' has fewer than 2 levels, cannot perform comparison.", target_factor))

    }



    # 3. 生成所有成对比较

    combos <- combn(factor_levels, 2, simplify = FALSE)



    contrast_registry_list <- list()

    de_store_list <- list() # 定义列表变量



    for (combo in combos) {

    left <- combo[1]

    right <- combo[2]

    contrast_id <- paste(left, right, sep = "_vs_")



    # DESeq2 contrast 格式: c(target_factor, numerator, denominator)

    contrast_vec <- c(target_factor, left, right)



    # 提取结果

    res <- tryCatch(

            {

        DESeq2::results(dds, contrast = contrast_vec)

            },

            error = function(e) {

        warning(sprintf("Error extracting contrast %s: %s", contrast_id, e$message))

        return(NULL)

            }

    )



    if (!is.null(res)) {

            # 添加到 registry

            contrast_registry_list[[contrast_id]] <- tibble::tibble(

        contrast_id = contrast_id,

        left_group = left,

        right_group = right,

        source_name = paste(left, "vs", right, sep = " "),

        backend = "deseq2"

            )



            # 标准化列名

            de_store_list[[contrast_id]] <- .standardize_de_columns(

        df = as.data.frame(res),

        backend = "deseq2"

            )

    }

    }



    contrast_registry <- dplyr::bind_rows(contrast_registry_list)



    # 4. 构建 expr_bundle

    expr_bundle <- .build_expr_bundle(dds, backend = "deseq2")



    # 🌟 修复点：返回 de_store_list 而非 de_store

    return(list(

    contrast_registry = contrast_registry,

    de_store = de_store_list,

    expr_bundle = expr_bundle

    ))

}





#' @title Standardize Differential Expression Table Column Names

#' @description Unify column names from different backends to: gene_symbol, logFC, stat, pvalue, padj

#' @keywords internal

.standardize_de_columns <- function(df, backend) {

    # 添加基因名列（保持大小写敏感，但确保存在）

    if ("gene_symbol" %in% colnames(df)) {

    # 保持原样

    } else if ("SYMBOL" %in% colnames(df)) {

    df$gene_symbol <- df$SYMBOL

    } else if ("row.names" %in% colnames(df)) {

    df$gene_symbol <- df$row.names

    } else {

    df$gene_symbol <- rownames(df)

    }



    # 标准化统计列（根据后端类型）

    if (backend == "deseq2") {

    # DESeq2: stat, log2FoldChange, pvalue, padj

    df$stat <- df$stat %||% df$WaldStatistic %||% NA_real_

    df$logFC <- df$log2FoldChange %||% df$logFC %||% NA_real_

    df$pvalue <- df$pvalue %||% df$p.value %||% NA_real_

    df$padj <- df$padj %||% df$adj.P.Val %||% NA_real_

    } else if (backend == "limma_voom") {

    # Limma-Voom: t统计量作为stat, P.Value作为pvalue, adj.P.Val作为padj

    df$stat <- df$stat %||% df$t %||% NA_real_

    df$logFC <- df$logFC %||% NA_real_

    df$pvalue <- df$pvalue %||% df$P.Value %||% NA_real_

    df$padj <- df$padj %||% df$adj.P.Val %||% NA_real_

    }



    # 核心列检查

    core_cols <- c("gene_symbol", "logFC", "stat", "pvalue", "padj")

    missing <- setdiff(core_cols, colnames(df))

    if (length(missing) > 0) {

    stop(sprintf("Standardization failed, missing columns: %s", paste(missing, collapse = ", ")))

    }



    # 保留所有原始列，但确保核心列在前

    other_cols <- setdiff(colnames(df), core_cols)

    return(df[, c(core_cols, other_cols), drop = FALSE])

}



#' @title Build Expression Data Bundle

#' @description Uniformly encapsulate expression matrix and metadata

#' @keywords internal



.build_expr_bundle <- function(obj, backend) {

    if (is.null(obj)) {

    return(list(

            raw_counts = NULL,

            display_expr = NULL,

            sample_meta = NULL,

            gene_meta = NULL

    ))

    }



    if (backend == "limma_voom") {

    if (inherits(obj, "DGEList")) {

            raw_counts <- obj$counts

            sample_meta <- obj$samples

            gene_meta <- obj$genes

            display_expr <- edgeR::cpm(obj, log = TRUE)

    } else {

            raw_counts <- as.matrix(obj)

            sample_meta <- data.frame(row.names = colnames(obj))

            gene_meta <- NULL

            display_expr <- log2(raw_counts + 1)

    }

    } else if (backend == "deseq2") {

    # IRON FIX (2026-07-01, D-4): namespace counts() to DESeq2::counts() so
    # we do not collide with Matrix::counts() or any other package export.
    raw_counts <- DESeq2::counts(obj, normalized = FALSE)

    gene_meta <- as.data.frame(rowData(obj))

    sample_meta <- as.data.frame(colData(obj))

    # 默认展示 log2(normalized counts + 1)

    norm_counts <- DESeq2::counts(obj, normalized = TRUE)

    display_expr <- log2(norm_counts + 1)

    }



    return(list(

    raw_counts = raw_counts,

    display_expr = display_expr,

    sample_meta = sample_meta,

    gene_meta = gene_meta,

    dge_list = if (backend == "limma_voom" && inherits(obj, "DGEList")) obj else NULL,

    dds_obj = if (backend == "deseq2") obj else NULL

    ))

}

#' @name precomputed_gseares
#' @docType data
#' @title Pre-computed GSEA results from the airway dataset
#' @description A pre-computed `GseaRes` object built from the airway dataset
#'   (DESeq2 + clusterProfiler::GSEA). Used in examples and vignettes so that
#'   users can try visualization and code-generation functions without having
#'   to re-run the full GSEA pipeline.
#' @format A `GseaRes` object (list-like).
#' @source Built from the `airway` Bioconductor dataset via DESeq2 +
#'   clusterProfiler::GSEA.
#' @examples
#' data(precomputed_gseares, package = "GSEAlens")
#' class(precomputed_gseares)
NULL

#' @name preprocessed_dds
#' @docType data
#' @title Pre-computed DESeq2 DESeqDataSet from the airway dataset
#' @description A slimmed `DESeqDataSet` object fitted with `DESeq()`,
#'   suitable as input to [GSEAlens::setup_gsea_env()].
#' @format A `DESeqDataSet` object.
#' @source Built from the `airway` Bioconductor dataset via DESeq2.
#' @examples
#' data(preprocessed_dds, package = "GSEAlens")
#' class(preprocessed_dds)
NULL

#' @name preprocessed_dds_se
#' @docType data
#' @title Pre-computed DESeqDataSet (SummarizedExperiment form)
#' @description A `DESeqDataSet` based on a `SummarizedExperiment` input,
#'   suitable for demonstrating the SE-based entry point of
#'   [GSEAlens::setup_gsea_env()].
#' @format A `DESeqDataSet` object.
#' @source Built from the `airway` Bioconductor dataset via DESeq2.
#' @examples
#' data(preprocessed_dds_se, package = "GSEAlens")
#' class(preprocessed_dds_se)
NULL

#' @name preprocessed_limma
#' @docType data
#' @title Pre-computed limma-voom fit and DGEList from the airway dataset
#' @description A list with elements `fit` (`MArrayLM`) and
#'   `gsea_limma_voom_data` (filtered `DGEList`), suitable as input to
#'   [GSEAlens::setup_gsea_env()] for the limma-voom workflow.
#' @format A list with elements `fit` and `gsea_limma_voom_data`.
#' @source Built from the `airway` Bioconductor dataset via limma-voom.
#' @examples
#' data(preprocessed_limma, package = "GSEAlens")
#' names(preprocessed_limma)
NULL

#' @name gsea_pathwaysets_toy
#' @docType data
#' @title Toy MSigDB pathway collection (Hallmark + KEGG_LEGACY)
#' @description A lightweight pathway object for examples and vignettes,
#'   containing 236 pathways (Hallmark + KEGG_LEGACY) pre-assembled by
#'   [GSEAlens::build_gsea_pathways()]. See
#'   `inst/scripts/make_gsea_pathwaysets_toy.R` for regeneration.
#' @format A list with elements `TERM2GENE`, `meta_dict`, `SuperTag`,
#'   `collections_used`, `species`.
#' @source Assembled from msigdbr (Hallmark + KEGG_LEGACY collections).
#' @examples
#' data(gsea_pathwaysets_toy, package = "GSEAlens")
#' names(gsea_pathwaysets_toy)
NULL

#' @name gsea_pathwaysets_toy_hallmark
#' @docType data
#' @title Toy MSigDB pathway collection (Hallmark only)
#' @description A minimal Hallmark-only pathway object for fast examples
#'   and tests. See `inst/scripts/make_gsea_pathwaysets_toy.R` for
#'   regeneration.
#' @format A list with elements `TERM2GENE`, `meta_dict`, `SuperTag`,
#'   `collections_used`, `species`.
#' @source Assembled from msigdbr (Hallmark collection).
#' @examples
#' data(gsea_pathwaysets_toy_hallmark, package = "GSEAlens")
#' names(gsea_pathwaysets_toy_hallmark)
NULL

# Section: Pathway Modal Module ----



#' @title Pathway Detail Modal UI

#' @keywords internal

mod_pathway_modal_ui <- function(id) {

    ns <- shiny::NS(id)

    NULL # Modal dynamically generated in server

}





#' @title Pathway Detail Modal Server

#' @description Modal with checkbox selection using standard DT approach.

#' @param id Module ID

#' @param data_prep Reactive data from data prep module

#' @param trigger_event Event that triggers modal opening

#' @param gsea_res GseaRes object

#' @param pending_genes_reactive Function to get current pending genes

#' @param update_pending_genes Function to update pending genes list

#' @keywords internal



mod_pathway_modal_server <- function(id, data_prep, trigger_event, gsea_res,

                                                                            pending_genes_reactive, update_pending_genes) {

    shiny::moduleServer(id, function(input, output, session) {

    ns <- session$ns



    current_data <- shiny::reactiveVal(NULL)



    # Modal内部checkbox勾选状态

    modal_checked_genes <- shiny::reactiveVal(character(0))



    # ============================================================

    # 【新增】JS同步层 - 监听modal_checked_genes变化并更新checkbox显示

    # ============================================================

    shiny::observe({

            current_selection <- modal_checked_genes()

            session$sendCustomMessage(

        type = ns("syncModalCheckbox"),

        message = list(ids = current_selection)

            )

    })



    # 监听弹窗触发

    shiny::observeEvent(trigger_event(), {

            pathway_id <- trigger_event()

            shiny::req(pathway_id)



            # 清空modal内部状态

            modal_checked_genes(character(0))



            data_list <- data_prep()

            shiny::req(data_list)



            # 提取通路基因和核心基因

            gsea_obj <- data_list$gsea_res

            pathway_genes <- gsea_obj@geneSets[[pathway_id]]



            res_df <- as.data.frame(gsea_obj@result)

            core_str <- res_df$core_enrichment[res_df$ID == pathway_id]

            core_genes <- if (length(core_str) > 0 && !is.na(core_str[1])) {

        unlist(strsplit(as.character(core_str[1]), "/"))

            } else {

        character(0)

            }



            nes_val <- res_df$NES[res_df$ID == pathway_id]



            current_data(list(

        pathway_id = pathway_id,

        pathway_genes = pathway_genes,

        core_genes = core_genes,

        nes = nes_val,

        data_list = data_list

            ))



            # Modal UI

            shiny::showModal(shiny::modalDialog(

        title = shiny::HTML(sprintf(

                    "<b>%s</b><br><small>%s vs %s | NES: %.2f</small>",

                    pathway_id,

                    data_list$left_group,

                    data_list$right_group,

                    nes_val

        )),

        size = "l",

        easyClose = TRUE,



        # 第一行：GSEA图和热图

        shiny::fluidRow(

                    shiny::column(5, shiny::div(

            class = "white-box",

            shiny::h4("Classical GSEA Enrichment Profile"),

            shiny::plotOutput(ns("modal_gsea_plot"), height = "400px")

                    )),

                    shiny::column(7, shiny::div(

            class = "white-box",

            shiny::h4("Core Gene Expression Heatmap"),

            shiny::div(

                            style = "height: 600px; overflow-y: auto; overflow-x: auto; border: 1px solid #ddd;",

                            shiny::plotOutput(ns("modal_heatmap"), height = "auto", width = "100%")

            )

                    ))

        ),

        shiny::hr(),



        # 【新增】JavaScript处理器 - 用于checkbox同步

        shiny::tags$head(shiny::tags$script(shiny::HTML(sprintf("

                    Shiny.addCustomMessageHandler('%s', function(message) {

            var ids = message.ids || [];

            var checkboxes = document.querySelectorAll('.modal-gene-checkbox');

            checkboxes.forEach(function(cb) {

                            var id = cb.getAttribute('data-id');

                            if (cb.checked !== ids.includes(id)) {

                cb.checked = ids.includes(id);

                            }

            });

                    });

        ", ns("syncModalCheckbox"))))),



        # 基因选择区域

        shiny::div(

                    class = "white-box",

                    shiny::h4("Leading Edge Gene Statistics Table"),



                    # CSS样式 - 按钮固定定位

                    shiny::tags$head(shiny::tags$style(shiny::HTML(sprintf("

            .modal-gene-checkbox { width: 18px; height: 18px; cursor: pointer; }

                    ", ns(""))))),



                    # 按钮行

                    shiny::fluidRow(

            shiny::column(

                            4,

                            shiny::div(

                style = "color: #666; font-size: 13px;",

                shiny::HTML("<b>Hint:</b> Click checkbox to select, then confirm")

                            )

            ),

            shiny::column(

                            8,

                            shiny::div(

                style = "display: flex; gap: 10px; justify-content: flex-end;",

                shiny::actionButton(

                                    ns("select_all_leading"),

                                    label = "Select All Leading Edge",

                                    class = "btn-info"

                ),

                shiny::actionButton(

                                    ns("clear_modal_selection"),

                                    label = "Clear All",

                                    class = "btn-warning"

                ),

                shiny::actionButton(

                                    ns("confirm_gene_selection"),

                                    label = "Confirm Selection + Send to Sidebar",

                                    class = "btn-success btn-lg",

                                    style = "font-weight: bold;"

                )

                            )

            )

                    ),

                    shiny::hr(),



                    # 表格区域

                    DT::dataTableOutput(ns("modal_gene_table"))

        ),

        footer = shiny::modalButton("Close")

            ))

    })



    # Select All Leading Edge按钮

    shiny::observeEvent(input$select_all_leading, {

            pdata <- current_data()

            if (is.null(pdata)) {

        return()

            }



            core_genes <- pdata$core_genes

            if (length(core_genes) == 0) {

        shiny::showNotification("No leading edge genes found", type = "warning")

        return()

            }



            # 更新reactive状态

            modal_checked_genes(unique(toupper(core_genes)))

            message(sprintf("[Modal] Select All: %d leading edge genes", length(core_genes)))

    })



    # Clear Selection按钮

    shiny::observeEvent(input$clear_modal_selection, {

            modal_checked_genes(character(0))

            message("[Modal] Clear all selection")

    })



    # Confirm Selection按钮

    shiny::observeEvent(input$confirm_gene_selection, {

            selected <- modal_checked_genes()



            if (length(selected) == 0) {

        shiny::showNotification("No genes selected! Please click checkboxes first.", type = "warning", duration = 3)

        return()

            }



            message(sprintf("[Modal Confirm] %d genes selected", length(selected)))



            # 调用update_pending_genes更新sidebar

            update_pending_genes(selected)



            # 清空modal内部状态

            modal_checked_genes(character(0))



            # 关闭Modal

            shiny::removeModal()



            shiny::showNotification(

        sprintf("%d genes synced to 'Select Genes of Interest'. Click 'Confirm and Apply' to finalize.", length(selected)),

        type = "message",

        duration = 5

            )

    })



    # 监听checkbox点击事件

    shiny::observeEvent(input$modal_gene_toggle, {

            toggle <- input$modal_gene_toggle

            if (is.null(toggle) || !is.list(toggle)) {

        return()

            }



            gene_name <- toggle$id

            is_checked <- isTRUE(toggle$checked) || identical(toggle$checked, "true")



            current <- modal_checked_genes()



            if (is_checked) {

        if (!(gene_name %in% current)) {

                    modal_checked_genes(c(current, gene_name))

        }

            } else {

        modal_checked_genes(setdiff(current, gene_name))

            }

    })



    # =========================================

    # Modal内容渲染

    # =========================================



    # 渲染 GSEA 图

    output$modal_gsea_plot <- shiny::renderPlot({

            pdata <- current_data()

            shiny::req(pdata)



            tryCatch(

        {

                    print(plot_directional_gsea(

            directional_gsea_obj = list(

                            gsea_res = pdata$data_list$gsea_res,

                            meta = list(

                left_group = pdata$data_list$left_group,

                right_group = pdata$data_list$right_group

                            )

            ),

            target_pathways = pdata$pathway_id,

            subPlot = 3,

            add_pval = TRUE

                    ))

        },

        error = function(e) {

                    graphics::plot(1, type = "n", axes = FALSE, xlab = "", ylab = "")

                    graphics::text(1, 1, sprintf("Plotting failed: %s", e$message), col = "red")

        }

            )

    })



    # 通路核心基因热图绘制

    output$modal_heatmap <- shiny::renderPlot(

            {

        pdata <- current_data()

        shiny::req(pdata)

        symbol_map <- .rebuild_symbol_map(gsea_res, pdata$data_list$contrast_id)

        sample_meta_raw <- gsea_res$expr_bundle$sample_meta



        if (inherits(sample_meta_raw, "DFrame") || inherits(sample_meta_raw, "DataFrame")) {

                    sample_names <- rownames(sample_meta_raw)

                    sample_meta <- as.data.frame(sample_meta_raw, stringsAsFactors = FALSE)

                    if (!is.null(sample_names) && (is.null(rownames(sample_meta)) || all(rownames(sample_meta) == ""))) {

            rownames(sample_meta) <- sample_names

                    }

        } else {

                    sample_meta <- as.data.frame(sample_meta_raw, stringsAsFactors = FALSE)

        }



        group_col <- NULL

        if (!is.null(gsea_res$backend_info$target_factor)) {

                    target <- gsea_res$backend_info$target_factor

                    if (target %in% colnames(sample_meta)) {

            group_col <- target

                    }

        }



        if (is.null(group_col)) {

                    candidates <- c(

            "group", "Group", "condition", "Condition",

            "treatment", "Treatment", "type", "Type"

                    )

                    for (col in candidates) {

            if (col %in% colnames(sample_meta)) {

                            group_col <- col

                            break

            }

                    }

        }



        if (is.null(group_col)) {

                    stop(sprintf(

            "Cannot find group column. Available columns: %s",

            paste(colnames(sample_meta), collapse = ", ")

                    ))

        }



        sample_meta$group <- as.character(sample_meta[[group_col]])



        left_grp <- pdata$data_list$left_group

        right_grp <- pdata$data_list$right_group



        sample_idx <- which(sample_meta$group %in% c(left_grp, right_grp))

        if (length(sample_idx) == 0) {

                    stop(sprintf(

            "No samples found for groups '%s' or '%s'.",

            left_grp, right_grp

                    ))

        }



        target_samples <- rownames(sample_meta)[sample_idx]



        expr_bundle <- gsea_res$expr_bundle

        cpm_mat <- NULL



        if (!is.null(expr_bundle$dds_obj)) {

                    tryCatch(

            {

                            raw_counts <- DESeq2::counts(expr_bundle$dds_obj, normalized = FALSE)

                            common_samples <- intersect(target_samples, colnames(raw_counts))

                            if (length(common_samples) > 0) {

                raw_counts <- raw_counts[, common_samples, drop = FALSE]

                lib_sizes <- colSums(raw_counts)

                cpm_mat <- t(t(raw_counts) / lib_sizes) * 1e6

                target_samples <- common_samples

                            }

            },

            error = function(e) {

                            message("DDS CPM calculation failed: ", e$message)

            }

                    )

        }



        if (is.null(cpm_mat) && !is.null(expr_bundle$raw_counts)) {

                    common_samples <- intersect(target_samples, colnames(expr_bundle$raw_counts))

                    if (length(common_samples) > 0) {

            rc <- expr_bundle$raw_counts[, common_samples, drop = FALSE]

            lib_sizes <- colSums(rc)

            cpm_mat <- t(t(rc) / lib_sizes) * 1e6

            target_samples <- common_samples

                    }

        }



        if (is.null(cpm_mat) || ncol(cpm_mat) == 0) {

                    stop("Cannot compute CPM matrix.")

        }



        symbol_map <- .rebuild_symbol_map(gsea_res, pdata$data_list$contrast_id)

        # 使用通用工具函数处理表达矩阵行名

        cpm_mat <- .convert_expr_rownames(cpm_mat, expr_bundle$gene_meta, gsea_res)

        # 获取更新后的基因标识符

        pathway_genes <- pdata$pathway_genes

        gene_identifiers <- rownames(cpm_mat)



        expr_genes_upper <- toupper(gene_identifiers)

        pathway_genes_upper <- toupper(pathway_genes)

        matched_idx <- which(expr_genes_upper %in% pathway_genes_upper)





        if (length(matched_idx) < 2) {

                    stop(sprintf(

            "Gene matching failed: only %d/%d pathway genes matched.",

            length(matched_idx), length(pathway_genes)

                    ))

        }



        plot_mat <- cpm_mat[matched_idx, , drop = FALSE]

        rownames(plot_mat) <- gene_identifiers[matched_idx]



        gene_vars <- apply(plot_mat, 1, var, na.rm = TRUE)

        valid_genes <- gene_vars > 1e-6 & !is.na(gene_vars)

        if (sum(valid_genes) < 2) {

                    stop("Insufficient genes with variance after filtering")

        }

        plot_mat <- plot_mat[valid_genes, , drop = FALSE]



        display_numbers <- round(plot_mat)



        z_mat <- t(scale(t(plot_mat)))

        z_mat[is.na(z_mat)] <- 0

        z_mat[z_mat > 1] <- 1

        z_mat[z_mat < -1] <- -1



        gene_list <- pdata$data_list$gsea_res@geneList



        gene_metrics <- vapply(rownames(plot_mat), function(g) {

                    idx <- match(toupper(g), toupper(names(gene_list)))

                    if (is.na(idx)) {

            return(0)

                    }

                    return(gene_list[idx])

        }, numeric(1))



        res_df <- as.data.frame(pdata$data_list$gsea_res@result)

        core_str <- res_df$core_enrichment[res_df$ID == pdata$pathway_id]

        core_genes <- character(0)

        if (length(core_str) > 0 && !is.na(core_str[1])) {

                    core_genes <- unlist(strsplit(as.character(core_str[1]), "/"))

        }

        is_leading <- toupper(rownames(plot_mat)) %in% toupper(core_genes)



        if (pdata$nes > 0) {

                    sort_order <- order(gene_metrics, decreasing = TRUE)

        } else {

                    sort_order <- order(gene_metrics, decreasing = FALSE)

        }



        plot_mat <- plot_mat[sort_order, , drop = FALSE]

        z_mat <- z_mat[sort_order, , drop = FALSE]

        display_numbers <- display_numbers[sort_order, , drop = FALSE]

        is_leading <- is_leading[sort_order]

        gene_metrics <- gene_metrics[sort_order]



        col_fun <- circlize::colorRamp2(c(-1, 0, 1), c("#67a9cf", "#f7f7f7", "#ef8a62"))



        grp_col <- c("#E41A1C", "#377EB8")

        names(grp_col) <- c(left_grp, right_grp)



        group_factor <- factor(

                    sample_meta$group[match(colnames(plot_mat), rownames(sample_meta))],

                    levels = c(left_grp, right_grp)

        )



        top_ann <- ComplexHeatmap::HeatmapAnnotation(

                    Group = group_factor,

                    col = list(Group = grp_col),

                    annotation_name_gp = grid::gpar(fontsize = 12, fontface = "bold"),

                    simple_anno_size = grid::unit(0.6, "cm")

        )



        leading_status <- ifelse(is_leading, "YES", "NO")

        leading_colors <- c("YES" = "#FF9800", "NO" = "transparent")



        right_ann <- ComplexHeatmap::rowAnnotation(

                    LeadingEdge = leading_status,

                    col = list(LeadingEdge = leading_colors),

                    annotation_name_gp = grid::gpar(fontsize = 12, fontface = "bold"),

                    simple_anno_size = grid::unit(0.4, "cm")

        )



        layer_fun <- function(j, i, x, y, w, h, fill) {

                    vals <- ComplexHeatmap::pindex(display_numbers, i, j)

                    grid::grid.text(

            label = vals, x = x, y = y,

            gp = grid::gpar(fontsize = 13, col = "black", fontface = "bold")

                    )

        }



        if (pdata$nes > 0) {

                    row_split_factor <- factor(

            ifelse(is_leading, "Leading Edge", "Other Genes"),

            levels = c("Leading Edge", "Other Genes")

                    )

        } else {

                    row_split_factor <- factor(

            ifelse(is_leading, "Other Genes", "Leading Edge"),

            levels = c("Other Genes", "Leading Edge")

                    )

        }



        row_names_gp <- grid::gpar(

                    fontsize = 15,

                    fontface = ifelse(is_leading, "bold", "plain"),

                    col = ifelse(is_leading, "#FF9800", "black")

        )



        ht <- ComplexHeatmap::Heatmap(

                    z_mat,

                    name = "Z-Score",

                    col = col_fun,

                    cluster_rows = FALSE,

                    cluster_columns = FALSE,

                    column_split = group_factor,

                    cluster_column_slices = FALSE,

                    row_split = row_split_factor,

                    cluster_row_slices = FALSE,

                    row_gap = grid::unit(2, "mm"),

                    top_annotation = top_ann,

                    right_annotation = right_ann,

                    layer_fun = layer_fun,

                    row_names_gp = row_names_gp,

                    column_names_gp = grid::gpar(fontsize = 15, fontface = "bold"),

                    rect_gp = grid::gpar(col = "white", lwd = 1),

                    show_heatmap_legend = TRUE,

                    width = NULL,

                    height = NULL

        )



        enriched_group <- ifelse(pdata$nes > 0, left_grp, right_grp)

        title_text <- sprintf(

                    "%s\nRow-Scaled Z-Score [-1, 1] | Enriched in: %s",

                    pdata$pathway_id, enriched_group

        )



        ComplexHeatmap::draw(ht,

                    merge_legend = TRUE,

                    column_title = title_text,

                    column_title_gp = grid::gpar(fontsize = 14, fontface = "bold")

        )

            },

            height = function() {

        pdata <- current_data()

        if (is.null(pdata)) {

                    return(400)

        }

        max(400, length(pdata$pathway_genes) * 25 + 150)

            }

    )



    # =========================================

    # 基因统计表 - 使用与主Table相同的checkbox方式

    # =========================================

    output$modal_gene_table <- DT::renderDataTable(

            {

        pdata <- current_data()

        shiny::req(pdata)

        symbol_map <- .rebuild_symbol_map(gsea_res, pdata$data_list$contrast_id)

        # 获取当前checkbox状态

        current_selection <- isolate(modal_checked_genes())



        genelist <- pdata$data_list$gsea_res@geneList

        all_genes <- pdata$pathway_genes

        core_genes <- pdata$core_genes



        all_gene_names <- names(genelist)

        display_gene_name <- .get_display_symbols(all_gene_names, symbol_map)



        gene_df <- data.frame(

                    Gene = display_gene_name,

                    Rank_Metric = round(as.numeric(genelist), 3),

                    Rank_in_List = seq_along(genelist),

                    stringsAsFactors = FALSE

        )



        gene_df <- gene_df[toupper(gene_df$Gene) %in% toupper(all_genes), ]

        gene_df$Is_Core <- ifelse(

                    toupper(gene_df$Gene) %in% toupper(core_genes),

                    "YES",

                    "-"

        )



        gene_df <- gene_df[order(

                    gene_df$Is_Core == "YES",

                    abs(gene_df$Rank_Metric),

                    decreasing = TRUE

        ), ]



        # 生成checkbox列 - 使用与主Table完全相同的方式

        # checkbox 使用原始大小写的基因名

        gene_df$Select <- vapply(seq_len(nrow(gene_df)), function(i) {

                    g <- gene_df$Gene[i] # 已经是原始大小写

                    is_checked <- ifelse(g %in% current_selection, 'checked="checked"', "")

                    sprintf(

            '<input type="checkbox" class="modal-gene-checkbox" data-id="%s" %s onclick="Shiny.setInputValue(\'%s\', {id: \'%s\', checked: this.checked}, {priority: \'event\'});"/>',

            g, is_checked, ns("modal_gene_toggle"), g

                    )

        }, character(1))



        gene_df <- gene_df[, c("Select", "Gene", "Rank_Metric", "Rank_in_List", "Is_Core")]

        colnames(gene_df) <- c("Select", "Gene", "Rank Metric", "Rank in List", "Leading Edge")



        DT::datatable(

                    gene_df,

                    rownames = FALSE,

                    escape = FALSE,

                    selection = "none",

                    extensions = "Scroller",

                    options = list(

            scrollX = TRUE,

            scrollY = "50vh",

            scroller = TRUE,

            deferRender = TRUE,

            pageLength = -1,

            dom = "frtip",

            columnDefs = list(

                            list(width = "50px", targets = 0, orderable = FALSE, className = "dt-center"),

                            list(width = "100px", targets = 1, className = "dt-center"),

                            list(width = "80px", targets = 2, className = "dt-center"),

                            list(width = "80px", targets = 3, className = "dt-center"),

                            list(width = "80px", targets = 4, className = "dt-center")

            )

                    )

        ) |>

                    DT::formatStyle(

            columns = "Leading Edge",

            backgroundColor = DT::styleEqual(c("YES", "-"), c("#FF9800", "transparent")),

            fontWeight = DT::styleEqual("YES", "bold"),

            color = DT::styleEqual("YES", "white")

                    )

            },

            server = FALSE

    )

    })

}

# Section: Pathway Database Builder ----



#' @title Build GSEA Pathway Database with Interactive Selection

#' @description Provides an interactive or automated gene set selection interface

#'   based on msigdbr, generating standardized pathway objects with intelligent

#'   semantic tagging. Supports both human (Homo sapiens) and mouse (Mus musculus)

#'   species.

#' @param species Character string specifying the species. Options:

#'   \itemize{

#'     \item "HS": Homo sapiens (human, default)

#'     \item "MM": Mus musculus (mouse)

#'   }

#' @param auto_select Optional parameter for automated selection. Can be:

#'   \itemize{

#'     \item NULL: Launch interactive menu (default)

#'     \item "ALL": Load entire MSigDB database for the specified species

#'     \item Character vector: Collection names (e.g., c("H", "C5:GO:BP") for human)

#'   }

#' @return A list object containing:

#'   \itemize{

#'     \item TERM2GENE: Data frame mapping pathway IDs to gene symbols

#'     \item meta_dict: Metadata dictionary with descriptions, URLs, and collection info

#'     \item SuperTag: Intelligent batch identifier for the selected collections

#'     \item collections_used: Data frame of selected collection metadata

#'     \item species: Species identifier ("HS" or "MM")

#'   }

#' @export

#' @examples
#' \donttest{
#' # Automated selection by collection name -- safe for non-interactive use
#' pathways <- build_gsea_pathways(species = "HS",
#'                                 auto_select = c("H", "C5:GO:BP"))
#'
#' # MM uses M-prefixed collection codes (MH, M5:GO:BP, ...)
#' pathways_mm <- build_gsea_pathways(species = "MM",
#'                                    auto_select = c("MH", "M5:GO:BP"))
#' }
#'
#' \dontrun{
#' # Interactive menu -- only runs when user is at a terminal
#' if (interactive()) {
#'   pathways <- build_gsea_pathways(species = "HS")
#' }
#'
#' # Load ALL collections -- downloads the entire MSigDB (slow, needs network)
#' pathways <- build_gsea_pathways(species = "HS", auto_select = "ALL")
#' }
#' @importFrom dplyr arrange left_join select distinct mutate all_of bind_rows filter

#' @importFrom msigdbr msigdbr_collections msigdbr

build_gsea_pathways <- function(species = "HS", auto_select = NULL) {

    # === Section: Species Parameter Standardization === ----



    species <- toupper(trimws(as.character(species)))



    # 物种配置表

    species_config <- list(

    "HS" = list(

            db_species = "HS",

            msigdbr_species = "Homo sapiens",

            display_name = "Homo sapiens (Human)",

            is_mouse = FALSE

    ),

    "MM" = list(

            db_species = "MM",

            msigdbr_species = "mouse",

            display_name = "Mus musculus (Mouse)",

            is_mouse = TRUE

    )

    )



    if (!species %in% names(species_config)) {

    stop(sprintf(

            "Invalid species '%s'. Please use 'HS' (human) or 'MM' (mouse).",

            species

    ))

    }



    cfg <- species_config[[species]]

    message(sprintf("[build_gsea_pathways] Species: %s", cfg$display_name))



    # === Section: MM Database Code to msigdbr Function Parameter Mapping === ----



    # msigdbr_collections(db_species = "MM") returns MM codes like M1, M2, MH

    # but msigdbr(species = "mouse", ...) only accepts human codes (C1, C2, H, etc.)

    mm_code_map <- list(

    "MH" = list(

            coll = "H", subcoll = "",

            desc = "MH Hallmark Gene Sets", tag = "MHall"

    ),

    "M1" = list(

            coll = "C1", subcoll = "",

            desc = "M1 Positional Gene Sets", tag = "M1Pos"

    ),

    "M2:CGP" = list(

            coll = "C2", subcoll = "CGP",

            desc = "M2 Chemical and Genetic Perturbations", tag = "MCGP"

    ),

    "M2:CP:BIOCARTA" = list(

            coll = "C2", subcoll = "CP:BIOCARTA",

            desc = "M2 BioCarta Pathways", tag = "MBioC"

    ),

    "M2:CP:REACTOME" = list(

            coll = "C2", subcoll = "CP:REACTOME",

            desc = "M2 Reactome Pathways", tag = "MReac"

    ),

    "M2:CP:WIKIPATHWAYS" = list(

            coll = "C2", subcoll = "CP:WIKIPATHWAYS",

            desc = "M2 WikiPathways", tag = "MWiki"

    ),

    "M3:GTRD" = list(

            coll = "C3", subcoll = "GTRD",

            desc = "M3 GTRD Transcription Factors", tag = "MTFTG"

    ),

    "M3:MIRDB" = list(

            coll = "C3", subcoll = "MIRDB",

            desc = "M3 miRDB", tag = "MMirDB"

    ),

    "M5:GO:BP" = list(

            coll = "C5", subcoll = "GO:BP",

            desc = "M5 GO Biological Process", tag = "MGoBP"

    ),

    "M5:GO:CC" = list(

            coll = "C5", subcoll = "GO:CC",

            desc = "M5 GO Cellular Component", tag = "MGoCC"

    ),

    "M5:GO:MF" = list(

            coll = "C5", subcoll = "GO:MF",

            desc = "M5 GO Molecular Function", tag = "MGoMF"

    ),

    "M5:MPT" = list(

            coll = "C5", subcoll = "HPO", # MPT maps to HPO

            desc = "M5 Mouse Phenotype Tumor", tag = "MPTumor"

    ),

    "M7" = list(

            coll = "C7", subcoll = "",

            desc = "M7 Immunologic Signatures", tag = "MImmS"

    ),

    "M8" = list(

            coll = "C8", subcoll = "",

            desc = "M8 Cell Type Signatures", tag = "MC8Cell"

    )

    )



    # === Section: Dynamically Fetch Available Gene Set Collections === ----



    avail_colls <- msigdbr::msigdbr_collections(db_species = cfg$db_species) |>

    dplyr::arrange(.data$gs_collection, .data$gs_subcollection)



    # === Section: Build Menu Dataframe === ----



    # For MM mode, map MM codes to human codes and display info

    menu_df <- lapply(seq_len(nrow(avail_colls)), function(i) {

    row <- avail_colls[i, ]

    mm_key <- paste0(

            row$gs_collection,

            ifelse(row$gs_subcollection == "" || is.na(row$gs_subcollection),

        "", paste0(":", row$gs_subcollection)

            )

    )



    if (cfg$is_mouse && mm_key %in% names(mm_code_map)) {

            mapped <- mm_code_map[[mm_key]]

            # Human collection code (for msigdbr function calls)

            human_coll <- mapped$coll

            human_subcoll <- mapped$subcoll

            # Display info

            short_tag <- mapped$tag

            display_desc <- mapped$desc

            combo_name <- mm_key # 保持 MM 代码格式

            # Combined name (for auto_select matching)

    } else {

            # 人类模式：直接使用

            human_coll <- row$gs_collection

            human_subcoll <- row$gs_subcollection

            short_tag <- paste0(

        row$gs_collection,

        ifelse(row$gs_subcollection == "" || is.na(row$gs_subcollection),

                    "", paste0("_", gsub(":", "_", row$gs_subcollection))

        )

            )

            display_desc <- row$gs_collection_name

            combo_name <- paste0(

        row$gs_collection,

        ifelse(row$gs_subcollection == "" || is.na(row$gs_subcollection),

                    "", paste0(":", row$gs_subcollection)

        )

            )

    }



    data.frame(

            gs_collection = row$gs_collection,

            gs_subcollection = row$gs_subcollection,

            gs_collection_name = row$gs_collection_name,

            num_genesets = row$num_genesets,

            human_coll = human_coll,

            human_subcoll = human_subcoll,

            short_tag = short_tag,

            description = display_desc,

            combo_name = combo_name,

            stringsAsFactors = FALSE

    )

    })



    menu_df <- dplyr::bind_rows(menu_df)



    # === Section: Interactive Interface Logic === ----



    if (is.null(auto_select)) {

    message("\n", rep("=", 70))

    message(sprintf("Welcome to GSEAlens Pathway Wizard (%s)", cfg$display_name))

    message(rep("=", 70))

    message("Available Gene Set Collections:")

    message(sprintf("%-6s %-18s %s", "Tag", "Code", "Description"))

    message(rep("-", 70))



    for (i in seq_len(nrow(menu_df))) {

            message(sprintf(

        "[%2d] %-6s | %-18s | %s",

        i, menu_df$short_tag[i], menu_df$combo_name[i], menu_df$description[i]

            ))

    }



    user_input <- readline(prompt = "\nEnter selection (comma-separated, e.g., 1,3,5): ")

    selected_idx <- as.integer(unlist(strsplit(user_input, "[, ]+")))

    selected_idx <- selected_idx[!is.na(selected_idx) &

            selected_idx >= 1 &

            selected_idx <= nrow(menu_df)]



    if (length(selected_idx) == 0) {

            stop("Invalid input: No valid selection provided.")

    }

    } else {

    # Auto-select mode

    if (length(auto_select) == 1 && toupper(auto_select) == "ALL") {

            message(sprintf("Loading complete MSigDB database for %s...", cfg$display_name))

            selected_idx <- seq_len(nrow(menu_df))

    } else if (is.character(auto_select)) {

            # === Section: Auto-Select Mode === ----



            # 1. First attempt direct matching

            matched <- match(auto_select, menu_df$combo_name)



            # 2. If matching fails, try matching short_tag

            if (any(is.na(matched))) {

        matched2 <- match(auto_select, menu_df$short_tag)

        na_idx <- which(is.na(matched))

        matched[na_idx] <- matched2[na_idx]

            }



            # 3. In MM mode, if still failing, try human code conversion

            if (cfg$is_mouse && any(is.na(matched))) {

        human_to_mm_map <- c(

                    "H" = "MH", "C1" = "M1", "C2" = "M2", "C3" = "M3",

                    "C4" = "M4", "C5" = "M5", "C6" = "M6", "C7" = "M7", "C8" = "M8"

        )

        still_missing <- auto_select[is.na(matched)]

        converted <- vapply(still_missing, function(x) {

                    if (x %in% names(human_to_mm_map)) {

            converted_code <- human_to_mm_map[[x]]

            message(sprintf("[build_gsea_pathways] Converted '%s' -> '%s' for MM mode", x, converted_code))

            return(converted_code)

                    }

                    return(x)

        }, character(1))

        matched3 <- match(converted, menu_df$combo_name)

        still_na <- which(is.na(matched))

        matched[still_na] <- matched3

            }



            # 4. Final validation check

            if (any(is.na(matched))) {

        missing <- auto_select[is.na(matched)]

        stop(sprintf(

                    "Collection not found: %s. Available: %s",

                    paste(missing, collapse = ", "),

                    paste(unique(menu_df$combo_name), collapse = ", ")

        ))

            }

            selected_idx <- matched[!is.na(matched)]

    } else if (is.numeric(auto_select)) {

            selected_idx <- auto_select

            if (any(selected_idx < 1 | selected_idx > nrow(menu_df))) {

        stop(sprintf("Index out of range. Valid range: 1-%d", nrow(menu_df)))

            }

    } else {

            stop("Invalid auto_select parameter type.")

    }

    }



    selected_rows <- menu_df[selected_idx, ]



    # === Section: Intelligent Batch Tag Generation === ----



    if (length(auto_select) == 1 && toupper(auto_select) == "ALL") {

    super_tag <- sprintf("ALL_%s_MSigDB_Global", species)

    } else {

    super_tag <- ifelse(

            nrow(selected_rows) <= 4,

            paste(selected_rows$short_tag, collapse = "_"),

            sprintf("Mix%d_%s_%s", nrow(selected_rows), species, selected_rows$short_tag[1])

    )

    }



    message(sprintf(

    "\nSelected %d collection(s). Batch Tag: [%s]",

    nrow(selected_rows), super_tag

    ))



    # === Section: Core Extraction Using Correct Species and Codes === ----



    pathway_list <- lapply(seq_len(nrow(selected_rows)), function(i) {

    row <- selected_rows[i, ]



    # Use human codes to call msigdbr

    if (row$human_subcoll == "" || is.na(row$human_subcoll)) {

            msigdbr::msigdbr(species = cfg$msigdbr_species, collection = row$human_coll)

    } else {

            msigdbr::msigdbr(

        species = cfg$msigdbr_species,

        collection = row$human_coll,

        subcollection = row$human_subcoll

            )

    }

    })



    all_pathways <- dplyr::bind_rows(pathway_list)



    # === Section: Dynamic Column Name Detection === ----



    cat_col_name <- if ("gs_cat" %in% colnames(all_pathways)) "gs_cat" else "gs_collection"

    subcat_col_name <- if ("gs_subcat" %in% colnames(all_pathways)) "gs_subcat" else "gs_subcollection"



    # === Section: Build TERM2GENE Mapping Table === ----



    TERM2GENE <- all_pathways |>

    dplyr::select(gs_name, gene_symbol)



    # === Section: Build Metadata Dictionary === ----



    TERM2NAME <- all_pathways |>

    dplyr::select(

            ID = gs_name,

            Description = gs_description,

            URL = gs_url,

            Collection = dplyr::any_of(cat_col_name),

            Subcollection = dplyr::any_of(subcat_col_name)

    ) |>

    dplyr::distinct(ID, .keep_all = TRUE) |>

    dplyr::mutate(

            Combo_Name = ifelse(

        Subcollection == "" | is.na(Subcollection),

        Collection,

        paste0(Collection, ":", Subcollection)

            ),

            Species = species

    )



    # === Section: Return Result List === ----



    result <- list(

    TERM2GENE = TERM2GENE,

    meta_dict = TERM2NAME,

    SuperTag = super_tag,

    collections_used = selected_rows,

    species = species

    )



    message(sprintf(

    "\n[build_gsea_pathways] Done! Built %d pathways x %d genes for %s",

    nrow(TERM2NAME),

    length(unique(TERM2GENE$gene_symbol)),

    cfg$display_name

    ))



    return(result)

}

# Tests for build_gsea_pathways()

# Type: function with msigdbr dependency (skipped when msigdbr absent)

# Coverage: invalid species error path (always runs); valid HS/MM paths and

#           returned structure (BUG-2 FIX: list element is "TERM2GENE" not "term2gene");

#           case-insensitive species; ALL auto_select shorthand.

# NOTE: auto_select = NULL triggers readline() and is intentionally NOT tested

#       here because it hangs non-interactive sessions (BUG-3 FIX).



test_that("build_gsea_pathways rejects invalid species with informative message", {

  expect_error(

    build_gsea_pathways(species = "INVALID"),

    regexp = "Invalid species"

  )

})



test_that("build_gsea_pathways rejects empty species string", {

  expect_error(

    build_gsea_pathways(species = ""),

    regexp = "Invalid species"

  )

})



test_that("build_gsea_pathways rejects unsupported species like 'RAT'", {

  expect_error(

    build_gsea_pathways(species = "RAT"),

    regexp = "Invalid species"

  )

})



test_that("build_gsea_pathways accepts HS (human) and returns expected structure", {

  skip_if_not_installed("msigdbr")

  result <- build_gsea_pathways(species = "HS", auto_select = c("H"))

  expect_type(result, "list")

  # BUG-2 FIX: returned list element name is uppercase 'TERM2GENE' (not 'term2gene')

  expect_true("TERM2GENE" %in% names(result))

  expect_true("meta_dict" %in% names(result))

  expect_true("species" %in% names(result))

  expect_equal(result$species, "HS")

  expect_s3_class(result$TERM2GENE, "data.frame")

})



test_that("build_gsea_pathways accepts MM (mouse)", {

  skip_if_not_installed("msigdbr")

  result <- build_gsea_pathways(species = "MM", auto_select = c("H"))

  expect_type(result, "list")

  expect_true("TERM2GENE" %in% names(result))

  expect_equal(result$species, "MM")

})



test_that("build_gsea_pathways is case-insensitive for species code", {

  skip_if_not_installed("msigdbr")

  result_lower <- build_gsea_pathways(species = "hs", auto_select = c("H"))

  expect_type(result_lower, "list")

  expect_equal(result_lower$species, "HS")

})



test_that("build_gsea_pathways accepts 'ALL' auto_select shorthand", {

  skip_if_not_installed("msigdbr")

  result <- build_gsea_pathways(species = "HS", auto_select = "ALL")

  expect_type(result, "list")

  expect_true("TERM2GENE" %in% names(result))

})

# Section: Capsule Import and GSEA Result Extraction ----



#' @title Load and Smart-Relocate GSEA Computation Capsule

#' @description Safely load a computation capsule. If the file is moved from its

#'   original project path (e.g., copied to desktop), the engine will automatically

#'   restore the standard folder structure in the current working directory and

#'   relocate the capsule. Fully compatible with GseaEnv and GseaRes objects.

#' @param file_path Character. Absolute or relative path to the capsule rds file.

#' @param auto_relocate Logical. Whether to automatically fix the folder structure

#'   and relocate based on the built-in lineage in the current directory? Default TRUE.

#' @param inspect Logical. Whether to automatically print capsule overview after

#'   loading? Default TRUE.

#' @return Returns the parsed GseaRes or GseaEnv capsule object.

#' @export



#' @examples
#' # Load the pre-computed GseaRes capsule shipped with the package, write it
#' # to a temp file, then import it (auto_relocate = FALSE to avoid side
#' # effects).
#' data(precomputed_gseares, package = "GSEAlens")
#' tmp_capsule <- file.path(tempdir(), "import_demo.rds")
#' saveRDS(precomputed_gseares, tmp_capsule)
#'
#' gsea_res <- import_gsea_capsule(
#'   tmp_capsule,
#'   auto_relocate = FALSE,
#'   inspect = FALSE
#' )
#' class(gsea_res)
import_gsea_capsule <- function(file_path, auto_relocate = TRUE, inspect = TRUE) {

    # 1. 文件存在性检查

    if (!file.exists(file_path)) {

    stop(sprintf("File does not exist: %s", file_path))

    }



    message("Waking up computation capsule...")

    capsule <- readRDS(file_path)



    # 2. 识别胶囊类型

    if (inherits(capsule, c("GseaEnv", "GseaEnvPro"))) {

    message("Successfully loaded [GseaEnv] environment encapsulation capsule (parallel computation not yet performed).")



    if (inspect && exists("inspect_gsea_env", mode = "function")) {

            inspect_gsea_env(capsule)

    }



    return(invisible(capsule))

    }



    # 3. 对于计算结果胶囊，执行智能归位逻辑

    if (!inherits(capsule, "GseaRes")) {

    warning("This file does not appear to be a standard GSEAlens capsule!")

    return(invisible(capsule))

    }



    # 4. 检查是否有系统记忆（project_info）

    project_info <- capsule$metadata$project_info



    if (is.null(project_info)) {

    message("This capsule was generated by an older version and lacks project lineage memory. Loading directly (auto-relocation not possible).")

    } else {

    # 5. 获取当前路径与预期路径

    current_abs <- normalizePath(file_path, winslash = "/", mustWork = FALSE)

    expected_abs <- normalizePath(project_info$rds_path, winslash = "/", mustWork = FALSE)



    # 6. 路径不一致，且开启了自动归位

    if (current_abs != expected_abs && auto_relocate) {

            message("[Lineage Alert] Capsule is currently in a non-standard path")

            message(sprintf("   Current location: %s", current_abs))

            message(sprintf("   Original project: [%s]", project_info$custom_series_name))



            # 7. 根据当前工作目录重建标准档案库

            local_series_dir <- file.path(getwd(), "GSEA_Output", project_info$custom_series_name)

            if (!dir.exists(local_series_dir)) {

        dir.create(local_series_dir, recursive = TRUE, showWarnings = FALSE)

        message(sprintf("   Created standard archive directory: %s", local_series_dir))

            }



            # 8. 执行文件归位

            target_file <- file.path(local_series_dir, basename(file_path))



            if (!file.exists(target_file) || target_file == current_abs) {

        file.copy(from = file_path, to = target_file, overwrite = TRUE)

        message("   Automatically relocated capsule to standard archive")

        message(sprintf("   New location: %s", target_file))



        # 9. 更新胶囊内部的路径信息，防止下次加载时重复警告

        capsule$metadata$project_info$output_dir <- file.path(getwd(), "GSEA_Output")

        capsule$metadata$project_info$series_dir <- local_series_dir

        capsule$metadata$project_info$rds_path <- target_file

            } else {

        message("   Backup already exists in standard archive (no duplicate needed).")

            }

    }

    }



    # 10. 自动调用探针函数打印概况

    if (inspect) {

    if (exists("inspect_gsea_res", mode = "function")) {

            inspect_gsea_res(capsule)

    } else {

            message("Successfully loaded [GseaRes] computation result capsule!")

    }

    }



    return(invisible(capsule))

}





#' @title Extract GSEA Results for Specific Contrast

#' @description Extract results for a specified contrast from a GseaRes capsule.

#'   Supports subsetting by gene set collection and automatically recalculates FDR.

#' @param gsea_res GseaRes object (generated by batch_calc_gsea)

#' @param contrast_id String. Contrast group ID (e.g., "A_vs_B").

#' @param target_collection String vector. Gene set collection to extract

#'   (e.g., "H", "C2:CP:KEGG_LEGACY"). Default "ALL" means extract all.

#' @return GseaTask object.

#' @examples

#' # Load a pre-computed GseaRes shipped with the package

#' data(precomputed_gseares, package = "GSEAlens")
#'
#' gsea_res <- precomputed_gseares

#' task <- extract_gsea_task(gsea_res, contrast_id = "untrt_vs_trt")

#' class(task)

#' @export

extract_gsea_task <- function(gsea_res, contrast_id, target_collection = "ALL") {

    # 1. 校验输入

    .check_gsea_res(gsea_res)



    if (!(contrast_id %in% names(gsea_res$results))) {

    stop(sprintf(

            "Contrast '%s' not found in results. Available: %s",

            contrast_id, paste(names(gsea_res$results), collapse = ", ")

    ))

    }



    task_info <- gsea_res$results[[contrast_id]]



    if (task_info$status != "Success") {

    stop(sprintf("Contrast '%s' computation failed or has no enrichment results, cannot extract.", contrast_id))

    }



    # 2. 获取元信息（从 registry 中精确匹配，不再解析字符串）

    # 注意：contrast_id 可能是反向的 (B_vs_A)，registry 中可能只有正向的 (A_vs_B)

    # 但我们在 batch_calc_gsea 中已经把反向任务写入了 results，却没有写入 registry (这是设计上的权衡，registry 记录原始生物学对比)

    # 这里我们需要智能处理 left/right group



    # 优先从 results 里的 genelist 名称推断 (最准确)

    # 或者从 registry 查找



    reg <- gsea_res$contrast_registry

    reg_match <- reg[reg$contrast_id == contrast_id, ]



    left_group <- NA

    right_group <- NA



    if (nrow(reg_match) > 0) {

    left_group <- reg_match$left_group[1]

    right_group <- reg_match$right_group[1]

    } else {

    # 如果 registry 里没找到 (可能是反向生成的任务)，尝试从 ID 解析

    parts <- strsplit(contrast_id, "_vs_")[[1]]

    if (length(parts) == 2) {

            left_group <- parts[1]

            right_group <- parts[2]

    } else {

            left_group <- contrast_id

            right_group <- "Background"

    }

    }



    # 3. 提取 GSEA 结果对象

    gsea_obj <- task_info$data

    res_df <- as.data.frame(gsea_obj@result)



    # 4. 切片逻辑

    is_slice_mode <- length(target_collection) != 1 || toupper(target_collection[1]) != "ALL"



    if (is_slice_mode) {

    # 查找匹配的行

    # 支持 Collection (如 "H") 或 Combo_Name (如 "C2:CP:KEGG_LEGACY")

    match_idx <- which(res_df$Collection %in% target_collection |

            res_df$Combo_Name %in% target_collection)



    if (length(match_idx) == 0) {

            stop(sprintf(

        "Slice failed! No pathways found for '%s' in '%s'.",

        contrast_id, paste(target_collection, collapse = ", ")

            ))

    }



    res_df <- res_df[match_idx, , drop = FALSE]



    # 关键：重算 FDR (消除全库背景的惩罚)

    res_df$p.adjust <- p.adjust(res_df$pvalue, method = "BH")

    if ("qvalue" %in% colnames(res_df)) res_df$qvalue <- res_df$p.adjust



    message(sprintf("[Smart Slice] Extracted %d pathways. FDR recalculated.", nrow(res_df)))



    # 更新 S4 对象

    gsea_obj@result <- res_df

    }



    # 5. 构建 GseaTask 对象

    # 整合 meta 信息，供下游绘图使用

    task_meta <- list(

    contrast_id = contrast_id,

    left_group = left_group,

    right_group = right_group,

    geneset_name = ifelse(is_slice_mode,

            paste(target_collection, collapse = "_"),

            gsea_res$geneset_info$name

    ),

    meta_dict = gsea_res$geneset_info$meta_dict,

    expr_bundle = gsea_res$expr_bundle,

    project_info = gsea_res$metadata$project_info,

    backend_info = gsea_res$backend_info

    )



    obj <- create_gsea_task(gsea_res = gsea_obj, meta = task_meta)



    return(obj)

}





# Section: Inspection Functions ----



#' @title View GSEA Result Overview

#' @description Print detailed summary of GseaRes object to console.

#' @param gsea_res GseaRes object

#' @return Invisibly returns the input object (for chaining).

#' @export



#' @examples

#' # Load a pre-computed GseaRes shipped with the package

#' data(precomputed_gseares, package = "GSEAlens")
#'
#' gsea_res <- precomputed_gseares

#' inspect_gsea_res(gsea_res)

inspect_gsea_res <- function(gsea_res) {

    if (!inherits(gsea_res, "GseaRes")) stop("Input object is not of GseaRes class.")



    bi <- gsea_res$backend_info

    results <- gsea_res$results



    message("")

    message(paste(rep("=", 60), collapse = ""))

    message("       GSEAlens Result Summary")

    message(paste(rep("=", 60), collapse = ""))



    # 1. 后端信息

    message("[1] Backend")

    message(sprintf("   Type       : %s", bi$backend))

    message(sprintf("   Run Time   : %s", gsea_res$metadata$run_time))

    message("")



    # 2. 计算状态统计

    message("[2] Calculation Status")

    status_count <- table(vapply(results, function(x) x$status, character(1)))

    for (name in names(status_count)) {

    icon <- if (name == "Success") "OK" else "FAIL"

    message(sprintf("   %s %s : %d", icon, name, status_count[[name]]))

    }

    message("")



    # 3. 详细列表，修改为全部显示

    message("[3] Contrast Details ")

    n_show <- length(results)

    for (i in seq_len(n_show)) {

    task_name <- names(results)[i]

    task <- results[[i]]



    # IRON FIX (2026-07-01, B-7): defensive S4/data.frame dispatch on task$data
    # so the loop survives edge cases (NULL, plain data.frame, or non-gseaResult S4).
    if (task$status == "Success") {

            n_sig <- if (methods::is(task$data, "gseaResult")) {
        sum(task$data@result$p.adjust < 0.05, na.rm = TRUE)
            } else if (is.data.frame(task$data) && "p.adjust" %in% colnames(task$data)) {
        sum(task$data$p.adjust < 0.05, na.rm = TRUE)
            } else if (is.data.frame(task$data) && "padj" %in% colnames(task$data)) {
        sum(task$data$padj < 0.05, na.rm = TRUE)
            } else {
        NA_integer_
            }

            message(sprintf("   [%d] %-20s | FDR<0.05: %s", i, task_name,
                                            if (is.na(n_sig)) "n/a" else as.character(n_sig)))

    } else {

            message(sprintf("   [%d] %-20s | Failed", i, task_name))

    }

    }

    message("")



    # 4. 下一步指引

    message("[4] Next Step")

    message("   Extract specific results for visualization:")

    message(sprintf('   > task <- extract_gsea_task(gsea_res, "%s")', names(results)[1]))

    message("   > plot_directional_gsea(task, ...)")

    message(paste(rep("=", 60), collapse = ""))



    invisible(gsea_res)

}



#' @export

print.GseaRes <- function(x, ...) {

    inspect_gsea_res(x)

}

#' @title Batch Parallel GSEA Calculation

#' @description Consumes a standard GseaEnv object and performs efficient parallel GSEA calculation.

#' @param gsea_env GseaEnv object

#' @param custom_series_name String. Analysis series name

#' @param output_dir String. Output directory for results, default "./GSEA_Output"

#' @param workers Number of parallel cores. Default 4

#' @param bidirectional Logical. Whether to automatically generate reverse contrasts, default TRUE

#' @param minGSSize Minimum gene set size, default 10

#' @param maxGSSize Maximum gene set size, default 500

#' @param pvalueCutoff P-value threshold, default 1

#' @param seed Integer or NULL. Base random seed for per-contrast GSEA

#'   permutation, default 123. Each contrast derives its own deterministic

#'   seed from the base seed and the contrast id, so results are reproducible

#'   across runs and independent of `workers`/`chunk_size`. On

#'   clusterProfiler <= 4.20.x (where the `seed` argument of `GSEA()` is

#'   silently dropped) reproducibility is guaranteed by seeding the worker R

#'   RNG via `withr::local_seed()`; on clusterProfiler >= 4.21.x the derived

#'   seed is also forwarded to `clusterProfiler::GSEA(seed = ...)`. Set to

#'   NULL to restore native (non-reproducible) engine sampling.

#' @param force Logical. Whether to force recalculation, default FALSE

#' @param use_progress Logical. Whether to show progress bar, default TRUE

#' @param chunk_size Integer. Number of tasks per worker per chunk, default NULL (auto)

#' @return GseaRes object

#' @examples


#' # Build a GseaEnv from pre-computed inputs shipped with the package

#' data(preprocessed_limma, package = "GSEAlens")
#'
#' precomp <- preprocessed_limma

#' data(gsea_pathwaysets_toy, package = "GSEAlens")
#'
#' pathways <- gsea_pathwaysets_toy

#' gsea_env <- setup_gsea_env(

#'   fit = precomp$fit,

#'   pathway_obj = pathways,

#'   expr_data = precomp$gsea_limma_voom_data

#' )

#' # Run batch GSEA (writes to a temporary directory; ~20s on the toy set)

#' gsea_res <- batch_calc_gsea(gsea_env, workers = 2, output_dir = tempdir())

#' @export

batch_calc_gsea <- function(gsea_env,

                            custom_series_name = "Auto_Analysis",

                            output_dir = tempdir(),

                            workers = 2,

                            bidirectional = TRUE,

                            minGSSize = 10,

                            maxGSSize = 500,

                            pvalueCutoff = 1,

                            seed = 123,

                            force = FALSE,

                            use_progress = FALSE,

                            chunk_size = NULL) {

    .check_gsea_env(gsea_env)



    start_time <- Sys.time()

    start_ms <- as.numeric(start_time) * 1000



    series_dir <- file.path(output_dir, custom_series_name)

    if (!dir.exists(series_dir)) {

    dir.create(series_dir, recursive = TRUE, showWarnings = FALSE)

    }



    rds_name <- sprintf(

    "GSEA_Capsule_[%s]_[%s].rds",

    custom_series_name, gsea_env$geneset$name

    )

    rds_path <- file.path(series_dir, rds_name)



    if (file.exists(rds_path) && !force) {

    message(sprintf("Cache hit! Existing GSEA capsule detected: %s", rds_name))

    return(readRDS(rds_path))

    }



    registry <- gsea_env$contrast_registry

    de_store <- gsea_env$de_store



    task_metadata <- list()

    for (i in seq_len(nrow(registry))) {

    row <- registry[i, ]

    cid <- row$contrast_id

    task_metadata[[cid]] <- list(

            task_id = cid,

            left_group = row$left_group,

            right_group = row$right_group,

            is_reversed = FALSE

    )

    if (bidirectional) {

            rev_cid <- paste(row$right_group, row$left_group, sep = "_vs_")

            task_metadata[[rev_cid]] <- list(

        task_id = rev_cid,

        left_group = row$right_group,

        right_group = row$left_group,

        is_reversed = TRUE

            )

    }

    }



    total_tasks <- length(task_metadata)

    message(sprintf("Ready: %d contrast tasks pending calculation...", total_tasks))



    # Raise future.globals.maxSize for the duration of this call to accommodate

    # large globals (full DE table + gene set dictionary). Restore the user's

    # original option on exit to avoid leaking the inflated limit.

    old_max_size <- getOption("future.globals.maxSize")

    options(future.globals.maxSize = 192 * 1024^3)

    on.exit(options(future.globals.maxSize = old_max_size), add = TRUE)



    # Use future::availableCores() instead of parallel::detectCores() so that

    # the package does not need to declare a dependency on the `parallel`

    # package. `future` is already in Imports and `availableCores()` honors

    # future options (e.g. future.availableWorkers) for finer control.

    total_cores <- as.integer(future::availableCores())

    # B10 fix: guard against NA / non-integer workers input
    workers <- suppressWarnings(as.integer(workers))
    if (is.na(workers) || workers < 1) workers <- 1
    use_cores <- min(total_cores, max(1L, workers))

    message(sprintf("Hardware scheduling: Using %d cores for parallel computation...", use_cores))



    if (is.null(chunk_size)) {

    chunk_size <- max(1, ceiling(total_tasks / (use_cores * 4)))

    }



    # B7 fix: Set the future backend for this call. Save the current plan so it
    # can be restored on exit (avoid clobbering a plan the user set elsewhere,
    # e.g. in global.R or another Shiny session). Restoring the old plan also
    # avoids the global side-effect of forcing everyone back to `sequential`.
    old_plan <- future::plan(future::multisession, workers = use_cores)

    on.exit(future::plan(old_plan), add = TRUE)



    if (use_progress) {

    if (!requireNamespace("progressr", quietly = TRUE)) {

            stop("Please install progressr package: install.packages('progressr')")

    }

    progressr::handlers(global = TRUE)

    progressr::handlers("progress")

    }



    worker_term2gene <- gsea_env$geneset$term2gene

    worker_meta_dict <- gsea_env$geneset$meta_dict

    worker_de_list <- as.list(de_store)



    task_names <- names(task_metadata)

    task_chunks <- split(task_names, ceiling(seq_along(task_names) / chunk_size))



    message(sprintf(

    "Chunk strategy: %d chunks, up to %d tasks per chunk",

    length(task_chunks), chunk_size

    ))



    # Derive one deterministic seed per task up front, in the MAIN process,
    # so future workers only receive a plain named integer vector and never
    # depend on helper-function visibility across worker serialization.
    # .derive_task_seed() ranks tasks against the alphabetically sorted full
    # task list, making each seed independent of workers / chunk_size.
    task_seeds <- vapply(
    task_names,
    function(tid) {
        v <- .derive_task_seed(seed, tid, task_names)
        if (is.null(v)) NA_integer_ else v
    },
    integer(1)
    )

    p <- progressr::progressor(steps = total_tasks, enable = use_progress)



    res_list <- future.apply::future_lapply(

    X = task_chunks,

    FUN = function(chunk_task_names,

                                        task_metadata,

                                        de_list,

                                        term2gene,

                                        meta_dict,

                                        minGSSize,

                                        maxGSSize,

                                        pvalueCutoff,

                                        task_seeds,

                                        progressor_fn) {

            if (!requireNamespace("clusterProfiler", quietly = TRUE) ||

        !requireNamespace("dplyr", quietly = TRUE)) {

        stop("Worker missing required packages")

            }



            chunk_results <- list()



            for (task_name in chunk_task_names) {

        task_info <- task_metadata[[task_name]]

        task_id <- task_info$task_id



        if (task_info$is_reversed) {

                    original_cid <- paste(task_info$right_group, task_info$left_group, sep = "_vs_")

        } else {

                    original_cid <- task_id

        }



        de_table <- de_list[[original_cid]]



        if (is.null(de_table) || nrow(de_table) == 0) {

                    chunk_results[[task_name]] <- list(

            name = task_id,

            status = "Failed",

            data = NULL,

            genelist = c()

                    )

                    if (!is.null(progressor_fn)) progressor_fn()

                    next

        }



        genelist <- tryCatch(

                    {

            .prepare_rank_vector_fast(de_table, flip = task_info$is_reversed)

                    },

                    error = function(e) {

            c()

                    }

        )



        if (length(genelist) == 0) {

                    chunk_results[[task_name]] <- list(

            name = task_id,

            status = "Failed",

            data = NULL,

            genelist = c()

                    )

                    if (!is.null(progressor_fn)) progressor_fn()

                    next

        }



        detect_case_format <- function(genes) {

                    sample <- head(unique(genes), 100)

                    sample_filtered <- sample[!grepl("^[0-9]", sample)]

                    if (length(sample_filtered) == 0) {

            return("mixed")

                    }

                    n_title <- sum(grepl("^[A-Z][a-z]", sample_filtered))

                    n_upper <- sum(grepl("^[A-Z]{2,}$", sample_filtered))

                    if (n_title > n_upper) {

            return("title_case")

                    }

                    if (n_upper > n_title) {

            return("upper_case")

                    }

                    return("mixed")

        }



        # Note: gene-case detection uses term2gene + names(genelist) only.
        # (geneset_species was previously derived from gsea_env$geneset$species
        #  inside the future worker but never read; removed 2026-07-01.)
        term2gene_format <- detect_case_format(term2gene$gene_symbol)

        de_format <- detect_case_format(names(genelist))



        if (term2gene_format == "upper_case") {

                    if (de_format == "title_case") {

            names(genelist) <- toupper(names(genelist))

                    } else if (de_format == "mixed") {

            warning(

                            "[Ambiguous Gene Symbol Case]\n",

                            "Gene set format: Uppercase (HS / human); DE gene format: Mixed (cannot be auto-detected).\n",

                            "Common cause: a large fraction of gene symbols start with digits (e.g., '12R5', '7SK'), ",

                            "or symbols are non-standard identifiers.\n",

                            "Case auto-conversion was SKIPPED. If DE genes are actually mouse title case (e.g., Gapdh, Irf7), ",

                            "this may silently return zero enrichments. Please manually verify symbol consistency."

            )

                    }

        } else if (term2gene_format == "title_case") {

                    if (de_format == "upper_case") {

            stop(

                            "[Species Mismatch Error]\nGene set species: Mouse (MM) - requires title case gene symbols\nDE gene format: Uppercase (e.g., GAPDH, IRF7) - appears to be human data\n\nPlease use human gene sets (species='HS') for uppercase DE data,\nor provide mouse DE data with title case gene symbols (e.g., Gapdh, Irf7)."

            )

                    } else if (de_format == "mixed") {

            warning(

                            "[Ambiguous Gene Symbol Case]\n",

                            "Gene set format: Title case (MM / mouse); DE gene format: Mixed (cannot be auto-detected).\n",

                            "Case auto-conversion was SKIPPED. If DE genes are actually human uppercase (e.g., GAPDH, IRF7), ",

                            "this may silently return zero enrichments. Please manually verify symbol consistency."

            )

                    }

        } else if (term2gene_format == "mixed") {

                    warning(

            "[Ambiguous Gene Symbol Case]\n",

            "TERM2GENE gene set format could not be auto-detected (possibly contains many non-standard ",

            "or digit-leading symbols). Case auto-conversion was SKIPPED. ",

            "Please manually verify that DE gene symbols and gene set symbols use the same convention."

                    )

        }



        seed_val <- task_seeds[[task_name]]

        if (length(seed_val) == 1L && !is.na(seed_val)) withr::local_seed(seed_val)

        gsea_res <- tryCatch(

                    {

            clusterProfiler::GSEA(

                            geneList = genelist,

                            TERM2GENE = term2gene,

                            minGSSize = minGSSize,

                            maxGSSize = maxGSSize,

                            pvalueCutoff = pvalueCutoff,

                            pAdjustMethod = "BH",

                            verbose = FALSE,

                            seed = if (is.na(seed_val)) FALSE else seed_val,

                            eps = 0

            )

                    },

                    error = function(e) {

            NULL

                    }

        )



        status <- if (!is.null(gsea_res) && nrow(gsea_res@result) > 0) "Success" else "Failed/NoEnrich"



        if (status == "Success" && !is.null(meta_dict)) {

                    gsea_res@result <- .enrich_gsea_result(gsea_res@result, meta_dict)

        }



        chunk_results[[task_name]] <- list(

                    name = task_id,

                    status = status,

                    data = gsea_res,

                    genelist = genelist

        )



        if (!is.null(progressor_fn)) progressor_fn()

            }



            return(chunk_results)

    },

    task_metadata = task_metadata,

    de_list = worker_de_list,

    term2gene = worker_term2gene,

    meta_dict = worker_meta_dict,

    minGSSize = minGSSize,

    maxGSSize = maxGSSize,

    pvalueCutoff = pvalueCutoff,

    task_seeds = task_seeds,

    progressor_fn = if (use_progress) p else NULL,

    future.seed = TRUE,

    future.scheduling = 1.0

    )



    # future::plan(future::sequential) is handled by on.exit() above, so that the

    # plan is restored even if future_lapply throws.



    all_cons <- showConnections(all = TRUE)

    if (nrow(all_cons) > 1) {

    for (con_idx in as.integer(rownames(all_cons))) {

            if (con_idx > 2) {

        try(suppressWarnings(close.connection(getConnection(con_idx))), silent = TRUE)

            }

    }

    }

# suppressWarnings: suppress "closing unused connection" noise during cleanup

        

    invisible(gc(verbose = FALSE, full = TRUE))



    res_list_flat <- do.call(c, res_list)

    names(res_list_flat) <- names(task_metadata)



    end_time <- Sys.time()

    end_ms <- as.numeric(end_time) * 1000



    final_obj <- create_gsea_res(

    metadata = list(

            run_time = format(Sys.time(), "%Y-%m-%d %H:%M:%S"),

            workers_used = use_cores,

            chunk_size = chunk_size,

            parameters = list(

        bidirectional = bidirectional,

        minGSSize = minGSSize,

        maxGSSize = maxGSSize,

        pvalueCutoff = pvalueCutoff,

        seed = seed

            ),

            project_info = list(

        custom_series_name = custom_series_name,

        output_dir = normalizePath(output_dir, mustWork = FALSE),

        series_dir = normalizePath(series_dir, mustWork = FALSE),

        rds_path = normalizePath(rds_path, mustWork = FALSE)

            ),

            gsea_benchmark = list(

        start_time = format(start_time, "%Y-%m-%d %H:%M:%OS3"),

        start_ms = start_ms,

        end_time = format(end_time, "%Y-%m-%d %H:%M:%OS3"),

        end_ms = end_ms,

        duration_sec = round((end_ms - start_ms) / 1000, 3),

        workers = use_cores,

        total_tasks = length(task_metadata),

        successful_tasks = sum(vapply(res_list_flat, function(x) x$status == "Success", logical(1)))

            )

    ),

    backend_info = gsea_env$backend_info,

    contrast_registry = gsea_env$contrast_registry,

    de_store = gsea_env$de_store,

    expr_bundle = gsea_env$expr_bundle,

    geneset_info = gsea_env$geneset,

    results = res_list_flat

    )



    saveRDS(final_obj, rds_path)



    success_count <- final_obj$metadata$gsea_benchmark$successful_tasks



    message(sprintf(

    "\nCalculation complete! Time elapsed: %.2f seconds",

    final_obj$metadata$gsea_benchmark$duration_sec

    ))

    message(sprintf("   Successfully analyzed: %d/%d contrasts", success_count, total_tasks))

    message(sprintf("   Results saved to: %s", rds_path))



    return(final_obj)

}





#' @title Derive Deterministic Per-Task Seed
#'
#' @param seed Integer or NULL. Base seed chosen by the user.
#' @param task_id Character. Contrast task identifier.
#' @param all_task_ids Character vector. All task ids in the current run.
#' @return Integer seed derived from the base seed and the task's rank in the
#'   alphabetically sorted task list, or NULL when `seed` is NULL, NA or
#'   non-numeric. Ranking
#'   against the sorted full task list (not the chunk-local order) makes the
#'   derived seed independent of `workers` and `chunk_size`.
#' @keywords internal
.derive_task_seed <- function(seed, task_id, all_task_ids) {
    if (is.null(seed) || length(seed) == 0L ||
        !is.numeric(seed) || is.na(seed[1])) return(NULL)
    seed <- as.numeric(seed[1])
    idx <- match(task_id, sort(all_task_ids))
    if (is.na(idx)) idx <- 1L
    as.integer((seed + idx - 1) %% .Machine$integer.max)
}

#' @title Fast Rank Vector Preparation

#' @param de_table DE result table

#' @param flip Whether to flip sign

#' @return Named numeric vector

#' @keywords internal

.prepare_rank_vector_fast <- function(de_table, flip = FALSE) {

    # Preserve original case; format standardization is performed in worker function

    # based on TERM2GENE

    vals_df <- de_table |>

    dplyr::filter(!is.na(gene_symbol), gene_symbol != "") |>

    dplyr::mutate(abs_stat = abs(stat)) |>

    dplyr::arrange(dplyr::desc(abs_stat)) |>

    dplyr::distinct(gene_symbol, .keep_all = TRUE)



    vals <- vals_df$stat

    names(vals) <- vals_df$gene_symbol



    if (flip) vals <- -vals

    sort(vals, decreasing = TRUE)

}





#' @title GSEA Result Metadata Injection

#' @param result_df GSEA result dataframe

#' @param meta_dict Metadata dictionary

#' @return Enriched dataframe

#' @keywords internal

.enrich_gsea_result <- function(result_df, meta_dict) {

    if (!is.data.frame(result_df) || nrow(result_df) == 0) {

    warning("result_df is invalid or empty")

    return(result_df)

    }



    if (!is.data.frame(meta_dict) || nrow(meta_dict) == 0) {

    warning("meta_dict is empty, cannot enrich results")

    return(result_df)

    }



    if (!"ID" %in% colnames(result_df) || !"ID" %in% colnames(meta_dict)) {

    stop("Both result_df and meta_dict must contain 'ID' column")

    }



    original_rownames <- rownames(result_df)

    original_ids <- result_df$ID



    required_cols <- c("ID", "Description", "URL", "Collection", "Subcollection", "Combo_Name")



    for (col in required_cols) {

    if (!col %in% colnames(meta_dict)) {

            warning(sprintf("meta_dict missing column '%s', creating placeholder", col))

            meta_dict[[col]] <- NA_character_

    }

    }



    if (all(is.na(meta_dict$Subcollection)) || is.null(meta_dict$Subcollection)) {

    meta_dict$Subcollection <- ""

    }



    if (all(is.na(meta_dict$Combo_Name))) {

    meta_dict$Combo_Name <- ifelse(

            is.na(meta_dict$Subcollection) | meta_dict$Subcollection == "",

            meta_dict$Collection,

            paste0(meta_dict$Collection, ":", meta_dict$Subcollection)

    )

    }



    core_stat_cols <- c(

    "ID", "setSize", "enrichmentScore", "NES", "pvalue",

    "p.adjust", "qvalue", "rank", "leading_edge", "core_enrichment"

    )



    conflict_cols <- c("Description", "URL", "Collection", "Subcollection", "Combo_Name")



    cols_to_keep <- setdiff(colnames(result_df), conflict_cols)

    result_work <- result_df[, cols_to_keep, drop = FALSE]



    meta_subset <- meta_dict[, required_cols, drop = FALSE]



    merged_df <- result_work |>

    dplyr::left_join(

            meta_subset,

            by = "ID",

            suffix = c("", "_meta")

    )



    if (nrow(merged_df) != nrow(result_work)) {

    warning(sprintf(

            "Row count changed during merge: %d -> %d",

            nrow(result_work), nrow(merged_df)

    ))

    }



    merged_df$Display_Collection <- dplyr::coalesce(

    merged_df$Combo_Name,

    merged_df$Collection,

    "Unknown"

    )

    merged_df$Display_Collection <- as.factor(merged_df$Display_Collection)



    merged_df$Pathway_Link <- ifelse(

    is.na(merged_df$URL) | merged_df$URL == "",

    sprintf("<b>%s</b>", merged_df$ID),

    sprintf('<a href="%s" target="_blank">%s</a>', merged_df$URL, merged_df$ID)

    )



    if ("Description_meta" %in% colnames(merged_df)) {

    merged_df$Description <- dplyr::coalesce(

            merged_df$Description_meta,

            merged_df$ID

    )

    merged_df$Description_meta <- NULL

    } else if (!"Description" %in% colnames(merged_df)) {

    merged_df$Description <- merged_df$ID

    }



    merged_df$Description[is.na(merged_df$Description)] <- merged_df$ID[is.na(merged_df$Description)]



    meta_temp_cols <- grep("_meta$", colnames(merged_df), value = TRUE)

    if (length(meta_temp_cols) > 0) {

    merged_df <- merged_df[, !(colnames(merged_df) %in% meta_temp_cols), drop = FALSE]

    }



    merged_df <- as.data.frame(merged_df)

    if ("ID" %in% colnames(merged_df)) {

    if (any(duplicated(merged_df$ID))) {

            warning("Duplicate IDs found in result, using original rownames")

            rownames(merged_df) <- original_rownames

    } else {

            rownames(merged_df) <- merged_df$ID

    }

    } else {

    rownames(merged_df) <- original_rownames

    }



    standard_cols <- c(

    "ID", "setSize", "enrichmentScore", "NES", "pvalue",

    "p.adjust", "qvalue", "rank", "leading_edge", "core_enrichment",

    "Description", "URL", "Collection", "Subcollection", "Combo_Name",

    "Display_Collection", "Pathway_Link"

    )



    missing_standard <- setdiff(standard_cols, colnames(merged_df))

    if (length(missing_standard) > 0) {

    warning(sprintf(

            "Final result missing standard columns: %s",

            paste(missing_standard, collapse = ", ")

    ))

    for (col in missing_standard) {

            merged_df[[col]] <- NA_character_

    }

    }



    present_cols <- intersect(standard_cols, colnames(merged_df))

    extra_cols <- setdiff(colnames(merged_df), standard_cols)

    merged_df <- merged_df[, c(present_cols, extra_cols), drop = FALSE]



    message(sprintf(

    "[.enrich_gsea_result] Successfully enriched: %d rows x %d columns",

    nrow(merged_df), ncol(merged_df)

    ))



    return(merged_df)

}

# Tests for deterministic seeding of batch_calc_gsea() (v0.99.34)
#
# Background: clusterProfiler 4.19.3-4.20.x (Bioc 3.23) silently drops the
# `seed` argument of GSEA() (it lands in `...` and is never forwarded to
# enrichit::gsea_gson), so the engine draws a fresh seed from the worker R RNG
# on every call. These tests pin the two guarantees of the fix:
#   1. same `seed` -> identical results across runs
#   2. results independent of `workers` / chunking

test_that(".derive_task_seed is deterministic and order-independent", {
  ids <- c("B_vs_A", "A_vs_B", "C_vs_A")
  expect_identical(
    .derive_task_seed(123, "A_vs_B", ids),
    .derive_task_seed(123, "A_vs_B", rev(ids))
  )
  expect_identical(
    .derive_task_seed(123, "A_vs_B", ids),
    .derive_task_seed(123, "A_vs_B", sample(ids))
  )
  # distinct tasks get distinct seeds
  seeds <- vapply(sort(ids), function(id) .derive_task_seed(50, id, ids), integer(1))
  expect_identical(length(unique(seeds)), 3L)
  # NULL / NA / non-numeric seeds hand control back to the engine
  expect_null(.derive_task_seed(NULL, "A_vs_B", ids))
  expect_null(.derive_task_seed(NA, "A_vs_B", ids))
  expect_null(.derive_task_seed("abc", "A_vs_B", ids))
})

build_seed_test_env <- function() {
  genes <- sprintf("G%03d", 1:300)
  stat <- seq(3, -3, length.out = 300)
  de_a_b <- data.frame(gene_symbol = genes, stat = stat, stringsAsFactors = FALSE)
  de_a_c <- data.frame(
    gene_symbol = genes, stat = stat * rev(seq_len(300)) / 300,
    stringsAsFactors = FALSE
  )
  # 4 gene sets of 40 genes each, deterministic membership (no RNG)
  t2g <- do.call(rbind, lapply(seq_len(4), function(i) {
    data.frame(
      gs_name = paste0("PW", i),
      gene_symbol = genes[((i - 1) * 40 + 1):(i * 40)],
      stringsAsFactors = FALSE
    )
  }))
  env <- list(
    backend_info = list(backend = "test"),
    contrast_registry = data.frame(
      contrast_id = c("A_vs_B", "A_vs_C"),
      left_group = c("A", "A"),
      right_group = c("B", "C"),
      stringsAsFactors = FALSE
    ),
    de_store = list(A_vs_B = de_a_b, A_vs_C = de_a_c),
    expr_bundle = list(),
    geneset = list(
      name = "seed_test_gs",
      term2gene = t2g,
      meta_dict = data.frame(
        ID = paste0("PW", 1:4), Collection = "H",
        stringsAsFactors = FALSE
      )
    )
  )
  class(env) <- "GseaEnv"
  env
}

test_that("batch_calc_gsea with a fixed seed is reproducible across runs", {
  env <- build_seed_test_env()
  out <- file.path(tempdir(), "seed-repro")
  r1 <- batch_calc_gsea(env, workers = 2, bidirectional = FALSE,
                        output_dir = out, force = TRUE)
  r2 <- batch_calc_gsea(env, workers = 2, bidirectional = FALSE,
                        output_dir = out, force = TRUE)
  for (task in c("A_vs_B", "A_vs_C")) {
    expect_identical(
      r1$results[[task]]$data@result$pvalue,
      r2$results[[task]]$data@result$pvalue
    )
  }
  expect_identical(r1$metadata$parameters$seed, 123)
})

test_that("batch_calc_gsea results do not depend on worker count", {
  env <- build_seed_test_env()
  out <- file.path(tempdir(), "seed-workers")
  r_seq <- batch_calc_gsea(env, workers = 1, bidirectional = FALSE,
                           output_dir = out, force = TRUE)
  r_par <- batch_calc_gsea(env, workers = 2, bidirectional = FALSE,
                           output_dir = out, force = TRUE)
  for (task in c("A_vs_B", "A_vs_C")) {
    expect_identical(
      r_seq$results[[task]]$data@result$pvalue,
      r_par$results[[task]]$data@result$pvalue
    )
  }
})

#!/usr/bin/env Rscript

# Description -------------------------------------------------------------

# Pre-compute the three input objects (`fit`, `dds_se`, `dds`) that the main

# GSEAlens vignette consumes, following EXACTLY the steps documented in

# `vignettes/GSEAlens-preprocessing.Rmd`.

#

# Why: running these steps during vignette compilation on the Bioconductor

# build machine is slow (DESeq2 DESeq() + limma-voom pipeline on airway) and

# triggers the "vignette rebuilding took > 60s" NOTE. By shipping the

# pre-computed objects in data/, the main vignette can set

# `eval=TRUE` on the downstream setup_gsea_env() chunks so reviewers see a

# fully runnable workflow.

#

# Run manually (maintainer only) when DESeq2/limma/airway versions bump:

#

#   Rscript inst/scripts/make_preprocessed_inputs.R

#

# Output files (written to data/):

#   - preprocessed_limma.rds    (list with fit + gsea_limma_voom_data)

#   - preprocessed_dds_se.rds   (DESeq-transformed RangedSummarizedExperiment)

#   - preprocessed_dds.rds      (DESeqDataSet from count matrix)

# -------------------------------------------------------------------------



stopifnot(

  requireNamespace("airway",    quietly = TRUE),

  requireNamespace("limma",     quietly = TRUE),

  requireNamespace("edgeR",     quietly = TRUE),

  requireNamespace("DESeq2",    quietly = TRUE),

  requireNamespace("SummarizedExperiment", quietly = TRUE)

)



library(airway)

library(edgeR)

library(limma)

library(DESeq2)

library(SummarizedExperiment)



data(airway)

expression_data <- airway



# ===== limma-voom workflow (verbatim from preprocessing vignette) =====

group_level <- expression_data$dex

design <- model.matrix(~0 + group_level)

colnames(design) <- levels(group_level)

compare_end <- combn(levels(group_level), 2, simplify = FALSE)

contrast_strings <- sapply(compare_end, function(x) paste(x[2], x[1], sep = " - "))

contrast_matrix <- makeContrasts(contrasts = contrast_strings, levels = design)



genes_df <- data.frame(

  gene_id = rowData(expression_data)$gene_id,

  symbol = rowData(expression_data)$symbol,

  gene_biotype = rowData(expression_data)$gene_biotype

)

genes_df$Length <- rowData(expression_data)$gene_seq_end -

                   rowData(expression_data)$gene_seq_start + 1



gsea_limma_voom_data <- DGEList(

  counts = assay(expression_data, "counts"),

  genes = genes_df,

  norm.factors = NULL,

  group = group_level,

  remove.zeros = TRUE

)



total_counts <- cpm(gsea_limma_voom_data) |> rowSums()

dup_symbols <- gsea_limma_voom_data$genes$symbol[duplicated(gsea_limma_voom_data$genes$symbol)]

keep <- rep(TRUE, nrow(gsea_limma_voom_data))

for (gene in dup_symbols) {

  idx <- which(gsea_limma_voom_data$genes$symbol == gene)

  best_idx <- idx[which.max(total_counts[idx])]

  remove_idx <- idx[idx != best_idx]

  keep[remove_idx] <- FALSE

}

gsea_limma_voom_data <- gsea_limma_voom_data[keep, ]

rownames(gsea_limma_voom_data) <- gsea_limma_voom_data$genes$symbol



keep_biotype <- gsea_limma_voom_data$genes$gene_biotype == "protein_coding"

gsea_limma_voom_data <- gsea_limma_voom_data[keep_biotype, ]

gsea_limma_voom_data <- normLibSizes(gsea_limma_voom_data, method = "TMM")

isexpr <- rowSums(cpm(gsea_limma_voom_data) > 1) >= 3

gsea_limma_voom_data <- gsea_limma_voom_data[isexpr, ]



VoomOutPut <- voom(gsea_limma_voom_data, design)

fit <- lmFit(object = VoomOutPut, design = design) |>

  contrasts.fit(contrasts = contrast_matrix) |>

  eBayes()



# ===== DESeq2 SummarizedExperiment workflow (verbatim) =====

dds_se <- DESeqDataSet(expression_data, design = ~ cell + dex)

gene_biotypes <- rowData(dds_se)$gene_biotype

keep_protein_coding <- gene_biotypes == "protein_coding"

dds_se <- dds_se[keep_protein_coding, ]



smallestGroupSize <- 3

keep <- rowSums(counts(dds_se) >= 10) >= smallestGroupSize

dds_se <- dds_se[keep, ]

rownames(dds_se) <- rowData(dds_se)$gene_name



total_counts <- rowSums(assay(dds_se))

dup_genes <- rownames(dds_se)[duplicated(rownames(dds_se))]

keep <- rep(TRUE, nrow(dds_se))

for (gene in dup_genes) {

  idx <- which(rownames(dds_se) == gene)

  best_idx <- idx[which.max(total_counts[idx])]

  remove_idx <- idx[idx != best_idx]

  keep[remove_idx] <- FALSE

}

dds_se <- dds_se[keep, ]

dds_se <- DESeq(dds_se)



# ===== DESeq2 Count Matrix workflow (verbatim) =====

DDS_rawdata <- expression_data

gene_biotypes <- rowData(DDS_rawdata)$gene_biotype

keep_protein_coding <- gene_biotypes == "protein_coding"

DDS_rawdata <- DDS_rawdata[keep_protein_coding, ]



smallestGroupSize <- 3

keep_epd <- rowSums(assay(DDS_rawdata, "counts") >= 10) >= smallestGroupSize

DDS_rawdata <- DDS_rawdata[keep_epd, ]

rownames(DDS_rawdata) <- rowData(DDS_rawdata)$gene_name



total_counts <- rowSums(assay(DDS_rawdata))

keep_name <- rep(TRUE, nrow(DDS_rawdata))

dup_genes <- rownames(DDS_rawdata)[duplicated(rownames(DDS_rawdata))]

for (gene in dup_genes) {

  idx <- which(rownames(DDS_rawdata) == gene)

  best_idx <- idx[which.max(total_counts[idx])]

  remove_idx <- idx[idx != best_idx]

  keep_name[remove_idx] <- FALSE

}

DDS_rawdata <- DDS_rawdata[keep_name, ]



cts <- assay(DDS_rawdata, "counts")

coldata <- as.data.frame(colData(DDS_rawdata))

coldata <- coldata[, c("cell", "dex")]

coldata$cell <- factor(coldata$cell)

coldata$dex <- factor(coldata$dex)



dds <- DESeqDataSetFromMatrix(countData = cts, colData = coldata, design = ~ dex)

dds <- DESeq(dds)



# ===== Save ----------------------------------------------------------------------

out_dir <- file.path("data")

dir.create(out_dir, showWarnings = FALSE, recursive = TRUE)



saveRDS(

  list(fit = fit, gsea_limma_voom_data = gsea_limma_voom_data),

  file.path(out_dir, "preprocessed_limma.rds"),

  compress = "xz"

)

# Slim preprocessed_dds_se.rds before saving to stay below the Bioconductor
# 5 MB extdata limit. The full DESeqDataSet from DESeq() on the pasilla data
# is ~8 MB on disk because each gene is wrapped in a GRangesList of ~20
# transcript sub-ranges and DESeq() also stores mu/H/cooks assay layers.
# GSEAlens only needs the gene-level fits, so the slimming is non-destructive
# for every code path that GSEAlens or its vignettes exercise.
#
# What is removed and why (all verified to be unused by GSEAlens):
#   * mu / H / cooks assays  -> DESeq() fitting intermediates. GSEAlens
#                               only reads the 'counts' assay and the
#                               Wald statistics stored in mcols (via
#                               DESeq2::results()). log2FoldChange / pvalue
#                               / padj are bit-identical before vs. after.
#   * GRangesList -> GRanges -> keep one representative range per gene.
#                               rowRanges() is never accessed by GSEAlens.
#   * 5 columns of Ensembl coordinate metadata (gene_seq_start, gene_seq_end,
#     seq_name, seq_strand, seq_coord_system) -> gene_id / gene_name /
#     symbol / gene_biotype are KEPT so users can still inspect gene info.
#
# Users who want the full, untrimmed DESeqDataSet for teaching DESeq2 itself
# should re-run this script without calling slim_dds_se(), or follow the
# preprocessing vignette to rebuild one from their own count matrix.
slim_dds_se <- function(dds_se) {
  # (1) Drop DESeq() intermediate assays (mu/H/cooks); keep counts only.
  SummarizedExperiment::assays(dds_se) <-
    SummarizedExperiment::assays(dds_se)["counts"]

  # (2) GRangesList -> GRanges: keep one representative range per gene.
  rr <- SummarizedExperiment::rowRanges(dds_se)
  if (inherits(rr, "GRangesList")) {
    grp_lens <- S4Vectors::elementNROWS(rr)
    first_idx <- cumsum(c(1L, head(grp_lens, -1L)))
    gr_flat <- unlist(rr, use.names = FALSE)
    gr_first <- gr_flat[first_idx]
    mcols(gr_first) <- mcols(rr)
    names(gr_first) <- rownames(dds_se)
    SummarizedExperiment::rowRanges(dds_se) <- gr_first
  }

  # (3) Drop 5 Ensembl coordinate columns; keep gene_id / gene_name /
  #     symbol / gene_biotype so gene-level metadata stays queryable.
  redundant_anno <- c("gene_seq_start", "gene_seq_end",
                      "seq_name", "seq_strand", "seq_coord_system")
  rr2 <- SummarizedExperiment::rowRanges(dds_se)
  mc2 <- mcols(rr2)
  mcols(rr2) <- mc2[, setdiff(colnames(mc2), redundant_anno), drop = FALSE]
  SummarizedExperiment::rowRanges(dds_se) <- rr2

  stopifnot(validObject(dds_se))
  dds_se
}

saveRDS(slim_dds_se(dds_se), file.path(out_dir, "preprocessed_dds_se.rds"), compress = "xz")
saveRDS(dds,    file.path(out_dir, "preprocessed_dds.rds"),    compress = "xz")



# ===== Report --------------------------------------------------------------------

files <- c("preprocessed_limma.rds", "preprocessed_dds_se.rds", "preprocessed_dds.rds")

sizes <- file.size(file.path(out_dir, files))

md5 <- tools::md5sum(file.path(out_dir, files))



cat("Wrote:\n")

for (i in seq_along(files)) {

  cat(sprintf("  %-32s  %7.1f KB  md5=%s\n",

              files[i], sizes[i] / 1024, md5[i]))

}

cat(sprintf("\nlimma: fit has %d genes; gsea_limma_voom_data has %d rows\n",

            nrow(fit$coefficients), nrow(gsea_limma_voom_data)))

cat(sprintf("dds_se: %d genes x %d samples; results names: %s\n",

            nrow(dds_se), ncol(dds_se),

            paste(resultsNames(dds_se), collapse = ", ")))

cat(sprintf("dds:    %d genes x %d samples; results names: %s\n",

            nrow(dds), ncol(dds),

            paste(resultsNames(dds), collapse = ", ")))



cat("\nairway  version:", as.character(packageVersion("airway")), "\n")

cat("DESeq2  version:", as.character(packageVersion("DESeq2")), "\n")

cat("limma   version:", as.character(packageVersion("limma")), "\n")

cat("edgeR   version:", as.character(packageVersion("edgeR")), "\n")

cat("Generation date:", format(Sys.Date(), "%Y-%m-%d"), "\n")

---
title: "GSEAlens：从 limma-voom 和 DESeq2 流程准备输入数据"
author: "Shenhui Xu"
date: "`r Sys.Date()`"
output:
  BiocStyle::html_document:
    toc_float: true
    number_sections: true
vignette: >
  %\VignetteIndexEntry{GSEAlens：准备输入数据}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, warning = FALSE, message = FALSE)
```

# 安装

GSEAlens 可从 Bioconductor 安装。使用以下命令安装稳定版本：

```{r install-package, eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("GSEAlens")
```

开发版本可从 GitHub 安装：

```{r install-github, eval=FALSE}
if (!requireNamespace("pak", quietly = TRUE))
    install.packages("pak")
pak::pkg_install("DDL095/GSEAlens")
```

# 简介

本 vignette 描述如何准备 `r Biocpkg("GSEAlens")` 所需的输入对象。**GSEAlens 本身
不执行差异表达分析（DEG）**；它接收来自 `r Biocpkg("limma")` 的已拟合 `MArrayLM`
对象或来自 `r Biocpkg("DESeq2")` 的 `DESeqDataSet` 对象。以下步骤基于这些包的标准
工作流程，此处提供是为了完整性。
完整的 DEG 工作流程文档请参阅：

- `r Biocpkg("limma")` 和 `r Biocpkg("edgeR")` 的 vignette（limma-voom 流程）

- `r Biocpkg("DESeq2")` 的 vignette

- `r Biocpkg("airway")` 包提供示例数据集

# 示例数据

我们使用 `r Biocpkg("airway")` 数据集（4 vs 4 样本，dex 处理）。

```{r setup-environment, results='hide'}
library(airway)
data(airway)
expression_data <- airway
```

# limma-voom 流程

GSEAlens 要求**无截距设计**（`~0+group`），使列名直接对应组别名称，从而精确构建
对比。

```{r limma-voom-data-preparation}
library(edgeR)
library(limma)
group_level <- expression_data$dex
design <- model.matrix(~0+group_level)
colnames(design) <- levels(group_level)
compare_end <- combn(levels(group_level), 2, simplify = FALSE)
contrast_strings <- sapply(compare_end, function(x) paste(x[2], x[1], sep = " - "))
contrast_matrix <- makeContrasts(contrasts = contrast_strings, levels = design)
genes_df <- data.frame(
  gene_id = SummarizedExperiment::rowData(expression_data)$gene_id,
  symbol = SummarizedExperiment::rowData(expression_data)$symbol,
  gene_biotype = SummarizedExperiment::rowData(expression_data)$gene_biotype
)
genes_df$Length <- SummarizedExperiment::rowData(expression_data)$gene_seq_end -
                   SummarizedExperiment::rowData(expression_data)$gene_seq_start + 1
gsea_limma_voom_data <- edgeR::DGEList(
  counts = SummarizedExperiment::assay(expression_data, "counts"),
  genes = genes_df,
  norm.factors = NULL,
  group = group_level,
  remove.zeros = TRUE
)
# 过滤低表达基因，保留蛋白质编码 RNA，去重复 symbol
total_counts <- edgeR::cpm(gsea_limma_voom_data) |> rowSums()
dup_symbols <- gsea_limma_voom_data$genes$symbol[duplicated(gsea_limma_voom_data$genes$symbol)]
keep <- rep(TRUE, nrow(gsea_limma_voom_data))
for (gene in dup_symbols) {
  idx <- which(gsea_limma_voom_data$genes$symbol == gene)
  best_idx <- idx[which.max(total_counts[idx])]
  remove_idx <- idx[idx != best_idx]
  keep[remove_idx] <- FALSE
}
gsea_limma_voom_data <- gsea_limma_voom_data[keep, ]
rownames(gsea_limma_voom_data) <- gsea_limma_voom_data$genes$symbol
keep_biotype <- gsea_limma_voom_data$genes$gene_biotype == "protein_coding"
gsea_limma_voom_data <- gsea_limma_voom_data[keep_biotype, ]
gsea_limma_voom_data <- edgeR::normLibSizes(gsea_limma_voom_data, method = "TMM")
isexpr <- rowSums(edgeR::cpm(gsea_limma_voom_data) > 1) >= 3
gsea_limma_voom_data <- gsea_limma_voom_data[isexpr, ]
```

进行 limma-voom 拟合，获取 `fit` 对象。

```{r limma-voom-processing}
VoomOutPut <- voom(gsea_limma_voom_data, design)
fit <- lmFit(object = VoomOutPut, design = design) |>
  contrasts.fit(contrasts = contrast_matrix) |>
  eBayes()
```

# DESeq2 流程（SummarizedExperiment 输入）

```{r deseq2-se-workflow}
library("DESeq2")
dds_se <- DESeqDataSet(expression_data, design = ~ cell + dex)
# 仅保留蛋白编码基因
gene_biotypes <- SummarizedExperiment::rowData(dds_se)$gene_biotype
keep_protein_coding <- gene_biotypes == "protein_coding"
dds_se <- dds_se[keep_protein_coding, ]
# 去除低表达基因（要求 >=10 reads 且至少出现在 >=3 个样本中）
smallestGroupSize <- 3
keep <- rowSums(DESeq2::counts(dds_se) >= 10) >= smallestGroupSize
dds_se <- dds_se[keep, ]
# 基因符号去重，保留表达量最高的那一行
rownames(dds_se) <- SummarizedExperiment::rowData(dds_se)$gene_name
total_counts <- rowSums(SummarizedExperiment::assay(dds_se))
dup_genes <- rownames(dds_se)[duplicated(rownames(dds_se))]
keep <- rep(TRUE, nrow(dds_se))
for (gene in dup_genes) {
  idx <- which(rownames(dds_se) == gene)
  best_idx <- idx[which.max(total_counts[idx])]
  remove_idx <- idx[idx != best_idx]
  keep[remove_idx] <- FALSE
}
dds_se <- dds_se[keep, ]
# 运行 DESeq2 拟合
dds_se <- DESeq(dds_se)
```

# DESeq2 流程（Count Matrix 输入）

```{r deseq2-matrix-workflow}
DDS_rawdata <- expression_data
# 仅保留蛋白编码基因
gene_biotypes <- SummarizedExperiment::rowData(DDS_rawdata)$gene_biotype
keep_protein_coding <- gene_biotypes == "protein_coding"
DDS_rawdata <- DDS_rawdata[keep_protein_coding, ]
# 去除低表达基因
smallestGroupSize <- 3
keep_epd <- rowSums(SummarizedExperiment::assay(DDS_rawdata, "counts") >= 10) >= smallestGroupSize
DDS_rawdata <- DDS_rawdata[keep_epd, ]
# 基因符号去重
rownames(DDS_rawdata) <- SummarizedExperiment::rowData(DDS_rawdata)$gene_name
total_counts <- rowSums(SummarizedExperiment::assay(DDS_rawdata))
keep_name <- rep(TRUE, nrow(DDS_rawdata))
dup_genes <- rownames(DDS_rawdata)[duplicated(rownames(DDS_rawdata))]
for (gene in dup_genes) {
  idx <- which(rownames(DDS_rawdata) == gene)
  best_idx <- idx[which.max(total_counts[idx])]
  remove_idx <- idx[idx != best_idx]
  keep_name[remove_idx] <- FALSE
}
DDS_rawdata <- DDS_rawdata[keep_name, ]
# 抽取 counts 矩阵和样本元数据
cts <- SummarizedExperiment::assay(DDS_rawdata, "counts")
coldata <- as.data.frame(SummarizedExperiment::colData(DDS_rawdata))
coldata <- coldata[, c("cell", "dex")]
coldata$cell <- factor(coldata$cell)
coldata$dex <- factor(coldata$dex)
# 从矩阵构建 DESeqDataSet 并运行 DESeq2
dds <- DESeqDataSetFromMatrix(countData = cts, colData = coldata, design = ~ dex)
dds <- DESeq(dds)
```

# 下一步

`fit`、`dds_se` 和 `dds` 准备就绪后，返回主 vignette：

```r

vignette("GSEAlens")

```

```{r sessionInfo}
sessionInfo()
```

---
title: "GSEAlens: An Interactive Exploration Platform for Gene Set Enrichment Analysis"
author: "Shenhui Xu"
date: "`r Sys.Date()`"
output:
  BiocStyle::html_document:
    toc_float: true
    number_sections: true
vignette: >
  %\VignetteIndexEntry{GSEAlens: An Interactive Exploration Platform for Gene Set Enrichment Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, warning = FALSE, message = FALSE)
```

# Package Introduction

**GSEAlens** derives its name from **Lens**, symbolizing how this package acts as a **magnifying glass** to help researchers deeply explore key pathways in GSEA enrichment analysis.

GSEAlens provides a web-based interactive platform for displaying pathway introductions and descriptions, integrating AI-assisted pathway enrichment result export functionality. By encapsulating workflows and standardizing input formats, this R package simplifies the process of viewing and exploring GSEA enrichment analysis results.

## Integration with Bioconductor Workflows

GSEAlens is designed to plug into standard Bioconductor RNA-seq and functional
enrichment workflows as a **post-DEG exploration layer**:

- **Upstream (input preparation)**: GSEAlens accepts fitted model objects from
  `r Biocpkg("limma")` (the `MArrayLM` object returned by `limma::eBayes()`,
  based on the `r Biocpkg("edgeR")` + limma-voom pipeline) or `DESeqDataSet`
  objects from `r Biocpkg("DESeq2")`. The expression matrix and sample metadata
  can be passed as `r Biocpkg("SummarizedExperiment")` objects, ensuring
  interoperability with the core Bioconductor data containers.

- **Enrichment computation**: Under the hood, GSEAlens wraps
  `r Biocpkg("clusterProfiler")` (`GSEA()` function) and uses gene set
  collections from `r CRANpkg("msigdbr")`. The statistical framework therefore
  inherits the methodology of `r Biocpkg("fgsea")` via `clusterProfiler`.

- **Parallelization**: GSEAlens uses `r CRANpkg("future")` (`future::multisession`)
  for multi-contrast parallel computation. The user's original `future::plan()` and
  `future.globals.maxSize` option are saved before the parallel run and restored
  on exit via `on.exit()`, so the global state is never polluted. We anecdotally
  observed `future::multisession` to be noticeably faster than
  `r Biocpkg("BiocParallel")` (`SnowParam` / PSOCK serialization) on Windows for
  this package's typical workload, which serializes large globals (the full DE
  table plus the gene set dictionary and metadata dictionary). This is an informal
  observation from development, not a formal benchmark; users are free to re-run
  the analysis with `BiocParallel` if it better suits their environment.

- **Visualization**: Plotting builds on `r Biocpkg("enrichplot")`,
  `r Biocpkg("ComplexHeatmap")`, `r CRANpkg("ggplot2")`,
  `r CRANpkg("patchwork")`, and `r CRANpkg("visNetwork")`, producing figures
  compatible with downstream publication pipelines.

- **Downstream**: The "Generate R Code" feature in the Shiny application emits
  self-contained scripts that can be embedded in `r CRANpkg("rmarkdown")` /
  Quarto reports or integrated into multi-step pipelines.

A typical end-to-end Bioconductor workflow is therefore:

```

RNA-seq counts

   |
   +--[edgeR + limma-voom]--> MArrayLM fit --+
   |                                         |
   +--[DESeq2]--------------> DESeqDataSet --+
                                             |
                                             v
                                   [setup_gsea_env]
                                             |
                                             v
                                  [batch_calc_gsea]
                                             |
                                             v
                                    Shiny app exploration
                                             |
                                             v
                                  Reproducible R code export

```

This makes GSEAlens a natural complement to existing Bioconductor enrichment
packages: where `r Biocpkg("clusterProfiler")`, `r Biocpkg("GSVA")`,
`r Biocpkg("gprofiler2")`, or `r Biocpkg("ReactomePA")` focus on **computing**
enrichment, GSEAlens focuses on **interactive interpretation** of the resulting
pathway lists.

## Installation

GSEAlens is available on Bioconductor. Install the stable release with:

```{r install-package, eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("GSEAlens")
```

The development version can be installed from GitHub:

```{r install-github, eval=FALSE}
if (!requireNamespace("pak", quietly = TRUE))
    install.packages("pak")
pak::pkg_install("DDL095/GSEAlens")
```

# Quick Start

This section demonstrates a complete GSEAlens workflow using the `r Biocpkg("airway")`
dataset. Because GSEAlens does **not** perform DEG analysis itself, we assume the
input objects (`fit`, `dds_se`, `dds`) have been prepared according to standard
`r Biocpkg("limma")` / `r Biocpkg("DESeq2")` workflows.

For detailed input preparation steps (limma-voom fitting, DESeq2 object construction,
gene filtering), please see the supplementary vignette `vignette("GSEAlens-preprocessing")`.

```{r setup-environment, results='hide'}
library(GSEAlens)
library(airway)
```

For reproducibility in this main vignette, the three input objects (`fit`,
`dds_se`, `dds`) are prepared following the preprocessing vignette. Because
re-running `DESeq2::DESeq()` and the limma-voom pipeline on every build would
make the vignette slow, we ship **pre-computed** versions of those objects in
`inst/extdata/`; the script that regenerates them lives in
`inst/scripts/make_preprocessed_inputs.R` and follows the preprocessing
vignette exactly.

**Note on the shipped `dds_se`**: to stay below the Bioconductor 5 MB `extdata`
limit, the `preprocessed_dds_se.rds` shipped here is a *slimmed* `DESeqDataSet`
produced by `slim_dds_se()` inside `make_preprocessed_inputs.R`. The slimmed
copy drops the `mu` / `H` / `cooks` assay layers (DESeq() fitting
intermediates) and flattens `rowRanges` from `GRangesList` to `GRanges` (one
representative range per gene). `DESeq2::results()` and every GSEAlens entry
point return bit-identical output on the slimmed vs. the full object. To
rebuild the full, untrimmed `DESeqDataSet` for teaching DESeq2 itself, see the
preprocessing vignette.

```{r load-prepared-objects}
data(preprocessed_limma, package = "GSEAlens")
preproc_limma         <- preprocessed_limma
fit                   <- preproc_limma$fit
gsea_limma_voom_data  <- preproc_limma$gsea_limma_voom_data
data(preprocessed_dds_se, package = "GSEAlens")
dds_se <- preprocessed_dds_se
data(preprocessed_dds, package = "GSEAlens")
dds <- preprocessed_dds
```

To prepare these objects from your own data, follow the supplementary
preprocessing vignette:

```r

vignette("GSEAlens-preprocessing")

```

## GSEAlens Processing

### Creating GSEA Pathway Object

Use the `build_gsea_pathways` function to construct a pathway object for GSEA enrichment analysis. The real call downloads multiple MSigDB collections and is slow on the Bioconductor build machine, so it is shown commented out; instead we load a pre-computed lightweight pathway object (Hallmark + KEGG_LEGACY, 236 pathways, see `inst/scripts/make_gsea_pathwaysets_toy.R` for regeneration instructions).

```{r build-gsea-pathways}
# Real call (slow on the Bioconductor build machine):
# gsea_pathwaysets <- build_gsea_pathways(
#   species = "HS", auto_select = c("H", "C2:CP:REACTOME", "C5:GO:BP")
# )
# For the vignette we load a pre-computed lightweight pathway object instead:
data(gsea_pathwaysets_toy, package = "GSEAlens")
gsea_pathwaysets <- gsea_pathwaysets_toy
```

### Assembling Computation Object

Through the `setup_gsea_env` function, assemble a `GSEAEnv` object for computational analysis. Different workflows use the same function with different data inputs. For the limma-voom workflow, since the `fit` object does not contain the original gene read counts, the filtered `DGEList` used to generate the `fit` object must be additionally provided (here `gsea_limma_voom_data`). The three supported backends are assembled below.

```{r setup-gsea-env}
# limma-voom workflow (needs the DGEList because fit alone lacks raw counts)
gseadata_limmavoom <- setup_gsea_env(fit = fit, pathway_obj = gsea_pathwaysets, expr_data = gsea_limma_voom_data)
# DESeq2 SummarizedExperiment workflow
gseadata_se <- setup_gsea_env(fit = dds_se, pathway_obj = gsea_pathwaysets)
# DESeq2 Count matrix workflow
gseadata_dds <- setup_gsea_env(fit = dds, pathway_obj = gsea_pathwaysets)
```

### Running Processing

All objects are processed using the `batch_calc_gsea` function with no differences.

Parallel computing note: Adjust the `workers` option based on your computer's performance to set the number of cores for computation. More contrasts recommend higher core settings for better computational efficiency.

```{r batch-calc-limma, eval=FALSE}
# Write vignette outputs to a temporary directory to avoid polluting the
# Bioconductor build machine's working directory.
out_dir <- tempdir()
# limma-voom workflow
gsea_res_limmavoom <- batch_calc_gsea(gseadata_limmavoom,
                                                 custom_series_name = "limmavoom_data",
                                                 output_dir = out_dir,
                                                 workers = 2,
                                                 force = TRUE)
# DESeq2 SummarizedExperiment workflow
gsea_res_se <- batch_calc_gsea(gseadata_se,
                                          custom_series_name = "dds_se_data",
                                          output_dir = out_dir,
                                          workers = 2,
                                          force = TRUE)
# DESeq2 Count matrix workflow
gsea_res_dds <- batch_calc_gsea(gseadata_dds,
                                           custom_series_name = "dds_data",
                                           output_dir = out_dir,
                                           workers = 2,
                                           force = TRUE)
```

### Interactive Analysis and Viewing

After running `batch_calc_gsea`, an RDS file (the "GSEA Capsule") is generated in
the output directory. You can either read it directly with `readRDS` or use
`import_gsea_capsule`, which automatically organizes related files into the
working directory of your `.Rmd` / `.R` script and performs data inspection.

```{r import-capsule, eval=FALSE}
gsea_res <- import_gsea_capsule("/path/to/your/files/")
# Or read the RDS directly:
# gsea_res <- readRDS("/path/of/your/file/")
```

## Interactive Exploration with the Shiny Application

GSEAlens provides an interactive Shiny application for visual exploration of GSEA
results. The app is launched by passing a `GseaRes` object (returned by
`batch_calc_gsea` or loaded via `import_gsea_capsule`) to `launch_gsea_app`.

### Launching the App

Launch the Shiny app by passing a `GseaRes` object (returned by `batch_calc_gsea` or loaded via `import_gsea_capsule`) to `launch_gsea_app`. Optionally pass an `addition_data` data frame (or path to `.csv` / `.rds` file) to merge pathway annotations into the main table; if `NULL`, the app auto-detects `addition_data_gsealens.rds` or `addition_data_gsealens.csv` in the working directory.

```{r launch-app, eval=FALSE}
# Basic launch
launch_gsea_app(gsea_res)
# With explicit pathway annotations:
# launch_gsea_app(gsea_res, addition_data = "pathway_annotations.csv")
```

### Application Layout

The app uses a **sidebar + main panel** layout with **6 tabs**. The sidebar
(`Data Preprocessing` module) provides global controls; the main panel hosts the
six feature tabs.

#### Sidebar -- Data Preprocessing

Provides global controls visible across all tabs:

- **Contrast selection**: choose which pairwise comparison to explore

- **Gene set subgroup filtering**: subset pathways by collection (e.g. H, C2, C5)

- **DEG marker selection**: highlight genes of interest

- **Group display order**: customize factor ordering for plots

- **Expression metrics**: switch between CPM / logCPM / VST / FPKM

Source: `R/08_shiny_mod_data_prep.R`.

#### Tab 1 -- Main Workspace

The default landing tab, combining two sub-modules:

- **Master Table** (`DT::datatable`): interactive table of all enriched pathways
  with sortable columns (NES, pvalue, p.adjust, setSize) and checkbox selection.
  Selected rows are pushed to other tabs.

- **Combined Pathway Plotting**: aggregates multiple selected pathways into a
  single composite figure (uses `patchwork` under the hood). The export modal
  includes a WYSIWYG Live Preview, PDF/PNG/SVG/TIFF output, and a "Copy R Code"
  button via `generate_combined_plot_code()`.

Click any pathway row to open the **Pathway Detail Modal**, which shows the
full description, leading-edge genes, and an option to add the pathway to the
plot queue.

Source: `R/09_shiny_mod_table.R`, `R/11_shiny_mod_modal.R`, `R/12_shiny_mod_multi_plot.R`.

#### Tab 2 -- Holographic Quadruple Linkage

Four synchronized panels:

1. Top-left: pathway selector (linked to Main Workspace selection)

2. Top-right: gene ranking table (ranked by `|stat|`)

3. Bottom-left: volcano plot for the selected contrast

4. Bottom-right: expression boxplot for the selected gene

Selections are **bidirectionally synchronized** -- clicking a gene in the table
highlights it in the volcano; clicking a point in the volcano scrolls the table.

Source: `R/10_shiny_mod_quadrant.R`.

#### Tab 3 -- Pathway Relationship Exploration

Network visualization of pathway-to-pathway relationships, where nodes are
pathways and edges represent shared genes (Jaccard similarity). Two selection
modes:

- **Single mode**: explore one pathway and its neighbors

- **Batch mode**: select multiple pathways from Main Workspace and visualize
  their interconnections
Two sub-panels are provided under this tab:

- **DotPlot panel**: horizontal dot plot where the X axis is NES, dot color
  encodes significance (`-log10(FDR)` / `-log10(P-value)` / `|NES|`,
  dot size encodes gene-set magnitude. A data-driven size scale
  (no fixed limits, no transform) is used so that dot sizes faithfully
  reflect the underlying gene-set magnitude range. This mirrors the
  `ggplot2::scale_size_continuous(range=c(3,8))` convention used by
  `enrichplot::dotplot`, where size limits are derived from the data
  rather than imposed as a fixed domain.

- **Network panel**: graph layout (Fruchterman-Reingold / Kamada-Kawai /
  Circle) with **two user-selectable edge-width encodings**:

  - *Weight-based* (default, `emapplot` convention): edge width is linearly
    proportional to the Jaccard value, faithfully reflecting the underlying
    similarity magnitude. Recommended for publication.

  - *Rank-based*: edge width is assigned by Jaccard rank, guaranteeing uniform
    visual spacing between edges regardless of absolute weight. Useful for
    dense networks with low weight variance.
  Node size reflects `|NES|`; node color reflects enrichment direction
  (red = up in left group, blue = up in right group).

**Export Center** (both panels): clicking "Export Publication Plot" opens a

modal with width / height / DPI / format (PDF, PNG, SVG, TIFF) controls and
two actions: download a static `ggplot2`-rendered image via `ggsave` (no
external dependencies such as kaleido/orca), or copy a fully reproducible
R script (`generate_dotplot_code()` / `generate_network_code()`) to the
clipboard. The static figures are byte-for-byte identical to what the copied
code would produce.

Source: `R/13_shiny_mod_pathway_relation.R`, helper `R/utils_hubgene.R`,
code generators in `R/15_code_generator.R`.

#### Tab 4 -- HubGene Network

Identifies and visualizes hub genes (highly connected genes across multiple
enriched pathways) using a `visNetwork` interactive plot. Adjustable parameters:

- **Physics simulation**: toggle on/off, adjust force-directed parameters

- **Pathway node size encoding**: three user-selectable modes

  - *By gene-set size* (`setSize`, default): matches the `enrichplot::cnetplot`
    convention; pathway node size is proportional to the number of genes in
    the set (sqrt-scaled). Recommended for biological interpretation.

  - *By significance* (`-log10(FDR)`): emphasizes the most statistically
    trustworthy pathways.

  - *Fixed size*: constant node size controlled by the slider (legacy behavior).
  The slider value always acts as the **base** size; the chosen encoding
  scales around it within `[0.6x, 1.4x]` to keep visNetwork's force-directed
  layout stable (size variance beyond ~2.3x causes visible layout jitter).
  Gene-node size is unaffected (always `base + degree * 3`).

- **Network statistics**: summary panel showing node count, edge count, density

**Export Center**: same modal pattern as Tab 3. The static reproduction uses

`generate_hubgene_code()` and renders pathway nodes as diamonds and gene nodes
as circles in a bipartite layout via `igraph` + `ggplot2` (no `ggraph`
dependency). The current size-encoding mode is preserved in the generated
script.

Source: `R/16_shiny_mod_hubgene_vis.R`, code generator in
`R/15_code_generator.R`.

#### Tab 5 -- AI Interpretation

Generates a structured prompt for an external LLM (e.g. GPT-4, Claude) to
interpret the selected pathways. Supports custom templates so users can enforce
a particular output format (e.g. "produce a 3-paragraph biological interpretation
citing leading-edge genes"). The generated prompt can be copied to clipboard.

Source: `R/17_shiny_mod_AI.R`.

**Note**: This tab only generates prompts; it does not call external APIs
directly. The author explicitly designed this to keep API keys and network
calls under user control.

#### Tab 6 -- Joint GSEA Canvas

Aggregates enrichment running-score curves for multiple selected pathways into
a single composite canvas. The image export modal includes a WYSIWYG Live
Preview, PDF/PNG/SVG/TIFF output, adjustable canvas margins, and a "Copy R
Code" button that generates a self-contained R script via
`generate_joint_canvas_code()` for reproduction outside the Shiny environment.

Source: `R/14_shiny_mod_joint_canvas.R`, code generator `R/15_code_generator.R`.

### Interpreting Outputs

A practical interpretation guide:

| Visualization | What to look for | Biological meaning |
|---|---|---|
| NES (Normalized Enrichment Score) | Sign and magnitude | Positive NES -> pathway up-regulated in the right-hand group of the contrast |
| p.adjust | < 0.05 threshold | Statistical significance after BH correction |
| Volcano (Tab 2) | Symmetry / asymmetry | Balanced volcano suggests global shift; skewed suggests targeted regulation |
| Pathway network (Tab 3) | Cluster structure | Tightly connected clusters indicate co-regulated biological modules |
| HubGene (Tab 4) | High-degree genes | Hub genes are candidate biomarkers or regulatory nodes |
| Joint Canvas (Tab 6) | Curve overlap | Overlapping running-score curves suggest coordinated regulation |

### Reproducible Code Export

Tabs 1 (Combined Pathway Plotting), 2, 3, 4, and 6 include a **"Copy R Code"**
button integrated directly into each module's image export modal, producing a
self-contained R script reproducing the current visualization. This is the
recommended way to generate publication-quality figures: iteratively refine
the plot in the Shiny app, then export the code for final customization.

# Package Intermediate Object Descriptions

## GseaEnv Object

The `GseaEnv` object returned by the `setup_gsea_env` function contains the following components:

| Component | Description |
|-----------------|-------------------------------------------------------|
| `backend_info` | Backend type information (limma-voom or DESeq2) |
| `contrast_registry` | Contrast registry containing all pairwise comparison information |
| `de_store` | Differential expression analysis results storage |
| `expr_bundle` | Expression data bundle (raw counts, normalized matrix, sample metadata) |
| `geneset` | Geneset information (TERM2GENE, metadata dictionary, species) |

## GseaRes Object

The `GseaRes` object returned by the `batch_calc_gsea` function contains the following components:

| Component | Description |
|-----------------|-------------------------------------------------------|
| `metadata` | Computation metadata (runtime, cores used, parameter settings) |
| `backend_info` | Backend type information |
| `contrast_registry` | Contrast registry |
| `de_store` | Differential expression analysis results storage |
| `expr_bundle` | Expression data bundle |
| `geneset_info` | Geneset information |
| `results` | GSEA results list, one entry per contrast |

## GseaTask Object

The `GseaTask` object returned by the `extract_gsea_task` function is used for single-contrast analysis:

| Component  | Description                                                    |
|----------------|-------------------------------------------------------|
| `gsea_res` | GSEA result object                                             |
| `meta`     | Metadata (contrast information, geneset name, expression data) |

# Session Info

```{r sessionInfo}
sessionInfo()
```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_class_validations.R
\name{create_gsea_res}
\alias{create_gsea_res}
\title{Define GseaRes Class}
\usage{
create_gsea_res(
  metadata,
  backend_info,
  contrast_registry,
  de_store,
  expr_bundle,
  geneset_info,
  results
)
}
\arguments{
\item{metadata}{Metadata information}

\item{backend_info}{Backend information object}

\item{contrast_registry}{Contrast registry}

\item{de_store}{Differential expression store}

\item{expr_bundle}{Expression data bundle}

\item{geneset_info}{Gene set information}

\item{results}{GSEA results}
}
\value{
A GseaRes object
}
\description{
GSEA computation result capsule.
}
\examples{
gsea_res <- create_gsea_res(
  metadata = list(),
  backend_info = list(),
  contrast_registry = data.frame(),
  de_store = data.frame(),
  expr_bundle = list(),
  geneset_info = list(),
  results = list()
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/09_shiny_mod_table.R
\name{mod_master_table_server}
\alias{mod_master_table_server}
\title{Master Workspace Table Server}
\usage{
mod_master_table_server(id, data_prep, addition_data = NULL)
}
\arguments{
\item{id}{Module ID}

\item{data_prep}{Reactive data from the data preprocessing module}

\item{addition_data}{Optional. A data frame with pathway annotations to merge.}
}
\value{
List containing selected_pathways, show_modal, and selection control functions
}
\description{
Provides data display, column display control with decoupled checkbox state.
  Supports merging addition_data columns into the main table.
  Includes external selection update capabilities for bidirectional sync.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{preprocessed_dds}
\alias{preprocessed_dds}
\title{Pre-computed DESeq2 DESeqDataSet from the airway dataset}
\format{
A `DESeqDataSet` object.
}
\source{
Built from the `airway` Bioconductor dataset via DESeq2.
}
\description{
A slimmed `DESeqDataSet` object fitted with `DESeq()`,
  suitable as input to [GSEAlens::setup_gsea_env()].
}
\examples{
data(preprocessed_dds, package = "GSEAlens")
class(preprocessed_dds)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/15_code_generator.R
\name{.serialize_df_tsv}
\alias{.serialize_df_tsv}
\title{Serialize a Data Frame to TSV}
\usage{
.serialize_df_tsv(df)
}
\arguments{
\item{df}{A \code{data.frame}.}
}
\value{
A character scalar containing the TSV (no trailing newline).
}
\description{
Internal helper used by the Quadrant module's "Export Boxplot
  Data" modal. Writes a \code{data.frame} as tab-separated values with a
  header row and \code{NA} rendered as empty strings. Values are quoted
  only when they contain tab / newline / double-quote characters, so
  clean sample IDs stay unquoted.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-accessors.R
\name{create_addition_template}
\alias{create_addition_template}
\title{Create Additional Data Template}
\usage{
create_addition_template(
  gsea_res,
  output_path = "addition_template_gsealens.csv",
  include_cols = c("Brief_Description", "Custom_Category", "Custom_Tag", "User_Notes")
)
}
\arguments{
\item{gsea_res}{A GseaRes object (generated by \code{batch_calc_gsea}).}

\item{output_path}{Character string specifying the output template file path.
Default: \code{"addition_template_gsealens.csv"}.}

\item{include_cols}{Character vector specifying additional columns to include
besides ID. Default includes standard annotation columns.}
}
\value{
Invisible \code{TRUE} on success.
}
\description{
Generate a template CSV file containing all pathway IDs from
  the GseaRes object. Users can edit this template to add custom annotations
  such as Brief_Description, Custom_Category, Custom_Tag, and User_Notes.
}
\examples{
# Load pre-computed GseaRes shipped with the package and write a template
# CSV to a temporary file (no side effects on the user's filesystem).
data(precomputed_gseares, package = "GSEAlens")

gsea_res <- precomputed_gseares
out_csv <- file.path(tempdir(), "addition_template_demo.csv")
create_addition_template(gsea_res, output_path = out_csv)

# The template contains one row per pathway ID, ready to edit.
template <- utils::read.csv(out_csv)
utils::head(template)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_hubgene.R
\name{generate_hubgene_hover_text}
\alias{generate_hubgene_hover_text}
\title{Generate HubGene Hover Text}
\usage{
generate_hubgene_hover_text(
  node,
  node_type,
  left_group = "A",
  right_group = "B"
)
}
\arguments{
\item{node}{Node data row (as list with named elements)}

\item{node_type}{Node type: "pathway" or "gene"}

\item{left_group}{Left group name}

\item{right_group}{Right group name}
}
\value{
Character string with HTML content for hover tooltip
}
\description{
Generate HTML text for plotly hover tooltips.
}
\examples{
# Example with a pathway node
pathway_node <- list(
  id = "HALLMARK_INFLAMMATORY_RESPONSE",
  NES = 2.3, FDR = 0.001, pvalue = 0.0005,
  n_core = 15, n_total = 50
)
hover_pathway <- generate_hubgene_hover_text(pathway_node, "pathway")
cat(hover_pathway)

# Example with a gene node
gene_node <- list(id = "GENE_A", log2FC = 1.8)
hover_gene <- generate_hubgene_hover_text(gene_node, "gene")
cat(hover_gene)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/17_shiny_mod_AI.R
\name{mod_ai_abs_page_ui}
\alias{mod_ai_abs_page_ui}
\title{AI Interpretation Page UI}
\usage{
mod_ai_abs_page_ui(id)
}
\arguments{
\item{id}{Module ID}
}
\description{
AI Interpretation Page UI
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04_calculation.R
\name{.prepare_rank_vector_fast}
\alias{.prepare_rank_vector_fast}
\title{Fast Rank Vector Preparation}
\usage{
.prepare_rank_vector_fast(de_table, flip = FALSE)
}
\arguments{
\item{de_table}{DE result table}

\item{flip}{Whether to flip sign}
}
\value{
Named numeric vector
}
\description{
Fast Rank Vector Preparation
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{gsea_pathwaysets_toy_hallmark}
\alias{gsea_pathwaysets_toy_hallmark}
\title{Toy MSigDB pathway collection (Hallmark only)}
\format{
A list with elements `TERM2GENE`, `meta_dict`, `SuperTag`,
  `collections_used`, `species`.
}
\source{
Assembled from msigdbr (Hallmark collection).
}
\description{
A minimal Hallmark-only pathway object for fast examples
  and tests. See `inst/scripts/make_gsea_pathwaysets_toy.R` for
  regeneration.
}
\examples{
data(gsea_pathwaysets_toy_hallmark, package = "GSEAlens")
names(gsea_pathwaysets_toy_hallmark)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/15_code_generator.R
\name{generate_combined_plot_code}
\alias{generate_combined_plot_code}
\title{Generate Combined Pathway Plotting Code}
\usage{
generate_combined_plot_code(
  gsea_res_var = "gsea_res",
  contrast_id,
  target_pathways,
  subPlot = 3,
  curve_colors = NULL,
  left_group = "Left",
  right_group = "Right"
)
}
\arguments{
\item{gsea_res_var}{Character. Name of the GseaRes variable the generated
code should reference (default \code{"gsea_res"}). The user replaces the
placeholder assignment with their own object.}

\item{contrast_id}{Character. Contrast ID passed to
\code{extract_gsea_task()}.}

\item{target_pathways}{Character vector. Pathway IDs to plot.}

\item{subPlot}{Integer (1-3). \code{plot_directional_gsea} subPlot argument
(1 = running score, 2 = rank position, 3 = both). Default 3.}

\item{curve_colors}{Character vector of color hex codes for pathway curves.
If NULL, falls back to a sensible default palette.}

\item{left_group}{Character. Left group label for axis annotation.}

\item{right_group}{Character. Right group label.}
}
\value{
Character scalar of executable R code.
}
\description{
Return an executable R script reproducing the Combined Pathway
  Plotting module (mod_multi_plot / module 12) as a static figure. Mirrors
  the interactive \code{plot_directional_gsea()} call the app performs when
  the user selects pathways in the main table's "Combined Display" column.
}
\examples{
code <- generate_combined_plot_code(
  contrast_id    = "untrt_vs_trt",
  target_pathways = c("HALLMARK_TNFA_SIGNALING_VIA_NFKB")
)
cat(code)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/15_code_generator.R
\name{generate_boxplot_data_code}
\alias{generate_boxplot_data_code}
\title{Generate Boxplot Expression-Data Extraction Code}
\usage{
generate_boxplot_data_code(
  GSEAlens_res,
  contrast_id,
  gene_symbol,
  expr_type = "logcpm",
  custom_order = NULL
)
}
\arguments{
\item{GSEAlens_res}{A \code{GseaRes} object.}

\item{contrast_id}{Character. Contrast ID (e.g. "Treat_vs_Control").}

\item{gene_symbol}{Character. Gene symbol as shown in the boxplot title.}

\item{expr_type}{Character. Expression matrix type passed to
\code{get_expr_matrix()} (e.g. "logcpm", "vst", "count"). Default "logcpm".}

\item{custom_order}{Character or NULL. Custom group ordering string
(e.g. "GroupA -> GroupB") or "default" / NULL to use natural order.
When non-default, the generated code will re-order the \code{Group}
factor accordingly.}
}
\value{
A character string of executable R code.
}
\description{
Produce a self-contained, reproducible R script that extracts
  the per-sample expression values (Sample / Group / Expression) of a
  single gene from a \code{GseaRes} object. The returned data frame
  matches exactly what is shown in the Quadrant module's expression
  boxplot (panel 4), so the user can paste it into other software
  (e.g. GraphPad Prism, Excel, R) for downstream plotting.
}
\examples{
\donttest{
data(precomputed_gseares, package = "GSEAlens")
code <- generate_boxplot_data_code(
  GSEAlens_res = precomputed_gseares,
  contrast_id  = "untrt_vs_trt",
  gene_symbol  = "GAPDH",
  expr_type    = "logcpm"
)
cat(code)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/06_visualization.R
\name{plot_directional_gsea}
\alias{plot_directional_gsea}
\title{Plot Directional GSEA}
\usage{
plot_directional_gsea(
  directional_gsea_obj,
  target_pathways,
  main_title = NULL,
  subPlot = 3,
  curveCol = NULL,
  add_pval = FALSE,
  show_contrast_in_axis = FALSE,
  ...
)
}
\arguments{
\item{directional_gsea_obj}{A wrapped GseaTask object (returned by extract_gsea_task)}

\item{target_pathways}{Vector of pathway IDs to plot (supports single or multiple)}

\item{main_title}{Main title name}

\item{subPlot}{Controls the number of subplots generated (1: enrichment plot only; 2: enrichment + heatmap track; 3: full rank track)}

\item{curveCol}{Custom curve color vector}

\item{add_pval}{Whether to add statistical annotations on the plot. Default FALSE.}

\item{show_contrast_in_axis}{Logical. Whether to display contrast group information on the x-axis of the Rank plot.
Default FALSE; recommended to set TRUE only in combined canvas mode.}

\item{...}{Additional parameters passed to internal engine}
}
\value{
ggplot2 composite object
}
\description{
Professional-grade GSEA plotting engine with seamless support for single or multiple pathway visualization.
  Automatically generates elegant legends and native red-blue gene distribution track.
}
\examples{
# Load pre-computed GseaRes and pick a task + an enriched pathway ID
data(precomputed_gseares, package = "GSEAlens")

gsea_res <- precomputed_gseares
task <- extract_gsea_task(gsea_res, contrast_id = "untrt_vs_trt")
res_df <- as.data.frame(task$gsea_res)
top_pathway <- res_df$ID[1]

# Render the directional GSEA plot (enrichment + rank + heatmap tracks)
p <- plot_directional_gsea(task, target_pathways = top_pathway)
class(p)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/15_code_generator.R
\name{extract_boxplot_data}
\alias{extract_boxplot_data}
\title{Extract Boxplot Data (Long and Wide)}
\usage{
extract_boxplot_data(
  GSEAlens_res,
  contrast_id,
  gene_symbol,
  expr_type = "logcpm"
)
}
\arguments{
\item{GSEAlens_res}{A \code{GseaRes} object.}

\item{contrast_id}{Character. Contrast ID (e.g. \code{"Treat_vs_Control"}).}

\item{gene_symbol}{Character. Gene symbol as shown in the boxplot title.}

\item{expr_type}{Character. Expression matrix type passed to
\code{get_expr_matrix()}. Default \code{"logcpm"}.}
}
\value{
A named list with elements \code{long}, \code{wide}, \code{gene},
  \code{contrast_id}, \code{expr_type}, \code{left_group},
  \code{right_group}, \code{sample_order}.
}
\description{
Internal helper used by the Quadrant module's "Export Boxplot
  Data" button. Computes the exact per-sample expression data frame that
  the Shiny boxplot (panel 4) is built on, and returns it in two formats:
\describe{
  \item{long}{Tidy data frame with columns \code{Sample}, \code{Group},
    \code{Expression}. One row per sample. Suitable for GraphPad Prism
    ("Grouped" data table) and Excel.}
  \item{wide}{A one-row data frame. The first column holds the gene
    symbol (column name \code{Gene}); the remaining columns hold the
    expression value of that gene in each sample (column names are the
    sample IDs, in the expression matrix's original column order).
    Suitable for direct copy-paste into Prism ("Column" data table) or
    any spreadsheet.}
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis-gsea-core.R
\name{.gs_info}
\alias{.gs_info}
\title{Internal GSEA Data Extractor}
\usage{
.gs_info(object, geneSetID)
}
\arguments{
\item{object}{A gseaResult object.}

\item{geneSetID}{Character, gene set ID.}
}
\value{
data.frame with runningScore, position, geneList, etc.
}
\description{
Extract running score data from gseaResult object.
  Adapted from GseaVis::gsInfo, using enrichit::gseaScores internally.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_hubgene.R
\name{color_by_direction}
\alias{color_by_direction}
\title{Color Nodes by Direction}
\usage{
color_by_direction(
  node_data,
  color_mode = "logFC",
  left_group = "A",
  right_group = "B"
)
}
\arguments{
\item{node_data}{Node data frame with type and direction columns}

\item{color_mode}{Color mode: "logFC", "pathway", or "uniform"}

\item{left_group}{Left group name for legend label}

\item{right_group}{Right group name for legend label}
}
\value{
Node data with added color and color_label columns
}
\description{
Assign colors based on NES (pathways) or log2FC (genes).
}
\examples{
# Build a minimal node data frame (pathway + gene nodes)
nodes <- data.frame(
  name   = c("P_POS", "P_NEG", "G_UP", "G_DOWN"),
  type   = c("pathway", "pathway", "gene", "gene"),
  NES    = c( 2.5, -2.0, NA, NA),
  log2FC = c( NA,  NA, 1.5, -1.2),
  stringsAsFactors = FALSE
)
colored <- color_by_direction(nodes, color_mode = "logFC")
head(colored$color)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_class_validations.R
\name{create_gsea_env}
\alias{create_gsea_env}
\title{Define GseaEnv Class}
\usage{
create_gsea_env(
  backend_info,
  contrast_registry,
  de_store,
  expr_bundle,
  geneset,
  raw_obj
)
}
\arguments{
\item{backend_info}{Backend information object}

\item{contrast_registry}{Contrast registry}

\item{de_store}{Differential expression store}

\item{expr_bundle}{Expression data bundle}

\item{geneset}{Gene set collection}

\item{raw_obj}{Raw data object}
}
\value{
A GseaEnv object
}
\description{
Standardized GSEA input environment object.
}
\examples{
gsea_env <- create_gsea_env(
  backend_info = list(),
  contrast_registry = data.frame(),
  de_store = data.frame(),
  expr_bundle = list(),
  geneset = character(),
  raw_obj = NULL
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/06_visualization.R
\name{visualization}
\alias{visualization}
\title{GSEA Visualization and Report Generation}
\value{
NULL (this is a documentation landing page; individual functions
  return their own objects, documented separately).
}
\description{
Provides static plotting functions and interactive HTML report generation
capabilities. This is a documentation landing page for the visualization
family of functions in this file.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_class_validations.R
\name{create_gsea_task}
\alias{create_gsea_task}
\title{Define GseaTask Class}
\usage{
create_gsea_task(gsea_res, meta)
}
\arguments{
\item{gsea_res}{GseaRes object}

\item{meta}{Metadata for the task}
}
\value{
A GseaTask object
}
\description{
Single contrast extraction result object.
}
\examples{
gsea_task <- create_gsea_task(
  gsea_res = NULL,
  meta = data.frame()
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/12_shiny_mod_multi_plot.R
\name{mod_multi_plot_server}
\alias{mod_multi_plot_server}
\title{Combined Pathway Plotting Server}
\usage{
mod_multi_plot_server(id, data_prep, table_controller, gsea_res = NULL)
}
\description{
Combined Pathway Plotting Server
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{precomputed_gseares}
\alias{precomputed_gseares}
\title{Pre-computed GSEA results from the airway dataset}
\format{
A `GseaRes` object (list-like).
}
\source{
Built from the `airway` Bioconductor dataset via DESeq2 +
  clusterProfiler::GSEA.
}
\description{
A pre-computed `GseaRes` object built from the airway dataset
  (DESeq2 + clusterProfiler::GSEA). Used in examples and vignettes so that
  users can try visualization and code-generation functions without having
  to re-run the full GSEA pipeline.
}
\examples{
data(precomputed_gseares, package = "GSEAlens")
class(precomputed_gseares)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04_calculation.R
\name{batch_calc_gsea}
\alias{batch_calc_gsea}
\title{Batch Parallel GSEA Calculation}
\usage{
batch_calc_gsea(
  gsea_env,
  custom_series_name = "Auto_Analysis",
  output_dir = tempdir(),
  workers = 2,
  bidirectional = TRUE,
  minGSSize = 10,
  maxGSSize = 500,
  pvalueCutoff = 1,
  seed = 123,
  force = FALSE,
  use_progress = FALSE,
  chunk_size = NULL
)
}
\arguments{
\item{gsea_env}{GseaEnv object}

\item{custom_series_name}{String. Analysis series name}

\item{output_dir}{String. Output directory for results, default "./GSEA_Output"}

\item{workers}{Number of parallel cores. Default 4}

\item{bidirectional}{Logical. Whether to automatically generate reverse contrasts, default TRUE}

\item{minGSSize}{Minimum gene set size, default 10}

\item{maxGSSize}{Maximum gene set size, default 500}

\item{pvalueCutoff}{P-value threshold, default 1}

\item{seed}{Integer or NULL. Base random seed for per-contrast GSEA
permutation, default 123. Each contrast derives its own deterministic
seed from the base seed and the contrast id, so results are reproducible
across runs and independent of `workers`/`chunk_size`. On
clusterProfiler <= 4.20.x (where the `seed` argument of `GSEA()` is
silently dropped) reproducibility is guaranteed by seeding the worker R
RNG via `withr::local_seed()`; on clusterProfiler >= 4.21.x the derived
seed is also forwarded to `clusterProfiler::GSEA(seed = ...)`. Set to
NULL to restore native (non-reproducible) engine sampling.}

\item{force}{Logical. Whether to force recalculation, default FALSE}

\item{use_progress}{Logical. Whether to show progress bar, default TRUE}

\item{chunk_size}{Integer. Number of tasks per worker per chunk, default NULL (auto)}
}
\value{
GseaRes object
}
\description{
Consumes a standard GseaEnv object and performs efficient parallel GSEA calculation.
}
\examples{
# Build a GseaEnv from pre-computed inputs shipped with the package
data(preprocessed_limma, package = "GSEAlens")

precomp <- preprocessed_limma
data(gsea_pathwaysets_toy, package = "GSEAlens")

pathways <- gsea_pathwaysets_toy
gsea_env <- setup_gsea_env(
  fit = precomp$fit,
  pathway_obj = pathways,
  expr_data = precomp$gsea_limma_voom_data
)
# Run batch GSEA (writes to a temporary directory; ~20s on the toy set)
gsea_res <- batch_calc_gsea(gsea_env, workers = 2, output_dir = tempdir())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/17_shiny_mod_AI.R
\name{mod_ai_abs_page_server}
\alias{mod_ai_abs_page_server}
\title{AI Interpretation Page Server}
\usage{
mod_ai_abs_page_server(id, gsea_res, data_prep_list, table_controller)
}
\arguments{
\item{id}{Module ID}

\item{gsea_res}{GseaRes object}

\item{data_prep_list}{Data preprocessing list}

\item{table_controller}{Table controller}
}
\value{
No return value; called for side effects within a Shiny module.
Extract Leading Edge genes and format as comma-separated
Determine confidence level
}
\description{
AI Interpretation Page Server. Called for side effects within a Shiny module.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/16_shiny_mod_hubgene_vis.R
\name{mod_hubgene_vis_server}
\alias{mod_hubgene_vis_server}
\title{HubGene Network (visNetwork) Module Server}
\usage{
mod_hubgene_vis_server(id, data_prep_list, table_controller, gsea_res)
}
\arguments{
\item{id}{Character string used to namespace the module.}

\item{data_prep_list}{A reactive expression returning a list containing
GSEA results, data frame, contrast groups, and metadata.}

\item{table_controller}{A list containing reactive expressions for
table interactions, including selected pathways.}

\item{gsea_res}{A GSEA result object used for symbol mapping.}
}
\value{
A list containing reactive expressions:
  \item{final_pathways}{Reactive character vector of selected pathway IDs}
}
\description{
Shiny module server for HubGene network visualization using visNetwork.
Handles pathway selection, network building, rendering with physics
simulation, and interactive node/edge display.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/15_code_generator.R
\name{generate_pathway_plot_code}
\alias{generate_pathway_plot_code}
\title{Generate Pathway Plotting Code}
\usage{
generate_pathway_plot_code(
  GSEAlens_res,
  contrast_id,
  target_pathways,
  user_genes = character(0),
  pathway_genes = character(0),
  expr_type = "logcpm"
)
}
\arguments{
\item{GSEAlens_res}{GseaRes object}

\item{contrast_id}{Character. Contrast ID}

\item{target_pathways}{Character vector. Pathway IDs}

\item{user_genes}{Character vector. Genes of interest}

\item{pathway_genes}{Character vector. Pathway genes (for coloring)}

\item{expr_type}{Character. Expression type}
}
\value{
A character string containing executable R code
}
\description{
Generate Pathway Plotting Code
}
\examples{
# Generate plotting code for the top enriched pathway in a pre-computed task
data(precomputed_gseares, package = "GSEAlens")

gsea_res <- precomputed_gseares
task <- extract_gsea_task(gsea_res, contrast_id = "untrt_vs_trt")
res_df <- as.data.frame(task$gsea_res)
top_pathway <- res_df$ID[1]

code <- generate_pathway_plot_code(
  GSEAlens_res   = gsea_res,
  contrast_id    = "untrt_vs_trt",
  target_pathways = top_pathway
)
cat(code)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-core-genes.R
\name{calculate_overlap_ratio}
\alias{calculate_overlap_ratio}
\title{Calculate ORA Ratio (for DotPlot)}
\usage{
calculate_overlap_ratio(
  pathway_genes,
  de_genes,
  ratio_mode = c("ora", "leading")
)
}
\arguments{
\item{pathway_genes}{Gene set of pathway (defined by TERM2GENE)}

\item{de_genes}{Differentially expressed genes (filtered by p-value threshold)}

\item{ratio_mode}{Character, "ora" (intersection/pathway size) or "leading"
(intersection/DE size)}
}
\value{
Numeric, ratio value
}
\description{
Calculate overlap ratio between pathway genes and differentially
  expressed genes.
}
\examples{
ratio <- calculate_overlap_ratio(c("GAPDH", "TP53", "MYC"), c("TP53", "MYC", "EGFR"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/07_shiny_app.R
\name{launch_gsea_app}
\alias{launch_gsea_app}
\title{GSEAlens Interactive Analysis Workstation}
\usage{
launch_gsea_app(gsea_res, addition_data = NULL)
}
\arguments{
\item{gsea_res}{GseaRes object (generated by batch_calc_gsea())}

\item{addition_data}{Optional. A data frame or file path containing
pathway annotations to merge into the main table. Can also be:
\itemize{
  \item A data.frame with 'ID' column as primary key
  \item A character vector: file path to .csv, .csv.gz, .rds, or .rds.gz
  \item NULL: automatically searches for "addition_data_gsealens.rds"
    or "addition_data_gsealens.csv" in the working directory
}
Additional columns will be merged to the main pathway table.}
}
\value{
Shiny application object
}
\description{
Modular Shiny application based on GseaRes object, supporting
  Limma and DESeq2 dual backends. Provides interactive visualization for
  GSEA pathway enrichment analysis including volcano plots, heatmaps,
  expression boxplots, and pathway relationship networks.
}
\examples{
if(interactive()){
# Basic usage
gsea_res <- batch_calc_gsea(gsea_env)
launch_gsea_app(gsea_res)

# With addition data from file path
launch_gsea_app(gsea_res, "pathway_annotations.csv")

# With pre-loaded data frame
add_data <- read.csv("annotations.csv")
launch_gsea_app(gsea_res, add_data)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_backends.R
\name{.extract_deseq2_data}
\alias{.extract_deseq2_data}
\title{Extract DESeq2 Data}
\usage{
.extract_deseq2_data(dds, target_factor = NULL)
}
\arguments{
\item{dds}{DESeqDataSet object (must have been processed with DESeq())}

\item{target_factor}{Character. Target factor. If NULL, automatically inferred as the last term in the design formula.}
}
\value{
A list containing contrast_registry, de_store, and expr_bundle
}
\description{
Extract contrast groups and differential analysis results from DESeqDataSet object.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/08_shiny_mod_data_prep.R
\name{mod_data_prep_server}
\alias{mod_data_prep_server}
\title{Data Preprocessing Module Server}
\usage{
mod_data_prep_server(id, gsea_res)
}
\arguments{
\item{id}{Module namespace ID}

\item{gsea_res}{GSEA results object containing contrast registry and backend info}
}
\value{
A list of reactive functions and values:
\itemize{
  \item \code{data}: Reactive data object with processed GSEA results
  \item \code{highlight_genes}: Reactive character vector of confirmed gene markers
  \item \code{boxplot_order}: Reactive value for boxplot group ordering
  \item \code{joint_contrasts}: Reactive vector of contrasts for joint canvas
  \item \code{joint_ncol}: Reactive number of columns for joint canvas
  \item \code{joint_generate}: Event reactive for canvas regeneration trigger
  \item \code{update_pending_genes}: Function to update pending genes from modal
  \item \code{get_pending_genes}: Function to retrieve current pending genes
}
}
\description{
Server logic for the data preprocessing module. Handles:
\itemize{
  \item Contrast and gene set selection
  \item Data processing and sorting
  \item Gene marker management (add, confirm, clear)
  \item Boxplot ordering configuration
  \item Joint canvas parameter passing
  \item Expression data type selection based on backend
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/13_shiny_mod_pathway_relation.R
\name{mod_pathway_relation_server}
\alias{mod_pathway_relation_server}
\title{Pathway Relationship Exploration Module Server}
\usage{
mod_pathway_relation_server(id, data_prep_list, gsea_res, table_result = NULL)
}
\description{
Dual-mode pathway network visualization with defensive rendering
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_backends.R
\name{.standardize_de_columns}
\alias{.standardize_de_columns}
\title{Standardize Differential Expression Table Column Names}
\usage{
.standardize_de_columns(df, backend)
}
\description{
Unify column names from different backends to: gene_symbol, logFC, stat, pvalue, padj
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-accessors.R
\name{.process_sample_meta}
\alias{.process_sample_meta}
\title{Internal Sample Metadata Processing}
\usage{
.process_sample_meta(sample_meta, expr_bundle)
}
\arguments{
\item{sample_meta}{Raw sample metadata (possibly DFrame).}

\item{expr_bundle}{Expression bundle containing raw_counts, dds_obj, and
other components for row name recovery.}
}
\description{
Automatically infers target_factor from dds_obj@design for
  DESeq2 workflows, handles DFrame conversion, and creates a unified
  'group' column for consistent downstream processing.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/15_code_generator.R
\name{generate_network_code}
\alias{generate_network_code}
\title{Generate Publication Pathway Network Code}
\usage{
generate_network_code(
  gsea_res_var = "gsea_res",
  contrast_id,
  pathways,
  min_shared_genes = 2,
  width_mode = "weight",
  seed = 42L,
  target_collection = "ALL",
  base_size = 12,
  margin_top = 18,
  margin_bottom = 18,
  margin_left = 18,
  margin_right = 18,
  fig_width = 10,
  fig_height = 8,
  fig_dpi = 300
)
}
\arguments{
\item{gsea_res_var}{Character. Name of the GseaRes variable.}

\item{contrast_id}{Character. Contrast ID to extract.}

\item{pathways}{Character vector. Pathway IDs to include in the network.}

\item{min_shared_genes}{Integer. Minimum shared core genes for an edge.}

\item{width_mode}{Character. "rank" or "weight".}

\item{seed}{Integer. Layout seed.}

\item{target_collection}{Character. Gene set collection(s) to extract.
Default \code{"ALL"}.}

\item{base_size}{Numeric.}

\item{margin_top}{Numeric. Top plot margin (mm).}

\item{margin_bottom}{Numeric. Bottom plot margin (mm).}

\item{margin_left}{Numeric. Left plot margin (mm).}

\item{margin_right}{Numeric. Right plot margin (mm).}

\item{fig_width}{Numeric. Figure width in inches for ggsave.}

\item{fig_height}{Numeric. Figure height in inches for ggsave.}

\item{fig_dpi}{Numeric. DPI for PNG raster output (default 300).}
}
\value{
Character scalar of executable R code.
}
\description{
Return an executable R script reproducing the Pathway
  Relationship module's network view as a static ggplot2 figure.
  Assumes a \code{GseaRes} object is already loaded. Uses igraph layout +
  geom_segment edges + geom_point nodes (no ggraph dependency).
}
\examples{
code <- generate_network_code(
  gsea_res_var = "gsea_res", contrast_id = "untrt_vs_trt",
  pathways = c("HALLMARK_TNFA_SIGNALING_VIA_NFKB", "HALLMARK_HYPOXIA"))
cat(code)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-core-genes.R
\name{build_edge_list_safely}
\alias{build_edge_list_safely}
\title{Build Pathway Similarity Network Edges (Safe Version)}
\usage{
build_edge_list_safely(core_genes_list, min_shared_genes = 2)
}
\arguments{
\item{core_genes_list}{Named list: pathway_id -> core_gene_vector}

\item{min_shared_genes}{Minimum number of shared genes between pathways to form an edge}
}
\value{
Data frame with columns: from, to, shared, weight (Jaccard index),
  overlap_coef, dice_coef, shared_genes
}
\description{
Safely construct edge list for pathway network based on Jaccard
  similarity of core genes. Includes defensive checks for edge cases.
  Now includes Overlap Coefficient and Dice Coefficient for comprehensive
  similarity assessment.
}
\examples{
# Build edges from two pathways sharing 2 core genes
core_genes <- list(
  PATHWAY_A = c("GENE_A", "GENE_B", "GENE_C"),
  PATHWAY_B = c("GENE_B", "GENE_C", "GENE_D")
)
edges <- build_edge_list_safely(core_genes, min_shared_genes = 2)
print(edges)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/09_shiny_mod_table.R
\name{mod_master_table_ui}
\alias{mod_master_table_ui}
\title{Master Workspace Table UI}
\usage{
mod_master_table_ui(id)
}
\arguments{
\item{id}{Module ID}
}
\value{
Shiny UI tagList
}
\description{
Interactive data table module for displaying GSEA pathway results
  with checkbox selection and modal integration.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/07_shiny_app.R
\name{.process_addition_data}
\alias{.process_addition_data}
\title{Process Addition Data}
\usage{
.process_addition_data(addition_data)
}
\arguments{
\item{addition_data}{A data.frame or character vector (file path)}
}
\value{
A validated data.frame with addition data
}
\description{
Internal function to handle various addition_data input types.
  Supports data.frame objects or file paths (.csv, .rds formats).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_class_validations.R
\name{.validate_limma_design}
\alias{.validate_limma_design}
\title{Validate Limma Design Matrix}
\usage{
.validate_limma_design(fit)
}
\arguments{
\item{fit}{MArrayLM object from limma}
}
\value{
TRUE if validation passes, otherwise stops with an error
}
\description{
Enforce no-intercept design (~ 0 + group) to ensure accurate contrast parsing.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-accessors.R
\name{.get_expr_internal}
\alias{.get_expr_internal}
\title{Internal Expression Matrix Extraction Logic}
\usage{
.get_expr_internal(expr_bundle, backend_info, type = "default", ...)
}
\description{
Core logic for extracting and computing expression matrices.
  Handles backend-specific computations (DESeq2, limma-voom) and normalizes
  sample ordering between expression matrices and metadata.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-accessors.R
\name{creat_addition_data_rdsfile}
\alias{creat_addition_data_rdsfile}
\title{Create Additional Data RDS File}
\usage{
creat_addition_data_rdsfile(
  csv_path,
  output_dir = getwd(),
  output_name = "addition_data_gsealens.rds",
  overwrite = FALSE
)
}
\arguments{
\item{csv_path}{Character string specifying the input CSV file path.}

\item{output_dir}{Character string specifying the output directory path.
Default: current working directory.}

\item{output_name}{Character string specifying the output filename.
Default: \code{"addition_data_gsealens.rds"}}

\item{overwrite}{Logical scalar indicating whether to overwrite an existing
file. Default: \code{FALSE}.}
}
\value{
Invisible \code{TRUE} on success. The RDS file path is printed to
  the console.
}
\description{
Convert a CSV file to standardized gzip-compressed RDS format
  with validation. The output file will be saved to the specified directory
  with a standardized naming convention.
}
\examples{
# Build a small CSV in a temp directory, then convert to RDS.
tmp_csv <- file.path(tempdir(), "annotations_demo.csv")
utils::write.csv(
  data.frame(
    ID = c("HALLMARK_HYPOXIA", "HALLMARK_INFLAMMATORY_RESPONSE"),
    Brief_Description = c("Hypoxia", "Inflammatory response"),
    stringsAsFactors = FALSE
  ),
  tmp_csv, row.names = FALSE
)

out_rds <- file.path(tempdir(), "annotations_demo.rds")
creat_addition_data_rdsfile(
  tmp_csv,
  output_dir = tempdir(),
  output_name = "annotations_demo.rds",
  overwrite = TRUE
)
file.exists(out_rds)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-accessors.R
\name{get_expr_matrix}
\alias{get_expr_matrix}
\title{Get Expression Matrix}
\usage{
get_expr_matrix(obj, type = "default", ...)
}
\arguments{
\item{obj}{A GseaEnv or GseaRes object.}

\item{type}{Character string specifying the type of expression values to return.
Options include: \code{"default"} (default display matrix), \code{"raw"}
(raw counts), \code{"cpm"} (counts per million), \code{"logcpm"}
(log2-transformed CPM), \code{"vst"} (variance stabilizing transformation,
DESeq2 only), \code{"fpkm"} (fragments per kilobase per million),
\code{"tpm"} (transcripts per million), and \code{"lognorm"} (log2 of
normalized counts).}

\item{...}{Additional arguments passed to backend-specific methods.}
}
\value{
A numeric matrix with genes as rows and samples as columns.
}
\description{
Extract expression matrix from GseaEnv or GseaRes object, supporting
  multiple normalization methods including raw counts, CPM, logCPM, VST, FPKM,
  TPM, and log-normalized counts.
}
\examples{
# Load pre-computed inputs shipped with the package
data(preprocessed_limma, package = "GSEAlens")

precomp <- preprocessed_limma
data(gsea_pathwaysets_toy, package = "GSEAlens")

pathways <- gsea_pathwaysets_toy
gsea_env <- setup_gsea_env(
  fit = precomp$fit,
  pathway_obj = pathways,
  expr_data = precomp$gsea_limma_voom_data
)
expr <- get_expr_matrix(gsea_env)
dim(expr)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/11_shiny_mod_modal.R
\name{mod_pathway_modal_ui}
\alias{mod_pathway_modal_ui}
\title{Pathway Detail Modal UI}
\usage{
mod_pathway_modal_ui(id)
}
\description{
Pathway Detail Modal UI
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_setup.R
\name{inspect_gsea_env}
\alias{inspect_gsea_env}
\title{View GSEA Environment Object Overview}
\usage{
inspect_gsea_env(env_obj)
}
\arguments{
\item{env_obj}{GseaEnv object}
}
\value{
Invisibly returns the input object (for chaining).
}
\description{
Print detailed information about the GseaEnv object to the console, including backend type,
contrast list, and gene set status.
}
\examples{
# Build a GseaEnv from pre-computed inputs shipped with the package
data(preprocessed_limma, package = "GSEAlens")

precomp <- preprocessed_limma
data(gsea_pathwaysets_toy, package = "GSEAlens")

pathways <- gsea_pathwaysets_toy
env <- setup_gsea_env(
  fit       = precomp$fit,
  pathway_obj = pathways,
  expr_data = precomp$gsea_limma_voom_data
)
inspect_gsea_env(env)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/07_shiny_app.R
\name{.auto_detect_addition_data}
\alias{.auto_detect_addition_data}
\title{Auto-detect Addition Data}
\usage{
.auto_detect_addition_data()
}
\value{
A validated data.frame or NULL if no file found
}
\description{
Search working directory for default addition_data files.
  Checks for 'addition_data_gsealens.rds' or 'addition_data_gsealens.csv'
  in the current working directory.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_hubgene.R
\name{extract_hub_genes}
\alias{extract_hub_genes}
\title{Extract Hub Genes from Selected Pathways (Enhanced with Leading Edge)}
\usage{
extract_hub_genes(gsea_task, pathway_ids, min_degree = 2, de_df = NULL)
}
\arguments{
\item{gsea_task}{GseaTask object}

\item{pathway_ids}{Character vector of pathway IDs to analyze}

\item{min_degree}{Minimum number of pathways a gene must appear in (default: 2)}

\item{de_df}{Optional differential expression data frame with stat column}
}
\value{
A data frame with columns:
  \item{gene}{Gene symbol}
  \item{degree}{Number of pathways the gene appears in}
  \item{pathways}{Comma-separated list of pathway IDs}
  \item{is_hub}{Logical indicating if gene is a hub (degree >= min_degree)}
  \item{stat}{Statistic value from de_df or geneList}
  \item{pathway_leading_edges}{List column with per-pathway leading edge status}
}
\description{
Extract hub genes with their leading edge status for each pathway.
A gene is considered a hub if it appears in at least min_degree pathways.
}
\examples{
# Load pre-computed GseaRes shipped with the package and pick a task
data(precomputed_gseares, package = "GSEAlens")

gsea_res <- precomputed_gseares
task <- extract_gsea_task(gsea_res, contrast_id = "untrt_vs_trt")

# Pick top 3 enriched pathway IDs from the task result
res_df <- as.data.frame(task$gsea_res)
top_ids <- utils::head(res_df$ID, 3)

# Extract hub genes appearing in >= 2 of these pathways
hubs <- extract_hub_genes(task, top_ids, min_degree = 2)
utils::head(hubs)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_class_validations.R
\name{.check_gsea_res}
\alias{.check_gsea_res}
\title{Validate GseaRes Object Integrity}
\usage{
.check_gsea_res(res_obj)
}
\arguments{
\item{res_obj}{GseaRes object to validate}
}
\value{
TRUE if validation passes, otherwise stops with an error
}
\description{
Internal function to ensure GseaRes object contains required components.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/14_shiny_mod_joint_canvas.R
\name{mod_joint_canvas_server}
\alias{mod_joint_canvas_server}
\title{Joint GSEA Fill Canvas Module Server (Sidebar Control Version)}
\usage{
mod_joint_canvas_server(id, gsea_res, data_prep_list, table_result)
}
\arguments{
\item{id}{Character string identifying the module namespace.}

\item{gsea_res}{Reactive expression containing GSEA results.}

\item{data_prep_list}{Reactive list containing data preparation parameters
including contrasts, ncol, and custom colors.}

\item{table_result}{Reactive list containing pathway selection results.}
}
\description{
Server-side logic for the joint GSEA canvas display module.
  This module:
  \enumerate{
    \item Receives sidebar control parameters (arrangement mode)
    \item Applies no maximum row limit
    \item Maintains aesthetic consistency with multi_plot
    \item Generates combined visualization with multiple contrasts and pathways
    \item Provides R code export functionality for reproducibility
  }
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis-gsea-core.R
\name{.gsea_nb_core}
\alias{.gsea_nb_core}
\title{Internal GSEA Native Plotting Engine}
\usage{
.gsea_nb_core(
  object,
  geneSetID,
  subPlot = 3,
  curveCol = c("#76BA99", "#EB4747", "#996699"),
  htCol = c("#08519C", "#A50F15"),
  lineSize = 0.8,
  addPval = FALSE,
  pvalX = 0.9,
  pvalY = 0.9,
  pvalSize = 4,
  pCol = "grey0",
  nesDigit = 2,
  pDigit = 2,
  show_contrast_in_axis = FALSE,
  left_group = "Left",
  right_group = "Right",
  base_size = 14,
  subRatio = c(0.5, 0.2, 0.3),
  rankSeq = 5000,
  htHeight = 0.3,
  htAlpha = 0.8
)
}
\arguments{
\item{object}{gseaResult object.}

\item{geneSetID}{Character vector of pathway IDs.}

\item{subPlot}{Integer, 1/2/3.}

\item{curveCol}{Curve color vector.}

\item{htCol}{Heatmap gradient colors.}

\item{lineSize}{Line width for enrichment curve.}

\item{addPval}{Logical, add NES/Pvalue text annotation (single pathway only).}

\item{pvalX}{X position for pval text (relative fraction).}

\item{pvalY}{Y position for pval text (relative fraction).}

\item{pvalSize}{Text size for pval annotation.}

\item{pCol}{Color for pval text.}

\item{nesDigit}{NES rounding digits.}

\item{pDigit}{P-value rounding digits.}

\item{show_contrast_in_axis}{Logical, if TRUE shows "Left vs Right" on X-axis.}

\item{left_group}{Left group label.}

\item{right_group}{Right group label.}

\item{base_size}{Theme base size.}

\item{subRatio}{Relative heights for 3 panels.}

\item{rankSeq}{X-axis break interval for rank plot.}

\item{htHeight}{Relative height of heatmap strip inside Panel 2.}

\item{htAlpha}{Alpha for heatmap rectangles.}
}
\value{
Named list: p1 (curve), p2 (heatmap strip), p3 (rank distribution),
  data_ga, gsdata, glist.
}
\description{
Core plotting logic migrated from GseaVis::gseaNb,
  stripped of unused features (newGsea, filePath, KEGG, geneExpHt, etc.).
  Returns a list of ggplot objects for downstream assembly via patchwork.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04_calculation.R
\name{.derive_task_seed}
\alias{.derive_task_seed}
\title{Derive Deterministic Per-Task Seed}
\usage{
.derive_task_seed(seed, task_id, all_task_ids)
}
\arguments{
\item{seed}{Integer or NULL. Base seed chosen by the user.}

\item{task_id}{Character. Contrast task identifier.}

\item{all_task_ids}{Character vector. All task ids in the current run.}
}
\value{
Integer seed derived from the base seed and the task's rank in the
  alphabetically sorted task list, or NULL when `seed` is NULL, NA or
  non-numeric. Ranking
  against the sorted full task list (not the chunk-local order) makes the
  derived seed independent of `workers` and `chunk_size`.
}
\description{
Derive Deterministic Per-Task Seed
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/13_shiny_mod_pathway_relation.R
\name{mod_pathway_relation_ui}
\alias{mod_pathway_relation_ui}
\title{Pathway Relationship Exploration Module UI}
\usage{
mod_pathway_relation_ui(id)
}
\description{
Network visualization module with dual-mode pathway selection
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_class_validations.R
\name{.validate_deseq2_design}
\alias{.validate_deseq2_design}
\title{Validate DESeq2 Target Factor}
\usage{
.validate_deseq2_design(dds, target_factor)
}
\arguments{
\item{dds}{DESeqDataSet object}

\item{target_factor}{String, the target factor to validate}
}
\value{
TRUE if validation passes, otherwise stops with an error
}
\description{
Check if target_factor exists in colData and is properly formatted.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-accessors.R
\name{.detect_gene_symbol_column}
\alias{.detect_gene_symbol_column}
\title{Detect Gene Symbol Column}
\usage{
.detect_gene_symbol_column(gene_meta)
}
\arguments{
\item{gene_meta}{A data frame containing gene metadata.}
}
\value{
The detected column name if successful, or \code{NULL} if detection
  fails.
}
\description{
Automatically detect the gene symbol column in gene metadata.
  Uses housekeeping gene probe matching for robust cross-species detection.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-accessors.R
\name{get_sample_meta}
\alias{get_sample_meta}
\title{Get Sample Metadata}
\usage{
get_sample_meta(obj)
}
\arguments{
\item{obj}{GseaEnv or GseaRes object}
}
\value{
A data frame with sample metadata. Includes a standardized 'group'
  column derived from the target factor (inferred from dds_obj@design for
  DESeq2 workflows). Sample row names are preserved for downstream matching.
}
\description{
Extract and process sample metadata, handling DFrame conversion
  from DESeq2 and unifying group column naming conventions across backends.
}
\examples{
# Load a pre-computed GseaRes shipped with the package
data(precomputed_gseares, package = "GSEAlens")

gsea_res <- precomputed_gseares
meta <- get_sample_meta(gsea_res)
head(meta)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_class_validations.R
\name{class_validations}
\alias{class_validations}
\title{Core Class Definitions and Validation Functions}
\description{
Define GseaEnv, GseaRes, GseaTask structures and provide internal validation logic.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/10_shiny_mod_quadrant.R
\name{mod_quadrant_ui}
\alias{mod_quadrant_ui}
\title{Quadrant Linkage Module}
\usage{
mod_quadrant_ui(id)
}
\arguments{
\item{id}{Module ID}
}
\value{
Shiny UI tagList
}
\description{
Four-quadrant interactive visualization module for pathway and gene analysis.
  Provides synchronized pathway selection, gene ranking, differential expression volcano,
  and expression boxplot visualizations with bidirectional table-volcano synchronization.

UI components for the four-quadrant interactive visualization
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-accessors.R
\name{get_geneset_info}
\alias{get_geneset_info}
\title{Get Gene Set Information}
\usage{
get_geneset_info(obj)
}
\arguments{
\item{obj}{A GseaEnv or GseaRes object.}
}
\value{
An object containing geneset_info with pathway definitions and
  metadata dictionary.
}
\description{
Retrieve gene set collection metadata including pathway definitions
  and annotation dictionary.
}
\examples{
# Load a pre-computed GseaRes shipped with the package
data(precomputed_gseares, package = "GSEAlens")

gsea_res <- precomputed_gseares
gs_info <- get_geneset_info(gsea_res)
names(gs_info)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-accessors.R
\name{.get_display_symbols}
\alias{.get_display_symbols}
\title{Batch Get Display Gene Symbols}
\usage{
.get_display_symbols(genes, symbol_map)
}
\arguments{
\item{genes}{A character vector of gene identifiers.}

\item{symbol_map}{A case mapping vector generated by \code{.rebuild_symbol_map}.}
}
\value{
A character vector of gene symbols in their original case.
}
\description{
Batch Get Display Gene Symbols
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/12_shiny_mod_multi_plot.R
\name{mod_multi_plot_ui}
\alias{mod_multi_plot_ui}
\title{Combined Pathway Plotting UI}
\usage{
mod_multi_plot_ui(id)
}
\description{
Combined Pathway Plotting UI
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/06_visualization.R
\name{generate_gsea_html_report}
\alias{generate_gsea_html_report}
\title{Generate GSEA HTML Report}
\usage{
generate_gsea_html_report(
  res_obj,
  output_base_dir = NULL,
  p_adjust_cutoff = 1,
  top_plots = c(15, 15),
  dpi = 200
)
}
\arguments{
\item{res_obj}{GseaTask object}

\item{output_base_dir}{Output folder path. If left empty (NULL), it will automatically follow the original project location of the calculation capsule.}

\item{p_adjust_cutoff}{FDR filtering threshold, default 1}

\item{top_plots}{Number of pathways to plot detailed sub-pages, format c(positive_count, negative_count)}

\item{dpi}{Resolution for GSEA enrichment plots, default 200}
}
\value{
Path to the generated HTML report (invisible)
}
\description{
Automatically avoids column name contamination, intelligently extracts expression matrices from objects,
and outputs comfortable web reports with dual p-value display and heatmaps.
Uses the ComplexHeatmap engine to draw high-quality heatmaps, perfectly replicating pheatmap aesthetics.
}
\examples{
\donttest{
data(precomputed_gseares, package = "GSEAlens")
task <- extract_gsea_task(precomputed_gseares, contrast_id = "untrt_vs_trt")
report_path <- generate_gsea_html_report(task,
                                         output_base_dir = tempdir())
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/05_extraction.R
\name{import_gsea_capsule}
\alias{import_gsea_capsule}
\title{Load and Smart-Relocate GSEA Computation Capsule}
\usage{
import_gsea_capsule(file_path, auto_relocate = TRUE, inspect = TRUE)
}
\arguments{
\item{file_path}{Character. Absolute or relative path to the capsule rds file.}

\item{auto_relocate}{Logical. Whether to automatically fix the folder structure
and relocate based on the built-in lineage in the current directory? Default TRUE.}

\item{inspect}{Logical. Whether to automatically print capsule overview after
loading? Default TRUE.}
}
\value{
Returns the parsed GseaRes or GseaEnv capsule object.
}
\description{
Safely load a computation capsule. If the file is moved from its
  original project path (e.g., copied to desktop), the engine will automatically
  restore the standard folder structure in the current working directory and
  relocate the capsule. Fully compatible with GseaEnv and GseaRes objects.
}
\examples{
# Load the pre-computed GseaRes capsule shipped with the package, write it
# to a temp file, then import it (auto_relocate = FALSE to avoid side
# effects).
data(precomputed_gseares, package = "GSEAlens")
tmp_capsule <- file.path(tempdir(), "import_demo.rds")
saveRDS(precomputed_gseares, tmp_capsule)

gsea_res <- import_gsea_capsule(
  tmp_capsule,
  auto_relocate = FALSE,
  inspect = FALSE
)
class(gsea_res)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-accessors.R
\name{.rebuild_symbol_map}
\alias{.rebuild_symbol_map}
\title{Rebuild Gene Symbol Case Mapping}
\usage{
.rebuild_symbol_map(gsea_res, contrast_id)
}
\arguments{
\item{gsea_res}{A GseaRes object.}

\item{contrast_id}{Character string specifying the contrast ID.}
}
\value{
A named character vector where names are uppercase gene symbols and
  values are the original case. Returns \code{NULL} if mapping cannot be
  constructed.
}
\description{
Reconstruct gene symbol case mapping from differential expression
  results for display purposes.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/05_extraction.R
\name{extract_gsea_task}
\alias{extract_gsea_task}
\title{Extract GSEA Results for Specific Contrast}
\usage{
extract_gsea_task(gsea_res, contrast_id, target_collection = "ALL")
}
\arguments{
\item{gsea_res}{GseaRes object (generated by batch_calc_gsea)}

\item{contrast_id}{String. Contrast group ID (e.g., "A_vs_B").}

\item{target_collection}{String vector. Gene set collection to extract
(e.g., "H", "C2:CP:KEGG_LEGACY"). Default "ALL" means extract all.}
}
\value{
GseaTask object.
}
\description{
Extract results for a specified contrast from a GseaRes capsule.
  Supports subsetting by gene set collection and automatically recalculates FDR.
}
\examples{
# Load a pre-computed GseaRes shipped with the package
data(precomputed_gseares, package = "GSEAlens")

gsea_res <- precomputed_gseares
task <- extract_gsea_task(gsea_res, contrast_id = "untrt_vs_trt")
class(task)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-benchmark.R
\name{plot_gsea_memory}
\alias{plot_gsea_memory}
\title{Plot GSEA Benchmark Memory Curve}
\usage{
plot_gsea_memory(aligned_data, highlight_phases = TRUE)
}
\arguments{
\item{aligned_data}{Aligned data frame returned by align_benchmark_data}

\item{highlight_phases}{Whether to color by phase, default TRUE}
}
\value{
A ggplot object
}
\description{
Generates a standard memory usage curve plot for publication.
}
\examples{
# Build a small synthetic aligned-data frame (mimics the output of
# align_benchmark_data() without needing an external CSV or GseaRes object).
syn <- data.frame(
  timestamp_ms = seq(0, 20000, by = 1000),
  rss_mb = c(120, 150, 220, 280, 310, 330, 340, 335, 320, 305,
             290, 270, 255, 245, 240, 235, 230, 225, 220, 215, 210),
  relative_sec = seq(0, 20, by = 1),
  phase = c(rep("startup", 4), rep("core_compute", 12), rep("cleanup", 5))
)
attr(syn, "gsea_info") <- list(duration_sec = 20, workers = 2)
class(syn) <- c("GseaBenchmarkAligned", "data.frame")
plot_gsea_memory(syn)
plot_gsea_memory(syn, highlight_phases = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-accessors.R
\name{.get_display_symbol}
\alias{.get_display_symbol}
\title{Get Display Gene Symbol}
\usage{
.get_display_symbol(gene, symbol_map)
}
\arguments{
\item{gene}{A gene identifier (case-insensitive).}

\item{symbol_map}{A case mapping vector generated by \code{.rebuild_symbol_map}.}
}
\value{
The original case gene symbol, or the input if no mapping exists.
}
\description{
Retrieve the original case gene symbol based on the case mapping.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/11_shiny_mod_modal.R
\name{mod_pathway_modal_server}
\alias{mod_pathway_modal_server}
\title{Pathway Detail Modal Server}
\usage{
mod_pathway_modal_server(
  id,
  data_prep,
  trigger_event,
  gsea_res,
  pending_genes_reactive,
  update_pending_genes
)
}
\arguments{
\item{id}{Module ID}

\item{data_prep}{Reactive data from data prep module}

\item{trigger_event}{Event that triggers modal opening}

\item{gsea_res}{GseaRes object}

\item{pending_genes_reactive}{Function to get current pending genes}

\item{update_pending_genes}{Function to update pending genes list}
}
\description{
Modal with checkbox selection using standard DT approach.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_hubgene.R
\name{get_hubgene_legend}
\alias{get_hubgene_legend}
\title{Get Color Legend Configuration}
\usage{
get_hubgene_legend(left_group = "A", right_group = "B")
}
\arguments{
\item{left_group}{Left group name (e.g., treatment or case)}

\item{right_group}{Right group name (e.g., control or reference)}
}
\value{
A named list containing:
  \item{pathway_up}{Color for up-regulated pathways}
  \item{pathway_down}{Color for down-regulated pathways}
  \item{gene_up}{Color for up-regulated genes}
  \item{gene_down}{Color for down-regulated genes}
  \item{labels}{Named list of legend labels}
}
\description{
Generate color legend data for HubGene network plot.
}
\examples{
legend <- get_hubgene_legend("Control", "Treatment")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSEAlens-package.R
\docType{package}
\name{GSEAlens-package}
\alias{GSEAlens-package}
\alias{GSEAlens}
\title{GSEAlens: Gene Set Enrichment Analysis Interactive Explorer}
\description{
GSEAlens provides an interactive exploration layer on top of standard
Bioconductor RNA-seq workflows. It consumes fitted model objects from
limma (MArrayLM) or DESeq2 (DESeqDataSet) as input and accepts expression
matrices and sample metadata as SummarizedExperiment objects, ensuring
interoperability with core Bioconductor data containers.

For core computation, GSEAlens wraps [clusterProfiler::GSEA()] as its
statistical engine (thereby inheriting the fgsea fast GSEA methodology)
and draws on MSigDB gene set collections via the msigdbr package from CRAN.
Multi-contrast parallel computation is handled by future
(future::multisession).

Visualization output relies on Bioconductor graphics packages including
enrichplot, ComplexHeatmap, and circlize, producing figures suitable for
publication pipelines. The package also includes a built-in Shiny
application for interactive exploration of enrichment results after DEG
analysis, with the ability to export self-contained reproducible R scripts.
}
\details{
**Main entry points:**

- [setup_gsea_env()]: Initialize a GSEA analysis environment from limma or
  DESeq2 fitted objects.
- [batch_calc_gsea()]: Run GSEA across multiple contrasts in parallel.
- [build_gsea_pathways()]: Build MSigDB pathway database (interactive or
  automated selection).
- [extract_gsea_task()]: Extract a single contrast for downstream
  visualization and reporting.

**Built-in Shiny application:**

Launch the interactive explorer via [launch_gsea_app()] after DEG analysis.
The app provides pathway-level drill-down, multi-contrast joint canvases,
quadrant plots, hub-gene networks, and one-click export of self-contained
reproducible R scripts.

**Vignettes:**

- `vignette("GSEAlens", package = "GSEAlens")`: Main workflow
- `vignette("GSEAlens-preprocessing", package = "GSEAlens")`: Data
  preprocessing
}
\examples{
library(GSEAlens)
packageDescription("GSEAlens")$Version
}
\references{
<https://github.com/DDL095/GSEAlens>
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/DDL095/GSEAlens}
  \item Report bugs at \url{https://github.com/DDL095/GSEAlens/issues}
}

}
\author{
Shenhui Xu \email{sealgod@qq.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/16_shiny_mod_hubgene_vis.R
\name{mod_hubgene_vis_ui}
\alias{mod_hubgene_vis_ui}
\title{HubGene Network Module UI}
\usage{
mod_hubgene_vis_ui(id)
}
\arguments{
\item{id}{Character string used to namespace the module.}
}
\value{
A Shiny tagList containing the module's UI elements.
}
\description{
Shiny module UI for visualizing HubGene networks using visNetwork.
Provides interactive controls for pathway selection modes, physics
simulation parameters, node sizing, and network statistics display.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04_calculation.R
\name{.enrich_gsea_result}
\alias{.enrich_gsea_result}
\title{GSEA Result Metadata Injection}
\usage{
.enrich_gsea_result(result_df, meta_dict)
}
\arguments{
\item{result_df}{GSEA result dataframe}

\item{meta_dict}{Metadata dictionary}
}
\value{
Enriched dataframe
}
\description{
GSEA Result Metadata Injection
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_backends.R
\name{backends}
\alias{backends}
\title{Backend Data Extractor}
\description{
Extract standardized data structures from limma or DESeq2 objects.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-imports.R
\name{utils-imports}
\alias{utils-imports}
\title{GSEAlens Global Imports}
\description{
Centralized management of all external dependency imports to avoid redundant definitions
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/15_code_generator.R
\name{generate_hubgene_code}
\alias{generate_hubgene_code}
\title{Generate Publication HubGene Network Code}
\usage{
generate_hubgene_code(
  gsea_res_var = "gsea_res",
  contrast_id,
  pathways,
  min_hub_degree = 2,
  pw_size_mode = "setsize",
  seed = 42L,
  target_collection = "ALL",
  base_size = 12,
  margin_top = 18,
  margin_bottom = 18,
  margin_left = 18,
  margin_right = 18,
  fig_width = 10,
  fig_height = 8,
  fig_dpi = 300
)
}
\arguments{
\item{gsea_res_var}{Character. Name of the GseaRes variable.}

\item{contrast_id}{Character. Contrast ID to extract.}

\item{pathways}{Character vector. Pathway IDs to include.}

\item{min_hub_degree}{Integer. Minimum hub degree for gene inclusion.}

\item{pw_size_mode}{Character. One of "fixed", "fdr", "setsize".}

\item{seed}{Integer.}

\item{target_collection}{Character. Gene set collection(s) to extract.
Default \code{"ALL"}.}

\item{base_size}{Numeric.}

\item{margin_top}{Numeric. Top plot margin (mm).}

\item{margin_bottom}{Numeric. Bottom plot margin (mm).}

\item{margin_left}{Numeric. Left plot margin (mm).}

\item{margin_right}{Numeric. Right plot margin (mm).}

\item{fig_width}{Numeric. Figure width in inches for ggsave.}

\item{fig_height}{Numeric. Figure height in inches for ggsave.}

\item{fig_dpi}{Numeric. DPI for PNG raster output (default 300).}
}
\value{
Character scalar of executable R code.
}
\description{
Return an executable R script reproducing the HubGene module's
  visNetwork view as a static ggplot2 figure. Assumes a \code{GseaRes}
  object is already loaded. Node size is controlled by the same three-mode
  logic (fixed / fdr / setsize) as the interactive view.
}
\examples{
code <- generate_hubgene_code(
  gsea_res_var = "gsea_res", contrast_id = "untrt_vs_trt",
  pathways = c("HALLMARK_TNFA_SIGNALING_VIA_NFKB"))
cat(code)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/08_shiny_mod_data_prep.R
\name{mod_data_prep_ui}
\alias{mod_data_prep_ui}
\title{Data Preprocessing Module UI}
\usage{
mod_data_prep_ui(id)
}
\arguments{
\item{id}{Module namespace ID}
}
\description{
User interface for the data preprocessing module, providing controls for:
\itemize{
  \item Contrast selection and gene set subgroup filtering
  \item Joint GSEA canvas for multi-pathway visualization
  \item Differential expression gene marker selection
  \item Group display order customization
  \item Expression metrics configuration
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{gsea_pathwaysets_toy}
\alias{gsea_pathwaysets_toy}
\title{Toy MSigDB pathway collection (Hallmark + KEGG_LEGACY)}
\format{
A list with elements `TERM2GENE`, `meta_dict`, `SuperTag`,
  `collections_used`, `species`.
}
\source{
Assembled from msigdbr (Hallmark + KEGG_LEGACY collections).
}
\description{
A lightweight pathway object for examples and vignettes,
  containing 236 pathways (Hallmark + KEGG_LEGACY) pre-assembled by
  [GSEAlens::build_gsea_pathways()]. See
  `inst/scripts/make_gsea_pathwaysets_toy.R` for regeneration.
}
\examples{
data(gsea_pathwaysets_toy, package = "GSEAlens")
names(gsea_pathwaysets_toy)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_backends.R
\name{.build_expr_bundle}
\alias{.build_expr_bundle}
\title{Build Expression Data Bundle}
\usage{
.build_expr_bundle(obj, backend)
}
\description{
Uniformly encapsulate expression matrix and metadata
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_class_validations.R
\name{.check_gsea_env}
\alias{.check_gsea_env}
\title{Validate GseaEnv Object Integrity}
\usage{
.check_gsea_env(env_obj)
}
\arguments{
\item{env_obj}{GseaEnv object to validate}
}
\value{
TRUE if validation passes, otherwise stops with an error
}
\description{
Internal function to ensure object structure conforms to specification.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/07_shiny_app.R
\name{.launch_gsea_shiny}
\alias{.launch_gsea_shiny}
\title{Launch GSEAlens Shiny App (Internal)}
\usage{
.launch_gsea_shiny(gsea_res, addition_data)
}
\arguments{
\item{gsea_res}{A valid GseaRes object}

\item{addition_data}{Optional data.frame with pathway annotations}
}
\value{
A Shiny app object
}
\description{
Core Shiny application launcher. Builds the UI layout and
  initializes all server modules for interactive GSEA analysis.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_hubgene.R
\name{build_hubgene_network}
\alias{build_hubgene_network}
\title{Build HubGene Network Data Structure (Enhanced with Leading Edge)}
\usage{
build_hubgene_network(
  gsea_task,
  pathway_ids,
  min_hub_degree = 2,
  de_df = NULL,
  res_df = NULL,
  seed = 123
)
}
\arguments{
\item{gsea_task}{GseaTask object}

\item{pathway_ids}{Character vector of selected pathway IDs}

\item{min_hub_degree}{Minimum degree threshold for hub genes (default: 2)}

\item{de_df}{Optional DE data frame containing stat column}

\item{res_df}{Optional pathway result data frame}

\item{seed}{Random seed for reproducibility (default: 123)}
}
\value{
A list with three components:
  \item{nodes}{A list containing pathway and gene data frames}
  \item{edges}{Edge data frame with source, target, and leading edge info}
  \item{hub_df}{Hub gene data frame from extraction}
}
\description{
Build complete network data with leading edge information for edges.
Now properly passes de_df to extract_hub_genes for stat extraction.
}
\examples{
# Load pre-computed GseaRes shipped with the package and pick a task
data(precomputed_gseares, package = "GSEAlens")

gsea_res <- precomputed_gseares
task <- extract_gsea_task(gsea_res, contrast_id = "untrt_vs_trt")

# Pick top 3 enriched pathway IDs from the task result
res_df <- as.data.frame(task$gsea_res)
top_ids <- utils::head(res_df$ID, 3)

# Build the hub-gene network from these pathways
network <- build_hubgene_network(task, top_ids)
names(network)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/07_shiny_app.R
\name{.validate_addition_data}
\alias{.validate_addition_data}
\title{Validate Addition Data}
\usage{
.validate_addition_data(data)
}
\arguments{
\item{data}{A data.frame to validate}
}
\value{
Validated data.frame with character ID column
}
\description{
Internal function to validate addition_data structure.
  Checks that input is a data.frame with required 'ID' column and
  removes duplicate entries if present.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-core-genes.R
\name{get_core_genes_list}
\alias{get_core_genes_list}
\title{Batch Extract Core Genes for Multiple Pathways}
\usage{
get_core_genes_list(gsea_task_obj, pathway_ids)
}
\arguments{
\item{gsea_task_obj}{GseaTask object (single contrast)}

\item{pathway_ids}{Character vector, pathway ID list}
}
\value{
Named list, names = pathway_id, values = core_genes vector
}
\description{
Batch data preparation for 03 module Network/UpSet/Chord visualizations.
}
\examples{
# Load a pre-computed GseaRes and extract a task
data(precomputed_gseares, package = "GSEAlens")

gsea_res <- precomputed_gseares
task <- extract_gsea_task(gsea_res, contrast_id = "untrt_vs_trt")
res_df <- as.data.frame(task$gsea_res)
pw_ids <- head(res_df$ID, 3)
genes_list <- get_core_genes_list(task, pw_ids)
length(genes_list)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/15_code_generator.R
\name{generate_volcano_code}
\alias{generate_volcano_code}
\title{Generate Publication Volcano Code}
\usage{
generate_volcano_code(
  gsea_res_var = "gsea_res",
  contrast_id,
  left_group = "Left",
  right_group = "Right",
  n_selected = 0L,
  selected_ids = character(0),
  show_annotations = FALSE,
  target_collection = "ALL",
  base_size = 12,
  margin_top = 15,
  margin_bottom = 15,
  margin_left = 18,
  margin_right = 18,
  fig_width = 8,
  fig_height = 6,
  fig_dpi = 300
)
}
\arguments{
\item{gsea_res_var}{Character. Name of the GseaRes variable.}

\item{contrast_id}{Character. Contrast ID to extract.}

\item{left_group}{Character.}

\item{right_group}{Character.}

\item{n_selected}{Integer. Number of selected pathways (subtitle).}

\item{selected_ids}{Character vector. Pathway IDs for ggrepel annotation.}

\item{show_annotations}{Logical. Add ggrepel labels for selected pathways.}

\item{target_collection}{Character. Gene set collection(s) to extract.
Default \code{"ALL"}.}

\item{base_size}{Numeric.}

\item{margin_top}{Numeric. Top plot margin (mm).}

\item{margin_bottom}{Numeric. Bottom plot margin (mm).}

\item{margin_left}{Numeric. Left plot margin (mm).}

\item{margin_right}{Numeric. Right plot margin (mm).}

\item{fig_width}{Numeric. Figure width in inches for ggsave.}

\item{fig_height}{Numeric. Figure height in inches for ggsave.}

\item{fig_dpi}{Numeric. DPI for PNG raster output (default 300).}
}
\value{
Character scalar of executable R code.
}
\description{
Return an executable ggplot2 script reproducing the Quadrant
  module's pathway volcano plot as a static, publication-ready figure.
  Assumes a \code{GseaRes} object is already loaded in the R environment.
}
\examples{
code <- generate_volcano_code(
  gsea_res_var = "gsea_res", contrast_id = "untrt_vs_trt",
  left_group = "Ctrl", right_group = "Trt")
cat(code)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_setup.R
\name{setup_gsea_env}
\alias{setup_gsea_env}
\title{Initialize GSEA Environment Object}
\usage{
setup_gsea_env(
  fit,
  pathway_obj,
  expr_data = NULL,
  target_factor = NULL,
  verbose = TRUE
)
}
\arguments{
\item{fit}{Differential analysis object. Supports \code{MArrayLM} (limma) or \code{DESeqDataSet} (DESeq2).}

\item{pathway_obj}{Gene set object. Typically generated by \code{build_gsea_pathways()}.}

\item{expr_data}{Optional. Expression matrix or DGEList. If NULL, extraction from fit object will be attempted (partial support only).}

\item{target_factor}{Only for DESeq2. Specifies the target factor of interest. If NULL, automatically inferred as the last term in the design formula.}

\item{verbose}{Logical. If TRUE (default), print progress messages. Set to FALSE for silent batch-mode use.}
}
\value{
Returns a standardized object of class \code{GseaEnv}.
}
\description{
Unified entry point supporting limma-voom (no intercept design) and DESeq2 objects.
Automatically extracts contrasts, differential analysis results, and expression matrices to generate
a standardized GseaEnv object.
}
\examples{
# Limma-voom workflow using pre-computed inputs shipped with the package
data(preprocessed_limma, package = "GSEAlens")

precomp <- preprocessed_limma
data(gsea_pathwaysets_toy, package = "GSEAlens")

pathways <- gsea_pathwaysets_toy
env <- setup_gsea_env(
  fit       = precomp$fit,
  pathway_obj = pathways,
  expr_data = precomp$gsea_limma_voom_data
)
class(env)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-core-genes.R
\name{validate_param}
\alias{validate_param}
\title{Safe Parameter Validation with Fallback (Global Strategy)}
\usage{
validate_param(
  value,
  default,
  min_val = 1,
  max_val = NULL,
  param_name = "parameter"
)
}
\arguments{
\item{value}{Input value}

\item{default}{Default value}

\item{min_val}{Minimum value (inclusive)}

\item{max_val}{Maximum value (inclusive, optional)}

\item{param_name}{Parameter name (for warning messages)}
}
\value{
Validated value
}
\description{
Unified handling of invalid parameters (e.g., ncol <= 0) to ensure
  03/04 module stability.
}
\examples{
workers <- validate_param(4, default = 2, min_val = 1, max_val = 8, param_name = "workers")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/15_code_generator.R
\name{generate_joint_canvas_code}
\alias{generate_joint_canvas_code}
\title{Generate Joint Canvas Code}
\usage{
generate_joint_canvas_code(
  GSEAlens_res,
  contrast_ids,
  target_pathways,
  ncol = 2
)
}
\arguments{
\item{GSEAlens_res}{GseaRes object}

\item{contrast_ids}{Character vector. Multiple contrast IDs}

\item{target_pathways}{Character vector. Pathway IDs}

\item{ncol}{Integer. Number of columns}
}
\value{
A character string containing executable R code
}
\description{
Generate Joint Canvas Code
}
\examples{
\donttest{
data(precomputed_gseares, package = "GSEAlens")
task <- extract_gsea_task(precomputed_gseares, contrast_id = "untrt_vs_trt")
target <- as.data.frame(task$gsea_res)$ID[1]
code <- generate_joint_canvas_code(GSEAlens_res = precomputed_gseares,
                                   contrast_ids = "untrt_vs_trt",
                                   target_pathways = target)
cat(code)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/03_pathways.R
\name{build_gsea_pathways}
\alias{build_gsea_pathways}
\title{Build GSEA Pathway Database with Interactive Selection}
\usage{
build_gsea_pathways(species = "HS", auto_select = NULL)
}
\arguments{
\item{species}{Character string specifying the species. Options:
\itemize{
  \item "HS": Homo sapiens (human, default)
  \item "MM": Mus musculus (mouse)
}}

\item{auto_select}{Optional parameter for automated selection. Can be:
\itemize{
  \item NULL: Launch interactive menu (default)
  \item "ALL": Load entire MSigDB database for the specified species
  \item Character vector: Collection names (e.g., c("H", "C5:GO:BP") for human)
}}
}
\value{
A list object containing:
  \itemize{
    \item TERM2GENE: Data frame mapping pathway IDs to gene symbols
    \item meta_dict: Metadata dictionary with descriptions, URLs, and collection info
    \item SuperTag: Intelligent batch identifier for the selected collections
    \item collections_used: Data frame of selected collection metadata
    \item species: Species identifier ("HS" or "MM")
  }
}
\description{
Provides an interactive or automated gene set selection interface
  based on msigdbr, generating standardized pathway objects with intelligent
  semantic tagging. Supports both human (Homo sapiens) and mouse (Mus musculus)
  species.
}
\examples{
\donttest{
# Automated selection by collection name -- safe for non-interactive use
pathways <- build_gsea_pathways(species = "HS",
                                auto_select = c("H", "C5:GO:BP"))

# MM uses M-prefixed collection codes (MH, M5:GO:BP, ...)
pathways_mm <- build_gsea_pathways(species = "MM",
                                   auto_select = c("MH", "M5:GO:BP"))
}

\dontrun{
# Interactive menu -- only runs when user is at a terminal
if (interactive()) {
  pathways <- build_gsea_pathways(species = "HS")
}

# Load ALL collections -- downloads the entire MSigDB (slow, needs network)
pathways <- build_gsea_pathways(species = "HS", auto_select = "ALL")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-accessors.R
\name{get_de_table}
\alias{get_de_table}
\title{Get Differential Expression Table}
\usage{
get_de_table(obj, contrast_id)
}
\arguments{
\item{obj}{A GseaRes object containing differential expression results.}

\item{contrast_id}{Character string specifying the contrast ID (e.g., "A_vs_B").
If the reverse contrast exists in storage, it will be retrieved and
logFC/stat signs will be inverted.}
}
\value{
A data frame containing at minimum: gene_symbol, logFC, and p-value
  columns. Additional columns (stat, pvalue, padj, t) may be present depending
  on the analysis backend.
}
\description{
Extract differential expression results for a specific contrast
  from the stored results. Automatically handles reverse contrasts (e.g.,
  B_vs_A) by retrieving the forward contrast and flipping the sign of logFC.
}
\examples{
# Load a pre-computed GseaRes shipped with the package
data(precomputed_gseares, package = "GSEAlens")

gsea_res <- precomputed_gseares
de_result <- get_de_table(gsea_res, contrast_id = "untrt_vs_trt")
head(de_result)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-accessors.R
\name{.convert_expr_rownames}
\alias{.convert_expr_rownames}
\title{Convert Expression Matrix Row Names to Gene Symbols}
\usage{
.convert_expr_rownames(expr_mat, gene_meta, gsea_res)
}
\arguments{
\item{expr_mat}{An expression matrix.}

\item{gene_meta}{A gene metadata data frame containing both ID and symbol columns.}

\item{gsea_res}{A GseaRes object (used to access backend information).}
}
\value{
The expression matrix with updated row names.
}
\description{
Automatically detect and convert expression matrix row names,
  supporting Ensembl ID to gene symbol conversion.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/15_code_generator.R
\name{generate_de_volcano_code}
\alias{generate_de_volcano_code}
\title{Generate Publication DE Volcano Code}
\usage{
generate_de_volcano_code(
  gsea_res_var = "gsea_res",
  contrast_id,
  user_genes = character(0),
  pathway_genes = character(0),
  logfc_thresh = 1,
  pval_thresh = 0.05,
  left_group = "Left",
  right_group = "Right",
  show_annotations = FALSE,
  base_size = 12,
  margin_top = 15,
  margin_bottom = 15,
  margin_left = 18,
  margin_right = 18,
  fig_width = 8,
  fig_height = 6,
  fig_dpi = 300
)
}
\arguments{
\item{gsea_res_var}{Character. Name of the GseaRes variable.}

\item{contrast_id}{Character. Contrast ID for DE table extraction.}

\item{user_genes}{Character vector of user-selected genes (upper-cased).}

\item{pathway_genes}{Character vector of pathway genes (upper-cased).}

\item{logfc_thresh}{Numeric. Default 1.}

\item{pval_thresh}{Numeric. Default 0.05.}

\item{left_group}{Character.}

\item{right_group}{Character.}

\item{show_annotations}{Logical. Add ggrepel labels.}

\item{base_size}{Numeric.}

\item{margin_top}{Numeric. Top plot margin (mm).}

\item{margin_bottom}{Numeric. Bottom plot margin (mm).}

\item{margin_left}{Numeric. Left plot margin (mm).}

\item{margin_right}{Numeric. Right plot margin (mm).}

\item{fig_width}{Numeric. Figure width in inches for ggsave.}

\item{fig_height}{Numeric. Figure height in inches for ggsave.}

\item{fig_dpi}{Numeric. DPI for PNG raster output (default 300).}
}
\value{
Character scalar of executable R code.
}
\description{
Return an executable ggplot2 script reproducing the Quadrant
  module's gene-level Differential Expression Volcano as a static figure.
  Assumes a \code{GseaRes} object is already loaded. Six-color classification
  (both / user / pathway / up / down / ns) mirrors the interactive view.
}
\examples{
code <- generate_de_volcano_code(
  gsea_res_var = "gsea_res", contrast_id = "untrt_vs_trt",
  left_group = "Ctrl", right_group = "Trt")
cat(code)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-core-genes.R
\name{get_core_genes_for_pathway}
\alias{get_core_genes_for_pathway}
\title{Extract Pathway Core Genes (Leading Edge)}
\usage{
get_core_genes_for_pathway(gsea_res_obj, pathway_id)
}
\arguments{
\item{gsea_res_obj}{GseaRes object or single GSEA result (list(status=, data=, ...))}

\item{pathway_id}{Character, pathway ID}
}
\value{
Character vector, core genes (uppercase normalized)
}
\description{
Parse the core_enrichment field from GSEA results to extract the gene list.
  This is the fundamental data interface for all visualization modules in 03/04.
}
\examples{
# Load a pre-computed GseaRes and extract a task
data(precomputed_gseares, package = "GSEAlens")

gsea_res <- precomputed_gseares
task <- extract_gsea_task(gsea_res, contrast_id = "untrt_vs_trt")
res_df <- as.data.frame(task$gsea_res)
pw_id <- res_df$ID[1]
core_genes <- get_core_genes_for_pathway(task, pw_id)
head(core_genes)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_backends.R
\name{.extract_limma_data}
\alias{.extract_limma_data}
\title{Extract Limma-Voom Data}
\usage{
.extract_limma_data(fit, expr_data = NULL)
}
\arguments{
\item{fit}{MArrayLM object (must have been processed with eBayes)}

\item{expr_data}{Optional. DGEList or expression matrix.}
}
\value{
A list containing contrast_registry, de_store, and expr_bundle
}
\description{
Extract contrast groups and differential analysis results from MArrayLM object.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-core-genes.R
\name{get_term_genes}
\alias{get_term_genes}
\title{Get Full Pathway Gene Set (TERM2GENE)}
\usage{
get_term_genes(gsea_res, pathway_id)
}
\arguments{
\item{gsea_res}{GseaRes object}

\item{pathway_id}{Pathway ID}
}
\value{
Character vector
}
\description{
Extract complete gene list for a pathway from geneset_info.
}
\examples{
# Load a pre-computed GseaRes shipped with the package
data(precomputed_gseares, package = "GSEAlens")

gsea_res <- precomputed_gseares
pw_id <- gsea_res$geneset_info$term2gene$gs_name[1]
genes <- get_term_genes(gsea_res, pw_id)
head(genes)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/14_shiny_mod_joint_canvas.R
\name{mod_joint_canvas_ui}
\alias{mod_joint_canvas_ui}
\title{Joint GSEA Fill Canvas Module UI (Display-Only Version)}
\usage{
mod_joint_canvas_ui(id)
}
\description{
All controls have been moved to the sidebar; only canvas results are displayed here.
  This module provides a display interface for joint GSEA visualization results
  with export functionality for reproducible R code generation.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/15_code_generator.R
\name{generate_boxplot_image_code}
\alias{generate_boxplot_image_code}
\title{Generate Publication Boxplot Image Code}
\usage{
generate_boxplot_image_code(
  box_data_long,
  gene_symbol = "Gene",
  expr_type = "Expression",
  left_group = NA_character_,
  right_group = NA_character_,
  use_zero_baseline = TRUE,
  custom_order = "default",
  base_size = 12,
  margin_top = 15,
  margin_bottom = 15,
  margin_left = 18,
  margin_right = 18,
  fig_width = 5,
  fig_height = 5,
  fig_dpi = 300
)
}
\arguments{
\item{box_data_long}{data.frame with columns: Sample, Group, Expression.
(Same long format returned by \code{extract_boxplot_data()$long}.)}

\item{gene_symbol}{Character. Gene name for the plot title.}

\item{expr_type}{Character. Y-axis label (e.g. "logcpm", "log2FC").}

\item{left_group}{Character. First group name (gets red fill).}

\item{right_group}{Character. Second group name (gets blue fill).}

\item{use_zero_baseline}{Logical. Add dashed red line at y = 0.}

\item{custom_order}{Character scalar. Custom group ordering
(e.g. "A->B,C") or "default".}

\item{base_size}{Numeric.}

\item{margin_top}{Numeric. Top plot margin (mm).}

\item{margin_bottom}{Numeric. Bottom plot margin (mm).}

\item{margin_left}{Numeric. Left plot margin (mm).}

\item{margin_right}{Numeric. Right plot margin (mm).}

\item{fig_width}{Numeric. Figure width in inches for ggsave.}

\item{fig_height}{Numeric. Figure height in inches for ggsave.}

\item{fig_dpi}{Numeric. DPI for PNG raster output (default 300).}
}
\value{
Character scalar of executable R code.
}
\description{
Return an executable ggplot2 script reproducing the Quadrant
  module's per-gene expression boxplot (panel 4) as a static,
  publication-ready figure. Mirrors the interactive plotly: boxplot +
  jittered points, group colors, optional zero baseline, optional custom
  group order.
}
\examples{
\donttest{
# Build a minimal long-format expression data frame (self-contained)
box_long <- data.frame(
  Sample     = c("S1", "S2", "S3", "S4"),
  Group      = factor(c("untrt", "untrt", "trt", "trt"),
                      levels = c("untrt", "trt")),
  Expression = c(8.1, 8.3, 9.5, 9.7)
)
code <- generate_boxplot_image_code(
  box_data_long = box_long,
  gene_symbol   = "GAPDH",
  expr_type     = "logcpm",
  left_group    = "untrt",
  right_group   = "trt"
)
cat(code)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{preprocessed_limma}
\alias{preprocessed_limma}
\title{Pre-computed limma-voom fit and DGEList from the airway dataset}
\format{
A list with elements `fit` and `gsea_limma_voom_data`.
}
\source{
Built from the `airway` Bioconductor dataset via limma-voom.
}
\description{
A list with elements `fit` (`MArrayLM`) and
  `gsea_limma_voom_data` (filtered `DGEList`), suitable as input to
  [GSEAlens::setup_gsea_env()] for the limma-voom workflow.
}
\examples{
data(preprocessed_limma, package = "GSEAlens")
names(preprocessed_limma)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{preprocessed_dds_se}
\alias{preprocessed_dds_se}
\title{Pre-computed DESeqDataSet (SummarizedExperiment form)}
\format{
A `DESeqDataSet` object.
}
\source{
Built from the `airway` Bioconductor dataset via DESeq2.
}
\description{
A `DESeqDataSet` based on a `SummarizedExperiment` input,
  suitable for demonstrating the SE-based entry point of
  [GSEAlens::setup_gsea_env()].
}
\examples{
data(preprocessed_dds_se, package = "GSEAlens")
class(preprocessed_dds_se)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-accessors.R
\name{read_addition_data}
\alias{read_addition_data}
\title{Read Additional Data}
\usage{
read_addition_data(file_path)
}
\arguments{
\item{file_path}{Character string specifying the file path. Supported formats:
\itemize{
  \item \code{.csv} / \code{.csv.gz} - Comma-separated values (optionally
    gzip compressed)
  \item \code{.rds} / \code{.rds.gz} - R serialized data (optionally gzip
    compressed)
}}
}
\value{
A data frame with validated ID column. Rows are deduplicated by ID,
  keeping the first occurrence of each unique ID.
}
\description{
Read and validate additional data from CSV or RDS file. Supports
  gzip-compressed formats (.gz). The data frame must contain an 'ID' column
  as the primary key for joining with pathway results.
}
\examples{
# Build a small CSV in a temp directory and read it back.
tmp_csv <- file.path(tempdir(), "pathway_annotations_demo.csv")
utils::write.csv(
  data.frame(
    ID = c("HALLMARK_HYPOXIA", "HALLMARK_INFLAMMATORY_RESPONSE"),
    Brief_Description = c("Hypoxia", "Inflammatory response"),
    stringsAsFactors = FALSE
  ),
  tmp_csv, row.names = FALSE
)

add_data <- read_addition_data(tmp_csv)
add_data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/05_extraction.R
\name{inspect_gsea_res}
\alias{inspect_gsea_res}
\title{View GSEA Result Overview}
\usage{
inspect_gsea_res(gsea_res)
}
\arguments{
\item{gsea_res}{GseaRes object}
}
\value{
Invisibly returns the input object (for chaining).
}
\description{
Print detailed summary of GseaRes object to console.
}
\examples{
# Load a pre-computed GseaRes shipped with the package
data(precomputed_gseares, package = "GSEAlens")

gsea_res <- precomputed_gseares
inspect_gsea_res(gsea_res)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-accessors.R
\name{utils-accessors}
\alias{utils-accessors}
\title{Data Accessor Utilities}
\description{
Provides unified data extraction interface, isolating underlying
  object structure differences. Fixes DESeq2 sample name matching issues.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-accessors.R
\name{get_contrast_registry}
\alias{get_contrast_registry}
\title{Get Contrast Registry}
\usage{
get_contrast_registry(obj)
}
\arguments{
\item{obj}{A GseaEnv or GseaRes object.}
}
\value{
The contrast registry object stored in the input object.
}
\description{
Retrieve the registered contrasts used for differential expression
  analysis, including contrast definitions and metadata.
}
\examples{
# Load a pre-computed GseaRes shipped with the package
data(precomputed_gseares, package = "GSEAlens")

gsea_res <- precomputed_gseares
registry <- get_contrast_registry(gsea_res)
print(registry)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/15_code_generator.R
\name{generate_dotplot_code}
\alias{generate_dotplot_code}
\title{Generate Publication DotPlot Code}
\usage{
generate_dotplot_code(
  gsea_res_var = "gsea_res",
  contrast_id,
  pathways,
  color_mode = "padj",
  size_mode = "core_size",
  left_group = "Left",
  right_group = "Right",
  palette = "D",
  point_range = c(3, 8),
  target_collection = "ALL",
  base_size = 12,
  margin_top = 18,
  margin_bottom = 18,
  margin_left = 18,
  margin_right = 18,
  fig_width = 9,
  fig_height = 7,
  fig_dpi = 300
)
}
\arguments{
\item{gsea_res_var}{Character. Name of the GseaRes variable in the user's
environment (default \code{"gsea_res"}).}

\item{contrast_id}{Character. Contrast ID to extract.}

\item{pathways}{Character vector. Pathway IDs to display.}

\item{color_mode}{Character. One of "padj", "pval", "nes".}

\item{size_mode}{Character. One of "core_size", "setsize".}

\item{left_group}{Character. Left contrast label.}

\item{right_group}{Character. Right contrast label.}

\item{palette}{Character. Viridis option letter ("A"-"H").}

\item{point_range}{Numeric length 2. Min/max point size in mm.}

\item{target_collection}{Character. Gene set collection(s) to extract
(e.g. \code{"H"}, \code{c("C2", "C2:CP")}). Default \code{"ALL"}.}

\item{base_size}{Numeric. theme_bw base font size.}

\item{margin_top}{Numeric. Top plot margin (mm).}

\item{margin_bottom}{Numeric. Bottom plot margin (mm).}

\item{margin_left}{Numeric. Left plot margin (mm).}

\item{margin_right}{Numeric. Right plot margin (mm).}

\item{fig_width}{Numeric. Figure width in inches for ggsave.}

\item{fig_height}{Numeric. Figure height in inches for ggsave.}

\item{fig_dpi}{Numeric. DPI for PNG raster output (default 300).}
}
\value{
Character scalar of executable R code.
}
\description{
Return an executable ggplot2 script reproducing the Pathway
  Relationship module's dotplot as a static, publication-ready figure.
  The generated script assumes a \code{GseaRes} object is already loaded
  in the R environment and derives plot data via \code{extract_gsea_task()}
  + \code{get_core_genes_list()}, producing concise reproducible code
  without embedding raw data.
}
\examples{
code <- generate_dotplot_code(
  gsea_res_var = "gsea_res", contrast_id = "untrt_vs_trt",
  pathways = c("HALLMARK_TNFA_SIGNALING_VIA_NFKB"))
cat(code)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/10_shiny_mod_quadrant.R
\name{mod_quadrant_server}
\alias{mod_quadrant_server}
\title{Quadrant Linkage Server}
\usage{
mod_quadrant_server(id, data_prep_list, gsea_res, table_controller)
}
\arguments{
\item{id}{Module ID}

\item{data_prep_list}{List from data prep module}

\item{gsea_res}{GseaRes object}

\item{table_controller}{Table controller with update_selection method}
}
\description{
Server logic with unified gene pool and bidirectional table-volcano sync.
  Now properly responds to table selection changes.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_hubgene.R
\name{prepare_hubgene_nodes}
\alias{prepare_hubgene_nodes}
\title{Prepare HubGene Nodes for Plotting}
\usage{
prepare_hubgene_nodes(network_data, layout = "fr", seed = 42)
}
\arguments{
\item{network_data}{Output from \code{\link{build_hubgene_network}}}

\item{layout}{Layout algorithm: "fr" (Fruchterman-Reingold),
"kk" (Kamada-Kawai), or "circle"}

\item{seed}{Random seed for reproducibility (default: 42)}
}
\value{
A named list with three components:
  \item{pathway}{Pathway node data frame with coordinates}
  \item{gene}{Gene node data frame with coordinates}
  \item{all}{Combined node data frame}
}
\description{
Prepare node data with layout positions for plotly visualization.
}
\examples{
# Build a minimal network_data object (2 pathways + 2 shared genes)
network_data <- list(
  nodes = list(
    pathway = data.frame(
      id = c("PATHWAY_A", "PATHWAY_B"), type = "pathway",
      NES = c(2.0, -1.5), direction = c("up", "down"),
      stringsAsFactors = FALSE
    ),
    gene = data.frame(
      id = c("GENE_X", "GENE_Y"), type = "gene",
      stringsAsFactors = FALSE
    )
  ),
  edges = data.frame(
    from = c("PATHWAY_A", "PATHWAY_A", "PATHWAY_B", "PATHWAY_B"),
    to   = c("GENE_X", "GENE_Y", "GENE_X", "GENE_Y"),
    stringsAsFactors = FALSE
  ),
  hub_df = data.frame(gene = c("GENE_X", "GENE_Y"), degree = c(2, 2),
                      stringsAsFactors = FALSE)
)
nodes <- prepare_hubgene_nodes(network_data, layout = "fr", seed = 42)
if (!is.null(nodes)) head(nodes$all[, c("id", "type")])
}
